@@ -1,4 +1,4 @@
-/* $Id: about.cpp,v 1.96 2005/02/12 22:19:58 ydirson Exp $ */
+/* $Id: about.cpp,v 1.113 2005/04/09 21:59:00 ettin Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -15,12 +15,12 @@
 #include "events.hpp"
 #include "font.hpp"
 #include "game_config.hpp"
+#include "gettext.hpp"
 #include "image.hpp"
 #include "key.hpp"
-#include "language.hpp"
 #include "display.hpp"
-#include "show_dialog.hpp"
 #include "sdl_utils.hpp"
+#include "video.hpp"
 #include "widgets/button.hpp"
 
 #include <sstream>
@@ -146,6 +146,7 @@ std::vector<std::string> get_text() {
 		"_" N_("+Basque Translation"),
 		"-   Alfredo Beaumont (ziberpunk)",
 		"-   Julen Landa (genars)",
+		"-   Mikel Olasagasti (Hey_neken)",
 		"+ ",
 
 		"_" N_("+Bulgarian Translation"),
@@ -160,6 +161,10 @@ std::vector<std::string> get_text() {
 		"-   Pau Rul·lan Ferragut",
 		"+ ",
 
+		"_" N_("+Chinese Translation"),
+		"-   Lim Choon Kiat",
+		"+ ",
+
 		"_" N_("+Czech Translation"),
 		"-   David Nečas (Yeti)",
 		"-   Mintaka",
@@ -176,14 +181,20 @@ std::vector<std::string> get_text() {
 
 		"_" N_("+Dutch Translation"),
 		"-   Lala",
+		"-   Maarten Albrecht",
 		"-   Pieter Vermeylen (Onne)",
 		"+ ",
 
 		"_" N_("+English (GB) Translation"),
 		"-   ott",
 		"+ ",
+		
+		"_" N_("+Estonian Translation"),
+		"-   Mart Tõnso",
+		"+ ",
 
 		"_" N_("+Finnish Translation"),
+		"-   Ankka",
 		"-   paxed",
 		"+ ",
 
@@ -192,6 +203,7 @@ std::vector<std::string> get_text() {
 		"-   Cédric Duval",
 		"-   DaringTremayne",
 		"-   Guillaume Duwelz-Rebert",
+		"-   Guillaume Massart",
 		"-   Guillaume Melquiond (silene)",
 		"-   Jean-Luc Richard (Le Gnome)",
 		"-   Philippe Plantier (Ayin)",
@@ -204,10 +216,11 @@ std::vector<std::string> get_text() {
 		"-   Andre Schmidt (schmidta)",
 		"-   Boris Stumm (quijote_)",
 		"-   Christoph Berg (chrber)",
-		"-   Gerfried Fuchs (Rhonda)",
+		"-   Gerfried Fuchs (Alfie)",
 		"-   Jan Heiner Laberenz (jan-heiner)",
 		"-   Kai Ensenbach (Pingu)",
 		"-   Nils Kneuper (Ivanovic)",
+		"-   Ruben Philipp Wickenhäuser (The Very Uhu)",
 		"-   Stephan Grochtmann (Schattenstephan)",
 		"+ ",
 
@@ -234,12 +247,19 @@ std::vector<std::string> get_text() {
 		"-   RokStar",
 		"+ ",
 		
+		"_" N_("+Japanese Translation"),
+		"-   Naoki Iimura",
+		"-   Nobuhito Okada",
+		"-   Yuji Matsumoto",
+		"+ ",
+
 		"_" N_("+Latin Translation"),
 		"-   Mark Polo (mpolo)",
 		"+ ",
 
 		"_" N_("+Norwegian Translation"),
 		"-   Hallvard Norheim Bø (Lysander)",
+		"-   Håvard Korsvoll",
 		"-   Erik J. Mesoy (Circon)",
 		"-   Susanne Mesoy (Rarlgland)",
 		"+ ",
@@ -247,7 +267,9 @@ std::vector<std::string> get_text() {
 		"_" N_("+Polish Translation"),
 		"-   Artur R. Czechowski",
 		"-   BOrsuk",
+                "-   Karol Nowak (grzywacz)",
 		"-   methinks",
+		"-   Michał Jedynak (Artanis)",
 		"-   Paweł Stradomski",
 		"-   Paweł Tomak",
 		"+ ",
@@ -258,6 +280,7 @@ std::vector<std::string> get_text() {
 		"-   Claus Aranha",
 		"-   Michel Loos",
 		"-   Renato Cunha",
+		"-   Sérgio de Miranda Costa",
 		"+ ",
 
 		"_" N_("+Russian Translation"),
@@ -269,9 +292,14 @@ std::vector<std::string> get_text() {
 		"_" N_("+Slovak Translation"),
 		"-   Viliam Bur",
 		"+ ",
+		
+		"_" N_("+Slovenian Translation"),
+		"-   Jaka Kranjc (lynx)",
+		"+ ",
 
 		"_" N_("+Spanish Translation"),
 		"-   David Martínez",
+		"-   Flamma",
 		"-   Franciso Muñoz (fmunoz)",
 		"-   Gabriel Rodríguez (Chewie)",
 		"-   Jose Gordillo (kilder)",
@@ -306,27 +334,28 @@ std::vector<std::string> get_text() {
 	return res;
 }
 
-void show_about(display& disp)
+void show_about(display &disp)
 {
+	CVideo &video = disp.video();
 	std::vector<std::string> text = get_text();
-	SDL_Rect rect = {0, 0, disp.x(), disp.y()};
+	SDL_Rect rect = {0, 0, video.getx(), video.gety()};
 
-	const surface_restorer restorer(&disp.video(), rect);
+	const surface_restorer restorer(&video, rect);
 
 	// Clear the screen
-	gui::draw_solid_tinted_rectangle(0,0,disp.x()-1,disp.y()-1,
-	                                 0,0,0,1.0,disp.video().getSurface());
+	draw_solid_tinted_rectangle(0,0,video.getx()-1,video.gety()-1,
+	                                 0,0,0,1.0,video.getSurface());
 	update_whole_screen();
 
 	const surface map_image(image::get_image(game_config::map_image,image::UNSCALED));
 	SDL_Rect map_rect;
-	map_rect.x = disp.x()/2 - map_image->w/2;
-	map_rect.y = disp.y()/2 - map_image->h/2;
+	map_rect.x = video.getx()/2 - map_image->w/2;
+	map_rect.y = video.gety()/2 - map_image->h/2;
 	map_rect.w = map_image->w;
 	map_rect.h = map_image->h;
 
-	gui::button close(disp,_("Close"));
-	close.set_location((disp.x()/2)-(close.width()/2), map_rect.y+map_rect.h+15);
+	gui::button close(video,_("Close"));
+	close.set_location((video.getx()/2)-(close.width()/2), map_rect.y+map_rect.h+15);
 
 
 	//substitute in the correct control characters for '+' and '-'
@@ -362,7 +391,7 @@ void show_about(display& disp)
 
 	do {
 		// draw map to screen, thus erasing all text
-		SDL_BlitSurface(map_image,&middle_src,disp.video().getSurface(),&middle_dest);
+		SDL_BlitSurface(map_image,&middle_src,video.getSurface(),&middle_dest);
 
 		// draw one screen full of text
 		const int line_spacing = 5;
@@ -371,7 +400,7 @@ void show_about(display& disp)
 		int cur_line = 0;
 
 		do {
-			SDL_Rect tr = font::draw_text(&disp,disp.screen_area(),font::SIZE_XLARGE,font::BLACK_COLOUR,
+			SDL_Rect tr = font::draw_text(&video,screen_area(),font::SIZE_XLARGE,font::BLACK_COLOUR,
 					              text[line], map_rect.x + map_rect.w / 8,y);
 			if(is_new_line) {
 				is_new_line = false;
@@ -399,8 +428,8 @@ void show_about(display& disp)
 		// mask off the upper and lower half of the map,
 		// so text will scroll into view rather than
 		// suddenly appearing out of nowhere
-		SDL_BlitSurface(map_image,&upper_src,disp.video().getSurface(),&upper_dest);
-		SDL_BlitSurface(map_image,&lower_src,disp.video().getSurface(),&lower_dest);
+		SDL_BlitSurface(map_image,&upper_src,video.getSurface(),&upper_dest);
+		SDL_BlitSurface(map_image,&lower_src,video.getSurface(),&lower_dest);
 
 		// handle events
 		events::pump();
@@ -409,7 +438,7 @@ void show_about(display& disp)
 
 		// update screen and wait, so the text does not scroll too fast
 		update_rect(map_rect);
-		disp.video().flip();
+		disp.flip();
 		SDL_Delay(20);
 
 	} while(!close.pressed());
