@@ -1,4 +1,4 @@
-/* $Id: about.hpp,v 1.12 2004/07/20 11:56:58 gakusho Exp $ */
+/* $Id: about.hpp,v 1.15 2005/03/24 21:35:50 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,14 +14,15 @@
 #ifndef ABOUT_H_INCLUDED
 #define ABOUT_H_INCLUDED
 
-#include "display.hpp"
+class display;
+
 #include <vector>
 #include <string>
 
 namespace about
 {
 
-void show_about(display& disp);
+void show_about(display &disp);
 std::vector<std::string> get_text();
 
 }
