@@ -1,4 +1,4 @@
-/* $Id: actions.cpp,v 1.190 2005/02/09 23:32:01 ydirson Exp $ */
+/* $Id: actions.cpp,v 1.204 2005/04/09 22:22:19 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -16,11 +16,13 @@
 #include "actions.hpp"
 #include "display.hpp"
 #include "events.hpp"
-#include "game.hpp"
 #include "game_config.hpp"
+#include "game_errors.hpp"
 #include "game_events.hpp"
+#include "gettext.hpp"
+#include "halo.hpp"
+#include "hotkeys.hpp"
 #include "key.hpp"
-#include "language.hpp"
 #include "log.hpp"
 #include "map.hpp"
 #include "pathfind.hpp"
@@ -32,7 +34,10 @@
 #include "statistics.hpp"
 #include "unit_display.hpp"
 #include "util.hpp"
+#include "video.hpp"
 #include "wassert.hpp"
+#include "wml_separators.hpp"
+#include "serialization/string_utils.hpp"
 #include "widgets/menu.hpp"
 
 #include <cmath>
@@ -40,8 +45,8 @@
 #include <string>
 #include <sstream>
 
-#define LOG_NG lg::info(lg::engine)
-#define ERR_NW lg::err(lg::network)
+#define LOG_NG LOG_STREAM(info, engine)
+#define ERR_NW LOG_STREAM(err, network)
 
 struct castle_cost_calculator : cost_calculator
 {
@@ -260,8 +265,15 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 	}
 
 	static const std::string plague_string("plague");
-	res.attacker_plague = !d->second.type().not_living() && (res.attacker_special == plague_string)
-	                      && !map.is_village(defender);
+	res.attacker_plague = !d->second.type().not_living() && 
+	  (res.attacker_special.substr(0,6) == plague_string) && 
+	  strcmp(d->second.type().undead_variation().c_str(),"null") && 
+	  !map.is_village(defender);
+	if(res.attacker_special.size()>8){ //plague(type) used
+	  res.attacker_plague_type=res.attacker_special.substr(7,res.attacker_special.size()-8);
+	}else{//plague type is that of attacker
+	  res.attacker_plague_type= a->second.type().id();
+	}
 	res.defender_plague = false;
 
 	static const std::string slow_string("slow");
@@ -309,9 +321,6 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 	if (counterattack) {
 		const attack_type& defend = defender_attacks[defend_with];
 		res.defender_special = defend.special();
-		if (res.defender_special == to_the_death_string) {
-			res.to_the_death = true;
-		}
 
 		//magical attacks always have a 70% chance to hit
 		if (res.defender_special == magical_string) {
@@ -401,8 +410,16 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 			res.amount_defender_drains = 0;
 		}
 
-		res.defender_plague = !a->second.type().not_living() && (res.defender_special == plague_string)
-		                      && !map.is_village(attacker);
+		res.defender_plague = !a->second.type().not_living() && 
+		  (res.defender_special.substr(0,6) == plague_string) &&
+		  strcmp(a->second.type().undead_variation().c_str(),"null") &&
+		  !map.is_village(attacker);
+		if(res.defender_special.size()>8){ //plague(type) used
+		        res.defender_plague_type=res.defender_special.substr(7,res.defender_special.size()-8);
+		}else{//plague type is that of attacker
+		        res.defender_plague_type= d->second.type().id();
+		}
+
 		res.defender_slows = (defend.special() == slow_string);
 
 		static const std::string first_strike = "firststrike";
@@ -523,7 +540,7 @@ battle_stats evaluate_battle_stats(const gamemap& map,
 static std::string unit_dump(std::pair< gamemap::location, unit > const &u)
 {
 	std::stringstream s;
-	s << u.second.type().name() << " (" << u.first.x + 1 << ',' << u.first.y + 1 << ')';
+	s << u.second.type().id() << " (" << u.first.x + 1 << ',' << u.first.y + 1 << ')';
 	return s.str();
 }
 
@@ -653,7 +670,7 @@ void attack(display& gui, const gamemap& map,
 				}
 			}
 
-			if(dies) {
+			if(dies) {//attacker kills defender
 				attackerxp = game_config::kill_experience*d->second.type().level();
 				if(d->second.type().level() == 0)
 					attackerxp = game_config::kill_experience/2;
@@ -665,6 +682,7 @@ void attack(display& gui, const gamemap& map,
 
 				gamemap::location loc = d->first;
 				gamemap::location attacker_loc = a->first;
+				std::string undead_variation = d->second.type().undead_variation();
 				const int defender_side = d->second.side();
 				LOG_NG << "firing die event\n";
 				game_events::fire("die",loc,a->first);
@@ -676,12 +694,29 @@ void attack(display& gui, const gamemap& map,
 					d = units.end();
 				}
 
-				//plague units make clones of themselves on the target hex
+				//plague units make new units on the target hex
 				if(stats.attacker_plague) {
-					a = units.find(attacker_loc);
-					if(a != units.end()) {
-						units.insert(std::pair<gamemap::location,unit>(loc,a->second));
-						gui.draw_tile(loc.x,loc.y);
+				        a = units.find(attacker_loc); 
+				        game_data::unit_type_map::const_iterator reanimitor;
+				        LOG_NG<<"trying to reanimate "<<stats.attacker_plague_type<<std::endl;
+				        reanimitor = info.unit_types.find(stats.attacker_plague_type);
+				        LOG_NG<<"found unit type:"<<reanimitor->second.id()<<std::endl;
+					
+					if(reanimitor != info.unit_types.end()) {
+					       unit newunit=unit(&reanimitor->second,a->second.side(),false,true);
+					       //apply variation					       
+					       if(strcmp(undead_variation.c_str(),"null")){
+						 config mod;
+						 config& variation=mod.add_child("effect");
+						 variation["apply_to"]="variation";
+						 variation["name"]=undead_variation;
+						 newunit.add_modification("variation",mod);
+					       }
+
+					       units.insert(std::pair<gamemap::location,unit>(loc,newunit));
+					       gui.draw_tile(loc.x,loc.y);
+					}else{
+					       LOG_NG<<"unit not reanimated"<<std::endl;
 					}
 				}
 				recalculate_fog(map,state,info,units,teams,defender_side-1);
@@ -799,7 +834,7 @@ void attack(display& gui, const gamemap& map,
 				}
 			}
 				
-			if(dies) {
+			if(dies) {//defender kills attacker
 				defenderxp = game_config::kill_experience*a->second.type().level();
 				if(a->second.type().level() == 0)
 					defenderxp = game_config::kill_experience/2;
@@ -809,6 +844,7 @@ void attack(display& gui, const gamemap& map,
 				defenderxp = 0;
 				attackerxp = 0;
 
+				std::string undead_variation = a->second.type().undead_variation();
 				gamemap::location loc = a->first;
 				gamemap::location defender_loc = d->first;
 				const int attacker_side = a->second.side();
@@ -821,12 +857,29 @@ void attack(display& gui, const gamemap& map,
 					a = units.end();
 				}
 
-				//plague units make clones of themselves on the target hex.
+				//plague units make new units on the target hex.
 				if(stats.defender_plague) {
-					d = units.find(defender_loc);
-					if(d != units.end()) {
-						units.insert(std::pair<gamemap::location,unit>(loc,d->second));
-						gui.draw_tile(loc.x,loc.y);
+				        d = units.find(defender_loc); 
+				        game_data::unit_type_map::const_iterator reanimitor;
+				        LOG_NG<<"trying to reanimate "<<stats.defender_plague_type<<std::endl;
+				        reanimitor = info.unit_types.find(stats.defender_plague_type);
+				        LOG_NG<<"found unit type:"<<reanimitor->second.id()<<std::endl;
+					
+					if(reanimitor != info.unit_types.end()) {
+					       unit newunit=unit(&reanimitor->second,d->second.side(),false,true);
+					       //apply variation
+					       if(strcmp(undead_variation.c_str(),"null")){
+						 config mod;
+						 config& variation=mod.add_child("effect");
+						 variation["apply_to"]="variation";
+						 variation["name"]=undead_variation;
+						 newunit.add_modification("variation",mod);
+					       }
+
+					       units.insert(std::pair<gamemap::location,unit>(loc,newunit));
+					       gui.draw_tile(loc.x,loc.y);
+					}else{
+					       LOG_NG<<"unit not reanimated"<<std::endl;
 					}
 				}
 				gui.recalculate_minimap();
@@ -1277,7 +1330,7 @@ unit get_advanced_unit(const game_data& info,
 	if(new_type != info.unit_types.end() && un != units.end()) {
 		return unit(&(new_type->second),un->second);
 	} else {
-		throw gamestatus::game_error("Could not find the unit being advanced"
+		throw game::game_error("Could not find the unit being advanced"
 		                             " to: " + advance_to);
 	}
 }
@@ -1293,8 +1346,9 @@ void advance_unit(const game_data& info,
 	const unit& new_unit = get_advanced_unit(info,units,loc,advance_to);
 
 	statistics::advance_unit(new_unit);
-	preferences::encountered_units().insert(new_unit.type().name());
-	lg::info(lg::config) << "Added '" << new_unit.type().name() << "' to encountered units\n";
+
+	preferences::encountered_units().insert(new_unit.type().id());
+	LOG_STREAM(info, config) << "Added '" << new_unit.type().id() << "' to encountered units\n";
 
 	units.erase(loc);
 	units.insert(std::pair<gamemap::location,unit>(loc,new_unit));
@@ -1637,7 +1691,11 @@ size_t move_unit(display* disp, const game_data& gamedata,
 	std::vector<gamemap::location>::const_iterator begin = route.begin();
 
 	std::vector<gamemap::location> steps(begin,step);
-	while(!steps.empty() && units.count(steps.back()) != 0) {
+	while (!steps.empty()) {
+		gamemap::location const &loc = steps.back();
+		if (units.count(loc) == 0)
+			break;
+		moves_left += u.movement_cost(map, map[loc.x][loc.y]);
 		steps.pop_back();
 	}
 
@@ -1678,7 +1736,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 	if(map.is_village(steps.back())) {
 		orig_village_owner = village_owner(steps.back(),teams);
 
-		if(orig_village_owner != team_num) {
+		if (size_t(orig_village_owner) != team_num) {
 			ui->second.set_movement(0);
 			get_village(steps.back(),teams,team_num,units);
 		}
@@ -1749,7 +1807,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 				msg_id = N_("Units sighted! ($friends friendly, $enemies enemy)");
 			}
 
-			string_map symbols;
+			utils::string_map symbols;
 			symbols["friends"] = lexical_cast<std::string>(nfriends);
 			symbols["enemies"] = lexical_cast<std::string>(nenemies);
 
@@ -1764,7 +1822,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 					msg << '\n' << _("(press $hotkey to continue)");
 				}
 			}
-			const std::string message = config::interpolate_variables_into_string(msg.str(),&symbols);
+			const std::string message = utils::interpolate_variables_into_string(msg.str(), &symbols);
 
 			font::add_floating_label(message,font::SIZE_XLARGE,font::BAD_COLOUR,
 			                         disp->map_area().w/2,disp->map_area().h/3,
