@@ -1,4 +1,4 @@
-/* $Id: ai.cpp,v 1.138 2005/02/17 23:47:12 j_daniel Exp $ */
+/* $Id: ai.cpp,v 1.146 2005/03/28 19:08:58 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -17,8 +17,6 @@
 #include "ai.hpp"
 #include "ai2.hpp"
 //#include "advanced_ai.hpp"
-#include "ai_attack.hpp"
-#include "ai_move.hpp"
 #include "dialogs.hpp"
 #include "game_config.hpp"
 #include "game_events.hpp"
@@ -29,7 +27,6 @@
 #include "playturn.hpp"
 #include "preferences.hpp"
 #include "replay.hpp"
-#include "show_dialog.hpp"
 #include "statistics.hpp"
 #include "unit_display.hpp"
 #include "util.hpp"
@@ -37,7 +34,7 @@
 
 #include <iostream>
 
-#define LOG_AI lg::info(lg::ai)
+#define LOG_AI LOG_STREAM(info, ai)
 
 ///a trivial ai that sits around doing absolutely nothing
 class idle_ai : public ai_interface {
@@ -198,7 +195,7 @@ ai_interface* create_ai(const std::string& name, ai_interface::info& info)
 	else if(name == "ai2")
 		return new ai2(info);
 	else if(name != "")
-		lg::err(lg::ai) << "AI not found: '" << name << "'\n";
+		LOG_STREAM(err, ai) << "AI not found: '" << name << "'\n";
 
 	return new ai(info);
 }
@@ -227,7 +224,7 @@ bool ai::recruit_usage(const std::string& usage)
 	for(std::map<std::string,unit_type>::const_iterator i =
 	    gameinfo_.unit_types.begin(); i != gameinfo_.unit_types.end(); ++i) {
 
-		const std::string& name = i->second.name();
+		const std::string& name = i->second.id();
 
 		if(i->second.usage() == usage && recruits.count(name)
 		   && current_team().gold() - i->second.cost() > min_gold
@@ -352,6 +349,17 @@ void ai_interface::sync_network()
 }
 
 gamemap::location ai_interface::move_unit(location from, location to, std::map<location,paths>& possible_moves)
+{
+	const location loc = move_unit_partial(from,to,possible_moves);
+	const unit_map::iterator u = info_.units.find(loc);
+	if(u != info_.units.end()) {
+		u->second.set_movement(0);
+	}
+
+	return loc;
+}
+
+gamemap::location ai_interface::move_unit_partial(location from, location to, std::map<location,paths>& possible_moves)
 {
 	LOG_AI << "ai_interface::move_unit " << (from.x + 1) << "," << (from.y + 1)
 		<< " -> " << (to.x + 1) << "," << (to.y + 1) << "\n";
@@ -366,14 +374,13 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 	log_scope2(ai, "move_unit");
 	unit_map::iterator u_it = info_.units.find(from);
 	if(u_it == info_.units.end()) {
-		lg::err(lg::ai) << "Could not find unit at " << from.x << ", " << from.y << "\n";
+		LOG_STREAM(err, ai) << "Could not find unit at " << from.x << ", " << from.y << "\n";
 		wassert(false);
 		return location();
 	}
 
 	if(from == to) {
 		LOG_AI << "moving unit at " << (from.x+1) << "," << (from.y+1) << " on spot. resetting moves\n";
-		u_it->second.set_movement(0);
 		return to;
 	}
 
@@ -398,6 +405,8 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 		}
 
 		if(rt != p.routes.end()) {
+			current_unit.set_movement(rt->second.move_left);
+
 			std::vector<location> steps = rt->second.steps;
 
 			if(steps.empty() == false) {
@@ -421,6 +430,7 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 					}
 
 					if(n != 6) {
+						current_unit.set_movement(0); //enter enemy ZoC, no movement left
 						break;
 					}
 				}
@@ -446,7 +456,6 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 		info_.units.erase(u_it);
 	}
 
-	current_unit.set_movement(0);
 	info_.units.insert(std::pair<location,unit>(to,current_unit));
 	if(info_.map.is_village(to)) {
 		get_village(to,info_.teams,info_.team_num-1,info_.units);
@@ -457,14 +466,14 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 		info_.disp.draw();
 	}
 
+	recorder.add_movement(from,to);
+
 	game_events::fire("moveto",to);
 
 	if((info_.teams.front().uses_fog() || info_.teams.front().uses_shroud()) && !info_.teams.front().fogged(to.x,to.y)) {
 		game_events::fire("sighted",to);
 	}
 
-	recorder.add_movement(from,to);
-
 	sync_network();
 
 	return to;
@@ -717,7 +726,7 @@ void ai::find_threats()
 	for(config::child_list::const_iterator j = protected_units.begin(); j != protected_units.end(); ++j) {
 
 		for(unit_map::const_iterator u = units_.begin(); u != units_.end(); ++u) {
-			if(game_events::unit_matches_filter(u,**j)) {
+			if(game_events::unit_matches_filter(u, *j)) {
 				items.push_back(protected_item(lexical_cast_default<double>((**j)["value"],1.0),lexical_cast_default<int>((**j)["radius"],20),u->first));
 			}
 		}
@@ -902,7 +911,7 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 
 		const location arrived_at = move_unit(from,to,possible_moves);
 		if(arrived_at != to) {
-			lg::warn(lg::ai) << "unit moving to attack has ended up unexpectedly at "
+			LOG_STREAM(warn, ai) << "unit moving to attack has ended up unexpectedly at "
 				<< (arrived_at.x + 1) << "," << (arrived_at.y + 1) << " when moving to "
 				<< (to.x + 1) << "," << (to.y + 1) << " moved from "
 				<< (from.x + 1) << "," << (from.y + 1) << "\n";
@@ -931,7 +940,7 @@ void ai_interface::attack_enemy(const location& u, const location& target, int w
 
 	if(info_.units.count(u) && info_.units.count(target)) {
 		if(info_.units.find(target)->second.stone()) {
-			lg::err(lg::ai) << "attempt to attack unit that is turned to stone\n";
+			LOG_STREAM(err, ai) << "attempt to attack unit that is turned to stone\n";
 			return;
 		}
 
@@ -1260,7 +1269,7 @@ bool ai::retreat_units(std::map<gamemap::location,paths>& possible_moves, const
 				}
 
 				if(best_pos.valid()) {
-					LOG_AI << "retreating '" << i->second.type().name() << "' " << i->first.x << ","
+					LOG_AI << "retreating '" << i->second.type().id() << "' " << i->first.x << ","
 						<< i->first.y << " -> " << best_pos.x << "," << best_pos.y << "\n";
 					move_unit(i->first,best_pos,possible_moves);
 					return true;
@@ -1336,7 +1345,7 @@ bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move
 		//we didn't arrive at our intended destination. We return true, meaning that
 		//the AI algorithm should be recalculated from the start.
 		if(arrived_at != move.second) {
-			lg::warn(lg::ai) << "didn't arrive at destination\n";
+			LOG_STREAM(warn, ai) << "didn't arrive at destination\n";
 			return true;
 		}
 
@@ -1368,7 +1377,7 @@ int ai::average_resistance_against(const unit_type& a, const unit_type& b) const
 
 	defense /= weighting_sum;
 
-	LOG_AI << "average defense of '" << a.name() << "': " << defense << "\n";
+	LOG_AI << "average defense of '" << a.id() << "': " << defense << "\n";
 
 	int sum = 0, weight_sum = 0;
 
@@ -1396,7 +1405,7 @@ int ai::compare_unit_types(const unit_type& a, const unit_type& b) const
 	const int a_effectiveness_vs_b = average_resistance_against(b,a);
 	const int b_effectiveness_vs_a = average_resistance_against(a,b);
 
-	LOG_AI << "comparison of '" << a.name() << " vs " << b.name() << ": "
+	LOG_AI << "comparison of '" << a.id() << " vs " << b.id() << ": "
 	          << a_effectiveness_vs_b << " - " << b_effectiveness_vs_a << " = "
 			  << (a_effectiveness_vs_b - b_effectiveness_vs_a) << "\n";
 	return a_effectiveness_vs_b - b_effectiveness_vs_a;
@@ -1554,8 +1563,8 @@ void ai::analyze_potential_recruit_movements()
 	for(std::map<std::string,int>::iterator j = unit_movement_scores_.begin(); j != unit_movement_scores_.end(); ++j) {
 		const game_data::unit_type_map::const_iterator info = gameinfo_.unit_types.find(j->first);
 		if(info == gameinfo_.unit_types.end()) {
+			continue;
 		}
-		continue;
 
 		const int best_score = best_scores[info->second.usage()];
 		if(best_score > 0) {
