@@ -1,4 +1,4 @@
-/* $Id: ai.hpp,v 1.56 2004/09/19 20:42:35 Sirp Exp $ */
+/* $Id: ai.hpp,v 1.57 2004/11/28 10:29:30 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -104,10 +104,9 @@ protected:
 
 	struct attack_analysis
 	{
-		void analyze(const gamemap& map, std::map<location,unit>& units,
-		             const gamestatus& status, const game_data& info, int sims,
-					 class ai& ai_obj, const move_map& dstsrc, const move_map& srcdst,
-					 const move_map& enemy_dstsrc, const move_map& enemy_srcdst);
+		void analyze(const gamemap& map, std::map<location,unit>& units, int sims,
+		             class ai& ai_obj, const move_map& dstsrc, const move_map& srcdst,
+		             const move_map& enemy_dstsrc, const move_map& enemy_srcdst);
 
 		double rating(double aggression, class ai& ai_obj) const;
 
