@@ -1,4 +1,4 @@
-/* $Id: ai_attack.cpp,v 1.58 2004/12/31 21:01:37 isaaccp Exp $ */
+/* $Id: ai_attack.cpp,v 1.61 2005/03/26 17:10:32 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,8 +14,9 @@
 #include "global.hpp"
 
 #include "actions.hpp"
-#include "ai_attack.hpp"
+#include "ai.hpp"
 #include "game_config.hpp"
+#include "gamestatus.hpp"
 #include "log.hpp"
 #include "pathfind.hpp"
 #include "util.hpp"
@@ -26,7 +27,7 @@
 #include <iostream>
 #include <set>
 
-#define LOG_AI lg::info(lg::ai)
+#define LOG_AI LOG_STREAM(info, ai)
 
 const int max_positions = 10000;
 
@@ -257,7 +258,7 @@ int ai::choose_weapon(const location& att, const location& def,
 		cur_stats = cache_itor->stats;
 
 		if(!(size_t(cache_itor->weapon) < itor->second.attacks().size())) {
-			lg::err(lg::ai) << "cached illegal weapon: " << cache_itor->weapon
+			LOG_STREAM(err, ai) << "cached illegal weapon: " << cache_itor->weapon
 			          << "/" << itor->second.attacks().size() << "\n";
 		}
 
