@@ -1,4 +1,4 @@
-/* $Id: ai_move.cpp,v 1.56 2005/01/21 23:17:15 Sirp Exp $ */
+/* $Id: ai_move.cpp,v 1.60 2005/03/27 23:06:16 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,8 +13,7 @@
 
 #include "global.hpp"
 
-#include "ai_move.hpp"
-#include "display.hpp"
+#include "ai.hpp"
 #include "game_config.hpp"
 #include "log.hpp"
 #include "map.hpp"
@@ -23,7 +22,7 @@
 
 #include <iostream>
 
-#define LOG_AI lg::info(lg::ai)
+#define LOG_AI LOG_STREAM(info, ai)
 
 struct move_cost_calculator : cost_calculator
 {
@@ -411,7 +410,7 @@ std::pair<gamemap::location,gamemap::location> ai::choose_move(std::vector<targe
 
 	//guardian units stay put
 	if(u->second.is_guardian()) {
-		LOG_AI << u->second.type().name() << " is guardian, staying still\n";
+		LOG_AI << u->second.type().id() << " is guardian, staying still\n";
 		return std::pair<location,location>(u->first,u->first);
 	}
 
