@@ -1,4 +1,4 @@
-/* $Id: builder.cpp,v 1.40 2004/12/31 21:01:37 isaaccp Exp $ */
+/* $Id: builder.cpp,v 1.49 2005/03/26 17:10:32 silene Exp $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -14,13 +14,16 @@
 #include "global.hpp"
 
 #include "array.hpp"
-#include "display.hpp"
+#include "builder.hpp"
+#include "config.hpp"
 #include "log.hpp"
+#include "pathutils.hpp"
 #include "terrain.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
-#define ERR_NG lg::err(lg::engine)
+#define ERR_NG LOG_STREAM(err, engine)
 
 namespace {
 
@@ -53,12 +56,12 @@ const int terrain_builder::rule_image::UNITPOS = 36 + 18;
 terrain_builder::rule_image::rule_image(int layer, bool global_image) : 
 	position(HORIZONTAL), layer(layer),
 	basex(0), basey(0), global_image(global_image)
-{};
+{}
 
 terrain_builder::rule_image::rule_image(int x, int y, bool global_image) : 
 	position(VERTICAL), layer(0),
 	basex(x), basey(y), global_image(global_image)
-{};
+{}
 
 terrain_builder::tile::tile() : last_tod("invalid_tod")
 {
@@ -89,11 +92,16 @@ void terrain_builder::tile::rebuild_cache(const std::string &tod) const
 
 	ordered_ri_list::const_iterator itor;
 	for(itor = horizontal_images.begin(); itor != horizontal_images.end(); ++itor) {
-		add_image_to_cache(tod, itor);
+		if (itor->first <= 0) 
+			add_image_to_cache(tod, itor);
 	}
 	for(itor = vertical_images.begin(); itor != vertical_images.end(); ++itor) {
 		add_image_to_cache(tod, itor);
 	}
+	for(itor = horizontal_images.begin(); itor != horizontal_images.end(); ++itor) {
+		if (itor->first > 0) 
+			add_image_to_cache(tod, itor);
+	}
 }
 
 void terrain_builder::tile::clear() 
@@ -495,7 +503,7 @@ void terrain_builder::add_images_from_config(rule_imagelist& images, const confi
 
 		} else if((**img)["position"] == "vertical") {
 
-			std::vector<std::string> base = config::split((**img)["base"]);
+			std::vector<std::string> base = utils::split((**img)["base"]);
 			int basex = 0, basey = 0;
 
 			if(base.size() >= 2) {
@@ -543,31 +551,28 @@ void terrain_builder::add_constraints(
 
 }
 
-void terrain_builder::add_constraint_item(std::vector<std::string> &list, const config& cfg, const std::string &item)
-{
-	if(!cfg[item].empty()) {
-		std::vector<std::string> item_string = config::split(cfg[item]);
-		
-		for(std::vector<std::string>::const_iterator itor = item_string.begin();
-				itor != item_string.end(); ++itor) {
-			list.push_back(*itor);
-		}
-	}
-}
-
 void terrain_builder::add_constraints(terrain_builder::constraint_set &constraints, const gamemap::location& loc, const config& cfg, const config& global_images)
 {
 	add_constraints(constraints, loc, cfg["type"], global_images);
 
 	terrain_constraint& constraint = constraints[loc];
-	
-	add_constraint_item(constraint.set_flag, cfg, "set_flag");
-	add_constraint_item(constraint.has_flag, cfg, "has_flag");
-	add_constraint_item(constraint.no_flag, cfg, "no_flag");
+
+	std::vector<std::string> item_string = utils::split(cfg["set_flag"]);
+	constraint.set_flag.insert(constraint.set_flag.end(),
+			item_string.begin(), item_string.end());
+
+	item_string = utils::split(cfg["has_flag"]);
+	constraint.has_flag.insert(constraint.has_flag.end(),
+			item_string.begin(), item_string.end());
+
+	item_string = utils::split(cfg["no_flag"]);
+	constraint.no_flag.insert(constraint.no_flag.end(),
+			item_string.begin(), item_string.end());
 
 	add_images_from_config(constraint.images, cfg, false);
 }
 
+
 void terrain_builder::parse_mapstring(const std::string &mapstring,
 		struct building_rule &br, anchormap& anchors,
 		const config& global_images)
@@ -575,11 +580,11 @@ void terrain_builder::parse_mapstring(const std::string &mapstring,
 	int lineno = 0;
 	int x = 0;
 
-	const std::vector<std::string> &lines = config::split(mapstring, '\n', 0);
+	const std::vector<std::string> &lines = utils::split(mapstring, '\n', 0);
 	std::vector<std::string>::const_iterator line = lines.begin();
 	
 	//Strips trailing empty lines
-	while(line != lines.end() && std::find_if(line->begin(),line->end(),config::notspace) == line->end()) {
+	while (line != lines.end() && std::find_if(line->begin(), line->end(), utils::notspace) == line->end()) {
 		line++;
 	}
 	//Break if there only are blank lines
@@ -604,7 +609,7 @@ void terrain_builder::parse_mapstring(const std::string &mapstring,
 
 		while(lpos < line->size()) {
 			std::string types = line->substr(lpos, 4);
-			config::strip(types);
+			utils::strip(types);
 			
 			//If there are numbers in the types string, consider it
 			//is an anchor
@@ -641,7 +646,7 @@ void terrain_builder::add_rotated_rules(building_ruleset& rules, building_rule&
 
 		add_rule(rules, tpl);
 	} else {
-		const std::vector<std::string>& rot = config::split(rotations, ',');
+		const std::vector<std::string>& rot = utils::split(rotations, ',');
 			
 		for(size_t angle = 0; angle < rot.size(); angle++) {
 			building_rule rule = rotate_rule(tpl, angle, rot);
@@ -678,8 +683,9 @@ void terrain_builder::parse_config(const config &cfg)
 		config::child_list tcs((*br)->get_children("tile"));
 		
 		for(config::child_list::const_iterator tc = tcs.begin(); tc != tcs.end(); tc++) {
-			//Adds the terrain constraint to the current built terrain's list of terrain 
-			//constraints, if it does not exist.
+			//Adds the terrain constraint to the current built
+			//terrain's list of terrain constraints, if it does not
+			//exist.
 			gamemap::location loc;
 			if((**tc)["x"].size()) {
 				loc.x = atoi((**tc)["x"].c_str());
@@ -688,7 +694,7 @@ void terrain_builder::parse_config(const config &cfg)
 				loc.y = atoi((**tc)["y"].c_str());
 			} 
 			if(!(**tc)["loc"].empty()) {
-				std::vector<std::string> sloc = config::split((**tc)["pos"]);
+				std::vector<std::string> sloc = utils::split((**tc)["loc"]);
 				if(sloc.size() == 2) {
 					loc.x = atoi(sloc[0].c_str());
 					loc.y = atoi(sloc[1].c_str());
@@ -700,7 +706,7 @@ void terrain_builder::parse_config(const config &cfg)
 			if((**tc)["pos"].size()) {
 				int pos = atoi((**tc)["pos"].c_str());
 				if(anchors.find(pos) == anchors.end()) {
-					lg::warn(lg::engine) << "Invalid anchor!\n";
+					LOG_STREAM(warn, engine) << "Invalid anchor!\n";
 					continue;
 				}
 
@@ -860,10 +866,7 @@ void terrain_builder::apply_rule(const terrain_builder::building_rule &rule, con
 
 		tile& btile = tile_map_[tloc];
 
-		//std::multimap<int, std::string>::const_iterator img;
-
 		for(img = constraint->second.images.begin(); img != constraint->second.images.end(); ++img) {
-			//animated<image::locator> th(img->second, locator_string_initializer());
 
 			if(img->position == rule_image::HORIZONTAL) {
 				btile.horizontal_images.insert(std::pair<int, const rule_image*>(
