@@ -5,12 +5,13 @@
 #include "pathfind.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
-#define LOG_NG lg::info(lg::engine)
+#define LOG_NG LOG_STREAM(info, engine)
 
 cave_map_generator::cave_map_generator(const config* cfg) : wall_('W'), clear_('u'), village_('D'), castle_('o'),
                                                             cfg_(cfg), width_(50), height_(50), village_density_(0),
-															flipx_(false), flipy_(false)
+							    flipx_(false), flipy_(false)
 {
 	if(cfg_ == NULL) {
 		static const config default_cfg;
@@ -137,7 +138,7 @@ void cave_map_generator::generate_chambers()
 		size_t min_xpos = 0, min_ypos = 0, max_xpos = width_, max_ypos = height_;
 
 		if(xpos != "") {
-			const std::vector<std::string>& items = config::split(xpos,'-');
+			const std::vector<std::string>& items = utils::split(xpos, '-');
 			if(items.empty() == false) {
 				min_xpos = atoi(items.front().c_str()) - 1;
 				max_xpos = atoi(items.back().c_str());
@@ -145,7 +146,7 @@ void cave_map_generator::generate_chambers()
 		}
 
 		if(ypos != "") {
-			const std::vector<std::string>& items = config::split(ypos,'-');
+			const std::vector<std::string>& items = utils::split(ypos, '-');
 			if(items.empty() == false) {
 				min_ypos = atoi(items.front().c_str()) - 1;
 				max_ypos = atoi(items.back().c_str());
