@@ -1,8 +1,11 @@
 #ifndef CAVEGEN_HPP_INCLUDED
 #define CAVEGEN_HPP_INCLUDED
 
+#include "config.hpp"
 #include "mapgen.hpp"
 
+#include <set>
+
 class cave_map_generator : public map_generator
 {
 public:
