@@ -3,7 +3,9 @@
 
 #include <string>
 #include "SDL.h"
-
+#include "serialization/string_utils.hpp"
+void copy_ucs2_to_clipboard(const ucs2_string& text);
+ucs2_string copy_ucs2_from_clipboard();
 void copy_to_clipboard(const std::string& text);
 std::string copy_from_clipboard();
 
