@@ -1,4 +1,4 @@
-/* $Id: dialogs.cpp,v 1.80 2005/01/19 20:11:34 silene Exp $ */
+/* $Id: dialogs.cpp,v 1.98 2005/04/09 09:46:12 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -18,15 +18,21 @@
 #include "filesystem.hpp"
 #include "font.hpp"
 #include "game_config.hpp"
+#include "game_errors.hpp"
+#include "gettext.hpp"
 #include "help.hpp"
 #include "language.hpp"
 #include "log.hpp"
+#include "map.hpp"
 #include "playturn.hpp"
 #include "preferences.hpp"
 #include "replay.hpp"
 #include "show_dialog.hpp"
 #include "util.hpp"
+#include "video.hpp"
 #include "wassert.hpp"
+#include "wml_separators.hpp"
+#include "serialization/string_utils.hpp"
 #include "widgets/menu.hpp"
 #include "widgets/progressbar.hpp"
 
@@ -37,8 +43,8 @@
 #include <time.h>
 #include <vector>
 
-#define LOG_DP lg::info(lg::display)
-#define ERR_G  lg::err(lg::general)
+#define LOG_DP LOG_STREAM(info, display)
+#define ERR_G  LOG_STREAM(err, general)
 
 namespace dialogs
 {
@@ -53,7 +59,7 @@ void advance_unit(const game_data& info,
 	if(u == units.end() || u->second.advances() == false)
 		return;
 
-	LOG_DP << "advance_unit: " << u->second.type().name() << "\n";
+	LOG_DP << "advance_unit: " << u->second.type().id() << "\n";
 
 	const std::vector<std::string>& options = u->second.type().advances_to();
 
@@ -69,10 +75,10 @@ void advance_unit(const game_data& info,
 	const config::child_list& mod_options = u->second.get_modification_advances();
 
 	for(config::child_list::const_iterator mod = mod_options.begin(); mod != mod_options.end(); ++mod) {
-		sample_units.push_back(::get_advanced_unit(info,units,loc,u->second.type().name()));
+		sample_units.push_back(::get_advanced_unit(info,units,loc,u->second.type().id()));
 		sample_units.back().add_modification("advance",**mod);
 		const unit_type& type = sample_units.back().type();
-		lang_options.push_back(IMAGE_PREFIX + type.image() + COLUMN_SEPARATOR + (**mod)["description"]);
+		lang_options.push_back(IMAGE_PREFIX + type.image() + COLUMN_SEPARATOR + (**mod)["description"].str());
 	}
 
 	LOG_DP << "options: " << options.size() << "\n";
@@ -130,7 +136,7 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 		}
 	}
 
-	const std::string& chosen_unit = choice < options.size() ? options[choice] : u->second.type().name();
+	const std::string& chosen_unit = choice < options.size() ? options[choice] : u->second.type().id();
 	::advance_unit(info,units,loc,chosen_unit);
 
 	u = units.find(loc);
@@ -158,14 +164,11 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 	return true;
 }
 
-void show_objectives(display& disp, config& level_info)
+void show_objectives(display& disp, const config& level, const std::string& objectives)
 {
 	static const std::string no_objectives(_("No objectives available"));
-	const std::string& name = level_info["name"];
-	const std::string& lang_objectives = level_info["objectives"];
+	const std::string& name = level["name"];
 
-	const std::string& objectives = lang_objectives.empty() ?
-	        level_info["objectives"] : lang_objectives;
 	gui::show_dialog(disp, NULL, "", font::LARGE_TEXT + name + "\n" +
 	         (objectives.empty() ? no_objectives : objectives), gui::OK_ONLY);
 
@@ -246,9 +249,9 @@ static const int save_preview_border = 10;
 class save_preview_pane : public gui::preview_pane
 {
 public:
-	save_preview_pane(display& disp, const config& game_config, gamemap* map, const game_data& data,
+	save_preview_pane(CVideo &video, const config& game_config, gamemap* map, const game_data& data,
 	                  const std::vector<save_info>& info, const std::vector<config*>& summaries)
-		: gui::preview_pane(disp), game_config_(&game_config), map_(map), data_(&data), info_(&info), summaries_(&summaries), index_(0)
+		: gui::preview_pane(video), game_config_(&game_config), map_(map), data_(&data), info_(&info), summaries_(&summaries), index_(0)
 	{
 		set_measurements(200, 400);
 	}
@@ -273,13 +276,13 @@ private:
 
 void save_preview_pane::draw_contents()
 {
-	if(index_ < 0 || index_ >= int(summaries_->size()) || info_->size() != summaries_->size()) {
+	if (size_t(index_) >= summaries_->size() || info_->size() != summaries_->size()) {
 		return;
 	}
 
 	const config& summary = *(*summaries_)[index_];
 
-	surface const screen = disp().video().getSurface();
+	surface const screen = video().getSurface();
 
 	SDL_Rect const &loc = location();
 	const SDL_Rect area = { loc.x + save_preview_border, loc.y + save_preview_border,
@@ -351,7 +354,7 @@ void save_preview_pane::draw_contents()
 
 	// escape all special characters in filenames
 	std::string name = (*info_)[index_].name;
-	str << font::BOLD_TEXT << config::escape(name) << "\n" << time_buf;
+	str << font::BOLD_TEXT << utils::escape(name) << "\n" << time_buf;
 
 	const std::string& campaign_type = summary["campaign_type"];
 	if(summary["corrupt"] == "yes") {
@@ -379,13 +382,13 @@ void save_preview_pane::draw_contents()
 			str << _("Scenario Start");
 		}
 
-		str << "\n" << _("Difficulty") << ": " << string_table[summary["difficulty"]];
+		str << "\n" << _("Difficulty: ") << string_table[summary["difficulty"]];
 		if(!summary["version"].empty()) {
-			str << "\n" << _("Version") << ": " << summary["version"];
+			str << "\n" << _("Version: ") << summary["version"];
 		}
 	}
 
-	font::draw_text(&disp(), area, font::SIZE_SMALL, font::NORMAL_COLOUR, str.str(), area.x, ypos, true);
+	font::draw_text(&video(), area, font::SIZE_SMALL, font::NORMAL_COLOUR, str.str(), area.x, ypos, true);
 }
 
 } //end anon namespace
@@ -455,7 +458,7 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	const events::event_context context;
 
 	if(generate_summaries) {
-		gui::progress_bar bar(disp);
+		gui::progress_bar bar(disp.video());
 		const SDL_Rect bar_area = {disp.x()/2 - 100, disp.y()/2 - 20, 200, 40};
 		bar.set_location(bar_area);
 
@@ -472,7 +475,8 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 
 			try {
 				summary["mod_time"] = str_cast(lexical_cast<int>(games[*s].time_modified));
-				load_game(data,games[*s].name,state);
+				std::string dummy;
+				load_game(data,games[*s].name,state,&dummy);
 				extract_summary_data_from_save(state,summary);
 			} catch(io_exception&) {
 				summary["corrupt"] = "yes";
@@ -480,7 +484,7 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 			} catch(config::error&) {
 				summary["corrupt"] = "yes";
 				ERR_G << "save '" << games[*s].name << "' could not be loaded (config parse error)\n";
-			} catch(gamestatus::load_game_failed&) {
+			} catch(game::load_game_failed&) {
 				summary["corrupt"] = "yes";
 				ERR_G << "save '" << games[*s].name << "' could not be loaded (load_game_failed exception)\n";
 			}
@@ -500,7 +504,7 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	gamemap map_obj(game_config,"");
 
 	std::vector<gui::preview_pane*> preview_panes;
-	save_preview_pane save_preview(disp,game_config,&map_obj,data,games,summaries);
+	save_preview_pane save_preview(disp.video(),game_config,&map_obj,data,games,summaries);
 	preview_panes.push_back(&save_preview);
 
 	//create an option for whether the replay should be shown or not
@@ -546,9 +550,10 @@ namespace {
 }
 
 unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const unit& u, TYPE type, bool on_left_side)
-                                        : gui::preview_pane(disp), details_button_(disp,_("Profile"),gui::button::TYPE_PRESS,"lite_small",gui::button::MINIMUM_SPACE),
-										  map_(map), units_(&unit_store_), index_(0), left_(on_left_side),
-										  weapons_(type == SHOW_ALL)
+				    : gui::preview_pane(disp.video()), disp_(disp),
+				      details_button_(disp.video(),_("Profile"),gui::button::TYPE_PRESS,"lite_small",gui::button::MINIMUM_SPACE),
+				      map_(map), units_(&unit_store_), index_(0), left_(on_left_side),
+				      weapons_(type == SHOW_ALL)
 {
 	unsigned w = font::relative_size(weapons_ ? 200 : 190);
 	unsigned h = font::relative_size(weapons_ ? 370 : 140);
@@ -557,9 +562,10 @@ unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const un
 }
 
 unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const std::vector<unit>& units, TYPE type, bool on_left_side)
-                                        : gui::preview_pane(disp), details_button_(disp,_("Profile"),gui::button::TYPE_PRESS,"lite_small",gui::button::MINIMUM_SPACE),
-										  map_(map), units_(&units), index_(0), left_(on_left_side),
-										  weapons_(type == SHOW_ALL)
+                                    : gui::preview_pane(disp.video()), disp_(disp),
+				      details_button_(disp.video(),_("Profile"),gui::button::TYPE_PRESS,"lite_small",gui::button::MINIMUM_SPACE),
+				      map_(map), units_(&units), index_(0), left_(on_left_side),
+				      weapons_(type == SHOW_ALL)
 {
 	set_measurements(font::relative_size(200), font::relative_size(370));
 }
@@ -596,7 +602,7 @@ void unit_preview_pane::draw_contents()
 
 	const bool right_align = left_side();
 
-	surface const screen = disp().video().getSurface();
+	surface const screen = video().getSurface();
 
 	SDL_Rect const &loc = location();
 	const SDL_Rect area = { loc.x + unit_preview_border, loc.y + unit_preview_border,
@@ -632,7 +638,7 @@ void unit_preview_pane::draw_contents()
 		desc << font::NORMAL_TEXT << u.description();
 		const std::string description = desc.str();
 		description_rect = font::text_area(description,font::SIZE_NORMAL);
-		description_rect = font::draw_text(&disp(),area,font::SIZE_NORMAL,font::NORMAL_COLOUR,desc.str(),right_align ? image_rect.x : image_rect.x + image_rect.w - description_rect.w,image_rect.y+image_rect.h+details_button_.location().h);
+		description_rect = font::draw_text(&video(),area,font::SIZE_NORMAL,font::NORMAL_COLOUR,desc.str(),right_align ? image_rect.x : image_rect.x + image_rect.w - description_rect.w,image_rect.y+image_rect.h+details_button_.location().h);
 	}
 
 	std::stringstream details;
@@ -658,11 +664,11 @@ void unit_preview_pane::draw_contents()
 	else if(u.hitpoints() > 2*(u.max_hitpoints()/3))
 		details << font::GOOD_TEXT;
 
-	details << _("HP") << ": " << u.hitpoints()
+	details << _("HP: ") << u.hitpoints()
 			<< "/" << u.max_hitpoints() << "\n";
 	
 	if(u.can_advance() == false) {
-		details << _("XP") << ": " << u.experience() << "/-";
+		details << _("XP: ") << u.experience() << "/-";
 	} else {
 		//if killing a unit the same level as us would level us up,
 		//then display in green
@@ -670,12 +676,12 @@ void unit_preview_pane::draw_contents()
 			details << font::GOOD_TEXT;
 		}
 
-		details << _("XP") << ": " << u.experience() << "/" << u.max_experience();
+		details << _("XP: ") << u.experience() << "/" << u.max_experience();
 	}
 	
 	if(weapons_) {
 		details << "\n"
-				<< _("Moves") << ": " << u.movement_left() << "/"
+				<< _("Moves: ") << u.movement_left() << "/"
 				<< u.total_movement()
 				<< "\n";
 
@@ -696,7 +702,7 @@ void unit_preview_pane::draw_contents()
 	
 	const std::string text = details.str();
 	
-	const std::vector<std::string> lines = config::split(text,'\n');
+	const std::vector<std::string> lines = utils::split(text, '\n');
 
 	SDL_Rect cur_area = area;
 
@@ -711,7 +717,7 @@ void unit_preview_pane::draw_contents()
 			xpos = cur_area.x + cur_area.w - line_area.w;
 		}
 
-		cur_area = font::draw_text(&disp(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,xpos,cur_area.y);
+		cur_area = font::draw_text(&video(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,xpos,cur_area.y);
 		cur_area.y += cur_area.h;
 	}
 }
@@ -720,23 +726,23 @@ void unit_preview_pane::process_event()
 {
 	if(map_ != NULL && details_button_.pressed() && index_ >= 0 && index_ < int(units_->size())) {
 
-		show_unit_description(disp(), (*units_)[index_]);
+		show_unit_description(disp_, (*units_)[index_]);
 	}
 }
 
-void show_unit_description(display& disp, const unit& u)
+void show_unit_description(display &disp, const unit& u)
 {
 	help::show_help(disp,"unit_" + u.type().id());
 }
 
 
 namespace {
-	static const int campaign_preview_border = 10;
+	static const int campaign_preview_border = font::relative_size(10);
 }
 
-campaign_preview_pane::campaign_preview_pane(display& disp,std::vector<std::pair<std::string,std::string> >* desc) : gui::preview_pane(disp),descriptions_(desc),index_(0)
+campaign_preview_pane::campaign_preview_pane(CVideo &video,std::vector<std::pair<std::string,std::string> >* desc) : gui::preview_pane(video),descriptions_(desc),index_(0)
 {
-	set_measurements(350, 400);
+	set_measurements(font::relative_size(350), font::relative_size(400));
 }
 
 bool campaign_preview_pane::show_above() const { return false; }
@@ -753,7 +759,7 @@ void campaign_preview_pane::set_selection(int index)
 
 void campaign_preview_pane::draw_contents()
 {
-	if(index_ < 0 || index_ >= descriptions_->size()) {
+	if (size_t(index_) >= descriptions_->size()) {
 		return;
 	}
 
@@ -766,15 +772,15 @@ void campaign_preview_pane::draw_contents()
 	/* background frame */
 	static const std::string default_style("mainmenu");
 	const std::string* style = &default_style;
-	gui::draw_dialog_frame(area.x,area.y,area.w,area.h,disp(),style);
+	gui::draw_dialog_frame(area.x,area.y,area.w,area.h,video(),style);
 
 	/* description text */
 	const std::string& desc_text = font::word_wrap_text((*descriptions_)[index_].first,font::SIZE_SMALL,area.w-2*campaign_preview_border);
-	const std::vector<std::string> lines = config::split(desc_text,'\n');
+	const std::vector<std::string> lines = utils::split(desc_text, '\n');
 	SDL_Rect txt_area = { area.x+campaign_preview_border,area.y,0,0 };
 
 	for(std::vector<std::string>::const_iterator line = lines.begin(); line != lines.end(); ++line) {
-		txt_area = font::draw_text(&disp(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,txt_area.x,txt_area.y);
+	  txt_area = font::draw_text(&video(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,txt_area.x,txt_area.y);
 		txt_area.y += txt_area.h;
 	}
 
@@ -794,7 +800,7 @@ void campaign_preview_pane::draw_contents()
 		dst_rect.x = area.x+(area.w-src_rect.w)/2;
 		dst_rect.y = txt_area.y+(max_height-src_rect.h)/2;
 
-		SDL_BlitSurface(img,&src_rect,disp().video().getSurface(),&dst_rect);
+		SDL_BlitSurface(img,&src_rect,video().getSurface(),&dst_rect);
 
 	}
 }
