@@ -1,4 +1,4 @@
-/* $Id: dialogs.hpp,v 1.29 2004/12/19 21:18:14 silene Exp $ */
+/* $Id: dialogs.hpp,v 1.33 2005/03/19 13:02:34 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,9 +13,9 @@
 #ifndef DIALOGS_H_INCLUDED
 #define DIALOGS_H_INCLUDED
 
-#include "actions.hpp"
-#include "config.hpp"
-#include "display.hpp"
+class config;
+class display;
+
 #include "map.hpp"
 #include "show_dialog.hpp"
 #include "unit.hpp"
@@ -36,7 +36,7 @@ void advance_unit(const game_data& info, const gamemap& map,unit_map& units, gam
 
 bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::location loc, display& gui, size_t choice);
 
-void show_objectives(display& disp, config& level_info);
+void show_objectives(display& disp, const config& level, const std::string& objectives);
 
 // Ask user if I should really save the game and what name I should use
 // returns 0 iff user wants to save the game
@@ -65,14 +65,16 @@ class unit_preview_pane : public gui::preview_pane
 public:
 	enum TYPE { SHOW_ALL, SHOW_BASIC };
 
-	unit_preview_pane(display& disp, const gamemap* map, const unit& u, TYPE type=SHOW_ALL, bool left_side=true);
-	unit_preview_pane(display& disp, const gamemap* map, const std::vector<unit>& units, TYPE type=SHOW_ALL, bool left_side=true);
+	unit_preview_pane(display &disp, const gamemap* map, const unit& u, TYPE type=SHOW_ALL, bool left_side=true);
+	unit_preview_pane(display &disp, const gamemap* map, const std::vector<unit>& units, TYPE type=SHOW_ALL, bool left_side=true);
 
 	bool show_above() const;
 	bool left_side() const;
 	void set_selection(int index);
 
 private:
+	display& disp_;
+
 	void draw_contents();
 	void process_event();
 
@@ -85,13 +87,13 @@ private:
 	bool weapons_;
 };
 
-void show_unit_description(display& disp, const unit& u);
+void show_unit_description(display &disp, const unit& u);
 
 
 class campaign_preview_pane : public gui::preview_pane
 {
 public:
-	campaign_preview_pane(display& disp,std::vector<std::pair<std::string,std::string> >* descriptions);
+	campaign_preview_pane(CVideo &video,std::vector<std::pair<std::string,std::string> >* descriptions);
 
 	bool show_above() const;
 	bool left_side() const;
