@@ -1,4 +1,4 @@
-/* $Id: display.cpp,v 1.289 2005/02/09 23:32:01 ydirson Exp $ */
+/* $Id: display.cpp,v 1.314 2005/04/06 21:35:39 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,11 +14,13 @@
 #include "global.hpp"
 
 #include "actions.hpp"
+#include "cursor.hpp"
 #include "display.hpp"
 #include "events.hpp"
 #include "font.hpp"
-#include "game.hpp"
 #include "game_config.hpp"
+#include "gamestatus.hpp"
+#include "gettext.hpp"
 #include "halo.hpp"
 #include "hotkeys.hpp"
 #include "image.hpp"
@@ -26,12 +28,12 @@
 #include "log.hpp"
 #include "preferences.hpp"
 #include "sdl_utils.hpp"
-#include "show_dialog.hpp"
 #include "sound.hpp"
 #include "team.hpp"
 #include "tooltips.hpp"
 #include "unit_display.hpp"
 #include "util.hpp"
+#include "video.hpp"
 #include "wassert.hpp"
 
 #include "SDL_image.h"
@@ -41,12 +43,17 @@
 #include <iostream>
 #include <sstream>
 
-#define ERR_DP lg::err(lg::display)
+#define ERR_DP LOG_STREAM(err, display)
 
 std::map<gamemap::location,fixed_t> display::debugHighlights_;
 
 namespace {
+#ifdef USE_TINY_GUI
+	const int DefaultZoom = 36;
+#else
 	const int DefaultZoom = 72;
+#endif
+
 	const int MaxZoom = 200;
 
 	int MinZoom(const gamemap& map, const SDL_Rect& viewport)
@@ -73,15 +80,15 @@ display::display(unit_map& units, CVideo& video, const gamemap& map,
 	invalidateAll_(true), invalidateUnit_(true),
 	invalidateGameStatus_(true), panelsDrawn_(false),
 	currentTeam_(0), activeTeam_(0), hideEnergy_(false),
-	deadAmount_(ftofxp(0.0)), advancingAmount_(0.0), updatesLocked_(0),
+	deadAmount_(ftofxp(0.0)), advancingAmount_(0.0),
 	turbo_(false), grid_(false), sidebarScaling_(1.0),
 	theme_(theme_cfg,screen_area()), builder_(cfg, level, map),
 	first_turn_(true), in_game_(false), map_labels_(*this,map),
 	tod_hex_mask1(NULL), tod_hex_mask2(NULL), diagnostic_label_(0),
-	help_string_(0), fps_handle_(0)
+	fps_handle_(0)
 {
 	if(non_interactive())
-		updatesLocked_++;
+		screen_.lock_updates(true);
 
 	std::fill(reportRects_,reportRects_+reports::NUM_REPORTS,empty_rect);
 
@@ -107,7 +114,7 @@ display::display(unit_map& units, CVideo& video, const gamemap& map,
 			flag = teams_[i].flag();
 		}
 
-		lg::info(lg::display) << "Adding flag for team " << i << " from animation " << flag << "\n";
+		LOG_STREAM(info, display) << "Adding flag for team " << i << " from animation " << flag << "\n";
 		flags_.push_back(animated<image::locator>(flag));
 		flags_.back().start_animation(0, animated<image::locator>::INFINITE_CYCLES);
 	}
@@ -237,7 +244,7 @@ void display::highlight_hex(gamemap::location hex)
 		invalidate_unit();
 }
 
-gamemap::location display::hex_clicked_on(int xclick, int yclick, gamemap::location::DIRECTION* nearest_hex)
+gamemap::location display::hex_clicked_on(int xclick, int yclick, gamemap::location::DIRECTION* nearest_hex, gamemap::location::DIRECTION* second_nearest_hex)
 {
 	const SDL_Rect& rect = map_area();
 	if(point_in_rect(xclick,yclick,rect) == false) {
@@ -247,10 +254,10 @@ gamemap::location display::hex_clicked_on(int xclick, int yclick, gamemap::locat
 	xclick -= rect.x;
 	yclick -= rect.y;
 
-	return pixel_position_to_hex(xpos_ + xclick, ypos_ + yclick, nearest_hex);
+	return pixel_position_to_hex(xpos_ + xclick, ypos_ + yclick, nearest_hex, second_nearest_hex);
 }
 
-gamemap::location display::pixel_position_to_hex(int x, int y, gamemap::location::DIRECTION* nearest_hex)
+gamemap::location display::pixel_position_to_hex(int x, int y, gamemap::location::DIRECTION* nearest_hex, gamemap::location::DIRECTION* second_nearest_hex)
 {
 	const int s = hex_size();
 	const int tesselation_x_size = s * 3 / 2;
@@ -291,26 +298,55 @@ gamemap::location display::pixel_position_to_hex(int x, int y, gamemap::location
 	const gamemap::location res(x_base + x_modifier, y_base + y_modifier);
 	
 	if(nearest_hex != NULL) {
-		const int westx = (get_location_x(res) - map_area().x + xpos_) + hex_size()/3;
-		const int eastx = westx + hex_size()/3;
+		const int centerx = (get_location_x(res) - map_area().x + xpos_) + hex_size()/2;
 		const int centery = (get_location_y(res) - map_area().y + ypos_) + hex_size()/2;
-
-		const bool west = x < westx;
-		const bool east = x > eastx;
-		const bool north = y < centery;
-
-		if(north && west) {
-			*nearest_hex = gamemap::location::NORTH_WEST;
-		} else if(north && east) {
-			*nearest_hex = gamemap::location::NORTH_EAST;
-		} else if(north) {
-			*nearest_hex = gamemap::location::NORTH;
-		} else if(west) {
-			*nearest_hex = gamemap::location::SOUTH_WEST;
-		} else if(east) {
-			*nearest_hex = gamemap::location::SOUTH_EAST;
-		} else {
-			*nearest_hex = gamemap::location::SOUTH;
+		const int x_offset = x - centerx;
+		const int y_offset = y - centery;
+		if(y_offset > 0) {
+			if(x_offset > y_offset/2) {
+				*nearest_hex = gamemap::location::SOUTH_EAST;
+				if(second_nearest_hex != NULL) {
+					if(x_offset/2 > y_offset)    *second_nearest_hex = gamemap::location::NORTH_EAST;
+					else                         *second_nearest_hex = gamemap::location::SOUTH;
+				}
+			}
+			else if(-x_offset > y_offset/2) {
+				*nearest_hex = gamemap::location::SOUTH_WEST;
+				if(second_nearest_hex != NULL) {
+					if(-x_offset/2 > y_offset)   *second_nearest_hex = gamemap::location::NORTH_WEST;
+					else                         *second_nearest_hex = gamemap::location::SOUTH;
+				}
+			}
+			else {
+				*nearest_hex = gamemap::location::SOUTH;
+				if(second_nearest_hex != NULL) {
+					if(x_offset > 0)             *second_nearest_hex = gamemap::location::SOUTH_EAST;
+					else                         *second_nearest_hex = gamemap::location::SOUTH_WEST;
+				}
+			}
+		}
+		else {
+			if(x_offset > -y_offset/2) {
+				*nearest_hex = gamemap::location::NORTH_EAST;
+				if(second_nearest_hex != NULL) {
+					if(x_offset/2 > -y_offset)   *second_nearest_hex = gamemap::location::SOUTH_EAST;
+					else                         *second_nearest_hex = gamemap::location::NORTH;
+				}
+			}
+			else if(-x_offset > -y_offset/2) {
+				*nearest_hex = gamemap::location::NORTH_WEST;
+				if(second_nearest_hex != NULL) {
+					if(-x_offset/2 > -y_offset)  *second_nearest_hex = gamemap::location::SOUTH_WEST;
+					else                         *second_nearest_hex = gamemap::location::NORTH;
+				}
+			}
+			else {
+				*nearest_hex = gamemap::location::NORTH;
+				if(second_nearest_hex != NULL) {
+					if(x_offset > 0)             *second_nearest_hex = gamemap::location::NORTH_EAST;
+					else                         *second_nearest_hex = gamemap::location::NORTH_WEST;
+				}
+			}
 		}
 	}
 
@@ -418,7 +454,7 @@ void display::default_zoom()
 
 void display::scroll_to_tile(int x, int y, SCROLL_TYPE scroll_type, bool check_fogged)
 {
-	if(update_locked() || (check_fogged && fogged(x,y)))
+	if(screen_.update_locked() || (check_fogged && fogged(x,y)))
 		return;
 
 	const gamemap::location loc(x,y);
@@ -516,7 +552,7 @@ void display::bounds_check_position()
 
 void display::redraw_everything()
 {
-	if(update_locked() || teams_.empty())
+	if(screen_.update_locked() || teams_.empty())
 		return;
 
 	bounds_check_position();
@@ -543,21 +579,43 @@ void display::redraw_everything()
 	draw(true,true);
 }
 
+void display::flip()
+{
+	if(video().faked())
+		return;
+
+	const surface frameBuffer = get_video_surface();
+
+	halo::render();
+	font::draw_floating_labels(frameBuffer);
+	events::raise_volatile_draw_event();
+	cursor::draw(frameBuffer);
+
+	video().flip();
+
+	cursor::undraw(frameBuffer);
+	events::raise_volatile_undraw_event();
+	font::undraw_floating_labels(frameBuffer);
+	halo::unrender();
+}
+
 namespace {
 
-void draw_panel(display& disp, const theme::panel& panel, std::vector<gui::button>& buttons)
+void draw_panel(CVideo& video, const theme::panel& panel, std::vector<gui::button>& buttons)
 {
 	//log_scope("draw panel");
 	surface surf(image::get_image(panel.image(),image::UNSCALED));
 
-	const SDL_Rect screen = disp.screen_area();
+	const SDL_Rect screen = screen_area();
 	SDL_Rect& loc = panel.location(screen);
-	if(surf->w != loc.w || surf->h != loc.h) {
-		surf.assign(scale_surface(surf,loc.w,loc.h));
-	}
+	if(!surf.null()) {
+		if(surf->w != loc.w || surf->h != loc.h) {
+			surf.assign(scale_surface(surf,loc.w,loc.h));
+		}
 
-	disp.blit_surface(loc.x,loc.y,surf);
-	update_rect(loc);
+		video.blit_surface(loc.x,loc.y,surf);
+		update_rect(loc);
+	}
 
 	for(std::vector<gui::button>::iterator b = buttons.begin(); b != buttons.end(); ++b) {
 		if(rects_overlap(b->location(),loc)) {
@@ -566,27 +624,29 @@ void draw_panel(display& disp, const theme::panel& panel, std::vector<gui::butto
 	}
 }
 
-void draw_label(display& disp, surface target, const theme::label& label)
+void draw_label(CVideo& video, surface target, const theme::label& label)
 {
 	//log_scope("draw label");
 
 	const std::string& text = label.text();
 	const std::string& icon = label.icon();
-	SDL_Rect& loc = label.location(disp.screen_area());
+	SDL_Rect& loc = label.location(screen_area());
 	
 	if(icon.empty() == false) {
 		surface surf(image::get_image(icon,image::UNSCALED));
-		if(surf->w != loc.w || surf->h != loc.h) {
-			surf.assign(scale_surface(surf,loc.w,loc.h));
-		}
+		if(!surf.null()) {
+			if(surf->w != loc.w || surf->h != loc.h) {
+				surf.assign(scale_surface(surf,loc.w,loc.h));
+			}
 
-		SDL_BlitSurface(surf,NULL,target,&loc);
+			SDL_BlitSurface(surf,NULL,target,&loc);
+		}
 
 		if(text.empty() == false) {
 			tooltips::add_tooltip(loc,text);
 		}
 	} else if(text.empty() == false) {
-		font::draw_text(&disp,loc,label.font_size(),font::NORMAL_COLOUR,text,loc.x,loc.y);
+		font::draw_text(&video,loc,label.font_size(),font::NORMAL_COLOUR,text,loc.x,loc.y);
 	}
 
 
@@ -607,12 +667,12 @@ void display::draw(bool update,bool force)
 
 		const std::vector<theme::panel>& panels = theme_.panels();
 		for(std::vector<theme::panel>::const_iterator p = panels.begin(); p != panels.end(); ++p) {
-			draw_panel(*this,*p,buttons_);
+			draw_panel(video(),*p,buttons_);
 		}
 
 		const std::vector<theme::label>& labels = theme_.labels();
 		for(std::vector<theme::label>::const_iterator i = labels.begin(); i != labels.end(); ++i) {
-			draw_label(*this,screen,*i);
+			draw_label(video(),screen,*i);
 		}
 		
 		//invalidate the reports so they are redrawn
@@ -685,7 +745,7 @@ void display::draw(bool update,bool force)
 
 void display::update_display()
 {
-	if(updatesLocked_ > 0)
+	if(screen_.update_locked())
 		return;
 
 	if(preferences::show_fps()) {
@@ -713,7 +773,7 @@ void display::update_display()
 		fps_handle_ = 0;
 	}
 
-	screen_.flip();
+	flip();
 }
 
 void display::draw_sidebar()
@@ -864,7 +924,7 @@ void display::draw_report(reports::TYPE report_num)
 			for(reports::report::iterator i = report.begin(); i != report.end(); ++i) {
 				if(i->text.empty() == false) {
 					// Draw a text element
-					area = font::draw_text(this,rect,item->font_size(),font::NORMAL_COLOUR,i->text,x,y);
+					area = font::draw_text(&screen_,rect,item->font_size(),font::NORMAL_COLOUR,i->text,x,y);
 					if(area.h > tallest) tallest = area.h;
 					if(i->text[i->text.size() - 1] == '\n') {
 						x = rect.x;
@@ -877,6 +937,10 @@ void display::draw_report(reports::TYPE report_num)
 					// Draw an image element
 					surface img(image::get_image(i->image,image::UNSCALED));
 
+					if(report_num == reports::TIME_OF_DAY && img != NULL && preferences::flip_time()) {
+						img = flip_surface(img);
+					}
+
 					if(img == NULL) {
 						ERR_DP << "could not find image for report: '" << i->image << "'\n";
 						continue;
@@ -969,11 +1033,11 @@ void display::draw_unit_details(int x, int y, const gamemap::location& loc,
 	else if(u.hitpoints() > 2*(u.max_hitpoints()/3))
 		details << font::GOOD_TEXT;
 
-	details << _("HP") << ": " << u.hitpoints()
+	details << _("HP: ") << u.hitpoints()
 			<< "/" << u.max_hitpoints() << "\n";
 	
 	if(u.can_advance() == false) {
-		details << _("XP") << ": " << u.experience() << "/-";
+		details << _("XP: ") << u.experience() << "/-";
 	} else {
 		//if killing a unit the same level as us would level us up,
 		//then display in green
@@ -981,11 +1045,11 @@ void display::draw_unit_details(int x, int y, const gamemap::location& loc,
 			details << font::GOOD_TEXT;
 		}
 
-		details << _("XP") << ": " << u.experience() << "/" << u.max_experience();
+		details << _("XP: ") << u.experience() << "/" << u.max_experience();
 	}
 	
 	details << "\n"
-			<< _("Moves") << ": " << u.movement_left() << "/"
+			<< _("Moves: ") << u.movement_left() << "/"
 			<< u.total_movement()
 			<< "\n";
 
@@ -993,7 +1057,7 @@ void display::draw_unit_details(int x, int y, const gamemap::location& loc,
 	for(std::vector<attack_type>::const_iterator at_it = attacks.begin();
 	    at_it != attacks.end(); ++at_it) {
 
-		const std::string& lang_weapon = at_it->name();
+		const t_string& lang_weapon = at_it->name();
 		const std::string& lang_type = at_it->type();
 		const std::string& lang_special = at_it->special();
 		details << "\n"
@@ -1012,7 +1076,7 @@ void display::draw_unit_details(int x, int y, const gamemap::location& loc,
 	//with on the right-side panel
 	const size_t font_size = this->y() >= 700 ? 13 : 10;
 
-	description_rect = font::draw_text(this,clipRect,font_size,font::NORMAL_COLOUR,
+	description_rect = font::draw_text(&screen_,clipRect,font_size,font::NORMAL_COLOUR,
 	                                   details.str(),x,y);
 
 	update_rect(description_rect);
@@ -1083,7 +1147,7 @@ void display::draw_minimap(int x, int y, int w, int h)
 	const Uint32 boxcolour = SDL_MapRGB(surf->format,0xFF,0xFF,0xFF);
 	const surface screen(screen_.getSurface());
 
-	gui::draw_rectangle(x+xbox,y+ybox,wbox,hbox,boxcolour,screen);
+	draw_rectangle(x+xbox,y+ybox,wbox,hbox,boxcolour,screen);
 
 	update_rect(minimap_location);
 }
@@ -1113,7 +1177,7 @@ void display::draw_halo_on_tile(int x, int y)
 void display::draw_unit_on_tile(int x, int y, surface unit_image_override,
 		fixed_t highlight_ratio, Uint32 blend_with)
 {
-	if(updatesLocked_)
+	if(screen_.update_locked())
 		return;
 
 	const gamemap::location loc(x,y);
@@ -1295,8 +1359,11 @@ void display::draw_unit_on_tile(int x, int y, surface unit_image_override,
 	if (u.can_recruit()) {
 		surface crown(image::get_image("misc/leader-crown.png",image::SCALED,image::NO_ADJUST_COLOUR));
 		if(!crown.null()) {
+			if(bar_alpha != ftofxp(1.0))
+				crown = adjust_surface_alpha(crown, bar_alpha);
+
 			SDL_Rect r = {0, 0, crown->w, crown->h};
-			blit_surface(xpos,ypos,crown,&r);
+			video().blit_surface(xpos,ypos,crown,&r);
 		}
 	}
 
@@ -1337,8 +1404,8 @@ void display::draw_bar(const std::string& image, int xpos, int ypos, size_t heig
 	SDL_Rect bot = {0,bar_loc.y+skip_rows,surf->w,0};
 	bot.h = surf->w - bot.y;
 
-	blit_surface(xpos,ypos,surf,&top);
-	blit_surface(xpos,ypos+top.h,surf,&bot);
+	video().blit_surface(xpos,ypos,surf,&top);
+	video().blit_surface(xpos,ypos+top.h,surf,&bot);
 
 	const size_t unfilled = (const size_t)(height*(1.0 - filled));
 
@@ -1378,7 +1445,7 @@ void display::draw_terrain_on_tile(int x, int y, image::TYPE image_type, ADJACEN
 
 void display::draw_tile(int x, int y, surface unit_image, fixed_t alpha, Uint32 blend_to)
 {
-	if(updatesLocked_)
+	if(screen_.update_locked())
 		return;
 
 	draw_halo_on_tile(x,y);
@@ -1603,22 +1670,22 @@ void display::draw_footstep(const gamemap::location& loc, int xloc, int yloc)
 		const int y = yloc + zoom_/2 - text_area.h/2;
 
 		//draw the text with a black outline
-		font::draw_text(this,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x-1,y-1);
-		font::draw_text(this,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x-1,y);
-		font::draw_text(this,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x-1,y+1);
-		font::draw_text(this,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x,y-1);
-		font::draw_text(this,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x+1,y-1);
-		font::draw_text(this,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x+1,y);
-		font::draw_text(this,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x+1,y+1);
-		font::draw_text(this,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x,y+1);
-		font::draw_text(this,rect,font::SIZE_LARGE,font::YELLOW_COLOUR,str,x,y);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x-1,y-1);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x-1,y);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x-1,y+1);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x,y-1);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x+1,y-1);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x+1,y);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x+1,y+1);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::DARK_COLOUR,str,x,y+1);
+		font::draw_text(&screen_,rect,font::SIZE_LARGE,font::YELLOW_COLOUR,str,x,y);
 	}
 }
 
 namespace {
 const std::string& get_direction(size_t n)
 {
-	const static std::string dirs[6] = {"-n","-ne","-se","-s","-sw","-nw"};
+	static std::string const dirs[6] = { "-n", "-ne", "-se", "-s", "-sw", "-nw" };
 	return dirs[n >= sizeof(dirs)/sizeof(*dirs) ? 0 : n];
 }
 
@@ -1626,13 +1693,13 @@ const std::string& get_direction(size_t n)
 
 bool angle_is_northern(size_t n)
 {
-	const static bool results[6] = {true,false,false,false,false,true};
+	static bool const results[6] = { true, false, false, false, false, true };
 	return results[n >= sizeof(results)/sizeof(*results) ? 0 : n];
 }
 
 const std::string& get_angle_direction(size_t n)
 {
-	const static std::string dirs[6] = {"-ne","-e","-se","-sw","-w","-nw"};
+	static std::string const dirs[6] = { "-ne", "-e", "-se", "-sw", "-w", "-nw" };
 	return dirs[n >= sizeof(dirs)/sizeof(*dirs) ? 0 : n];
 }
 
@@ -1770,19 +1837,6 @@ surface display::get_flag(gamemap::TERRAIN terrain, int x, int y)
 	return surface(NULL);
 }
 
-void display::blit_surface(int x, int y, surface surf, SDL_Rect* srcrect, SDL_Rect* clip_rect)
-{
-	const surface target(video().getSurface());
-	SDL_Rect dst = {x,y,0,0};
-	
-	if(clip_rect != NULL) {
-		const clip_rect_setter clip_setter(target,*clip_rect);
-		SDL_BlitSurface(surf,srcrect,target,&dst);
-	} else {
-		SDL_BlitSurface(surf,srcrect,target,&dst);
-	}
-}
-
 surface display::get_minimap(int w, int h)
 {
 	if(minimap_ != NULL && (minimap_->w != w || minimap_->h != h)) {
@@ -1846,8 +1900,9 @@ void display::draw_unit(int x, int y, surface image,
 		bool upside_down, fixed_t alpha, Uint32 blendto, double blend_ratio, double submerged,
 		surface ellipse_back, surface ellipse_front)
 {
-	//calculate the y position of the ellipse. It should be the same as the y position of the image, unless
-	//the image is partially submerged, in which case the ellipse should appear to float 'on top of' the water
+	//calculate the y position of the ellipse. It should be the same as the
+	//y position of the image, unless the image is partially submerged, in
+	//which case the ellipse should appear to float 'on top of' the water
 	const int ellipse_ypos = y - (ellipse_back != NULL && submerged > 0.0 ? int(double(ellipse_back->h)*submerged) : 0)/2;
 	if(ellipse_back != NULL) {
 		draw_unit(x,ellipse_ypos,ellipse_back,false,blendto == 0 ? alpha : ftofxp(1.0),0,0.0);
@@ -1879,7 +1934,7 @@ void display::draw_unit(int x, int y, surface image,
 
 	SDL_Rect clip_rect = map_area();
 	SDL_Rect srcrect = {0,0,surf->w,submerge_height};
-	blit_surface(x,y,surf,&srcrect,&clip_rect);
+	video().blit_surface(x,y,surf,&srcrect,&clip_rect);
 
 	if(submerge_height != surf->h) {
 		surf.assign(adjust_surface_alpha(surf,ftofxp(0.2)));
@@ -1888,7 +1943,7 @@ void display::draw_unit(int x, int y, surface image,
 		srcrect.h = surf->h-submerge_height;
 		y += submerge_height;
 
-		blit_surface(x,y,surf,&srcrect,&clip_rect);
+		video().blit_surface(x,y,surf,&srcrect,&clip_rect);
 	}
 
 	if(ellipse_front != NULL) {
@@ -2049,19 +2104,6 @@ void display::set_advancing_unit(const gamemap::location& loc, double amount)
 	draw_tile(loc.x,loc.y);
 }
 
-void display::lock_updates(bool value)
-{
-	if(value == true)
-		++updatesLocked_;
-	else
-		--updatesLocked_;
-}
-
-bool display::update_locked() const
-{
-	return updatesLocked_ > 0;
-}
-
 bool display::turbo() const
 {
 	bool res = turbo_;
@@ -2142,54 +2184,35 @@ const theme::menu* display::menu_pressed()
 	return NULL;
 }
 
+void display::enable_menu(const std::string& item, bool enable)
+{
+	for(std::vector<theme::menu>::const_iterator menu = theme_.menus().begin();
+			menu != theme_.menus().end(); ++menu) {
+		
+		std::vector<std::string>::const_iterator hasitem = 
+			std::find(menu->items().begin(), menu->items().end(), item);
+
+		if(hasitem != menu->items().end()) {
+			const size_t index = menu - theme_.menus().begin();
+			wassert(index < buttons_.size());
+			buttons_[index].enable(enable);
+		}
+	}
+}
+
 void display::begin_game()
 {
 	in_game_ = true;
 	create_buttons();
 }
 
-int display::set_help_string(const std::string& str)
-{
-	font::remove_floating_label(help_string_);
-
-	const SDL_Color colour = {0x0,0x00,0x00,0x77};
-
-	int size = font::SIZE_LARGE;
-
-	while(size > 0) {
-		if(font::line_width(str, size) > x()) {
-			size--;
-		} else {
-			break;
-		}
-	}
-
-	help_string_ = font::add_floating_label(str,size,font::NORMAL_COLOUR,x()/2,y(),0.0,0.0,-1,screen_area(),font::CENTER_ALIGN,&colour,5);
-	const SDL_Rect& rect = font::get_floating_label_rect(help_string_);
-	font::move_floating_label(help_string_,0.0,-double(rect.h));
-	return help_string_;
-}
-
-void display::clear_help_string(int handle)
-{
-	if(handle == help_string_) {
-		font::remove_floating_label(handle);
-		help_string_ = 0;
-	}
-}
-
-void display::clear_all_help_strings()
-{
-	clear_help_string(help_string_);
-}
-
 void display::create_buttons()
 {
 	buttons_.clear();
 
 	const std::vector<theme::menu>& buttons = theme_.menus();
 	for(std::vector<theme::menu>::const_iterator i = buttons.begin(); i != buttons.end(); ++i) {
-		gui::button b(*this,i->title(),gui::button::TYPE_PRESS,i->image());
+		gui::button b(screen_,i->title(),gui::button::TYPE_PRESS,i->image());
 		const SDL_Rect& loc = i->location(screen_area());
 		b.set_location(loc.x,loc.y);
 
@@ -2212,7 +2235,7 @@ void display::remove_observer(const std::string& name)
 }
 
 namespace {
-	const int max_chat_messages = 6;
+	const unsigned int max_chat_messages = 6;
 	const int chat_message_border = 5;
 	const int chat_message_x = 10;
 	const int chat_message_y = 10;
@@ -2231,13 +2254,19 @@ void display::add_chat_message(const std::string& speaker, int side, const std::
 		msg = message;
 		action = false;
 	}
-	gui::text_to_lines(msg,80);
+	msg = font::word_wrap_text(msg,font::SIZE_SMALL,mapx()*3/4);
 
 	int ypos = chat_message_x;
 	for(std::vector<chat_message>::const_iterator m = chat_messages_.begin(); m != chat_messages_.end(); ++m) {
 		ypos += font::get_floating_label_rect(m->handle).h;
 	}
 
+	SDL_Color speaker_colour = {255,255,255,255};
+	if(side >= 1) {
+		speaker_colour = team::get_side_colour(side);
+	}
+	
+	SDL_Color message_colour = chat_message_colour;
 	std::stringstream str;
 	std::stringstream message_str;
 	message_str << msg;
@@ -2245,6 +2274,7 @@ void display::add_chat_message(const std::string& speaker, int side, const std::
 		if(action) {
 			str << "<" << speaker;
 			message_str << ">";
+			message_colour = speaker_colour;
 		} else {
 			str << "<" << speaker << ">";
 		}
@@ -2252,22 +2282,19 @@ void display::add_chat_message(const std::string& speaker, int side, const std::
 		if(action) {
 			str << "*" << speaker;
 			message_str << "*";
+			message_colour = speaker_colour;
 		} else {
 			str << "*" << speaker << "*";
 		}
 	}
 
-	SDL_Color speaker_colour = {255,255,255,255};
-	if(side >= 1) {
-		speaker_colour = team::get_side_colour(side);
-	}
 
 	const SDL_Rect rect = map_area();
 	const int speaker_handle = font::add_floating_label(str.str(),font::SIZE_SMALL,speaker_colour,
-	                                                   rect.x+chat_message_x,rect.y+ypos,
-													   0,0,-1,rect,font::LEFT_ALIGN,&chat_message_bg,chat_message_border);
+		rect.x+chat_message_x,rect.y+ypos,
+		0,0,-1,rect,font::LEFT_ALIGN,&chat_message_bg,chat_message_border);
 
-	const int message_handle = font::add_floating_label(message_str.str(),font::SIZE_SMALL,chat_message_colour,
+	const int message_handle = font::add_floating_label(message_str.str(),font::SIZE_SMALL,message_colour,
 		rect.x + chat_message_x + font::get_floating_label_rect(speaker_handle).w,rect.y+ypos,
 		0,0,-1,rect,font::LEFT_ALIGN,&chat_message_bg,chat_message_border);
 
@@ -2283,7 +2310,7 @@ void display::clear_chat_messages()
 
 void display::prune_chat_messages(bool remove_all)
 {
-	const int message_ttl = remove_all ? 0 : 1200000;
+	const unsigned int message_ttl = remove_all ? 0 : 1200000;
 	if(chat_messages_.empty() == false && (chat_messages_.front().created_at+message_ttl < SDL_GetTicks() || chat_messages_.size() > max_chat_messages)) {
 		const int movement = font::get_floating_label_rect(chat_messages_.front().handle).h;
 
