@@ -1,4 +1,4 @@
-/* $Id: display.hpp,v 1.93 2005/02/09 23:32:01 ydirson Exp $ */
+/* $Id: display.hpp,v 1.101 2005/03/24 21:35:50 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,8 +13,8 @@
 #ifndef DISPLAY_H_INCLUDED
 #define DISPLAY_H_INCLUDED
 
-#include "config.hpp"
-#include "gamestatus.hpp"
+class config;
+class gamestatus;
 #include "image.hpp"
 #include "key.hpp"
 #include "map.hpp"
@@ -107,6 +107,8 @@ public:
 	//invalidates entire screen, including all tiles and sidebar.
 	void redraw_everything();
 
+	void flip();
+
 	//draws invalidated items. If update is true, will also copy the
 	//display to the frame buffer. If force is true, will not skip frames,
 	//even if running behind.
@@ -138,12 +140,12 @@ public:
 	//given x,y co-ordinates of an onscreen pixel, will return the
 	//location of the hex that this pixel corresponds to. Returns an
 	//invalid location is the mouse isn't over any valid location.
-	gamemap::location hex_clicked_on(int x, int y, gamemap::location::DIRECTION* nearest_hex=NULL);
+	gamemap::location hex_clicked_on(int x, int y, gamemap::location::DIRECTION* nearest_hex=NULL, gamemap::location::DIRECTION* second_nearest_hex=NULL);
 	
 	//given x,y co-ordinates of a pixel on the map, will return the
 	//location of the hex that this pixel corresponds to. Returns an
 	//invalid location is the mouse isn't over any valid location.
-	gamemap::location pixel_position_to_hex(int x, int y, gamemap::location::DIRECTION* nearest_hex=NULL);
+	gamemap::location pixel_position_to_hex(int x, int y, gamemap::location::DIRECTION* nearest_hex=NULL, gamemap::location::DIRECTION* second_nearest_hex=NULL);
 	
 	//given x,y co-ordinates of the mouse, will return the location of the
 	//hex in the minimap that the mouse is currently over, or an invalid
@@ -206,9 +208,6 @@ public:
 	//gets the underlying screen object.
 	CVideo& video() { return screen_; }
 
-	//blits a surface with black as alpha
-	void blit_surface(int x, int y, surface surface, SDL_Rect* srcrect=NULL, SDL_Rect* clip_rect=NULL);
-
 	//function to invalidate all tiles.
 	void invalidate_all();
 
@@ -268,13 +267,9 @@ public:
 	//by amount.
 	void set_advancing_unit(const gamemap::location& loc, double amount);
 
-	//function to stop the screen being redrawn. Anything that happens while
-	//the update is locked will be hidden from the user's view.
-	//note that this function is re-entrant, meaning that if lock_updates(true)
-	//is called twice, lock_updates(false) must be called twice to unlock
-	//updates.
-	void lock_updates(bool value);
-	bool update_locked() const;
+	//compat methods to be dropped after full migration
+	void lock_updates(bool value) {screen_.lock_updates(value); };
+	bool update_locked() const {return screen_.update_locked(); };
 
 	//functions to set/get whether 'turbo' mode is on. When turbo mode is on,
 	//everything moves much faster.
@@ -304,6 +299,9 @@ public:
 
 	const theme::menu* menu_pressed();
 
+	//finds the menu which has a given item in it, and enables or disables it.
+	void enable_menu(const std::string& item, bool enable);
+
 	void add_observer(const std::string& name);
 	void remove_observer(const std::string& name);
 	const std::set<std::string>& observers() const { return observers_; }
@@ -352,13 +350,6 @@ public:
 
 	bool in_game() const { return in_game_; }
 
-	//functions to set and clear 'help strings'. A 'help string' is like a tooltip, but it appears
-	//at the bottom of the screen, so as to not be intrusive. Setting a help string sets what
-	//is currently displayed there.
-	int set_help_string(const std::string& str);
-	void clear_help_string(int handle);
-	void clear_all_help_strings();
-
 private:
 	display(const display&);
 	void operator=(const display&);
@@ -458,8 +449,6 @@ private:
 	gamemap::location advancingUnit_;
 	double advancingAmount_;
 
-	int updatesLocked_;
-
 	bool turbo_, grid_;
 	double sidebarScaling_;
 
@@ -482,7 +471,7 @@ private:
 
 		int speaker_handle;
 		int handle;
-		int created_at;
+		Uint32 created_at;
 	};
 
 	void prune_chat_messages(bool remove_all=false);
@@ -503,9 +492,6 @@ private:
 
 	int diagnostic_label_;
 
-	//variables for help strings
-	int help_string_;
-
 	//animated flags for each team
 	std::vector<animated<image::locator> > flags_;
 
@@ -513,31 +499,6 @@ private:
 	int fps_handle_;
 };
 
-//an object which will lock the display for the duration of its lifetime.
-struct update_locker
-{
-	update_locker(display& d, bool lock=true) : disp(d), unlock(lock) {
-		if(lock) {
-			disp.lock_updates(true);
-		}
-	}
-
-	~update_locker() {
-		unlock_update();
-	}
-
-	void unlock_update() {
-		if(unlock) {
-			disp.lock_updates(false);
-			unlock = false;
-		}
-	}
-
-private:
-	display& disp;
-	bool unlock;
-};
-
 bool angle_is_northern(size_t n);
 const std::string& get_angle_direction(size_t n);
 
