@@ -19,10 +19,9 @@
 #include "../filesystem.hpp"
 #include "../font.hpp"
 #include "../game_config.hpp"
-#include "../gamestatus.hpp"
+#include "../gettext.hpp"
 #include "../key.hpp"
 #include "../widgets/menu.hpp"
-#include "../language.hpp"
 #include "../pathfind.hpp"
 #include "../playlevel.hpp"
 #include "../preferences.hpp"
@@ -31,6 +30,9 @@
 #include "../team.hpp"
 #include "../util.hpp"
 #include "../video.hpp"
+#include "../wml_separators.hpp"
+#include "serialization/parser.hpp"
+#include "serialization/string_utils.hpp"
 
 #include "editor.hpp"
 #include "map_manip.hpp"
@@ -97,7 +99,8 @@ map_editor::map_editor(display &gui, gamemap &map, config &theme, config &game_c
 		// Perform some initializations that should only be performed
 		// the first time the editor object is created.
 		try {
-			prefs_.read(read_file(prefs_filename));
+			scoped_istream stream = istream_file(prefs_filename);
+			read(prefs_, *stream);
 		}
 		catch (config::error e) {
 			std::cerr << "Error when reading " << prefs_filename << ": "
@@ -133,7 +136,8 @@ map_editor::~map_editor() {
 	old_bg_terrain_ = palette_.selected_bg_terrain();
 	old_brush_size_ = brush_.selected_brush_size();
 	try {
-		write_file(prefs_filename, prefs_.write());
+		scoped_ostream prefs_file = ostream_file(prefs_filename);
+		write(*prefs_file, prefs_);
 	}
 	catch (io_exception& e) {
 		std::cerr << "Error when writing to " << prefs_filename << ": "
@@ -523,12 +527,16 @@ void map_editor::edit_select_all() {
 void map_editor::edit_draw() {
 	left_button_func_changed(DRAW);
 }
+void map_editor::edit_refresh() {
+	image::flush_cache();
+	redraw_everything();
+}
 
 std::string map_editor::load_map(const std::string filename) {
 	bool load_successful = true;
 	std::string msg;
 	std::string new_map;
-	string_map symbols;
+	utils::string_map symbols;
 	symbols["filename"] = filename;
 	if (!file_exists(filename) || is_directory(filename)) {
 		load_successful = false;
@@ -546,7 +554,7 @@ std::string map_editor::load_map(const std::string filename) {
 	if (!load_successful) {
 		const std::string failed_msg = _("Load failed: ");
 		const std::string show_msg = failed_msg +
-			config::interpolate_variables_into_string(msg, &symbols);
+			utils::interpolate_variables_into_string(msg, &symbols);
 		gui::show_dialog(gui_, NULL, "", show_msg, gui::OK_ONLY);
 		throw load_map_exception();
 	}
@@ -607,6 +615,7 @@ bool map_editor::can_execute_command(hotkey::HOTKEY_COMMAND command) const {
 	case hotkey::HOTKEY_EDIT_FLIP:
 	case hotkey::HOTKEY_EDIT_SELECT_ALL:
 	case hotkey::HOTKEY_EDIT_DRAW:
+	case hotkey::HOTKEY_EDIT_REFRESH:
 		return true;
 	default:
 		return false;
@@ -1022,7 +1031,7 @@ bool map_editor::save_map(const std::string fn, const bool display_confirmation)
 		}
 	}
 	catch (io_exception& e) {
-		string_map symbols;
+		utils::string_map symbols;
 		symbols["msg"] = e.what();
 		const std::string msg = vgettext("Could not save the map: $msg",symbols);
 		gui::show_dialog(gui_, NULL, "", msg, gui::OK_ONLY);
@@ -1141,8 +1150,8 @@ void map_editor::update_l_button_palette() {
 			else {
 				color = SDL_MapRGB(screen->format,0x00,0x00,0x00);
 			}
-			gui::draw_rectangle(draw_rect.x, draw_rect.y, draw_rect.w, draw_rect.h,
-								color, gui_.video().getSurface());
+			draw_rectangle(draw_rect.x, draw_rect.y, draw_rect.w, draw_rect.h,
+				       color, gui_.video().getSurface());
 			update_rect(draw_rect);
 		}
 	}
