@@ -16,9 +16,10 @@
 #include "../show_dialog.hpp"
 #include "../config.hpp"
 #include "../events.hpp"
-#include "../language.hpp"
 #include "../game_config.hpp"
+#include "../gettext.hpp"
 #include "../mapgen.hpp"
+#include "../map_create.hpp"
 #include "../filesystem.hpp"
 #include "../font.hpp"
 #include "../events.hpp"
@@ -27,6 +28,7 @@
 #include "../language.hpp"
 #include "../map.hpp"
 #include "../preferences.hpp"
+#include "../video.hpp"
 
 #include "editor_dialogs.hpp"
 
@@ -66,17 +68,17 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 	SDL_Rect dialog_rect = {xpos-10,ypos-10,width+20,height+20};
 	surface_restorer restorer(&disp.video(),dialog_rect);
 
-	gui::draw_dialog_frame(xpos,ypos,width,height,disp);
+	gui::draw_dialog_frame(xpos,ypos,width,height,disp.video());
 
-	SDL_Rect title_rect = font::draw_text(NULL,disp.screen_area(),24,font::NORMAL_COLOUR,
+	SDL_Rect title_rect = font::draw_text(NULL,screen_area(),24,font::NORMAL_COLOUR,
 					      _("Create New Map"),0,0);
 
 	const std::string& width_label = _("Width:");
 	const std::string& height_label = _("Height:");
 
-	SDL_Rect width_rect = font::draw_text(NULL, disp.screen_area(), 14, font::NORMAL_COLOUR,
+	SDL_Rect width_rect = font::draw_text(NULL, screen_area(), 14, font::NORMAL_COLOUR,
 										  width_label, 0, 0);
-	SDL_Rect height_rect = font::draw_text(NULL, disp.screen_area(), 14, font::NORMAL_COLOUR,
+	SDL_Rect height_rect = font::draw_text(NULL, screen_area(), 14, font::NORMAL_COLOUR,
 										   height_label, 0, 0);
 
 	const int text_right = xpos + horz_margin +
@@ -88,10 +90,10 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 	width_rect.y = ypos + title_rect.h + vertical_margin*2;
 	height_rect.y = width_rect.y + width_rect.h + vertical_margin;
 
-	gui::button new_map_button(disp, _("Generate New Map"));
-	gui::button random_map_button(disp, _("Generate Random Map"));
-	gui::button random_map_setting_button(disp, _("Random Generator Settings"));
-	gui::button cancel_button(disp, _("Cancel"));
+	gui::button new_map_button(disp.video(), _("Generate New Map"));
+	gui::button random_map_button(disp.video(), _("Generate Random Map"));
+	gui::button random_map_setting_button(disp.video(), _("Random Generator Settings"));
+	gui::button cancel_button(disp.video(), _("Cancel"));
 
 	new_map_button.set_location(xpos + horz_margin,height_rect.y + height_rect.h + vertical_margin);
 	random_map_button.set_location(xpos + horz_margin,ypos + height - random_map_button.height()-14*2-vertical_margin);
@@ -108,14 +110,14 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 	SDL_Rect slider_rect = { slider_left,width_rect.y,slider_right-slider_left,width_rect.h};
 
 	slider_rect.y = width_rect.y;
-	gui::slider width_slider(disp);
+	gui::slider width_slider(disp.video());
 	width_slider.set_location(slider_rect);
 	width_slider.set_min(map_min_width);
 	width_slider.set_max(map_max_width);
 	width_slider.set_value(map_width);
 
 	slider_rect.y = height_rect.y;
-	gui::slider height_slider(disp);
+	gui::slider height_slider(disp.video());
 	height_slider.set_location(slider_rect);
 	height_slider.set_min(map_min_height);
 	height_slider.set_max(map_max_height);
@@ -128,7 +130,7 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 		const config* const toplevel_cfg = game_config.find_child("multiplayer","id","ranmap");
 		const config* const cfg = toplevel_cfg == NULL ? NULL : toplevel_cfg->child("generator");
 		if (cfg == NULL) {
-			config dummy_cfg("");
+			config dummy_cfg;
 			random_map_generator.assign(create_map_generator("", &dummy_cfg));
 		}
 		else {
@@ -188,24 +190,24 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 		if (draw) {
 			map_width = width_slider.value();
 			map_height = height_slider.value();
-			gui::draw_dialog_frame(xpos,ypos,width,height,disp);
-			title_rect = font::draw_text(&disp,disp.screen_area(),24,font::NORMAL_COLOUR,
+			gui::draw_dialog_frame(xpos,ypos,width,height,disp.video());
+			title_rect = font::draw_text(&disp.video(),screen_area(),24,font::NORMAL_COLOUR,
 										 _("Create New Map"),
 										 xpos+(width-title_rect.w)/2,ypos+10);
 
-			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,
+			font::draw_text(&disp.video(),screen_area(),14,font::NORMAL_COLOUR,
 							width_label,width_rect.x,width_rect.y);
-			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,
+			font::draw_text(&disp.video(),screen_area(),14,font::NORMAL_COLOUR,
 							height_label,height_rect.x,height_rect.y);
 			
 			std::stringstream width_str;
 			width_str << map_width;
-			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,width_str.str(),
+			font::draw_text(&disp.video(),screen_area(),14,font::NORMAL_COLOUR,width_str.str(),
 							slider_right+horz_margin,width_rect.y);
 			
 			std::stringstream height_str;
 			height_str << map_height;
-			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,height_str.str(),
+			font::draw_text(&disp.video(),screen_area(),14,font::NORMAL_COLOUR,height_str.str(),
 							slider_right+horz_margin,height_rect.y);
 			
 		}
@@ -242,13 +244,13 @@ void preferences_dialog(display &disp, config &prefs) {
 
 	SDL_Rect clip_rect = {0,0,disp.x(),disp.y()};
 
-	gui::button close_button(disp,_("Close Window"));
+	gui::button close_button(disp.video(),_("Close Window"));
 
 	std::vector<gui::button*> buttons;
 	buttons.push_back(&close_button);
 
 	surface_restorer restorer;
-	gui::draw_dialog(xpos,ypos,width,height,disp,_("Preferences"),NULL,&buttons,&restorer);
+	gui::draw_dialog(xpos,ypos,width,height,disp.video(),_("Preferences"),NULL,&buttons,&restorer);
 
 	const std::string& scroll_label = _("Scroll Speed:");
 
@@ -271,30 +273,30 @@ void preferences_dialog(display &disp, config &prefs) {
 	SDL_Rect slider_rect = { slider_left, scroll_pos, slider_right - slider_left, 10  };
 
 	slider_rect.y = scroll_pos;
-	gui::slider scroll_slider(disp);
+	gui::slider scroll_slider(disp.video());
 	scroll_slider.set_location(slider_rect);
 	scroll_slider.set_min(1);
 	scroll_slider.set_max(100);
 	scroll_slider.set_value(preferences::scroll_speed());
 
-	gui::button fullscreen_button(disp,_("Full Screen"),
+	gui::button fullscreen_button(disp.video(),_("Full Screen"),
 	                              gui::button::TYPE_CHECK);
 
 	fullscreen_button.set_check(preferences::fullscreen());
 
 	fullscreen_button.set_location(slider_left,scroll_pos + 80);
 
-	gui::button grid_button(disp,_("Show Grid"),
+	gui::button grid_button(disp.video(),_("Show Grid"),
 	                        gui::button::TYPE_CHECK);
 	grid_button.set_check(preferences::grid());
 
 	grid_button.set_location(slider_left + fullscreen_button.width() + 100,
 							 scroll_pos + 80);
 
-	gui::button resolution_button(disp,_("Video Mode"));
+	gui::button resolution_button(disp.video(),_("Video Mode"));
 	resolution_button.set_location(slider_left,scroll_pos + 80 + 50);
 
-	gui::button hotkeys_button (disp,_("Hotkeys"));
+	gui::button hotkeys_button (disp.video(),_("Hotkeys"));
 	hotkeys_button.set_location(slider_left + fullscreen_button.width() + 100,
 								scroll_pos + 80 + 50);
 
@@ -315,7 +317,7 @@ void preferences_dialog(display &disp, config &prefs) {
 
 		if(redraw_all) {
 			restorer.cancel();
-			gui::draw_dialog(xpos,ypos,width,height,disp,_("Preferences"),NULL,&buttons,&restorer);
+			gui::draw_dialog(xpos,ypos,width,height,disp.video(),_("Preferences"),NULL,&buttons,&restorer);
 			fullscreen_button.set_dirty();
 			close_button.set_dirty();
 			resolution_button.set_dirty();
@@ -323,10 +325,10 @@ void preferences_dialog(display &disp, config &prefs) {
 			hotkeys_button.set_dirty();
 			scroll_slider.set_dirty();
 
-			font::draw_text(&disp,clip_rect,14,font::NORMAL_COLOUR,scroll_label,
+			font::draw_text(&disp.video(),clip_rect,14,font::NORMAL_COLOUR,scroll_label,
 	    	                scroll_rect.x,scroll_rect.y);
 
-			update_rect(disp.screen_area());
+			update_rect(screen_area());
 
 			redraw_all = false;
 		}
@@ -380,17 +382,17 @@ resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h) {
 	SDL_Rect dialog_rect = {xpos-10,ypos-10,width+20,height+20};
 	surface_restorer restorer(&disp.video(),dialog_rect);
 
-	gui::draw_dialog_frame(xpos,ypos,width,height,disp);
+	gui::draw_dialog_frame(xpos,ypos,width,height,disp.video());
 
-	SDL_Rect title_rect = font::draw_text(NULL,disp.screen_area(),24,font::NORMAL_COLOUR,
+	SDL_Rect title_rect = font::draw_text(NULL,screen_area(),24,font::NORMAL_COLOUR,
 					      _("Resize Map"),0,0);
 
 	const std::string& width_label = _("Width:");
 	const std::string& height_label = _("Height:");
 
-	SDL_Rect width_rect = font::draw_text(NULL, disp.screen_area(), 14, font::NORMAL_COLOUR,
+	SDL_Rect width_rect = font::draw_text(NULL, screen_area(), 14, font::NORMAL_COLOUR,
 										  width_label, 0, 0);
-	SDL_Rect height_rect = font::draw_text(NULL, disp.screen_area(), 14, font::NORMAL_COLOUR,
+	SDL_Rect height_rect = font::draw_text(NULL, screen_area(), 14, font::NORMAL_COLOUR,
 										   height_label, 0, 0);
 
 	const int text_right = xpos + horz_margin +
@@ -402,8 +404,8 @@ resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h) {
 	width_rect.y = ypos + title_rect.h + vertical_margin*2;
 	height_rect.y = width_rect.y + width_rect.h + vertical_margin;
 
-	gui::button cancel_button(disp, _("Cancel"));
-	gui::button ok_button(disp, _("Ok"));
+	gui::button cancel_button(disp.video(), _("Cancel"));
+	gui::button ok_button(disp.video(), _("Ok"));
 
 	cancel_button.set_location(xpos + width - cancel_button.width() - horz_margin,
 	                           ypos + height - cancel_button.height()-14);
@@ -418,14 +420,14 @@ resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h) {
 	SDL_Rect slider_rect = { slider_left,width_rect.y,slider_right-slider_left,width_rect.h};
 
 	slider_rect.y = width_rect.y;
-	gui::slider width_slider(disp);
+	gui::slider width_slider(disp.video());
 	width_slider.set_location(slider_rect);
 	width_slider.set_min(map_min_width);
 	width_slider.set_max(map_max_width);
 	width_slider.set_value(map_width);
 
 	slider_rect.y = height_rect.y;
-	gui::slider height_slider(disp);
+	gui::slider height_slider(disp.video());
 	height_slider.set_location(slider_rect);
 	height_slider.set_min(map_min_height);
 	height_slider.set_max(map_max_height);
@@ -441,24 +443,24 @@ resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h) {
 		if (draw) {
 			map_width = width_slider.value();
 			map_height = height_slider.value();
-			gui::draw_dialog_frame(xpos,ypos,width,height,disp);
-			title_rect = font::draw_text(&disp,disp.screen_area(),24,font::NORMAL_COLOUR,
+			gui::draw_dialog_frame(xpos,ypos,width,height,disp.video());
+			title_rect = font::draw_text(&disp.video(),screen_area(),24,font::NORMAL_COLOUR,
 										 _("Resize Map"),
 										 xpos+(width-title_rect.w)/2,ypos+10);
 
-			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,
+			font::draw_text(&disp.video(),screen_area(),14,font::NORMAL_COLOUR,
 							width_label,width_rect.x,width_rect.y);
-			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,
+			font::draw_text(&disp.video(),screen_area(),14,font::NORMAL_COLOUR,
 							height_label,height_rect.x,height_rect.y);
 			
 			std::stringstream width_str;
 			width_str << map_width;
-			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,width_str.str(),
+			font::draw_text(&disp.video(),screen_area(),14,font::NORMAL_COLOUR,width_str.str(),
 							slider_right+horz_margin,width_rect.y);
 			
 			std::stringstream height_str;
 			height_str << map_height;
-			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,height_str.str(),
+			font::draw_text(&disp.video(),screen_area(),14,font::NORMAL_COLOUR,height_str.str(),
 							slider_right+horz_margin,height_rect.y);
 			
 		}
