@@ -17,14 +17,20 @@
 #include "../filesystem.hpp"
 #include "../font.hpp"
 #include "../game_config.hpp"
+#include "../gamestatus.hpp"
 #include "../gettext.hpp"
 #include "../image.hpp"
 #include "../language.hpp"
 #include "../map.hpp"
 #include "../preferences.hpp"
+#include "../random.hpp"
 #include "../team.hpp"
 #include "../util.hpp"
+#include "../video.hpp"
 #include "../wesconfig.h"
+#include "serialization/parser.hpp"
+#include "serialization/preprocessor.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <cctype>
 #include <cstdlib>
@@ -35,6 +41,8 @@
 int main(int argc, char** argv)
 {
 	game_config::editor = true;
+	rng generator;
+	const set_random_generator generator_setter(&generator);
 
 	int arg;
 	for(arg = 1; arg != argc; ++arg) {
@@ -82,7 +90,7 @@ int main(int argc, char** argv)
 			if(arg+1 != argc) {
 				++arg;
 				const std::string val(argv[arg]);
-				const std::vector<std::string> res = config::split(val,'x');
+				const std::vector<std::string> res = utils::split(val, 'x');
 				if(res.size() == 2) {
 					const int xres = lexical_cast_default<int>(res.front());
 					const int yres = lexical_cast_default<int>(res.back());
@@ -204,15 +212,31 @@ int main(int argc, char** argv)
 	// settings. Maybe they should be moved? Or set an EDITOR define and
 	// make it load that way maybe.
 	defines_map["MULTIPLAYER"] = preproc_define();
+
+	//Set the locale first, then read the configuration, or else WML
+	//strings are not correctly translated. Does this work on on the win32
+	//platform?
+	const bool lang_res = ::set_language(get_locale());
+	if(!lang_res) {
+		std::cerr << "No translation for locale '" << get_locale().language
+		          << "', default to system locale\n";
+
+		const bool lang_res = ::set_language(known_languages[0]);
+		if(!lang_res) {
+			std::cerr << "Language data not found\n";
+		}
+	}
+
+	//Read the configuration af
 	config cfg;
 	try {
-		cfg.read(preprocess_file("data/game.cfg", &defines_map));
+		scoped_istream stream = preprocess_file("data/game.cfg", &defines_map);
+		read(cfg, *stream);
 	}
 	catch (config::error e) {
 		std::cerr << "Error when reading game config: '" << e.message << "'" << std::endl;
 	}
-
-	set_language(known_languages[0]);
+	font::load_font_config();
 
 	if(mapdata.empty()) {
 		for(int i = 0; i != 20; ++i) {
@@ -227,7 +251,7 @@ int main(int argc, char** argv)
 	// Add a dummy team so the reports will be handled properly.
 	teams.push_back(team(cfg));
 	config* theme_cfg = cfg.find_child("theme", "name", "editor");
-	config dummy_theme("");
+	config dummy_theme;
 	if (!theme_cfg) {
 		std::cerr << "Editor theme could not be loaded." << std::endl;
 		theme_cfg = &dummy_theme;
@@ -241,7 +265,7 @@ int main(int argc, char** argv)
 			std::cerr << "creating map...\n";
 			gamemap map(cfg, mapdata);
 
-			const config dummy_cfg("");
+			const config dummy_cfg;
 			display gui(units, video, map, status, teams,
 				    *theme_cfg, cfg, dummy_cfg);
 	
