@@ -16,11 +16,11 @@
 #include "editor_palettes.hpp"
 #include "editor_layout.hpp"
 #include "../sdl_utils.hpp"
-#include "../show_dialog.hpp"
 #include "../image.hpp"
 #include "../reports.hpp"
-#include "../language.hpp"
+#include "../gettext.hpp"
 #include "../util.hpp"
+#include "../video.hpp"
 #include "../wassert.hpp"
 
 
@@ -33,9 +33,9 @@ bool is_invalid_terrain(char c) {
 
 terrain_palette::terrain_palette(display &gui, const size_specs &sizes,
 								 const gamemap &map)
-	: gui::widget(gui), size_specs_(sizes), gui_(gui), tstart_(0), map_(map),
-	  top_button_(gui, "", gui::button::TYPE_PRESS, "uparrow-button"),
-	  bot_button_(gui, "", gui::button::TYPE_PRESS, "downarrow-button") {
+	: gui::widget(gui.video()), size_specs_(sizes), gui_(gui), tstart_(0), map_(map),
+	  top_button_(gui.video(), "", gui::button::TYPE_PRESS, "uparrow-button"),
+	  bot_button_(gui.video(), "", gui::button::TYPE_PRESS, "downarrow-button") {
 	terrains_ = map_.get_terrain_list();
 	terrains_.erase(std::remove_if(terrains_.begin(), terrains_.end(), is_invalid_terrain),
 					terrains_.end());
@@ -289,7 +289,7 @@ void terrain_palette::draw(bool force) {
 		else {
 			color = SDL_MapRGB(screen->format,0x00,0x00,0x00);
 		}
-		gui::draw_rectangle(dstrect.x, dstrect.y, image->w, image->h, color, screen);
+		draw_rectangle(dstrect.x, dstrect.y, image->w, image->h, color, screen);
 		if (counter_from_zero % 2 != 0)
 			y += size_specs_.terrain_space;
 	}
@@ -329,7 +329,7 @@ void terrain_palette::update_report() {
 // }
 
 brush_bar::brush_bar(display &gui, const size_specs &sizes)
-	: gui::widget(gui), size_specs_(sizes), gui_(gui), selected_(0), total_brush_(3),
+	: gui::widget(gui.video()), size_specs_(sizes), gui_(gui), selected_(0), total_brush_(3),
 	  size_(30) {
 	adjust_size();
 }
@@ -421,7 +421,7 @@ void brush_bar::draw(bool force) {
 		const Uint32 color = (unsigned)i == selected_brush_size() ? 
 			SDL_MapRGB(screen->format,0xFF,0x00,0x00) :
 			SDL_MapRGB(screen->format,0x00,0x00,0x00);
-		gui::draw_rectangle(dstrect.x, dstrect.y, image->w, image->h, color, screen);
+		draw_rectangle(dstrect.x, dstrect.y, image->w, image->h, color, screen);
 		x += image->w;
 	}
 	update_rect(loc);
