@@ -1,7 +1,4 @@
-//disable the very annoying VC++ warning 4786
-#ifdef WIN32
-#pragma warning(disable:4786)
-#endif
+#include "global.hpp"
 
 #include "clipboard.hpp"
 #include "cursor.hpp"
@@ -9,11 +6,11 @@
 #include "mouse.hpp"
 #include "preferences.hpp"
 #include "video.hpp"
+#include "wassert.hpp"
 
 #include "SDL.h"
 
 #include <algorithm>
-#include <cassert>
 #include <deque>
 #include <utility>
 #include <vector>
@@ -144,7 +141,7 @@ event_context::event_context(bool create) : create_(create)
 event_context::~event_context()
 {
 	if(create_) {
-		assert(event_contexts.empty() == false);
+		wassert(event_contexts.empty() == false);
 
 		event_contexts.pop_back();
 	}
@@ -158,7 +155,7 @@ handler::handler() : unicode_(SDL_EnableUNICODE(1))
 
 handler::~handler()
 {
-	assert(event_contexts.empty() == false);
+	wassert(event_contexts.empty() == false);
 
 	for(std::deque<context>::reverse_iterator i = event_contexts.rbegin(); i != event_contexts.rend(); ++i) {
 		if(i->remove_handler(this)) {
@@ -237,7 +234,7 @@ void pump()
 			case SDL_VIDEORESIZE: {
 				const SDL_ResizeEvent* const resize = reinterpret_cast<SDL_ResizeEvent*>(&event);
 
-				if(resize->w < 800 || resize->h < 600) {
+				if(resize->w < min_allowed_width || resize->h < min_allowed_height) {
 					resize_dimensions.first = 0;
 					resize_dimensions.second = 0;
 				} else {
@@ -332,7 +329,7 @@ void raise_process_event()
 		//events may cause more event handlers to be added and/or removed,
 		//so we must use indexes instead of iterators here.
 		for(size_t i1 = 0, i2 = event_handlers.size(); i1 != i2 && i1 < event_handlers.size(); ++i1) {
-			event_handlers[i1]->process();
+			event_handlers[i1]->process_event();
 		}
 	}
 }
