@@ -1,4 +1,4 @@
-/* $Id: filesystem.cpp,v 1.59 2005/01/28 23:38:52 ydirson Exp $ */
+/* $Id: filesystem.cpp,v 1.67 2005/04/05 21:47:27 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -63,7 +63,7 @@ BPath be_path;
 #include "scoped_resource.hpp"
 #include "util.hpp"
 
-#define LOG_G lg::info(lg::general)
+#define LOG_G LOG_STREAM(info, general)
 
 #ifdef USE_ZIPIOS
 #include <sstream>
@@ -75,7 +75,7 @@ BPath be_path;
 #include "zipios++/xcoll.hpp"
 
 namespace {
-	xzipios::XCColl the_collection;
+	xzipios::XCColl* the_collection = NULL;
 
 	void register_zipdir(const std::string directory) {
 		// look for zip files
@@ -88,7 +88,7 @@ namespace {
 				if ((fname.size() > suffix.size()) &&
 				    (0 == fname.compare(fname.size() - suffix.size(), suffix.size(), suffix))) {
 					zipios::ZipFile zip(game_config::path + "/" + fname);
-					the_collection.addCollection(zip);
+					the_collection->addCollection(zip);
 					LOG_G << "zip collection " << fname << 
 						" has " << zip.size() << " elements\n";
 				}
@@ -103,18 +103,25 @@ namespace {
 bool filesystem_init()
 {
 #ifdef USE_ZIPIOS
+	if (the_collection != NULL) {
+		// this is a re-read, cleanup first !
+		free (the_collection);
+	}
+
+	the_collection = new xzipios::XCColl;
+
 	if (!get_user_data_dir().empty()) {
 		LOG_G << "looking at user dir " << get_user_data_dir() << "\n";
 		zipios::DirectoryCollection dir(get_user_data_dir());
 		LOG_G << "user collection has " << dir.size() << " elements\n";
-		the_collection.addCollection(dir);
+		the_collection->addCollection(dir);
 		register_zipdir(get_user_data_dir());
 	}
 	if (!game_config::path.empty()) {
 		LOG_G << "looking at system dir " << game_config::path << "\n";
 		zipios::DirectoryCollection dir(game_config::path);
 		LOG_G << "system collection has " << dir.size() << " elements\n";
-		the_collection.addCollection(dir);
+		the_collection->addCollection(dir);
 		register_zipdir(game_config::path);
 	}
 #endif
@@ -137,8 +144,8 @@ void get_files_in_dir(const std::string& directory,
                       FILE_NAME_MODE mode)
 {
 #ifdef USE_ZIPIOS
-	if (the_collection.hasSubdir(directory)) {
-		the_collection.childrenOf(directory, files, dirs);
+	if (the_collection->hasSubdir(directory)) {
+		the_collection->childrenOf(directory, files, dirs);
 	}
 	else
 #endif
@@ -453,9 +460,9 @@ std::string read_file(const std::string& fname)
 	LOG_G << "Reading " << fname << "\n";
 #ifdef USE_ZIPIOS
 	if(!fname.empty() && fname[0] != '/') {
-		zipios::ConstEntryPointer p = the_collection.getEntry(fname);
+		zipios::ConstEntryPointer p = the_collection->getEntry(fname);
 		if (p != 0) {
-			std::istream* s = the_collection.getInputStream(p);
+			std::istream* s = the_collection->getInputStream(p);
 			if (s != NULL) {
 				std::string contents = read_stream(*s);
 				delete s;
@@ -482,6 +489,36 @@ std::string read_file(const std::string& fname)
 	}
 }
 
+std::istream *istream_file(std::string const &fname)
+{
+	LOG_G << "Streaming " << fname << " for reading.\n";
+#ifdef USE_ZIPIOS
+	if (!fname.empty() && fname[0] != '/') {
+		zipios::ConstEntryPointer p = the_collection->getEntry(fname);
+		if (p != 0)
+			if (std::istream *s = the_collection->getInputStream(p))
+				return s;
+	}
+#else
+	if (!fname.empty() && fname[0] != '/' && !game_config::path.empty()) {
+		std::ifstream *s = new std::ifstream((game_config::path + "/" + fname).c_str(),std::ios_base::binary);
+		if (s->is_open())
+			return s;
+		delete s;
+	}
+#endif
+
+	// FIXME: why do we rely on this even with relative paths ?
+	// still useful with zipios, for things like cache and prefs
+	return new std::ifstream(fname.c_str(),std::ios_base::binary);
+}
+
+std::ostream *ostream_file(std::string const &fname)
+{
+	LOG_G << "Streaming " << fname << " for writing.\n";
+	return new std::ofstream(fname.c_str(), std::ios_base::binary);
+}
+
 //throws io_exception if an error occurs
 void write_file(const std::string& fname, const std::string& data)
 {
@@ -549,7 +586,7 @@ bool is_directory(const std::string& fname)
 {
 #ifdef USE_ZIPIOS
 	if(!fname.empty() && fname[0] != '/') {
-		return the_collection.hasSubdir(fname);
+		return the_collection->hasSubdir(fname);
 	}
 
 #else
@@ -565,10 +602,10 @@ bool is_directory(const std::string& fname)
 bool file_exists(const std::string& name)
 {
 #ifdef USE_ZIPIOS
-	if (the_collection.getEntry(name))
+	if (the_collection->getEntry(name))
 		return true;
 #endif
-	std::ifstream file(name.c_str());
+	std::ifstream file(name.c_str(),std::ios_base::binary);
 	if (file.rdstate() != 0)
 	        return false;
 	file.close();
@@ -740,7 +777,7 @@ void binary_paths_manager::set_paths(const config& cfg)
 
 	const config::child_list& items = cfg.get_children("binary_path");
 	for(config::child_list::const_iterator i = items.begin(); i != items.end(); ++i) {
-		const std::string path = (**i)["path"] + "/";
+		const std::string path = (**i)["path"].str() + "/";
 		if(binary_paths.count(path) == 0) {
 			binary_paths.insert(path);
 			paths_.push_back(path);
@@ -794,3 +831,24 @@ std::string get_binary_file_location(const std::string& type, const std::string&
 	return "";
 }
 
+void scoped_istream::operator=(std::istream *s)
+{
+	delete stream;
+	stream = s;
+}
+
+scoped_istream::~scoped_istream()
+{
+	delete stream;
+}
+
+void scoped_ostream::operator=(std::ostream *s)
+{
+	delete stream;
+	stream = s;
+}
+
+scoped_ostream::~scoped_ostream()
+{
+	delete stream;
+}
