@@ -1,4 +1,4 @@
-/* $Id: font.hpp,v 1.47 2004/11/27 10:24:50 silene Exp $ */
+/* $Id: font.hpp,v 1.54 2005/04/02 14:06:35 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -16,10 +16,12 @@
 #include "SDL.h"
 #include "SDL_ttf.h"
 
-#include "display.hpp"
-#include "video.hpp"
+class CVideo;
+#include "sdl_utils.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <string>
+#include <vector>
 
 namespace font {
 
@@ -30,8 +32,16 @@ struct manager {
 	struct error {};
 };
 
-//function which sets the name of the font to use
-void set_font();
+//structure used to describe a font, and the subset of the Unicode character
+//set it covers.
+struct subset_descriptor
+{
+	std::string name;
+	std::vector<std::pair<size_t, size_t> > present_codepoints;
+};
+
+//sets the font list to be used.
+void set_font_list(const std::vector<subset_descriptor>& fontlist);
 
 //various standard colours
 extern const SDL_Color NORMAL_COLOUR, GOOD_COLOUR, BAD_COLOUR, BLACK_COLOUR,
@@ -49,17 +59,17 @@ const int SIZE_NORMAL = 14;
 #endif
 inline int relative_size(int size)
 {
-  return (SIZE_NORMAL * size / 14);
+	return (SIZE_NORMAL * size / 14);
 }
 // automatic computation of other font sizes, to be made a default for theme-provided values
 const int
-  SIZE_TINY	= relative_size(10),
-  SIZE_SMALL	= relative_size(12),
+	SIZE_TINY       = relative_size(10),
+	SIZE_SMALL      = relative_size(12),
 
-  SIZE_15	= relative_size(15),
-  SIZE_PLUS	= relative_size(16),
-  SIZE_LARGE	= relative_size(18),
-  SIZE_XLARGE	= relative_size(24)
+	SIZE_15         = relative_size(15),
+	SIZE_PLUS       = relative_size(16),
+	SIZE_LARGE      = relative_size(18),
+	SIZE_XLARGE     = relative_size(24)
   ;  
 //function to draw text on the screen. The text will be clipped to area.
 //If the text runs outside of area horizontally, an ellipsis will be displayed
@@ -78,7 +88,7 @@ const int
 //a bounding rectangle of the text is returned. If gui is NULL, then the
 //text will not be drawn, and a bounding rectangle only will be returned.
 
-SDL_Rect draw_text(display* gui, const SDL_Rect& area, int size,
+SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
                    const SDL_Color& colour, const std::string& text,
                    int x, int y, bool use_tooltips = false, int style = 0);
 
@@ -87,7 +97,10 @@ SDL_Rect text_area(const std::string& text, int size, int style=0);
 
 // Returns a SDL surface containing the text rendered in a given colour.
 surface get_rendered_text(const std::string& text, int size, const SDL_Color& colour, int style=0);
-	
+
+//measures the width and height of a single ucs2 text line
+SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::const_iterator last, int font_size = SIZE_NORMAL, int style = TTF_STYLE_NORMAL);
+
 // Returns the maximum height of a font, in pixels
 int get_max_height(int size);
 	
@@ -97,14 +110,20 @@ bool is_format_char(char c);
 /// Determine the width of a line of text given a certain font size.
 /// The font type used is the default wesnoth font type.
 ///
-int line_width(const std::string line, int font_size, int style=TTF_STYLE_NORMAL);
+int line_width(const std::string& line, int font_size, int style=TTF_STYLE_NORMAL);
+
+///
+/// Determine the size of a line of text given a certain font size. Similar to
+/// line_width, but for both coordinates.
+///
+SDL_Rect line_size(const std::string& line, int font_size, int style=TTF_STYLE_NORMAL);
 
 ///
 /// If the text exceedes the specified max width, wrap it one a word basis.
 /// If the is not possible, e.g. the word is too big to fit, wrap it on a
 /// char basis.
 ///
-std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width);
+std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height=-1);
 
 ///
 /// If the text excedes the specified max width, end it with an ellipsis (...)
@@ -121,7 +140,7 @@ std::string make_text_ellipsis(const std::string& text, int font_size, int max_w
 ///
 /// @return a bounding rectangle of the text.
 /// 
-SDL_Rect draw_wrapped_text(display* gui, const SDL_Rect& area, int font_size,
+SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
 			     const SDL_Color& colour, const std::string& text,
 			     int x, int y, int max_width);
 
@@ -173,6 +192,11 @@ SDL_Rect get_floating_label_rect(int handle);
 void draw_floating_labels(surface screen);
 void undraw_floating_labels(surface screen);
 
+bool load_font_config();
+
+//function to chop up one long string of text into lines
+size_t text_to_lines(std::string& text, size_t max_length);
+
 }
 
 #endif
