@@ -1,4 +1,4 @@
-/* $Id: game.cpp,v 1.191 2005/02/19 09:26:31 isaaccp Exp $ */
+/* $Id: game.cpp,v 1.226 2005/04/09 20:11:19 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -26,8 +26,10 @@
 #include "filesystem.hpp"
 #include "font.hpp"
 #include "game_config.hpp"
+#include "game_errors.hpp"
 #include "game_events.hpp"
 #include "gamestatus.hpp"
+#include "gettext.hpp"
 #include "hotkeys.hpp"
 #include "intro.hpp"
 #include "key.hpp"
@@ -35,7 +37,6 @@
 #include "log.hpp"
 #include "mapgen.hpp"
 #include "multiplayer.hpp"
-#include "multiplayer_client.hpp"
 #include "network.hpp"
 #include "pathfind.hpp"
 #include "playlevel.hpp"
@@ -52,6 +53,12 @@
 #include "unit.hpp"
 #include "video.hpp"
 #include "wassert.hpp"
+#include "wml_separators.hpp"
+#include "serialization/binary_or_text.hpp"
+#include "serialization/binary_wml.hpp"
+#include "serialization/parser.hpp"
+#include "serialization/preprocessor.hpp"
+#include "serialization/string_utils.hpp"
 #include "widgets/button.hpp"
 #include "widgets/menu.hpp"
 
@@ -59,6 +66,7 @@
 
 #include <cmath>
 #include <cstdlib>
+#include <ctime>
 #include <fstream>
 #include <iostream>
 #include <iterator>
@@ -86,7 +94,7 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 		//if the starting state is specified, then use that,
 		//otherwise get the scenario data and start from there.
 		if(state.starting_pos.empty() == false) {
-			std::cerr << "loading starting position: '" << state.starting_pos.write() << "'\n";
+			std::cerr << "loading starting position...\n";
 			starting_pos = state.starting_pos;
 			scenario = &starting_pos;
 		} else {
@@ -94,10 +102,6 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 			scenario = game_config.find_child(type,"id",state.scenario);
 			std::cerr << "scenario found: " << (scenario != NULL ? "yes" : "no") << "\n";
 		}
-
-		if(!recorder.at_end()) {
-			statistics::clear_current_scenario();
-		}
 	} else {
 		std::cerr << "loading snapshot...\n";
 		//load from a save-snapshot.
@@ -141,7 +145,7 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 							config snapshot;
 
 							recorder.save_game(label, snapshot, state.starting_pos);
-						} catch(gamestatus::save_game_failed&) {
+						} catch(game::save_game_failed&) {
 							gui::show_error_message(disp, _("The game could not be saved"));
 							retry = true;
 						};
@@ -163,10 +167,10 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 			if(res != VICTORY) {
 				return res;
 			}
-		} catch(gamestatus::load_game_failed& e) {
+		} catch(game::load_game_failed& e) {
 			gui::show_error_message(disp, _("The game could not be loaded: ") + e.message);
 			return QUIT;
-		} catch(gamestatus::game_error& e) {
+		} catch(game::game_error& e) {
 			gui::show_error_message(disp, _("Error while playing the game: ") + e.message);
 			return QUIT;
 		} catch(gamemap::incorrect_format_exception& e) {
@@ -198,7 +202,7 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 				if(should_save == 0) {
 					try {
 						save_game(state);
-					} catch(gamestatus::save_game_failed&) {
+					} catch(game::save_game_failed&) {
 						gui::show_error_message(disp, _("The game could not be saved"));
 						retry = true;
 					}
@@ -221,86 +225,6 @@ LEVEL_RESULT play_game(display& disp, game_state& state, config& game_config,
 
 namespace {
 
-//this function reads the game configuration, searching for valid cached copies first
-void read_game_cfg(preproc_map& defines, std::vector<line_source>& line_src, config& cfg, bool use_cache)
-{
-	log_scope("read_game_cfg");
-
-	if(defines.size() < 4) {
-		bool is_valid = true;
-		std::stringstream str;
-		str << "-v" << game_config::version;
-		for(preproc_map::const_iterator i = defines.begin(); i != defines.end(); ++i) {
-			if(i->second.value != "" || i->second.arguments.empty() == false) {
-				is_valid = false;
-				break;
-			}
-
-			str << "-" << i->first;
-		}
-		std::string localename = get_locale().localename;
-		str << "-lang_" << (localename.empty() ? "default" : localename);
-
-		if(is_valid) {
-			const std::string& cache = get_cache_dir();
-			if(cache != "") {
-				const std::string fname = cache + "/game.cfg-cache" + str.str();
-				const std::string fname_checksum = fname + ".checksum";
-
-				file_tree_checksum dir_checksum;
-				
-				if(use_cache) {
-					try {
-						if(file_exists(fname_checksum)) {
-							const config checksum_cfg(read_file(fname_checksum));
-							dir_checksum = file_tree_checksum(checksum_cfg);
-						}
-					} catch(config::error&) {
-						std::cerr << "cache checksum is corrupt\n";
-					} catch(io_exception&) {
-						std::cerr << "error reading cache checksum\n";
-					}
-				}
-
-				if(use_cache && file_exists(fname) && file_create_time(fname) > data_tree_checksum().modified && dir_checksum == data_tree_checksum()) {
-					std::cerr << "found valid cache at '" << fname << "' using it\n";
-					log_scope("read cache");
-					compression_schema schema;
-
-					try {
-						cfg.read_compressed(read_file(fname),schema);
-						return;
-					} catch(config::error&) {
-						std::cerr << "cache is corrupt. Loading from files\n";
-					} catch(io_exception&) {
-						std::cerr << "error reading cache. Loading from files\n";
-					}
-				}
-
-				std::cerr << "no valid cache found. Writing cache to '" << fname << "'\n";
-				
-				//read the file and then write to the cache
-				cfg.read(preprocess_file("data/game.cfg",&defines,&line_src),&line_src);
-				try {
-					compression_schema schema;
-					write_file(fname,cfg.write_compressed(schema));
-
-					config checksum_cfg;
-					data_tree_checksum().write(checksum_cfg);
-					std::cerr << "wrote checksum: '" << checksum_cfg.write() << "'\n";
-					write_file(fname_checksum,checksum_cfg.write());
-				} catch(io_exception&) {
-					std::cerr << "could not write to cache '" << fname << "'\n";
-				}
-
-				return;
-			}
-		}
-	}
-
-	std::cerr << "caching cannot be done. Reading file\n";
-	cfg.read(preprocess_file("data/game.cfg",&defines,&line_src),&line_src);
-}
 
 bool less_campaigns_rank(const config* a, const config* b) {
 	return lexical_cast_default<int>((*a)["rank"],1000) <
@@ -335,6 +259,7 @@ private:
 	game_controller(const game_controller&);
 	void operator=(const game_controller&);
 
+	void read_game_cfg(preproc_map& defines, std::vector<line_source>& line_src, config& cfg, bool use_cache);
 	void refresh_game_cfg();
 
 	void download_campaigns();
@@ -392,7 +317,7 @@ game_controller::game_controller(int argc, char** argv, bool use_sound)
 			if(arg_+1 != argc_) {
 				++arg_;
 				const std::string val(argv_[arg_]);
-				const std::vector<std::string> res = config::split(val,'x');
+				const std::vector<std::string> res = utils::split(val, 'x');
 				if(res.size() == 2) {
 					const int xres = lexical_cast_default<int>(res.front());
 					const int yres = lexical_cast_default<int>(res.back());
@@ -453,7 +378,7 @@ display& game_controller::disp()
 		}
 
 		static display::unit_map dummy_umap;
-		static config dummy_cfg("");
+		static config dummy_cfg;
 		static gamemap dummy_map(dummy_cfg, "1");
 		static gamestatus dummy_status(dummy_cfg, 0);
 		static std::vector<team> dummy_teams;
@@ -570,6 +495,9 @@ bool game_controller::init_config()
 {
 	units_data_.clear();
 	defines_map_.clear();
+	//Resets old_defines_map_, to force refresh_game_cfg to reload
+	//everything.
+	old_defines_map_.clear();
 
 	//load in the game's configuration files
 #if defined(__APPLE__)
@@ -596,12 +524,12 @@ bool game_controller::init_config()
 
 bool game_controller::init_language()
 {
-	const bool lang_res = set_language(get_locale());
+	const bool lang_res = ::set_language(get_locale());
 	if(!lang_res) {
 		std::cerr << "No translation for locale '" << get_locale().language
 		          << "', default to system locale\n";
 
-		const bool lang_res = set_language(known_languages[0]);
+		const bool lang_res = ::set_language(known_languages[0]);
 		if(!lang_res) {
 			std::cerr << "Language data not found\n";
 		}
@@ -642,7 +570,7 @@ bool game_controller::play_multiplayer_mode()
    	std::map<int,std::string> side_types, side_controllers, side_algorithms;
    	std::map<int,string_map> side_parameters;
 
-   	int sides_counted = 0;
+   	size_t sides_counted = 0;
 
    	for(++arg_; arg_ < argc_; ++arg_) {
    		const std::string val(argv_[arg_]);
@@ -650,7 +578,7 @@ bool game_controller::play_multiplayer_mode()
    			continue;
    		}
 
-   		std::vector<std::string> name_value = config::split(val,'=');
+   		std::vector<std::string> name_value = utils::split(val, '=');
    		if(name_value.size() > 2) {
    			std::cerr << "invalid argument '" << val << "'\n";
    			return false;
@@ -661,7 +589,7 @@ bool game_controller::play_multiplayer_mode()
    			const std::string name_head = name.substr(0,name.size()-1);
    			const char name_tail = name[name.size()-1];
    			const bool last_digit = isdigit(name_tail) ? true:false;
-   			const int side = name_tail - '0';
+   			const size_t side = name_tail - '0';
 
    			if(last_digit && side > sides_counted) {
    				std::cerr << "counted sides: " << side << "\n";
@@ -679,7 +607,7 @@ bool game_controller::play_multiplayer_mode()
    			} else if(last_digit && name_head == "--side") {
    				side_types[side] = value;
    			} else if(last_digit && name_head == "--parm") {
-   				const std::vector<std::string> name_value = config::split(value,':');
+   				const std::vector<std::string> name_value = utils::split(value, ':');
    				if(name_value.size() != 2) {
    					std::cerr << "argument to '" << name << "' must be in the format name:value\n";
    					return false;
@@ -778,9 +706,9 @@ bool game_controller::play_multiplayer_mode()
 
    	try {
    		play_level(units_data_,game_config_,&level,video_,state_,story);
-   	} catch(gamestatus::error& e) {
+   	} catch(game::error& e) {
    		std::cerr << "caught error: '" << e.message << "'\n";
-   	} catch(gamestatus::load_game_exception& e) {
+   	} catch(game::load_game_exception& e) {
    		//the user's trying to load a game, so go into the normal title screen loop and load one
    		loaded_game_ = e.game;
    		loaded_game_show_replay_ = e.show_replay;
@@ -815,8 +743,17 @@ bool game_controller::load_game()
 		//to load a save file, we first load the file in, then we re-parse game
 		//data with the save's #defines, and then we finally parse the save file,
 		//with the game data ready to go.
+
 		config cfg;
-		read_save_file(game,cfg);
+		std::string error_log;
+		read_save_file(game,cfg,&error_log);
+		if(!error_log.empty()) {
+			// FIXME: fix this string after the string-freeze is
+			// over, to clearly state this is a warning.
+			gui::show_error_message(disp(), 
+					_("The file you have tried to load is corrupt: '") +
+					error_log);
+		}
 
 		defines_map_.clear();
 		defines_map_[cfg["difficulty"]] = preproc_define();
@@ -843,7 +780,7 @@ bool game_controller::load_game()
 			}
 		}
 
-	} catch(gamestatus::error& e) {
+	} catch(game::error& e) {
 		gui::show_error_message(disp(), _("The file you have tried to load is corrupt: '") + e.message + '\'');
 		return false;
 	} catch(config::error& e) {
@@ -852,31 +789,38 @@ bool game_controller::load_game()
 	} catch(io_exception&) {
 		gui::show_error_message(disp(), _("File I/O Error while reading the game"));
 		return false;
-	}
-
+	} 
 	recorder = replay(state_.replay_data);
+	recorder.start_replay();
+	recorder.set_skip(0);
 
 	std::cerr << "has snapshot: " << (state_.snapshot.child("side") ? "yes" : "no") << "\n";
 
-	//only play replay data if the user has selected to view the replay,
-	//or if there is no starting position data to use.
-	if(!show_replay && state_.snapshot.child("side") != NULL) {
-		std::cerr << "setting replay to end...\n";
-		recorder.set_to_end();
-		if(!recorder.at_end()) {
-			std::cerr << "recorder is not at the end!!!\n";
-		}
-	} else {
-
-		recorder.start_replay();
-
-		//set whether the replay is to be skipped or not
-		if(show_replay) {
-			recorder.set_skip(0);
+	if(state_.snapshot.child("side") == NULL) {
+		// No snapshot; this is a start-of-scenario
+		if (show_replay) {
+			// There won't be any turns to replay, but the
+			// user gets to watch the intro sequence again ...
+			std::cerr << "replaying (start of scenario)\n";
 		} else {
 			std::cerr << "skipping...\n";
 			recorder.set_skip(-1);
 		}
+	} else {
+		// We have a snapshot. But does the user want to see a replay?
+		if(show_replay) {
+			// clear the stats for the current level
+			// they will be regenerated during the replay
+			statistics::replay_verify_stats = statistics::write_stats();
+			statistics::clear_current_scenario();
+			std::cerr << "replaying (snapshot)\n";
+		} else {
+			std::cerr << "setting replay to end...\n";
+			recorder.set_to_end();
+			if(!recorder.at_end()) {
+				std::cerr << "recorder is not at the end!!!\n";
+			}
+		}
 	}
 
 	if(state_.campaign_type == "tutorial") {
@@ -903,14 +847,16 @@ bool game_controller::load_game()
 		try {
 			play_level(units_data_,game_config_,&starting_pos,video_,state_,story);
 			recorder.clear();
-		} catch(gamestatus::load_game_failed& e) {
+		} catch(game::load_game_failed& e) {
 			gui::show_error_message(disp(), _("The game could not be loaded: ") + e.message);
-		} catch(gamestatus::game_error& e) {
+		} catch(game::game_error& e) {
 			gui::show_error_message(disp(), _("Error while playing the game: ") + e.message);
-		} catch(gamestatus::load_game_exception& e) {
+		} catch(game::load_game_exception& e) {
 			//this will make it so next time through the title screen loop, this game is loaded
 			loaded_game_ = e.game;
 			loaded_game_show_replay_ = e.show_replay;
+		} catch(gamemap::incorrect_format_exception& e) {
+			gui::show_error_message(disp(), std::string(_("The game map could not be loaded: ")) + e.msg_);
 		}
 
 		return false;
@@ -960,7 +906,7 @@ bool game_controller::new_campaign()
 
 	int res = 0;
 
-	dialogs::campaign_preview_pane campaign_preview(disp(),&campaign_desc);
+	dialogs::campaign_preview_pane campaign_preview(disp().video(),&campaign_desc);
 	std::vector<gui::preview_pane*> preview_panes;
 	preview_panes.push_back(&campaign_preview);
 
@@ -985,9 +931,9 @@ bool game_controller::new_campaign()
 	state_.scenario = campaign["first_scenario"];
 
 	const std::string difficulty_descriptions = campaign["difficulty_descriptions"];
-	std::vector<std::string> difficulty_options = config::split(difficulty_descriptions,';');
+	std::vector<std::string> difficulty_options = utils::split(difficulty_descriptions, ';');
 
-	const std::vector<std::string> difficulties = config::split(campaign["difficulties"]);
+	const std::vector<std::string> difficulties = utils::split(campaign["difficulties"]);
 
 	if(difficulties.empty() == false) {
 		if(difficulty_options.size() != difficulties.size()) {
@@ -1050,7 +996,8 @@ std::string format_file_size(const std::string& size_str)
 
 void game_controller::download_campaigns()
 {
-	std::string host = "campaigns.wesnoth.org";
+	std::string host = preferences::campaign_server();
+
 	const int res = gui::show_dialog(disp(),NULL,_("Connect to Server"),
 	        _("You will now connect to a campaign server to download campaigns."),
 	        gui::OK_CANCEL,NULL,NULL,_("Server: "),&host);
@@ -1058,8 +1005,9 @@ void game_controller::download_campaigns()
 		return;
 	}
 
-	const std::vector<std::string> items = config::split(host,':');
+	const std::vector<std::string> items = utils::split(host, ':');
 	host = items.front();
+	preferences::set_campaign_server(host);
 
 	try {
 		const network::manager net_manager;
@@ -1116,11 +1064,11 @@ void game_controller::download_campaigns()
 				std::replace(title.begin(),title.end(),'_',' ');
 			}
 			
-			options.push_back(IMAGE_PREFIX + (**i)["icon"] + COLUMN_SEPARATOR +
+			options.push_back(IMAGE_PREFIX + (**i)["icon"].str() + COLUMN_SEPARATOR +
 			                  title + COLUMN_SEPARATOR +
-			                  (**i)["version"] + COLUMN_SEPARATOR +
-			                  (**i)["author"] + COLUMN_SEPARATOR +
-			                  (**i)["downloads"] + COLUMN_SEPARATOR +
+			                  (**i)["version"].str() + COLUMN_SEPARATOR +
+			                  (**i)["author"].str() + COLUMN_SEPARATOR +
+			                  (**i)["downloads"].str() + COLUMN_SEPARATOR +
 			                  format_file_size((**i)["size"]));
 		}
 
@@ -1168,6 +1116,12 @@ void game_controller::download_campaigns()
 
 		unarchive_campaign(cfg);
 
+		// when using zipios, force a reread zip and directory indices
+		if (!filesystem_init()) {
+			gui::show_error_message(disp(), _("Cannot rescan the filesystem"));
+			return;
+		}
+
 		//force a reload of configuration information
 		const bool old_cache = use_caching_;
 		use_caching_ = false;
@@ -1197,7 +1151,7 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 		return;
 	} else if(data.child("error")) {
 		gui::show_error_message(disp(), _("The server responded with an error: \"") +
-		                        (*data.child("error"))["message"] + '"');
+		                        (*data.child("error"))["message"].str() + '"');
 		return;
 	} else if(data.child("message")) {
 		const int res = gui::show_dialog(disp(),NULL,_("Terms"),(*data.child("message"))["message"],gui::OK_CANCEL);
@@ -1209,7 +1163,7 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 	config cfg;
 	get_campaign_info(campaign,cfg);
 
-	std::string& passphrase = cfg["passphrase"];
+	std::string passphrase = cfg["passphrase"];
 	if(passphrase.empty()) {
 		passphrase.resize(8);
 		for(size_t n = 0; n != 8; ++n) {
@@ -1218,6 +1172,7 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 
 		set_campaign_info(campaign,cfg);
 	}
+	cfg["passphrase"] = passphrase;
 
 	cfg["name"] = campaign;
 
@@ -1235,7 +1190,7 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 		gui::show_error_message(disp(), _("Connection timed out"));
 	} else if(data.child("error")) {
 		gui::show_error_message(disp(), _("The server responded with an error: \"") +
-		                        (*data.child("error"))["message"] + '"');
+		                        (*data.child("error"))["message"].str() + '"');
 	} else if(data.child("message")) {
 		gui::show_dialog(disp(),NULL,_("Response"),(*data.child("message"))["message"],gui::OK_ONLY);
 	}
@@ -1260,7 +1215,7 @@ void game_controller::delete_campaign(const std::string& campaign, network::conn
 		gui::show_error_message(disp(), _("Connection timed out"));
 	} else if(data.child("error")) {
 		gui::show_error_message(disp(), _("The server responded with an error: \"") +
-		                        (*data.child("error"))["message"] + '"');
+		                        (*data.child("error"))["message"].str() + '"');
 	} else if(data.child("message")) {
 		gui::show_dialog(disp(),NULL,_("Response"),(*data.child("message"))["message"],gui::OK_ONLY);
 	}
@@ -1282,7 +1237,7 @@ bool game_controller::play_multiplayer()
 	host_or_join.push_back(pre + "ai.png" + sep1 + _("Human vs AI") + sep2 + _("Play a game against AI opponents"));
 
 	std::string login = preferences::login();
-	const int res = gui::show_dialog(disp(),NULL,_("Multiplayer"),"",gui::OK_CANCEL,&host_or_join,NULL,_("Login") + std::string(": "),&login);
+	const int res = gui::show_dialog(disp(),NULL,_("Multiplayer"),"",gui::OK_CANCEL,&host_or_join,NULL,_("Login: "),&login);
 
 	if(res >= 0) {
 		preferences::set_login(login);
@@ -1296,29 +1251,24 @@ bool game_controller::play_multiplayer()
 			std::vector<std::string> chat;
 			config game_data;
 
-			const std::string controller = (res == 2 ? "network" : (res == 3 ? "human" : "ai"));
+			const mp::controller cntr = (res == 2 ? 
+					mp::CNTR_NETWORK : 
+					(res == 3 ? mp::CNTR_LOCAL : mp::CNTR_COMPUTER ));
 			const bool is_server = res == 2;
 
-			multiplayer_game_setup_dialog mp_dialog(disp(),units_data_,game_config_,state_,is_server,controller);
-			lobby::RESULT res = lobby::CONTINUE;
-			while(res == lobby::CONTINUE) {
-				res = lobby::enter(disp(),game_data,game_config_,&mp_dialog,chat);
-			}
+			mp::start_server(disp(), game_config_, units_data_, cntr, is_server);
 
-			if(res == lobby::CREATE) {
-				mp_dialog.start_game();
-			}
 		} else if(res == 0 || res == 1) {
 			std::string host;
 			if(res == 0) {
 				host = preferences::official_network_host();
 			}
 
-			play_multiplayer_client(disp(),units_data_,game_config_,state_,host);
+			mp::start_client(disp(), game_config_, units_data_, host);
 		}
-	} catch(gamestatus::load_game_failed& e) {
+	} catch(game::load_game_failed& e) {
 		gui::show_error_message(disp(), _("The game could not be loaded: ") + e.message);
-	} catch(gamestatus::game_error& e) {
+	} catch(game::game_error& e) {
 		gui::show_error_message(disp(), _("Error while playing the game: ") + e.message);
 	} catch(network::error& e) {
 		std::cerr << "caught network error...\n";
@@ -1332,7 +1282,7 @@ bool game_controller::play_multiplayer()
 		}
 	} catch(gamemap::incorrect_format_exception& e) {
 		gui::show_error_message(disp(), std::string(_("The game map could not be loaded: ")) + e.msg_);
-	} catch(gamestatus::load_game_exception& e) {
+	} catch(game::load_game_exception& e) {
 		//this will make it so next time through the title screen loop, this game is loaded
 		loaded_game_ = e.game;
 		loaded_game_show_replay_ = e.show_replay;
@@ -1358,10 +1308,10 @@ bool game_controller::change_language()
 	std::transform(langdefs.begin(),langdefs.end(),std::back_inserter(langs),languagedef_name);
 
 	const int res = gui::show_dialog(disp(),NULL,_("Language"),
-	                         _("Choose your preferred language") + std::string(":"),
+	                         _("Choose your preferred language:"),
 	                         gui::OK_CANCEL,&langs);
 	if(size_t(res) < langs.size()) {
-		set_language(known_languages[res]);
+		::set_language(known_languages[res]);
 		preferences::set_language(known_languages[res].localename);
 
 		//force a reload of configuration information
@@ -1371,11 +1321,105 @@ bool game_controller::change_language()
 		use_caching_ = old_cache;
 	}
 
+	font::load_font_config();
 	hotkey::load_descriptions();
 
 	return false;
 }
 
+//this function reads the game configuration, searching for valid cached copies first
+void game_controller::read_game_cfg(preproc_map& defines, std::vector<line_source>& line_src, config& cfg, bool use_cache)
+{
+	log_scope("read_game_cfg");
+
+	if(defines.size() < 4) {
+		bool is_valid = true;
+		std::stringstream str;
+		str << "-v" << game_config::version;
+		for(preproc_map::const_iterator i = defines.begin(); i != defines.end(); ++i) {
+			if(i->second.value != "" || i->second.arguments.empty() == false) {
+				is_valid = false;
+				break;
+			}
+
+			str << "-" << i->first;
+		}
+		std::string localename = get_locale().localename;
+		str << "-lang_" << (localename.empty() ? "default" : localename);
+
+		if(is_valid) {
+			const std::string& cache = get_cache_dir();
+			if(cache != "") {
+				const std::string fname = cache + "/game.cfg-cache" + str.str();
+				const std::string fname_checksum = fname + ".checksum";
+
+				file_tree_checksum dir_checksum;
+				
+				if(use_cache) {
+					try {
+						if(file_exists(fname_checksum)) {
+							config checksum_cfg;
+							scoped_istream stream = istream_file(fname_checksum);
+							read(checksum_cfg, *stream);
+							dir_checksum = file_tree_checksum(checksum_cfg);
+						}
+					} catch(config::error&) {
+						std::cerr << "cache checksum is corrupt\n";
+					} catch(io_exception&) {
+						std::cerr << "error reading cache checksum\n";
+					}
+				}
+
+				if(use_cache && file_exists(fname) && file_create_time(fname) > data_tree_checksum().modified && dir_checksum == data_tree_checksum()) {
+					std::cerr << "found valid cache at '" << fname << "' using it\n";
+					log_scope("read cache");
+					try {
+						scoped_istream stream = istream_file(fname);
+						read_compressed(cfg, *stream);
+						return;
+					} catch(config::error&) {
+						std::cerr << "cache is corrupt. Loading from files\n";
+					} catch(io_exception&) {
+						std::cerr << "error reading cache. Loading from files\n";
+					}
+				}
+
+				std::cerr << "no valid cache found. Writing cache to '" << fname << "'\n";
+				
+				//read the file and then write to the cache
+				scoped_istream stream = preprocess_file("data/game.cfg", &defines, &line_src);
+
+				std::string error_log;
+				read(cfg, *stream, &line_src, &error_log);
+				if(!error_log.empty()) {
+					// FIXME: fix this string after the string-freeze is
+					// over, to clearly state this is a warning.
+					gui::show_error_message(disp(), 
+							_("Error loading game configuration files: '") +
+							error_log);
+
+				}
+				try {
+					scoped_ostream cache = ostream_file(fname);
+					write_compressed(*cache, cfg);
+					config checksum_cfg;
+					data_tree_checksum().write(checksum_cfg);
+					scoped_ostream checksum = ostream_file(fname_checksum);
+					write(*checksum, checksum_cfg);
+				} catch(io_exception&) {
+					std::cerr << "could not write to cache '" << fname << "'\n";
+				}
+
+				return;
+			}
+		}
+	}
+
+	std::cerr << "caching cannot be done. Reading file\n";
+	scoped_istream stream = preprocess_file("data/game.cfg", &defines, &line_src);
+	read(cfg, *stream, &line_src);
+}
+
 void game_controller::refresh_game_cfg()
 {
 	try {
@@ -1424,7 +1468,7 @@ void game_controller::play_game(RELOAD_GAME_DATA reload)
 			the_end(disp());
 			about::show_about(disp());
 		}
-	} catch(gamestatus::load_game_exception& e) {
+	} catch(game::load_game_exception& e) {
 
 		//this will make it so next time through the title screen loop, this game is loaded
 		loaded_game_ = e.game;
@@ -1461,10 +1505,16 @@ int play_game(int argc, char** argv)
 			<< "  --log-error=\"domain1,domain2,...\", --log-warning=..., --log-info=...\n"
 			<< "                    Set the severity level of the debug domains\n"
 			<< "                    \"all\" can be used to match any debug domain\n"
-			<< "  --nocache         Disables caching of game data\n";
+			<< "  --nocache         Disables caching of game data\n"
+			<< "  --nosound         Disables sounds\n"
+			<< "  --compress file1 file2 Compresses the text-WML file file1 into the\n"
+			<< "                    binary-WML file file2\n"
+			<< "  --decompress file1 file2 Uncompresses the binary-WML file file2 into\n"
+			<< "                    the text-WML file file2\n"
+			;
  			return 0;
  		} else if(val == "--version" || val == "-v") {
- 			std::cout << "Battle for Wesnoth " << game_config::version
+ 			std::cout << _("Battle for Wesnoth") << " " << game_config::version
  			          << "\n";
  			return 0;
  		} else if(val == "--path") {
@@ -1497,6 +1547,43 @@ int play_game(int argc, char** argv)
 				}
 				p = q;
 			}
+		} else if(val == "--compress" || val == "--decompress") {
+			if(argc != arg+3) {
+				std::cerr << "format of " << val << " command: " << val << " <input file> <output file>\n";
+				return 0;
+			}
+
+			const std::string input(argv[arg+1]);
+			const std::string output(argv[arg+2]);
+
+			scoped_istream stream = istream_file(input);
+			if (stream->fail()) {
+				std::cerr << "could not read file '" << input << "'\n";
+				return 0;
+			}
+
+			config cfg;
+
+			const bool compress = val == "--compress";
+			try {
+				const bool is_compressed = detect_format_and_read(cfg, *stream);
+				if(is_compressed && compress) {
+					std::cerr << input << " is already compressed\n";
+					return 0;
+				} else if(!is_compressed && !compress) {
+					std::cerr << input << " is already decompressed\n";
+					return 0;
+				}
+
+				scoped_ostream output_stream = ostream_file(output);
+				write_possibly_compressed(*output_stream, cfg, compress);
+			} catch(config::error& e) {
+				std::cerr << input << " is not a valid Wesnoth file: " << e.message << "\n";
+			} catch(io_exception& e) {
+				std::cerr << "IO error: " << e.what() << "\n";
+			}
+
+			return 0;
 		}
 	}
 
@@ -1524,6 +1611,16 @@ int play_game(int argc, char** argv)
 	textdomain (PACKAGE);
 
 	bool res;
+
+	// do initialize fonts before reading the game config, to have game
+	// config error messages displayed. fonts will be re-initialized later
+	// when the language is read from the game config.
+	res = font::load_font_config();
+	if(res == false) {
+		std::cerr << "could not initialize fonts\n";
+		return 0;
+	}
+
 #ifdef WIN32
 	res = game.init_config();
 	if(res == false) {
@@ -1544,6 +1641,12 @@ int play_game(int argc, char** argv)
 		return 0;
 	}
 
+	res = font::load_font_config();
+	if(res == false) {
+		std::cerr << "could not re-initialize fonts for the current language\n";
+		return 0;
+	}
+
 #ifndef WIN32
 	// it is better for gettext-native platforms to read the config
 	// files after having pre-initialized the language, maybe...
@@ -1633,6 +1736,10 @@ int play_game(int argc, char** argv)
 int main(int argc, char** argv)
 {
 	try {
+		std::cerr << "Battle for Wesnoth v" << VERSION << "\n";
+		time_t t = time(NULL);
+		std::cerr << "Started on " << ctime(&t) << "\n";
+
 		std::cerr << "started game: " << SDL_GetTicks() << "\n";
 		const int res = play_game(argc,argv);
 		std::cerr << "exiting with code " << res << "\n";
@@ -1650,7 +1757,7 @@ int main(int argc, char** argv)
 	} catch(end_level_exception&) {
 		std::cerr << "caught end_level_exception (quitting)\n";
 	} catch(std::bad_alloc&) {
-		std::cerr << "ran out of memory: game aborted\n";
+		std::cerr << "Ran out of memory. Aborted.\n";
 	} /*catch(...) {
 		std::cerr << "Unhandled exception. Exiting\n";
 	}*/
