@@ -1,4 +1,4 @@
-/* $Id: game_config.cpp,v 1.92 2004/11/18 22:00:12 ydirson Exp $ */
+/* $Id: game_config.cpp,v 1.94 2005/03/10 00:00:06 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,6 +11,7 @@
    See the COPYING file for more details.
 */
 
+#include "config.hpp"
 #include "global.hpp"
 
 #include "game_config.hpp"
@@ -31,7 +32,7 @@ namespace game_config
 	int kill_experience = 8;
 	int leadership_bonus = 25;
 	const std::string version = VERSION;
-	bool debug = false, editor = false;
+	bool debug = false, editor = false, ignore_replay_errors = false;
 
 	std::string game_icon = "wesnoth-icon.png", game_title, game_logo, title_music;
 	int title_logo_x = 0, title_logo_y = 0, title_buttons_x = 0, title_buttons_y = 0, title_buttons_padding = 0,
