@@ -1,4 +1,4 @@
-/* $Id: game_config.hpp,v 1.25 2004/09/22 00:05:33 cedricd Exp $ */
+/* $Id: game_config.hpp,v 1.27 2005/03/10 00:00:06 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,7 +13,7 @@
 #ifndef GAME_CONFIG_H_INCLUDED
 #define GAME_CONFIG_H_INCLUDED
 
-#include "config.hpp"
+class config;
 
 #include <string>
 
@@ -32,7 +32,7 @@ namespace game_config
 	extern int leadership_bonus;
 	extern const std::string version;
 
-	extern bool debug, editor;
+	extern bool debug, editor, ignore_replay_errors;
 
 	extern std::string path;
 
