@@ -1,4 +1,4 @@
-/* $Id: game_events.hpp,v 1.23 2004/09/12 21:55:20 Sirp Exp $ */
+/* $Id: game_events.hpp,v 1.27 2005/03/27 23:06:16 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,16 +13,20 @@
 #ifndef GAME_EVENTS_H_INCLUDED
 #define GAME_EVENTS_H_INCLUDED
 
-#include "config.hpp"
-#include "display.hpp"
+class config;
+class t_string;
+class display;
+
 #include "gamestatus.hpp"
 #include "map.hpp"
 #include "team.hpp"
 #include "unit.hpp"
-#include "unit_types.hpp"
+#include "variable.hpp"
 
+#include <vector>
 #include <map>
 
+
 //this file defines the game's events mechanism. Events might be units
 //moving or fighting, or when victory or defeat occurs. A scenario's
 //configuration file will define actions to take when certain events
@@ -34,12 +38,6 @@
 namespace game_events
 {
 
-std::string& get_variable(const std::string& varname);
-const std::string& get_variable_const(const std::string& varname);
-config& get_variable_cfg(const std::string& varname);
-
-void set_variable(const std::string& varname, const std::string& value);
-
 //the game event manager loads the scenario configuration object, and
 //ensures that events are handled according to the scenario configuration
 //for its lifetime.
@@ -49,17 +47,19 @@ void set_variable(const std::string& varname, const std::string& value);
 struct manager {
 	//note that references will be maintained, and must remain valid
 	//for the life of the object.
-	manager(config& scenario_cfg, display& disp, gamemap& map,
-	        std::map<gamemap::location,unit>& units, std::vector<team>& teams,
-		    game_state& state_of_game, gamestatus& status, game_data& data);
+	manager(const config& scenario_cfg, display& disp, gamemap& map,
+			std::map<gamemap::location,unit>& units, std::vector<team>& teams,
+			game_state& state_of_game, gamestatus& status, const game_data& data);
 	~manager();
+
+	variable::manager variable_manager;
 };
 
 void write_events(config& cfg);
 
-bool unit_matches_filter(const unit& u, const config& filter);
-bool unit_matches_filter(unit_map::const_iterator itor, const config& filter);
-bool unit_matches_filter(const unit& u, const config& filter);
+bool unit_matches_filter(const unit& u, const vconfig filter);
+bool unit_matches_filter(unit_map::const_iterator itor, const vconfig filter);
+bool unit_matches_filter(const unit& u, const vconfig filter);
 
 //function to fire an event. Events may have up to two arguments, both of
 //which must be locations.
@@ -68,7 +68,7 @@ bool fire(const std::string& event,
           const gamemap::location& loc2=gamemap::location::null_location);
 
 bool conditional_passed(const std::map<gamemap::location,unit>* units,
-                        const config& cond);
+                        const vconfig cond);
 bool pump();
 
 }
