@@ -1,4 +1,4 @@
-/* $Id: gamestatus.cpp,v 1.53 2005/01/28 20:17:46 silene Exp $ */
+/* $Id: gamestatus.cpp,v 1.70 2005/04/05 19:42:05 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -15,21 +15,41 @@
 
 #include "filesystem.hpp"
 #include "game_config.hpp"
+#include "game_errors.hpp"
 #include "gamestatus.hpp"
 #include "language.hpp"
 #include "log.hpp"
+#include "preferences.hpp"
 #include "statistics.hpp"
 #include "util.hpp"
+#include "serialization/binary_or_text.hpp"
+#include "serialization/binary_wml.hpp"
+#include "serialization/parser.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <algorithm>
 #include <cstdio>
+#include <iostream>
 #include <iterator>
 #include <sstream>
 
+#define LOG_NG lg::info(lg::engine)
+#define WRN_NG lg::warn(lg::engine)
+#define ERR_NG lg::err(lg::engine)
+
+player_info* game_state::get_player(const std::string& id) {
+	std::map< std::string, player_info >::iterator found = players.find(id);
+	if (found == players.end()) {
+		LOG_STREAM(warn, engine) << "player " << id << " does not exist.\n";
+		return NULL;
+	} else
+		return &found->second;
+}
+
 time_of_day::time_of_day(const config& cfg)
                  : lawful_bonus(atoi(cfg["lawful_bonus"].c_str())),
                    image(cfg["image"]), name(cfg["name"]), id(cfg["id"]),
-				   image_mask(cfg["mask"]),
+		   image_mask(cfg["mask"]),
                    red(atoi(cfg["red"].c_str())),
                    green(atoi(cfg["green"].c_str())),
                    blue(atoi(cfg["blue"].c_str()))
@@ -138,7 +158,7 @@ const time_of_day& gamestatus::get_time_of_day_turn(int nturn) const
 		return times_[(nturn-1)%times_.size()];
 	} else {
 		config dummy_cfg;
-		const static time_of_day default_time(dummy_cfg);
+		static time_of_day const default_time(dummy_cfg);
 		return default_time;
 	}
 }
@@ -172,7 +192,7 @@ const time_of_day& gamestatus::get_time_of_day(bool illuminated, const gamemap::
 	}
 
 	config dummy_cfg;
-	const static time_of_day default_time(dummy_cfg);
+	static time_of_day const default_time(dummy_cfg);
 	return default_time;
 }
 
@@ -207,7 +227,7 @@ player_info read_player(const game_data& data, const config* cfg)
 
 	const std::string& can_recruit_str = (*cfg)["can_recruit"];
 	if(can_recruit_str != "") {
-		const std::vector<std::string> can_recruit = config::split(can_recruit_str);
+		const std::vector<std::string> can_recruit = utils::split(can_recruit_str);
 		std::copy(can_recruit.begin(),can_recruit.end(),std::inserter(res.can_recruit,res.can_recruit.end()));
 	}
 
@@ -406,37 +426,36 @@ void delete_game(const std::string& name)
 	remove((get_saves_dir() + "/" + modified_name).c_str());
 }
 
-void read_save_file(const std::string& name, config& cfg)
+void read_save_file(const std::string& name, config& cfg, std::string* error_log)
 {
 	std::string modified_name = name;
 	std::replace(modified_name.begin(),modified_name.end(),' ','_');
 
 	//try reading the file both with and without underscores
-	std::string file_data = read_file(get_saves_dir() + "/" + modified_name);
-	if(file_data.empty()) {
-		file_data = read_file(get_saves_dir() + "/" + name);
-	}
+	scoped_istream file_stream = istream_file(get_saves_dir() + "/" + modified_name);
+	if (file_stream->fail())
+		file_stream = istream_file(get_saves_dir() + "/" + name);
 
 	cfg.clear();
-	cfg.read(file_data);
+	detect_format_and_read(cfg, *file_stream, error_log);
 
 	if(cfg.empty()) {
 		std::cerr << "Could not parse file data into config\n";
-		throw gamestatus::load_game_failed();
+		throw game::load_game_failed();
 	}
 }
 
-void load_game(const game_data& data, const std::string& name, game_state& state)
+void load_game(const game_data& data, const std::string& name, game_state& state, std::string* error_log)
 {
 	log_scope("load_game");
 
 	config cfg;
-	read_save_file(name,cfg);
+	read_save_file(name,cfg,error_log);
 	
 	state = read_game(data,&cfg);
 }
 
-//throws gamestatus::save_game_failed
+//throws game::save_game_failed
 void save_game(const game_state& state)
 {
 	log_scope("save_game");
@@ -449,9 +468,10 @@ void save_game(const game_state& state)
 		write_game(state,cfg);
 
 		const std::string fname = get_saves_dir() + "/" + name;
-
-		write_file(fname,cfg.write());
-
+		{
+			scoped_ostream savefile = ostream_file(fname);
+			write_possibly_compressed(*savefile, cfg, preferences::compress_saves());
+		}
 
 		config& summary = save_summary(state.label);
 		extract_summary_data_from_save(state,summary);
@@ -461,7 +481,7 @@ void save_game(const game_state& state)
 		write_save_index();
 
 	} catch(io_exception& e) {
-		throw gamestatus::save_game_failed(e.what());
+		throw game::save_game_failed(e.what());
 	}
 }
 
@@ -474,7 +494,8 @@ config& save_index()
 {
 	if(save_index_loaded == false) {
 		try {
-			save_index_cfg.read(read_file(get_save_index_file()));
+			scoped_istream stream = istream_file(get_save_index_file());
+			detect_format_and_read(save_index_cfg, *stream);
 		} catch(io_exception& e) {
 			std::cerr << "error reading save index: '" << e.what() << "'\n";
 		} catch(config::error&) {
@@ -515,7 +536,8 @@ void write_save_index()
 {
 	log_scope("write_save_index()");
 	try {
-		write_file(get_save_index_file(),save_index().write());
+		scoped_ostream stream = ostream_file(get_save_index_file());
+		write_compressed(*stream, save_index());
 	} catch(io_exception& e) {
 		std::cerr << "error writing to save index file: '" << e.what() << "'\n";
 	}
@@ -538,7 +560,7 @@ void extract_summary_data_from_save(const game_state& state, config& out)
 	if(has_snapshot) {
 		out["turn"] = state.snapshot["turn_at"];
 		if(state.snapshot["turns"] != "-1") {
-			out["turn"] += "/" + state.snapshot["turns"];
+			out["turn"] = out["turn"].str() + "/" + state.snapshot["turns"].str();
 		}
 	}
 
@@ -552,7 +574,7 @@ void extract_summary_data_from_save(const game_state& state, config& out)
 	    p!=state.players.end(); ++p) {
 		for(std::vector<unit>::const_iterator u = p->second.available_units.begin(); u != p->second.available_units.end(); ++u) {
 			if(u->can_recruit()) {
-				leader = u->type().name();
+				leader = u->type().id();
 			}
 		}
 	}
@@ -597,3 +619,108 @@ void extract_summary_data_from_save(const game_state& state, config& out)
 		}
 	}
 }
+
+namespace {
+const size_t MaxLoop = 1024;
+}
+
+void game_state::get_variable_internal(const std::string& key, config& cfg,
+		t_string** varout, config** cfgout)
+{
+	//we get the variable from the [variables] section of the game state. Variables may
+	//be in the format 
+	const std::string::const_iterator itor = std::find(key.begin(),key.end(),'.');
+	if(itor != key.end()) {
+		std::string element(key.begin(),itor);
+		const std::string sub_key(itor+1,key.end());
+
+		size_t index = 0;
+		const std::string::iterator index_start = std::find(element.begin(),element.end(),'[');
+		const bool explicit_index = index_start != element.end();
+
+		if(explicit_index) {
+			const std::string::iterator index_end = std::find(index_start,element.end(),']');
+			const std::string index_str(index_start+1,index_end);
+			index = size_t(atoi(index_str.c_str()));
+			if(index > MaxLoop) {
+				LOG_NG << "get_variable_internal: index greater than " << MaxLoop
+				       << ", truncated\n";
+				index = MaxLoop;
+			}
+
+			element = std::string(element.begin(),index_start);
+		}
+
+		const config::child_list& items = cfg.get_children(element);
+
+		//special case -- '.length' on an array returns the size of the array
+		if(explicit_index == false && sub_key == "length") {
+			if(items.empty()) {
+				if(varout != NULL) {
+					static t_string zero_str = "0";
+					*varout = &zero_str;
+				}
+			} else {
+				int size = minimum<int>(MaxLoop,int(items.size()));
+				(*items.back())["__length"] = lexical_cast<std::string>(size);
+
+				if(varout != NULL) {
+					*varout = &(*items.back())["__length"];
+				}
+			}
+
+			return;
+		}
+		
+		while(cfg.get_children(element).size() <= index) {
+			cfg.add_child(element);
+		}
+
+		if(cfgout != NULL) {
+			*cfgout = cfg.get_children(element)[index];
+		}
+		
+		get_variable_internal(sub_key,*cfg.get_children(element)[index],varout,cfgout);
+	} else {
+		if(varout != NULL) {
+			*varout = &cfg[key];
+		}
+	}
+}
+
+t_string& game_state::get_variable(const std::string& key)
+{
+	t_string* res = NULL;
+	get_variable_internal(key, variables, &res, NULL);
+	if(res != NULL) {
+		return *res;
+	}
+	
+	static t_string empty_string;
+	return empty_string;
+}
+
+const t_string& game_state::get_variable_const(const std::string& key)
+{
+	return get_variable(key);
+}
+
+config& game_state::get_variable_cfg(const std::string& key)
+{
+	config* res = NULL;
+	get_variable_internal(key + ".", variables, NULL, &res);
+
+	if(res != NULL) {
+		return *res;
+	}
+
+	static config empty_cfg;
+	return empty_cfg;
+}
+
+void game_state::set_variable(const std::string& key, const t_string& value)
+{
+	variables[key] = value;
+}
+
+
