@@ -1,6 +1,7 @@
 #include "global.hpp"
 
 #include "gettext.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <cstring>
 
@@ -24,6 +25,7 @@ const char* sgettext (const char *msgid)
 
 const char* dsgettext (const char * domainname, const char *msgid)
 {
+	bind_textdomain_codeset(domainname, "UTF-8");
 	const char *msgval = dgettext (domainname, msgid);
 	if (msgval == msgid) {
 		msgval = strrchr (msgid, '^');
@@ -35,10 +37,9 @@ const char* dsgettext (const char * domainname, const char *msgid)
 	return msgval;
 }
 
-std::string vgettext (const char *msgid, const string_map& symbols)
+std::string vgettext (const char *msgid, const utils::string_map& symbols)
 {
 	const std::string orig(gettext(msgid));
-	const std::string msg = config::interpolate_variables_into_string(orig,
-			&symbols);
+	const std::string msg = utils::interpolate_variables_into_string(orig, &symbols);
 	return msg;
 }
