@@ -1,4 +1,4 @@
-/* $Id: hotkeys.cpp,v 1.66 2005/01/09 00:02:52 silene Exp $ */
+/* $Id: hotkeys.cpp,v 1.70 2005/03/26 17:10:32 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -16,9 +16,10 @@
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "config.hpp"
+#include "display.hpp"
 #include "events.hpp"
 #include "hotkeys.hpp"
-#include "language.hpp"
+#include "gettext.hpp"
 #include "playlevel.hpp"
 #include "preferences.hpp"
 #include "show_dialog.hpp"
@@ -31,7 +32,7 @@
 #include <cstdlib>
 #include <map>
 
-#define ERR_G lg::err(lg::general)
+#define ERR_G LOG_STREAM(err, general)
 
 namespace {
 
@@ -91,6 +92,7 @@ const struct {
 	{ hotkey::HOTKEY_EDIT_FLIP, "editflip", N_("Flip Map"),true },
 	{ hotkey::HOTKEY_EDIT_SELECT_ALL, "editselectall", N_("Select All"),true },
 	{ hotkey::HOTKEY_EDIT_DRAW, "editdraw", N_("Draw Terrain"),true },
+	{ hotkey::HOTKEY_EDIT_REFRESH, "editrefresh", N_("Refresh Image Cache"), true },
 
 	{ hotkey::HOTKEY_DELAY_SHROUD, "delayshroud", N_("Delay Shroud Updates"), false },
 	{ hotkey::HOTKEY_UPDATE_SHROUD, "updateshroud", N_("Update Shroud Now"), false },
@@ -621,6 +623,10 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 			if(executor)
 				executor->edit_draw();
 			break;
+		 case HOTKEY_EDIT_REFRESH:
+			if(executor)
+				executor->edit_refresh();
+			break;
 		default:
 			std::cerr << "command_executor: unknown command number " << command << ", ignoring.\n";
 			break;
