@@ -1,4 +1,4 @@
-/* $Id: hotkeys.hpp,v 1.55 2005/02/06 10:40:12 isaaccp Exp $ */
+/* $Id: hotkeys.hpp,v 1.58 2005/03/23 08:52:10 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,10 +13,14 @@
 #ifndef HOTKEYS_HPP_INCLUDED
 #define HOTKEYS_HPP_INCLUDED
 
-#include "config.hpp"
-#include "display.hpp"
 #include "events.hpp"
-#include "key.hpp"
+#include "SDL.h"
+
+#include <string>
+#include <vector>
+
+class config;
+class display;
 
 //the hotkey system allows hotkey definitions to be loaded from
 //configuration objects, and then detect if a keyboard event
@@ -46,6 +50,7 @@ enum HOTKEY_COMMAND {
 	HOTKEY_EDIT_FILL_SELECTION, HOTKEY_EDIT_CUT, HOTKEY_EDIT_COPY,
 	HOTKEY_EDIT_PASTE, HOTKEY_EDIT_REVERT, HOTKEY_EDIT_RESIZE,
 	HOTKEY_EDIT_FLIP, HOTKEY_EDIT_SELECT_ALL, HOTKEY_EDIT_DRAW,
+	HOTKEY_EDIT_REFRESH,
 	HOTKEY_USER_CMD,
 	HOTKEY_NULL 
 };
@@ -173,6 +178,7 @@ public:
 	virtual void edit_flip() {}
 	virtual void edit_select_all() {}
 	virtual void edit_draw() {}
+	virtual void edit_refresh() {}
 
 	//Gets the action's image (if any). Displayed left of the action text in menus.
 	virtual std::string get_action_image(hotkey::HOTKEY_COMMAND command) const { return ""; }
