@@ -4,8 +4,6 @@
 #include "filesystem.hpp"
 #include "game_config.hpp"
 #include "image.hpp"
-#include "display.hpp"
-#include "font.hpp"
 #include "log.hpp"
 #include "sdl_utils.hpp"
 #include "team.hpp"
@@ -19,8 +17,8 @@
 #include <map>
 #include <string>
 
-#define LOG_DP lg::info(lg::display)
-#define ERR_DP lg::err(lg::display)
+#define LOG_DP LOG_STREAM(info, display)
+#define ERR_DP LOG_STREAM(err, display)
 
 namespace {
 
