@@ -1,4 +1,4 @@
-/* $Id: intro.hpp,v 1.17 2004/08/28 23:00:13 Sirp Exp $ */
+/* $Id: intro.hpp,v 1.21 2005/03/24 21:35:52 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,10 +13,9 @@
 #ifndef INTRO_HPP_INCLUDED
 #define INTRO_HPP_INCLUDED
 
+class config;
+class display;
 #include "SDL.h"
-#include "config.hpp"
-#include "display.hpp"
-#include "gamestatus.hpp"
 
 #include <string>
 
@@ -32,8 +31,8 @@
 //
 //each part of the sequence will be displayed in turn, with the
 //user able to go to the next part, or skip it entirely.
-void show_intro(display& screen, const config& data, const config& level);
+void show_intro(display &disp, const config& data, const config& level);
 
-void the_end(display& screen);
+void the_end(display &disp);
 
 #endif
