@@ -1,4 +1,4 @@
-/* $Id: key.cpp,v 1.14 2004/11/18 22:00:12 ydirson Exp $ */
+/* $Id: key.cpp,v 1.15 2005/03/09 17:07:43 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -12,31 +12,8 @@
 */
 
 #include "global.hpp"
-
 #include "key.hpp"
 
-#define KEY_TEST 0
-
-#if (KEY_TEST == 1)
-
-#include "video.hpp"
-
-int main( void )
-{
-	SDL_Init(SDL_INIT_VIDEO);
-	CVideo video( 640, 480, 16, 0 );
-	CKey key;
-	printf( "press enter (escape exits)...\n" );
-	for(;;) {
-		if( key[KEY_RETURN] != 0 )
-			printf( "key(ENTER) pressed\n" );
-		if( key[SDLK_ESCAPE] != 0 )
-			return 1;
-	}
-}
-
-#endif
-
 CKey::CKey() : is_enabled(true)
 {
 	static int num_keys = 300;
