@@ -1,4 +1,5 @@
-/* $Id: language.cpp,v 1.68 2005/02/12 22:24:12 ydirson Exp $ */
+/* $Id: language.cpp,v 1.83 2005/03/28 10:09:24 ettin Exp $ */
+/* vim:set encoding=utf-8: */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,11 +15,15 @@
 #include "global.hpp"
 
 #include "config.hpp"
-#include "font.hpp"
+#include "filesystem.hpp"
+#include "gettext.hpp"
 #include "language.hpp"
 #include "preferences.hpp"
 #include "util.hpp"
+#include "serialization/parser.hpp"
+#include "serialization/preprocessor.hpp"
 
+#include <algorithm>
 #include <cctype>
 #include <cerrno>
 #include <cstdlib>
@@ -33,20 +38,22 @@ namespace {
 language_def known_languages[] = {
 	language_def("", N_("System default language")),
 	language_def("bg_BG", "Български (Bulgarski)"),
-	language_def("es_ES", "Castellano"),
 	language_def("ca_ES", "Català"),
 	language_def("cs_CZ", "Čeština"),
 	language_def("da_DK", "Dansk"),
 	language_def("de_DE", "Deutsch"),
 	language_def("el_GR", "Ελληνικά (Ellinika)"),
+	language_def("et_EE", "Eesti"),
 	language_def("en_GB", "English (GB)"),
-	language_def("C",     "English (hm... nearly US)"),
+	language_def("C",     "English (US)"),
+	language_def("es_ES", "Español"),
 	language_def("eu_ES", "Euskera"),
 	language_def("fr_FR", "Français"),
 	language_def("it_IT", "Italiano"),
 	language_def("la_IT", "Latina"),
 	language_def("hu_HU", "Magyar"),
 	language_def("nl_NL", "Nederlands"),
+	language_def("ja_JP", "日本語 (Nihongo)"),
 	language_def("no_NO", "Norsk"),
 	language_def("pl_PL", "Polski"),
 	language_def("pt_BR", "Português do Brasil"),
@@ -55,6 +62,7 @@ language_def known_languages[] = {
 	language_def("sl_SI", "Slovenščina"),
 	language_def("fi_FI", "Suomi"),
 	language_def("sv_SE", "Svenska"),
+	language_def("zh_CN", "中文 (Zhongwen)"),
 
 	// end of list marker, do not remove
 	language_def("", "")
@@ -77,13 +85,13 @@ bool language_def::operator== (const language_def& a)
 
 symbol_table string_table;
 
-const std::string& symbol_table::operator[](const std::string& key) const
+const t_string& symbol_table::operator[](const std::string& key) const
 {
 	const string_map::const_iterator i = strings_.find(key);
 	if(i != strings_.end()) {
 		return i->second;
 	} else {
-		static std::string empty_string;
+		static t_string empty_string;
 		// Let's do it the painful way (untlb means untranslatABLE).
 		// It will cause problem if somebody stores more than one reference at once
 		// but I don't really care since this path is an error path and it should
@@ -93,7 +101,7 @@ const std::string& symbol_table::operator[](const std::string& key) const
 	}
 }
 
-const std::string& symbol_table::operator[](const char* key) const
+const t_string& symbol_table::operator[](const char* key) const
 {
 	return (*this)[std::string(key)];
 }
@@ -148,12 +156,12 @@ bool set_language(const language_def& locale)
 
 	current_language = locale;
 	wesnoth_setlocale (LC_MESSAGES, locale.localename.c_str());
-	font::set_font();
 	known_languages[0].language = gettext("System default language");
 
 	// fill string_table (should be moved somwhere else some day)
 	try {
-		cfg.read(preprocess_file("data/translations/english.cfg"));
+		scoped_istream stream = preprocess_file("data/translations/english.cfg");
+		read(cfg, *stream);
 	} catch(config::error& e) {
 		std::cerr << "Could not read english.cfg\n";
 		throw e;
@@ -161,7 +169,7 @@ bool set_language(const language_def& locale)
 
 	config* langp = cfg.child("language");
 	if (langp == NULL) {
-	 	std::cerr << "No [language] block found in english.cfg";	
+	 	std::cerr << "No [language] block found in english.cfg\n";	
 		return false;
 	}
 
@@ -210,159 +218,3 @@ const language_def& get_locale()
 	return known_languages[0];
 }
 
-class invalid_utf8_exception : public std::exception {
-};
-
-namespace 
-{
-std::string wstring_to_utf8(const wide_string &src)
-{
-	wchar_t ch;
-	wide_string::const_iterator i;
-	int j;
-	Uint32 bitmask;
-	std::string ret;
-
-	try {
-
-		for(i = src.begin(); i != src.end(); ++i) {
-			int count;
-			ch = *i;
-			
-			/* Determine the bytes required */
-			count = 1;
-			if(ch >= 0x80)
-				count++;
-
-			bitmask = 0x800;
-			for(j = 0; j < 5; ++j) {
-				if(ch >= bitmask)
-					count++;
-				bitmask <<= 5;
-			}
-			
-			if(count > 6)
-				throw invalid_utf8_exception();
-
-			if(count == 1) {
-				push_back(ret,ch);
-			} else {
-				for(j = count-1; j >= 0; --j) {
-					unsigned char c = (ch >> (6*j)) & 0x3f;
-					c |= 0x80;
-					if(j == count-1)
-						c |= 0xff << (8 - count);
-					push_back(ret,c);
-				}
-			}
-
-		}
-
-		return ret;
-	}
-	catch(invalid_utf8_exception e) {
-		std::cerr << "Invalid wide character string\n";
-		return ret;
-	}
-}
-
-int byte_size_from_utf8_first(unsigned char ch)
-{
-	int count;
-
-	if ((ch & 0x80) == 0)
-		count = 1;
-	else if ((ch & 0xE0) == 0xC0)
-		count = 2;
-	else if ((ch & 0xF0) == 0xE0)
-		count = 3;
-	else if ((ch & 0xF8) == 0xF0)
-		count = 4;
-	else if ((ch & 0xFC) == 0xF8)
-		count = 5;
-	else if ((ch & 0xFE) == 0xFC)
-		count = 6;
-	else
-		throw invalid_utf8_exception(); /* stop on invalid characters */
-
-	return count;
-}
-	
-wide_string utf8_to_wstring(const std::string &src)
-{
-	wide_string ret;	
-	
-	try {
-		for(size_t i = 0, l = src.size(); i != l;) {
-			wchar_t ch = (unsigned char)src[i];
-			const int count = byte_size_from_utf8_first(ch);
-
-			if (i + count > l)
-				throw invalid_utf8_exception();
-
-			/* Convert the first character */
-			if (count != 1) {
-				ch &= 0xFF >> (count + 1);
-			}
-
-			/* Convert the continuation bytes */
-			for(size_t j = i + 1; j != i + count; ++j) {
-				unsigned char ch2 = src[j];
-				if ((ch2 & 0xC0) != 0x80)
-					throw invalid_utf8_exception();
-
-				ch = (ch << 6) | (ch2 & 0x3F);
-			}
-			i += count;
-
-			push_back(ret,ch);
-		}
-	}
-	catch(invalid_utf8_exception e) {
-		std::cerr << "Invalid UTF-8 string: \"" << src << "\"\n";
-		return ret;
-	}
-
-	return ret;
-}
-
-}
-
-std::vector<std::string> split_utf8_string(const std::string &src)
-{
-	std::vector<std::string> ret;
-	
-	try {
-		for(size_t i = 0; i < src.size(); /* nop */) {
-			const int size = byte_size_from_utf8_first(src[i]);
-			if(i + size > src.size())
-				throw invalid_utf8_exception();
-
-			ret.push_back(src.substr(i, size));
-			i += size;
-		}
-	}
-	
-	catch(invalid_utf8_exception e) {
-		std::cerr << "Invalid UTF-8 string: \"" << src << "\"\n";
-		return ret;
-	}
-
-	return ret;
-}
-
-std::string wstring_to_string(const wide_string &src)
-{
-	return wstring_to_utf8(src);
-}
-
-std::string wchar_to_string(const wchar_t c) {
-	wide_string s;
-	s.push_back(c);
-	return wstring_to_utf8(s);
-}
-
-wide_string string_to_wstring(const std::string &src)
-{
-	return utf8_to_wstring(src);
-}
