@@ -1,4 +1,4 @@
-/* $Id: map.cpp,v 1.49 2005/02/19 09:26:31 isaaccp Exp $ */
+/* $Id: map.cpp,v 1.58 2005/03/26 17:10:32 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,12 +13,13 @@
 
 #include "global.hpp"
 
-#include "game.hpp"
+#include "config.hpp"
 #include "log.hpp"
 #include "map.hpp"
-#include "pathfind.hpp"
+#include "pathutils.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <algorithm>
 #include <cctype>
@@ -26,8 +27,8 @@
 #include <iostream>
 #include <sstream>
 
-#define ERR_CF lg::err(lg::config)
-#define LOG_G lg::info(lg::general)
+#define ERR_CF LOG_STREAM(err, config)
+#define LOG_G LOG_STREAM(info, general)
 
 gamemap::location gamemap::location::null_location;
 
@@ -198,8 +199,14 @@ void gamemap::read(const std::string& data)
 	villages_.clear();
 	std::fill(startingPositions_,startingPositions_+sizeof(startingPositions_)/sizeof(*startingPositions_),location());
 
+	//ignore leading newlines
+	std::string::const_iterator i = data.begin();
+	while(i != data.end() && (*i == '\r' || *i == '\n')) {
+		++i;
+	}
+
 	size_t x = 0, y = 0;
-	for(std::string::const_iterator i = data.begin(); i != data.end(); ++i) {
+	for(; i != data.end(); ++i) {
 		char c = *i;
 		if(c == '\r') {
 			continue;
@@ -249,15 +256,16 @@ std::string gamemap::write() const
 	for(int j = 0; j != y(); ++j) {
 		for(int i = 0; i != x(); ++i) {
 			int n;
-			for(n = 0; n != 10; ++n) {
+			for(n = 0; n != STARTING_POSITIONS; ++n) {
 				if(startingPositions_[n] == location(i,j))
 					break;
 			}
 
-			if(n < 10)
+			if(n < STARTING_POSITIONS) {
 				str << n;
-			else
+			} else {
 				str << tiles_[i][j];
+			}
 		}
 
 		str << "\n";
@@ -266,6 +274,73 @@ std::string gamemap::write() const
 	return str.str();
 }
 
+void gamemap::overlay(const gamemap& m, const config& rules_cfg, const int xpos, const int ypos)
+{
+	const config::child_list& rules = rules_cfg.get_children("rule");
+
+	const int xend = minimum<int>(xpos+m.x(),x());
+	const int yend = minimum<int>(ypos+m.y(),y());
+	for(int x = xpos; x < xend; ++x) {
+		for(int y = ypos; y < yend; ++y) {
+			const TERRAIN t = m[x-xpos][y-ypos];
+			const TERRAIN current = (*this)[x][y];
+
+			if(t == FOGGED || t == VOID_TERRAIN) {
+				continue;
+			}
+
+			//see if there is a matching rule
+			config::child_list::const_iterator rule = rules.begin();
+			for( ; rule != rules.end(); ++rule) {
+				static const std::string src_key = "old", src_not_key = "old_not",
+				                         dst_key = "new", dst_not_key = "new_not";
+				const config& cfg = **rule;
+				const std::string& src = cfg[src_key];
+				if(src != "" && std::find(src.begin(),src.end(),current) == src.end()) {
+					continue;
+				}
+
+				const std::string& src_not = cfg[src_not_key];
+				if(src_not != "" && std::find(src_not.begin(),src_not.end(),current) != src_not.end()) {
+					continue;
+				}
+
+				const std::string& dst = cfg[dst_key];
+				if(dst != "" && std::find(dst.begin(),dst.end(),t) == dst.end()) {
+					continue;
+				}
+
+				const std::string& dst_not = cfg[dst_not_key];
+				if(dst_not != "" && std::find(dst_not.begin(),dst_not.end(),t) != dst_not.end()) {
+					continue;
+				}
+
+				break;
+			}
+
+
+			if(rule != rules.end()) {
+				const config& cfg = **rule;
+				const std::string& terrain = cfg["terrain"];
+				if(terrain != "") {
+					set_terrain(location(x,y),terrain[0]);
+				} else if(cfg["use_old"] != "yes") {
+					set_terrain(location(x,y),t);
+				}
+			} else {
+				set_terrain(location(x,y),t);
+			}
+		}
+	}
+
+	for(const location* pos = m.startingPositions_; pos != m.startingPositions_ + sizeof(m.startingPositions_)/sizeof(*m.startingPositions_); ++pos) {
+		if(pos->valid()) {
+			startingPositions_[pos - m.startingPositions_] = *pos;
+		}
+	}
+	
+}
+
 int gamemap::x() const { return tiles_.size(); }
 int gamemap::y() const { return tiles_.empty() ? 0 : tiles_.front().size(); }
 
@@ -317,7 +392,7 @@ gamemap::TERRAIN gamemap::get_terrain(const gamemap::location& loc) const
 
 const gamemap::location& gamemap::starting_position(int n) const
 {
-	if(n < sizeof(startingPositions_)/sizeof(*startingPositions_)) {
+	if(size_t(n) < sizeof(startingPositions_)/sizeof(*startingPositions_)) {
 		return startingPositions_[n];
 	} else {
 		static const gamemap::location null_loc;
@@ -381,24 +456,40 @@ void gamemap::set_terrain(const gamemap::location& loc, gamemap::TERRAIN ter)
 	if(!on_board(loc))
 		return;
 
+	const bool old_village = is_village(loc);
+	const bool new_village = is_village(ter);
+
+	if(old_village && !new_village) {
+		villages_.erase(std::remove(villages_.begin(),villages_.end(),loc),villages_.end());
+	} else if(!old_village && new_village) {
+		villages_.push_back(loc);
+	}
+
+	//If the terrain is set under a starting position, do also erase the
+	//starting position.
+	for(int i = 0; i < STARTING_POSITIONS; ++i) {
+		if(loc == startingPositions_[i])
+			startingPositions_[i] = location();
+	}
+
 	tiles_[loc.x][loc.y] = ter;
 
 	location adj[6];
 	get_adjacent_tiles(loc,adj);
 
-	for(int i = 0; i < 6; ++i) 
-		remove_from_border_cache(adj[i]);
+	for(int n = 0; n < 6; ++n) 
+		remove_from_border_cache(adj[n]);
 }
 
 std::vector<gamemap::location> parse_location_range(const std::string& x, const std::string& y)
 {
 	std::vector<gamemap::location> res;
-	const std::vector<std::string> xvals = config::split(x);
-	const std::vector<std::string> yvals = config::split(y);
+	const std::vector<std::string> xvals = utils::split(x);
+	const std::vector<std::string> yvals = utils::split(y);
 
 	for(unsigned int i = 0; i != minimum(xvals.size(),yvals.size()); ++i) {
-		const std::pair<int,int> xrange = config::parse_range(xvals[i]);
-		const std::pair<int,int> yrange = config::parse_range(yvals[i]);
+		const std::pair<int,int> xrange = utils::parse_range(xvals[i]);
+		const std::pair<int,int> yrange = utils::parse_range(yvals[i]);
 
 		for(int x = xrange.first; x <= xrange.second; ++x) {
 			for(int y = yrange.first; y <= yrange.second; ++y) {
@@ -423,8 +514,8 @@ const std::map<gamemap::TERRAIN,size_t>& gamemap::get_weighted_terrain_frequenci
 	const size_t weight_at_edge = 100;
 	const size_t additional_weight_at_center = 200;
 
-	for(size_t i = 0; i != x(); ++i) {
-		for(size_t j = 0; j != y(); ++j) {
+	for(size_t i = 0; i != size_t(x()); ++i) {
+		for(size_t j = 0; j != size_t(y()); ++j) {
 			const size_t distance = distance_between(location(i,j),center);
 			terrainFrequencyCache_[(*this)[i][j]] += weight_at_edge + (furthest_distance-distance)*additional_weight_at_center;
 		}
