@@ -1,4 +1,4 @@
-/* $Id: map.hpp,v 1.32 2005/02/19 09:26:31 isaaccp Exp $ */
+/* $Id: map.hpp,v 1.37 2005/03/28 12:29:10 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,7 +14,8 @@
 #ifndef MAP_H_INCLUDED
 #define MAP_H_INCLUDED
 
-#include "config.hpp"
+class config;
+
 #include "terrain.hpp"
 
 #include <map>
@@ -110,6 +111,9 @@ public:
 
 	std::string write() const;
 
+	//overlays another map onto this one at the given position.
+	void overlay(const gamemap& m, const config& rules, int x=0, int y=0);
+
 	//dimensions of the map.
 	int x() const;
 	int y() const;
@@ -174,7 +178,9 @@ private:
 
 	std::vector<std::vector<TERRAIN> > tiles_;
 	std::vector<location> villages_;
-	location startingPositions_[10];
+
+	enum { STARTING_POSITIONS = 10 };
+	location startingPositions_[STARTING_POSITIONS];
 
 	mutable std::map<location,TERRAIN> borderCache_;
 	mutable std::map<TERRAIN,size_t> terrainFrequencyCache_;
