@@ -1,8 +1,14 @@
+#include "global.hpp"
+
 #include "display.hpp"
 #include "font.hpp"
 #include "language.hpp"
 #include "map_label.hpp"
 
+namespace {
+	const size_t max_label_size = 32;
+}
+
 map_labels::map_labels(const display& disp, const gamemap& map) : disp_(disp), map_(map)
 {}
 
@@ -42,6 +48,10 @@ const std::string& map_labels::get_label(int index) const {
 	return font::get_floating_label_text(index);
 }
 
+int map_labels::get_max_chars() {
+	return max_label_size;
+}
+
 const std::string& map_labels::get_label(const gamemap::location& loc) const
 {
 	const label_map::const_iterator itor = labels_.find(loc);
@@ -55,7 +65,6 @@ const std::string& map_labels::get_label(const gamemap::location& loc) const
 
 void map_labels::set_label(const gamemap::location& loc, const std::string& str)
 {
-	const size_t max_label_size = 32;
 	std::string text = str;
 	if(text.size() > max_label_size) {
 		text.resize(max_label_size);
@@ -73,11 +82,13 @@ void map_labels::set_label(const gamemap::location& loc, const std::string& str)
 
 	SDL_Color colour = font::NORMAL_COLOUR;
 
+#if 0
 	if(map_.get_terrain_info(map_.get_terrain(loc)).is_light()) {
 		colour.r = 0;
 		colour.g = 0;
 		colour.b = 0;
 	}
+#endif
 
 	const gamemap::location loc_nextx(loc.x+1,loc.y);
 	const gamemap::location loc_nexty(loc.x,loc.y+1);
