@@ -11,50 +11,18 @@
 #include <ctime>
 #include <vector>
 
-#include "cavegen.hpp"
 #include "language.hpp"
 #include "log.hpp"
 #include "mapgen.hpp"
-#include "mapgen_dialog.hpp"
 #include "pathfind.hpp"
 #include "race.hpp"
 #include "scoped_resource.hpp"
 #include "util.hpp"
+#include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
-#define ERR_CF lg::err(lg::config)
-#define LOG_NG lg::info(lg::engine)
-
-//function to generate a random map, from a string which describes
-//the generator to use and its arguments
-std::string random_generate_map(const std::string& parms, const config* cfg)
-{
-	//the first token is the name of the generator, tokens after
-	//that are arguments to the generator
-	std::vector<std::string> parameters = config::split(parms,' ');
-	util::scoped_ptr<map_generator> generator(create_map_generator(parameters.front(),cfg));
-	if(generator == NULL) {
-		ERR_CF << "could not find map generator '" << parameters.front() << "'\n";
-		return std::string();
-	}
-
-	parameters.erase(parameters.begin());
-	return generator.get()->create_map(parameters);
-}
-
-config random_generate_scenario(const std::string& parms, const config* cfg)
-{
-	//the first token is the name of the generator, tokens after
-	//that are arguments to the generator
-	std::vector<std::string> parameters = config::split(parms,' ');
-	util::scoped_ptr<map_generator> generator(create_map_generator(parameters.front(),cfg));
-	if(generator == NULL) {
-		ERR_CF << "could not find map generator '" << parameters.front() << "'\n";
-		return config();
-	}
-
-	parameters.erase(parameters.begin());
-	return generator->create_scenario(parameters);
-}
+#define ERR_CF LOG_STREAM(err, config)
+#define LOG_NG LOG_STREAM(info, engine)
 
 config map_generator::create_scenario(const std::vector<std::string>& args)
 {
@@ -438,7 +406,7 @@ int rank_castle_location(int x, int y, const is_valid_terrain& valid_terrain, in
 
 		if(distance < min_distance) {
 			avg_distance = 0;
-			break;
+			return -1;
 		}
 
 		avg_distance += distance;
@@ -523,10 +491,11 @@ gamemap::location place_village(const std::vector<std::vector<gamemap::TERRAIN>
 	return best_loc;
 }
 
-std::string generate_name(const unit_race& name_generator, const std::string& id, std::string* base_name=NULL,
-						  std::map<std::string,std::string>* additional_symbols=NULL)
+std::string generate_name(const unit_race& name_generator, const std::string& id, 
+		std::string* base_name=NULL,
+		utils::string_map* additional_symbols=NULL)
 {
-	const std::vector<std::string>& options = config::split(string_table[id]);
+	const std::vector<std::string>& options = utils::split(string_table[id].str());
 	if(options.empty() == false) {
 		const size_t choice = rand()%options.size();
 		LOG_NG << "calling name generator...\n";
@@ -537,7 +506,7 @@ std::string generate_name(const unit_race& name_generator, const std::string& id
 		}
 
 		LOG_NG << "assigned base name..\n";
-		std::map<std::string,std::string> table;
+		utils::string_map  table;
 		if(additional_symbols == NULL) {
 			additional_symbols = &table;
 		}
@@ -546,7 +515,7 @@ std::string generate_name(const unit_race& name_generator, const std::string& id
 
 		(*additional_symbols)["name"] = name;
 		LOG_NG << "interpolation variables into '" << options[choice] << "'\n";
-		return config::interpolate_variables_into_string(options[choice],additional_symbols);
+		return utils::interpolate_variables_into_string(options[choice], additional_symbols);
 	}
 
 	return "";
@@ -638,10 +607,8 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 {
 	log_scope("map generation");
 
-	//odd widths are nasty, so make them even
-	if(is_odd(width)) {
-		++width;
-	}
+	//odd widths are nasty
+	wassert(is_even(width));
 
 	int ticks = SDL_GetTicks();
 
@@ -673,8 +640,6 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 		naming = *names_info;
 	}
 
-	LOG_NG << "random map config: '" << cfg.write() << "'\n";
-	LOG_NG << "making dummy race for naming: '" << naming.write() << "'\n";
 	//make a dummy race for generating names
 	unit_race name_generator(naming);
 
@@ -867,7 +832,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 		const size_t min_distance = atoi((*castle_config)["min_distance"].c_str());
 
 		location best_loc;
-		int best_ranking = -1;
+		int best_ranking = 0;
 		for(int x = min_x; x != max_x; ++x) {
 			for(int y = min_y; y != max_y; ++y) {
 				const location loc(x,y);
@@ -1007,7 +972,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 					}
 
 					if(direction != -1) {
-						const std::vector<std::string> items = config::split(convert_to_bridge);
+						const std::vector<std::string> items = utils::split(convert_to_bridge);
 						if(size_t(direction) < items.size() && items[direction].empty() == false) {
 							terrain[x][y] = items[direction][0];
 						}
@@ -1129,7 +1094,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 								size_t field_count = 0, forest_count = 0, mountain_count = 0, hill_count = 0;
 
-								std::map<std::string,std::string> symbols;
+								utils::string_map symbols;
 
 								size_t n;
 								for(n = 0; n != 6; ++n) {
@@ -1206,17 +1171,6 @@ generator_map generators;
 
 }
 
-map_generator* create_map_generator(const std::string& name, const config* cfg)
-{
-	if(name == "default" || name == "") {
-		return new default_map_generator(cfg);
-	} else if(name == "cave") {
-		return new cave_map_generator(cfg);
-	} else {
-		return NULL;
-	}
-}
-
 #ifdef TEST_MAPGEN
 
 int main(int argc, char** argv)
