@@ -1,13 +1,14 @@
 #ifndef MAP_GEN_HPP_INCLUDED
 #define MAP_GEN_HPP_INCLUDED
 
-#include "config.hpp"
-#include "display.hpp"
+class config;
+class display;
 
+#include "map.hpp"
+
+#include <map>
 #include <string>
-
-std::string random_generate_map(const std::string& parms, const config* cfg);
-config random_generate_scenario(const std::string& parms, const config* cfg);
+#include <vector>
 
 class map_generator
 {
@@ -34,8 +35,6 @@ public:
 	virtual config create_scenario(const std::vector<std::string>& args);
 };
 
-map_generator* create_map_generator(const std::string& name, const config* cfg);
-
 std::string default_generate_map(size_t width, size_t height, size_t island_size, size_t island_off_center,
                                  size_t iterations, size_t hill_size,
 								 size_t max_lakes, size_t nvillages, size_t nplayers,
