@@ -1,4 +1,4 @@
-/* $Id: pathfind.cpp,v 1.61 2005/01/27 20:30:51 silene Exp $ */
+/* $Id: pathfind.cpp,v 1.65 2005/03/26 17:10:32 silene Exp $ */
 /*
 Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
 Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,17 +13,19 @@ See the COPYING file for more details.
 
 #include "global.hpp"
 
-#include "game.hpp"
+#include "astarnode.hpp"
+#include "gamestatus.hpp"
 #include "log.hpp"
 #include "pathfind.hpp"
 #include "util.hpp"
-#include "astarnode.hpp"
 #include "wassert.hpp"
 
+class gamestatus;
+
 #include <cmath>
 #include <iostream>
 
-#define LOG_PF lg::info(lg::engine)
+#define LOG_PF LOG_STREAM(info, engine)
 
 typedef std::vector<gamemap::location> vector_location;
 typedef std::vector<a_star_node*> vector_a_star_node;
