@@ -1,4 +1,4 @@
-/* $Id: pathfind.hpp,v 1.35 2005/01/21 20:16:43 Sirp Exp $ */
+/* $Id: pathfind.hpp,v 1.36 2005/03/10 20:17:17 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,7 +14,7 @@
 #define PATHFIND_H_INCLUDED
 
 #include "array.hpp"
-#include "gamestatus.hpp"
+class gamestatus;
 #include "log.hpp"
 #include "map.hpp"
 #include "pathutils.hpp"
