@@ -1,4 +1,4 @@
-/* $Id: playlevel.cpp,v 1.176 2005/02/06 10:40:12 isaaccp Exp $ */
+/* $Id: playlevel.cpp,v 1.199 2005/03/28 19:08:58 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,21 +13,27 @@
 
 #include "global.hpp"
 
+#include "ai_interface.hpp"
 #include "cursor.hpp"
+#include "dialogs.hpp"
 #include "events.hpp"
 #include "filesystem.hpp"
+#include "game_errors.hpp"
+#include "gamestatus.hpp"
+#include "gettext.hpp"
 #include "game_events.hpp"
 #include "halo.hpp"
 #include "help.hpp"
 #include "hotkeys.hpp"
 #include "intro.hpp"
-#include "language.hpp"
 #include "log.hpp"
 #include "mapgen.hpp"
+#include "map_create.hpp"
 #include "network.hpp"
 #include "playlevel.hpp"
 #include "playturn.hpp"
 #include "preferences.hpp"
+#include "random.hpp"
 #include "replay.hpp"
 #include "scoped_resource.hpp"
 #include "sound.hpp"
@@ -35,11 +41,12 @@
 #include "tooltips.hpp"
 #include "unit_display.hpp"
 #include "util.hpp"
+#include "video.hpp"
 
 #include <iostream>
 #include <iterator>
 
-#define LOG_NG lg::info(lg::engine)
+#define LOG_NG LOG_STREAM(info, engine)
 
 namespace {
 	int placing_score(const config& side, const gamemap& map, const gamemap::location& pos)
@@ -114,11 +121,21 @@ namespace {
 	}
 }
 
-LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
+LEVEL_RESULT play_level(const game_data& gameinfo, const config& game_config,
 		config* level, CVideo& video,
 		game_state& state_of_game,
 		const std::vector<config*>& story)
 {
+	//if the recorder has no event, adds an "game start" event to the
+	//recorder, whose only goal is to initialize the RNG
+	if(recorder.empty()) {
+		recorder.add_start();
+	} else {
+		recorder.pre_replay();
+	}
+
+	const set_random_generator generator_setter(&recorder);
+
 	//guarantee the cursor goes back to 'normal' at the end of the level
 	const cursor::setter cursor_setter(cursor::NORMAL);
 
@@ -224,7 +241,9 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 		LOG_NG << "initializing team...\n";
 
-		if (first_human_team == -1 && ((**ui)["controller"] == "human" || (**ui)["persistent"] == "1")) {
+		if ((**ui)["controller"] == preferences::client_type() && (**ui)["description"] == preferences::login()) {
+			first_human_team = ui - unit_cfg.begin();
+		} else if(first_human_team == -1 && ((**ui)["controller"] == "human" || (**ui)["persistent"] == "1")) {
 			first_human_team = ui - unit_cfg.begin();
 		}
 
@@ -243,6 +262,11 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 		teams.push_back(team(**ui,ngold));
 
+		//if this team has no objectives, set its objectives to the
+		//level-global "objectives"
+		if(teams.back().objectives().empty())
+			teams.back().set_objectives((*level)["objectives"]);
+
 		//if this side tag describes the leader of the side
 		if((**ui)["no_leader"] != "yes" && (**ui)["controller"] != "null") {
 			unit new_unit(gameinfo, **ui);
@@ -271,11 +295,11 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 			}
 
 			if(map.empty()) {
-				throw gamestatus::load_game_failed("Map not found");
+				throw game::load_game_failed("Map not found");
 			}
 
 			if(!start_pos.valid() && new_unit.side() == 1) {
-				throw gamestatus::load_game_failed("No starting position for side 1");
+				throw game::load_game_failed("No starting position for side 1");
 			}
 
 			if(start_pos.valid()) {
@@ -340,14 +364,14 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 	// system.
 	for (unit_map::const_iterator help_unit_it = units.begin();
 		 help_unit_it != units.end(); help_unit_it++) {
-		const std::string name = help_unit_it->second.type().name();
+		const std::string name = help_unit_it->second.type().id();
 		preferences::encountered_units().insert(name);
 	}
 
 	// Add all units that are recallable as encountred units.
 	for(std::map<std::string, player_info>::iterator pi = state_of_game.players.begin(); pi!=state_of_game.players.end(); ++pi) {
 		for(std::vector<unit>::iterator help_recall_it = pi->second.available_units.begin(); help_recall_it != pi->second.available_units.end(); help_recall_it++) {
-			preferences::encountered_units().insert(help_recall_it->type().name());
+			preferences::encountered_units().insert(help_recall_it->type().id());
 		}
 	}
 
@@ -374,6 +398,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 	LOG_NG << "initializing display... " << (SDL_GetTicks() - ticks) << "\n";
 	const config dummy_cfg;
 	display gui(units,video,map,status,teams,theme_cfg != NULL ? *theme_cfg : dummy_cfg, game_config, *level);
+	theme::set_known_themes(&game_config);
 
 	LOG_NG << "done initializing display... " << (SDL_GetTicks() - ticks) << "\n";
 
@@ -384,7 +409,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 	}
 
 	const preferences::display_manager prefs_disp_manager(&gui);
-	const tooltips::manager tooltips_manager(gui);
+	const tooltips::manager tooltips_manager(gui.video());
 
 	LOG_NG << "b... " << (SDL_GetTicks() - ticks) << "\n";
 
@@ -395,6 +420,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 	if(recorder.skipping() == false) {
 		for(std::vector<config*>::const_iterator story_i = story.begin(); story_i != story.end(); ++story_i) {
+
 			show_intro(gui,**story_i, *level);
 		}
 	}
@@ -422,7 +448,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 	//find a list of 'items' (i.e. overlays) on the level, and add them
 	const config::child_list& overlays = level->get_children("item");
 	for(config::child_list::const_iterator overlay = overlays.begin(); overlay != overlays.end(); ++overlay) {
-		gui.add_overlay(gamemap::location(**overlay),(**overlay)["image"]);
+		gui.add_overlay(gamemap::location(**overlay),(**overlay)["image"], (**overlay)["halo"]);
 	}
 
 	int turn = 1, player_number = 0;
@@ -440,10 +466,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 		
 		gui.begin_game();
 		gui.adjust_colours(0,0,0);
-
-		if(!loading_game) {
-			game_events::fire("prestart");
-		}
+		
 
 		LOG_NG << "scrolling... " << (SDL_GetTicks() - ticks) << "\n";
 		if(first_human_team != -1) {
@@ -463,33 +486,47 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 		if(first_player < 0 || first_player >= int(teams.size())) {
 			first_player = 0;
 		}
-
-		LOG_NG << "starting main loop\n" << (SDL_GetTicks() - ticks) << "\n";
+		
+		events::raise_draw_event();
+		for(std::vector<team>::iterator t = teams.begin(); t != teams.end(); ++t) {
+			clear_shroud(gui,status,map,gameinfo,units,teams,(t-teams.begin()));
+		}
+		gui.recalculate_minimap();
+		gui.draw();
+		
+		if(!loading_game) {
+			game_events::fire("prestart");
+			events::raise_draw_event();
+			gui.draw();
+			for(std::vector<team>::iterator t = teams.begin(); t != teams.end(); ++t) {
+				clear_shroud(gui,status,map,gameinfo,units,teams,(t-teams.begin()));
+			}
+			gui.recalculate_minimap();
+		}
+		
 
 		std::deque<config> data_backlog;
 		
+		LOG_NG << "starting main loop\n" << (SDL_GetTicks() - ticks) << "\n";
 		for(bool first_time = true; true; first_time = false, first_player = 0) {
-			player_number = 0;
-
 			if(first_time) {
 				const hotkey::basic_handler key_events_handler(&gui);
 
 				LOG_NG << "first_time..." << (recorder.skipping() ? "skipping" : "no skip") << "\n";
-				update_locker lock_display(gui,recorder.skipping());
+				update_locker lock_display(gui.video(),recorder.skipping());
 				events::raise_draw_event();
 				if(!loading_game) {
 					game_events::fire("start");
-					game_events::set_variable("turn_number", "1");
+					state_of_game.set_variable("turn_number", "1");
 				}
-
 				gui.draw();
-				
+
 				for(std::vector<team>::iterator t = teams.begin(); t != teams.end(); ++t) {
 					clear_shroud(gui,status,map,gameinfo,units,teams,(t-teams.begin()));
 				}
-
 				gui.recalculate_minimap();
 			}
+			player_number = 0;
 
 			gui.new_turn();
 			gui.invalidate_game_status();
@@ -512,7 +549,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 				std::stringstream player_number_str;
 				player_number_str << player_number;
-				game_events::set_variable("side_number",player_number_str.str());
+				state_of_game.set_variable("side_number",player_number_str.str());
 
 				//fire side turn event only if real side change occurs not counting changes from void to a side
 				if (team_it != teams.begin()+first_player || !first_time) {
@@ -584,15 +621,9 @@ redo_turn:
 				if(!replaying && team_it->is_human()) {
 					LOG_NG << "is human...\n";
 
-					if(first_time && team_it == teams.begin()) {
-						if(lvl["objectives"].empty() == false) {
-							dialogs::show_objectives(gui,*level);
-						}
-					}
-
 					try {
 						play_turn(gameinfo,state_of_game,status,game_config,
-						          level, key, gui, map, teams, player_number,
+						          *level, key, gui, map, teams, player_number,
 						          units, textbox_info, replay_sender);
 					} catch(end_turn_exception& end_turn) {
 						if (end_turn.redo == player_number)
@@ -611,9 +642,9 @@ redo_turn:
 					const cursor::setter cursor_setter(cursor::WAIT);
 
 					turn_info turn_data(gameinfo,state_of_game,status,
-						                game_config,level,key,gui,
+						                game_config,*level,key,gui,
 						                map,teams,player_number,units,
-										turn_info::BROWSE_AI,textbox_info,replay_sender);
+								turn_info::BROWSE_AI,textbox_info,replay_sender);
 
 					ai_interface::info ai_info(gui,map,gameinfo,units,teams,player_number,status,turn_data);
 					util::scoped_ptr<ai_interface> ai_obj(create_ai(team_it->ai_algorithm(),ai_info));
@@ -630,8 +661,10 @@ redo_turn:
 					LOG_NG << "is networked...\n";
 
 					turn_info turn_data(gameinfo,state_of_game,status,
-					                    game_config,level,key,gui,
-										map,teams,player_number,units,turn_info::BROWSE_NETWORKED,textbox_info,replay_sender);
+					                    game_config,*level,key,gui,
+							    map,teams,player_number,units,
+							    turn_info::BROWSE_NETWORKED,
+							    textbox_info,replay_sender);
 
 					for(;;) {
 
@@ -701,9 +734,9 @@ redo_turn:
 
 			{
 				LOG_NG << "turn event..." << (recorder.skipping() ? "skipping" : "no skip") << "\n";
-				update_locker lock_display(gui,recorder.skipping());
+				update_locker lock_display(gui.video(),recorder.skipping());
 				const std::string turn_num = event_stream.str();
-				game_events::set_variable("turn_number",turn_num);
+				state_of_game.set_variable("turn_number",turn_num);
 				game_events::fire("turn " + turn_num);
 				game_events::fire("new turn");
 			}
@@ -721,9 +754,8 @@ redo_turn:
 				info["condition"] = "game over";
 				network::send_data(cfg);
 			} else
-				gui::show_dialog(gui, NULL, "",
-				                 _("Victory") + std::string(" / ") + _("Defeat"),
-				                 gui::OK_ONLY);
+				gui::show_dialog(gui, NULL, _("Game Over"),
+				                 _("The game is over."), gui::OK_ONLY);
 		}
 
 		if(end_level.result == QUIT) {
@@ -806,23 +838,23 @@ redo_turn:
 						report << font::BOLD_TEXT << i->save_id() << "\n";
 					}
 
-					report << _("Remaining gold") << ": "
+					report << _("Remaining gold: ")
 					       << remaining_gold << "\n";
 					if(end_level.gold_bonus) {
-						report << _("Early finish bonus") << ": "
+						report << _("Early finish bonus: ")
 						       << finishing_bonus_per_turn
 						       << " " << _("per turn") << "\n"
-						       << _("Turns finished early") << ": "
+						       << _("Turns finished early: ")
 						       << turns_left << "\n"
-						       << _("Bonus") << ": "
+						       << _("Bonus: ")
 						       << finishing_bonus << "\n"
-						       << _("Gold") << ": "
+						       << _("Gold: ")
 						       << (remaining_gold+finishing_bonus);
 					}
 
 					// xgettext:no-c-format
 					report << "\n" << _("80% of gold is retained for the next scenario") << "\n"
-					       << _("Retained Gold") << ": " << player->gold;
+					       << _("Retained Gold: ") << player->gold;
 				}
 			}
 
@@ -849,8 +881,8 @@ redo_turn:
 		}
 
 		turn_info turn_data(gameinfo,state_of_game,status,
-					        game_config,level,key,gui,
-							map,teams,player_number,units,turn_info::BROWSE_NETWORKED,textbox_info,replay_sender);
+				game_config,*level,key,gui,
+				map,teams,player_number,units,turn_info::BROWSE_NETWORKED,textbox_info,replay_sender);
 
 		turn_data.save_game(_("A network disconnection has occured, and the game cannot continue. Do you want to save the game?"),gui::YES_NO);
 		if(disconnect) {
