@@ -1,4 +1,4 @@
-/* $Id: playlevel.hpp,v 1.19 2005/01/22 10:47:16 silene Exp $ */
+/* $Id: playlevel.hpp,v 1.22 2005/03/22 23:42:36 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,26 +13,14 @@
 #ifndef PLAY_LEVEL_HPP_INCLUDED
 #define PLAY_LEVEL_HPP_INCLUDED
 
-#include "actions.hpp"
-#include "ai_interface.hpp"
-#include "config.hpp"
-#include "dialogs.hpp"
-#include "display.hpp"
-#include "game_config.hpp"
-#include "gamestatus.hpp"
-#include "key.hpp"
-#include "pathfind.hpp"
-#include "show_dialog.hpp"
-#include "team.hpp"
-#include "unit_types.hpp"
-#include "unit.hpp"
-#include "video.hpp"
+class config;
+class CVideo;
+struct game_state;
 
-#include <cmath>
-#include <fstream>
-#include <iostream>
-#include <sstream>
-#include <string>
+#include "game_config.hpp"
+#include "unit_types.hpp"
+
+#include <vector>
 
 enum LEVEL_RESULT { VICTORY, DEFEAT, QUIT, LEVEL_CONTINUE, LEVEL_CONTINUE_NO_SAVE };
 
@@ -49,7 +37,7 @@ struct end_turn_exception {
 	int redo;
 };
 
-LEVEL_RESULT play_level(game_data& gameinfo, const config& terrain_config,
+LEVEL_RESULT play_level(const game_data& gameinfo, const config& terrain_config,
 		config* level, CVideo& video,
 		game_state& state_of_game,
 		const std::vector<config*>& story);
