@@ -1,4 +1,4 @@
-/* $Id: playturn.cpp,v 1.328 2005/02/06 10:40:12 isaaccp Exp $ */
+/* $Id: playturn.cpp,v 1.357 2005/04/10 11:53:15 isaaccp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,11 +14,14 @@
 #include "global.hpp"
 
 #include "actions.hpp"
+#include "dialogs.hpp"
 #include "events.hpp"
+#include "game_errors.hpp"
+#include "game_events.hpp"
+#include "gettext.hpp"
 #include "help.hpp"
 #include "hotkeys.hpp"
 #include "image.hpp"
-#include "language.hpp"
 #include "log.hpp"
 #include "map_label.hpp"
 #include "mouse.hpp"
@@ -34,6 +37,8 @@
 #include "unit_display.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
+#include "wml_separators.hpp"
+#include "serialization/string_utils.hpp"
 #include "widgets/menu.hpp"
 
 #include <cmath>
@@ -56,9 +61,9 @@ command_disabler::~command_disabler()
 	--commands_disabled;
 }
 
-void play_turn(game_data& gameinfo, game_state& state_of_game,
-               gamestatus& status, const config& terrain_config,
-               config* level, CKey& key, display& gui, gamemap& map,
+void play_turn(const game_data& gameinfo, game_state& state_of_game,
+               const gamestatus& status, const config& terrain_config,
+               const config& level, CKey& key, display& gui, gamemap& map,
                std::vector<team>& teams, int team_num,
                std::map<gamemap::location,unit>& units,
                turn_info::floating_textbox& textbox,
@@ -135,21 +140,24 @@ void play_turn(game_data& gameinfo, game_state& state_of_game,
 	turn_data.send_data();
 }
 
-turn_info::turn_info(game_data& gameinfo, game_state& state_of_game,
-                     gamestatus& status, const config& terrain_config, config* level,
-                     CKey& key, display& gui, gamemap& map,
-                     std::vector<team>& teams, int team_num,
-                     unit_map& units, TURN_MODE mode, floating_textbox& textbox, replay_network_sender& replay_sender)
+turn_info::turn_info(const game_data& gameinfo, game_state& state_of_game,
+                     const gamestatus& status, const config& terrain_config, 
+		     const config& level, CKey& key, display& gui, gamemap& map,
+		     std::vector<team>& teams, int team_num, unit_map& units,
+		     TURN_MODE mode, floating_textbox& textbox,
+		     replay_network_sender& replay_sender)
   : paths_wiper(gui),
     gameinfo_(gameinfo), state_of_game_(state_of_game), status_(status),
     terrain_config_(terrain_config), level_(level),
     key_(key), gui_(gui), map_(map), teams_(teams), team_num_(team_num),
     units_(units), browse_(mode != PLAY_TURN), allow_network_commands_(mode == BROWSE_NETWORKED),
     left_button_(false), right_button_(false), middle_button_(false),
-	minimap_scrolling_(false), enemy_paths_(false), last_nearest_(gamemap::location::NORTH),
+	minimap_scrolling_(false), enemy_paths_(false), last_nearest_(gamemap::location::NORTH), last_second_nearest_(gamemap::location::NORTH),
     path_turns_(0), end_turn_(false), start_ncmd_(-1), textbox_(textbox), replay_sender_(replay_sender)
 {
 	enemies_visible_ = enemies_visible();
+
+	gui_.enable_menu("endturn", !browse_);
 }
 
 void turn_info::turn_slice()
@@ -183,6 +191,12 @@ void turn_info::turn_slice()
 	
 	if(use_left_right && key_[SDLK_RIGHT] || mousex > gui_.x()-scroll_threshold)
 		gui_.scroll(preferences::scroll_speed(),0);
+
+	if(!browse_ && current_team().objectives_changed()) {
+		dialogs::show_objectives(gui_, level_, current_team().objectives());
+		current_team().reset_objectives_changed();
+	}
+
 }
 
 bool turn_info::turn_over() const { return end_turn_; }
@@ -291,11 +305,11 @@ void turn_info::mouse_motion(int x, int y)
 		if(minimap_scrolling_) return;
 	}
 
-	gamemap::location::DIRECTION nearest_hex;
+	gamemap::location::DIRECTION nearest_hex, second_nearest_hex;
 	const team& current_team = teams_[team_num_-1];
-	const gamemap::location new_hex = gui_.hex_clicked_on(x,y,&nearest_hex);
+	const gamemap::location new_hex = gui_.hex_clicked_on(x,y,&nearest_hex,&second_nearest_hex);
 
-	if(new_hex != last_hex_ || nearest_hex != last_nearest_) {
+	if(new_hex != last_hex_ || nearest_hex != last_nearest_ || second_nearest_hex != last_second_nearest_) {
 		if(new_hex.valid() == false) {
 			current_route_.steps.clear();
 			gui_.set_route(NULL);
@@ -312,8 +326,7 @@ void turn_info::mouse_motion(int x, int y)
 
 		gamemap::location attack_from;
 		if(selected_unit != units_.end() && mouseover_unit != units_.end()) {
-			const gamemap::location preferred = new_hex.get_direction(nearest_hex);
-			attack_from = current_unit_attacks_from(new_hex,&preferred);
+			attack_from = current_unit_attacks_from(new_hex, nearest_hex, second_nearest_hex);
 		}
 
 		if(selected_unit != units_.end() && (current_paths_.routes.count(new_hex) ||
@@ -335,18 +348,17 @@ void turn_info::mouse_motion(int x, int y)
 			gui_.set_paths(NULL);
 		}
 		
-		if(new_hex == selected_hex_) {
+		const gamemap::location& dest = attack_from.valid() ? attack_from : new_hex;
+		const unit_map::const_iterator dest_un = find_unit(dest);
+		if(dest == selected_hex_ || dest_un != units_.end()) {
 			current_route_.steps.clear();
 			gui_.set_route(NULL);
 		} else if(!current_paths_.routes.empty() && map_.on_board(selected_hex_) &&
 		   map_.on_board(new_hex)) {
-
-			const gamemap::location& dest = attack_from.valid() ? attack_from : new_hex;
-
+			
 			unit_map::const_iterator un = find_unit(selected_hex_);
-			const unit_map::const_iterator dest_un = find_unit(dest);
-
-			if((new_hex != last_hex_ || attack_from.valid()) && un != units_.end() && dest_un == units_.end() && !un->second.stone()) {
+			
+			if((new_hex != last_hex_ || attack_from.valid()) && un != units_.end() && !un->second.stone()) {
 				const shortest_path_calculator calc(un->second,current_team,
 				                                    visible_units(),teams_,map_,status_);
 				const bool can_teleport = un->second.type().teleports();
@@ -388,6 +400,7 @@ void turn_info::mouse_motion(int x, int y)
 
 	last_hex_ = new_hex;
 	last_nearest_ = nearest_hex;
+	last_second_nearest_ = second_nearest_hex;
 }
 
 namespace {
@@ -476,7 +489,7 @@ void turn_info::mouse_press(const SDL_MouseButtonEvent& event)
 			if (m != NULL)
 				show_menu(m->items(),event.x,event.y,true);
 			else
-				lg::warn(lg::display) << "no context menu found...\n";
+				LOG_STREAM(warn, display) << "no context menu found...\n";
 		}
 	} else if(is_middle_click(event) && event.state == SDL_PRESSED) {
 		// clicked on a hex on the minimap? then initiate minimap scrolling
@@ -646,7 +659,7 @@ bool turn_info::attack_enemy(unit_map::iterator attacker, unit_map::iterator def
 		preview_panes.push_back(&defender_preview);
 
 		res = gui::show_dialog(gui_,NULL,_("Attack Enemy"),
-				_("Choose weapon")+std::string(":\n"),
+				_("Choose weapon:")+std::string("\n"),
 				gui::OK_CANCEL,&items,&preview_panes,"",NULL,-1,NULL,NULL,-1,-1,
 				NULL,&buttons);
 	}
@@ -776,8 +789,8 @@ void turn_info::left_click(const SDL_MouseButtonEvent& event)
 		return;
 	}
 
-	gamemap::location::DIRECTION nearest_hex;
-	gamemap::location hex = gui_.hex_clicked_on(event.x,event.y,&nearest_hex);
+	gamemap::location::DIRECTION nearest_hex, second_nearest_hex;
+	gamemap::location hex = gui_.hex_clicked_on(event.x,event.y,&nearest_hex,&second_nearest_hex);
 
 	unit_map::iterator u = find_unit(selected_hex_);
 
@@ -799,8 +812,7 @@ void turn_info::left_click(const SDL_MouseButtonEvent& event)
 
 	//see if we're trying to do a move-and-attack
 	if(!browse_ && u != units_.end() && enemy != units_.end() && !current_route_.steps.empty()) {
-		const gamemap::location preferred = hex.get_direction(nearest_hex);
-		const gamemap::location& attack_from = current_unit_attacks_from(hex,&preferred);
+		const gamemap::location& attack_from = current_unit_attacks_from(hex, nearest_hex, second_nearest_hex);
 		if(attack_from.valid()) {
 			if(move_unit_along_current_route(false)) { //move the unit without updating shroud
 				u = find_unit(attack_from);
@@ -906,7 +918,7 @@ void turn_info::show_attack_options(unit_map::const_iterator u)
 	}
 }
 
-gamemap::location turn_info::current_unit_attacks_from(const gamemap::location& loc, const gamemap::location* preferred) const
+gamemap::location turn_info::current_unit_attacks_from(const gamemap::location& loc, const gamemap::location::DIRECTION preferred, const gamemap::location::DIRECTION second_preferred) const
 {
 	const unit_map::const_iterator current = find_unit(selected_hex_);
 	if(current == units_.end() || current->second.side() != team_num_) {
@@ -918,7 +930,7 @@ gamemap::location turn_info::current_unit_attacks_from(const gamemap::location&
 		return gamemap::location();
 	}
 
-	int best_defense = 100;
+	int best_rating = 100;//smaller is better
 	gamemap::location res;
 	gamemap::location adj[6];
 	get_adjacent_tiles(loc,adj);
@@ -928,7 +940,7 @@ gamemap::location turn_info::current_unit_attacks_from(const gamemap::location&
 		}
 
 		if(adj[n] == selected_hex_) {
-			return gamemap::location();
+			return selected_hex_;
 		}
 
 		if(find_unit(adj[n]) != units_.end()) {
@@ -936,9 +948,18 @@ gamemap::location turn_info::current_unit_attacks_from(const gamemap::location&
 		}
 
 		if(current_paths_.routes.count(adj[n])) {
-			const int defense = preferred != NULL && *preferred == adj[n] ? 0 : current->second.defense_modifier(map_,map_.get_terrain(loc));
-			if(defense < best_defense || res.valid() == false) {
-				best_defense = defense;
+			static const size_t NDIRECTIONS = gamemap::location::NDIRECTIONS;
+			int difference = abs(int(preferred - n));
+			if(difference > NDIRECTIONS/2) {
+				difference = NDIRECTIONS - difference;
+			}
+			int second_difference = abs(int(second_preferred - n));
+			if(second_difference > NDIRECTIONS/2) {
+				second_difference = NDIRECTIONS - second_difference;
+			}
+			const int rating = difference * 2 + (second_difference > difference);
+			if(rating < best_rating || res.valid() == false) {
+				best_rating = rating;
 				res = adj[n];
 			}
 		}
@@ -977,7 +998,7 @@ void turn_info::move_unit_to_loc(const unit_map::const_iterator& ui, const gamem
 }
 
 void turn_info::start_interactive_turn() {
-	lg::info(lg::engine) << "done gotos\n";
+	LOG_STREAM(info, engine) << "done gotos\n";
 	start_ncmd_ = recorder.ncommands();
 }
 
@@ -1163,7 +1184,7 @@ void turn_info::show_menu(const std::vector<std::string>& items_arg, int xloc, i
 	static const std::string style = "menu2";
 	const int res = gui::show_dialog(gui_,NULL,"","",
 			gui::MESSAGE,&menu,NULL,"",NULL,-1,NULL,NULL,xloc,yloc,&style);
-	if(res < 0 || res >= items.size())
+	if (size_t(res) >= items.size())
 		return;
 
 	const hotkey::HOTKEY_COMMAND cmd = hotkey::get_hotkey(items[res]).get_id();
@@ -1275,7 +1296,7 @@ void turn_info::end_turn()
 	write_game_snapshot(snapshot);
 	try {
 		recorder.save_game(_("Auto-Save"), snapshot, state_of_game_.starting_pos);
-	} catch(gamestatus::save_game_failed&) {
+	} catch(game::save_game_failed&) {
 		gui::show_dialog(gui_,NULL,"",_("Could not auto save the game. Please save the game manually."),gui::MESSAGE);
 		//do not bother retrying, since the user can just save the game
 	}
@@ -1325,7 +1346,7 @@ void turn_info::undo()
 		player_info* const player = state_of_game_.get_player(teams_[team_num_-1].save_id());
 
 		if(player == NULL) {
-			lg::err(lg::engine) << "trying to undo a recall for side " << team_num_
+			LOG_STREAM(err, engine) << "trying to undo a recall for side " << team_num_
 				<< ", which has no recall list!\n";
 		} else {
 			// Undo a recall action
@@ -1351,7 +1372,7 @@ void turn_info::undo()
 		const unit_map::iterator u = units_.find(route.front());
 		if(u == units_.end()) {
 			//this can actually happen if the scenario designer has abused the [allow_undo] command
-			lg::err(lg::engine) << "Illegal 'undo' found. Possible abuse of [allow_undo]?\n";
+			LOG_STREAM(err, engine) << "Illegal 'undo' found. Possible abuse of [allow_undo]?\n";
 			return;
 		}
 	
@@ -1413,7 +1434,7 @@ void turn_info::redo()
 	if(action.is_recall()) {
 		player_info *player=state_of_game_.get_player(teams_[team_num_-1].save_id());
 		if(!player) {
-			lg::err(lg::engine) << "trying to redo a recall for side " << team_num_
+			LOG_STREAM(err, engine) << "trying to redo a recall for side " << team_num_
 				<< ", which has no recall list!\n";
 		} else {
 			// Redo recall
@@ -1494,7 +1515,7 @@ void turn_info::load_game()
 	bool show_replay = false;
 	const std::string game = dialogs::load_game_dialog(gui_,terrain_config_,gameinfo_,&show_replay);
 	if(game != "") {
-		throw gamestatus::load_game_exception(game,show_replay);
+		throw game::load_game_exception(game,show_replay);
 	}
 }
 
@@ -1538,7 +1559,7 @@ void turn_info::save_game(const std::string& message, gui::DIALOG_TYPE dialog_ty
 			if(dialog_type != gui::NULL_DIALOG) {
 				gui::show_dialog(gui_,NULL,_("Saved"),_("The game has been saved"), gui::OK_ONLY);
 			}
-		} catch(gamestatus::save_game_failed&) {
+		} catch(game::save_game_failed&) {
 			gui::show_dialog(gui_,NULL,_("Error"),_("The game could not be saved"),gui::MESSAGE);
 			//do not bother retrying, since the user can just try to save the game again
 		};
@@ -1605,7 +1626,7 @@ unit_map::iterator turn_info::current_unit()
 
 void turn_info::write_game_snapshot(config& start) const
 {
-	start.values = level_->values;
+	start.values = level_.values;
 
 	start["snapshot"] = "yes";
 
@@ -1744,7 +1765,7 @@ void turn_info::recruit()
 		const std::map<std::string,unit_type>::const_iterator
 				u_type = gameinfo_.unit_types.find(*it);
 		if(u_type == gameinfo_.unit_types.end()) {
-			lg::err(lg::engine) << "could not find unit '" << *it << "'";
+			LOG_STREAM(err, engine) << "could not find unit '" << *it << "'\n";
 			return;
 		}
 
@@ -1778,7 +1799,7 @@ void turn_info::recruit()
 		preview_panes.push_back(&unit_preview);
 
 		recruit_res = gui::show_dialog(gui_,NULL,_("Recruit"),
-				_("Select unit") + std::string(":\n"),
+				_("Select unit:") + std::string("\n"),
 				gui::OK_CANCEL,&items,&preview_panes,"",NULL,-1,NULL,NULL,-1,-1,
 				NULL,NULL,"recruit_and_recall");
 	}
@@ -1876,9 +1897,9 @@ gui::dialog_button_action::RESULT delete_recall_unit::button_pressed(int menu_se
 		}
 
 		if(message != "") {
-			string_map symbols;
+			utils::string_map symbols;
 			symbols["noun"] = (u.gender() == unit_race::MALE ? _("him") : _("her"));
-			message = config::interpolate_variables_into_string(message,&symbols);
+			message = utils::interpolate_variables_into_string(message, &symbols);
 
 			const int res = gui::show_dialog(disp_,NULL,"",message,gui::YES_NO);
 			if(res != 0) {
@@ -1903,7 +1924,7 @@ void turn_info::recall()
 
 	player_info *player = state_of_game_.get_player(teams_[team_num_-1].save_id());
 	if(!player) {
-		lg::err(lg::engine) << "cannot recall a unit for side " << team_num_
+		LOG_STREAM(err, engine) << "cannot recall a unit for side " << team_num_
 			<< ", which has no recall list!\n";
 		return;
 	}
@@ -1917,14 +1938,14 @@ void turn_info::recall()
 
 	gui_.draw(); //clear the old menu
 
-	if((*level_)["disallow_recall"] == "yes") {
+	if(level_["disallow_recall"] == "yes") {
 		gui::show_dialog(gui_,NULL,"",_("You are separated from your soldiers and may not recall them"));
 	} else if(recall_list.empty()) {
 		gui::show_dialog(gui_,NULL,"",_("There are no troops available to recall\n\
 (You must have veteran survivors from a previous scenario)"));
 	} else if(current_team.gold() < game_config::recall_cost) {
 		std::stringstream msg;
-		string_map i18n_symbols;
+		utils::string_map i18n_symbols;
 		i18n_symbols["cost"] = lexical_cast<std::string>(game_config::recall_cost);
 		msg << vgettext("You must have at least $cost gold pieces to recall a unit", i18n_symbols);
 		gui::show_dialog(gui_,NULL,"",msg.str());
@@ -1936,8 +1957,8 @@ void turn_info::recall()
 			option << IMAGE_PREFIX << u->type().image() << COLUMN_SEPARATOR
 			       << u->type().language_name() << COLUMN_SEPARATOR
 			       << description << COLUMN_SEPARATOR
-			       << _("level") << ": " << u->type().level() << COLUMN_SEPARATOR
-			       << _("XP") << ": " << u->experience() << "/";
+			       << _("level: ") << u->type().level() << COLUMN_SEPARATOR
+			       << _("XP: ") << u->experience() << "/";
 
 			if(u->can_advance() == false) {
 				option << "-";
@@ -1962,7 +1983,7 @@ void turn_info::recall()
 			preview_panes.push_back(&unit_preview);
 
 			res = gui::show_dialog(gui_,NULL,_("Recall"),
-					_("Select unit") + std::string(":\n"),
+					_("Select unit:") + std::string("\n"),
 					gui::OK_CANCEL,&options,
 					&preview_panes,"",NULL,-1,
 					NULL,NULL,-1,-1,NULL,&buttons);
@@ -2006,7 +2027,7 @@ bool turn_info::has_friends() const
 
 void turn_info::speak()
 {
-	create_textbox(floating_textbox::TEXTBOX_MESSAGE,_("Message") + std::string(":"), has_friends() ? _("Send to allies only") : "", preferences::message_private());
+	create_textbox(floating_textbox::TEXTBOX_MESSAGE,_("Message:"), has_friends() ? _("Send to allies only") : "", preferences::message_private());
 }
 
 void turn_info::do_speak(const std::string& message, bool allies_only)
@@ -2030,7 +2051,6 @@ void turn_info::do_speak(const std::string& message, bool allies_only)
 		cfg["team_name"] = teams_[gui_.viewing_team()].team_name();
 	}
 
-	lg::info(lg::config) << "logging speech: '" << cfg.write() << "'\n";
 	recorder.speak(cfg);
 	gui_.add_chat_message(cfg["description"],side,message,
 		                  private_message ? display::MESSAGE_PRIVATE : display::MESSAGE_PUBLIC);
@@ -2058,7 +2078,7 @@ void turn_info::create_unit()
 		                          gui::OK_CANCEL,&options,&preview_panes);
 	}
 
-	if(choice >= 0 && choice < unit_choices.size()) {
+	if (size_t(choice) < unit_choices.size()) {
 		units_.erase(last_hex_);
 		units_.insert(std::pair<gamemap::location,unit>(last_hex_,unit_choices[choice]));
 		gui_.invalidate(last_hex_);
@@ -2090,7 +2110,8 @@ void turn_info::preferences()
 
 void turn_info::objectives()
 {
-	dialogs::show_objectives(gui_,*level_);
+	dialogs::show_objectives(gui_, level_, current_team().objectives());
+	current_team().reset_objectives_changed();
 }
 
 void turn_info::unit_list()
@@ -2310,7 +2331,7 @@ void turn_info::do_search(const std::string& new_search)
 	bool found = false;
 	gamemap::location loc = last_search_hit_;
 	//If this is a location search, just center on that location.
-	std::vector<std::string> args = config::split(last_search_,',');
+	std::vector<std::string> args = utils::split(last_search_, ',');
 	if(args.size() == 2) {
 		int x, y;
 		x = lexical_cast_default<int>(args[0], 0)-1;
@@ -2359,9 +2380,9 @@ void turn_info::do_search(const std::string& new_search)
 	} else {
 		last_search_hit_ = gamemap::location();
 		//Not found, inform the player
-		string_map symbols;
+		utils::string_map symbols;
 		symbols["search"] = last_search_;
-		const std::string msg = config::interpolate_variables_into_string(
+		const std::string msg = utils::interpolate_variables_into_string(
 			_("Couldn't find label or unit containing the string '$search'."),&symbols);
 		gui::show_dialog(gui_,NULL,"",msg);
 	}
@@ -2384,13 +2405,27 @@ void turn_info::do_command(const std::string& str)
 		} else if (teams_[index].is_human()) {
 			teams_[index].make_ai();
 			close_textbox();
-			throw end_turn_exception(side);
+			if(team_num_ == side) {
+				throw end_turn_exception(side);
+			}
 		} else if (teams_[index].is_ai()) {
 			teams_[index].make_human();
+		}       
+	} else if (cmd == "theme") {
+	        int action = 0;
+		std::vector<std::string> options=theme::get_known_themes();
+		std::string current_theme=_("Saved Theme Preference: ")+preferences::theme();
+		action = gui::show_dialog(gui_,NULL,"",current_theme,gui::OK_CANCEL,&options);
+		if(action >-1){
+		  preferences::set_theme(options[action]);
+		//it would be preferable for the new theme to take effect 
+		//immediately, however, this will have to do for now.
+		  gui::show_dialog(gui_,NULL,"",_("New theme will take effect on next new or loaded game."),gui::MESSAGE);
 		}
-	} else if(cmd == "ban") {
+
+	} else if(cmd == "ban" || cmd == "kick") {
 		config cfg;
-		config& ban = cfg.add_child("ban");
+		config& ban = cfg.add_child(cmd);
 		ban["username"] = data;
 
 		network::send_data(cfg);
@@ -2411,10 +2446,12 @@ void turn_info::do_command(const std::string& str)
 		throw end_level_exception(QUIT);
 	} else if(cmd == "q!" || cmd == "q") {
 		throw end_level_exception(QUIT);
-	} else if(cmd == "debug") {
-		game_config::debug = (data != "off") ? true : false;
+	} else if(cmd == "ignore_replay_errors") {
+		game_config::ignore_replay_errors = (data != "off") ? true : false;
 	} else if(cmd == "n" && game_config::debug) {
 		throw end_level_exception(VICTORY);
+	} else if(cmd == "debug" && network::nconnections() == 0) {
+		game_config::debug = true;
 	} else if(game_config::debug && cmd == "unit") {
 		const unit_map::iterator i = current_unit();
 		if(i != units_.end()) {
@@ -2431,6 +2468,19 @@ void turn_info::do_command(const std::string& str)
 				gui_.invalidate_unit();
 			}
 		}
+	} else if(game_config::debug && cmd == "create" && map_.on_board(last_hex_)) {
+		const game_data::unit_type_map::const_iterator i = gameinfo_.unit_types.find(data);
+		if(i == gameinfo_.unit_types.end()) {
+			return;
+		}
+
+		units_.erase(last_hex_);
+		units_.insert(std::pair<gamemap::location,unit>(last_hex_,unit(&i->second,1,false)));
+		gui_.invalidate(last_hex_);
+		gui_.invalidate_unit();
+	} else if(game_config::debug && cmd == "gold") {
+		current_team().spend_gold(-lexical_cast_default<int>(data,1000));
+		gui_.redraw_everything();
 	}
 }
 
@@ -2456,7 +2506,7 @@ void turn_info::label_terrain()
 
 	std::string label = gui_.labels().get_label(last_hex_);
 	const int res = gui::show_dialog(gui_,NULL,_("Place Label"),"",gui::OK_CANCEL,
-	                                 NULL,NULL,_("Label") + std::string(":"),&label,
+	                                 NULL,NULL,_("Label:"),&label,
 					 map_labels::get_max_chars());
 	if(res == 0) {
 		gui_.labels().set_label(last_hex_,label);
@@ -2627,7 +2677,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 	if(cfg["side_drop"] != "") {
 		const size_t side = atoi(cfg["side_drop"].c_str())-1;
 		if(side >= teams_.size()) {
-			lg::err(lg::network) << "unknown side " << side << " is dropping game\n";
+			LOG_STREAM(err, network) << "unknown side " << side << " is dropping game\n";
 			throw network::error("");
 		}
 
@@ -2690,7 +2740,7 @@ void turn_info::create_textbox(floating_textbox::MODE mode, const std::string& l
 	textbox_.mode = mode;
 
 	if(check_label != "") {
-		textbox_.check.assign(new gui::button(gui_,check_label,gui::button::TYPE_CHECK));
+		textbox_.check.assign(new gui::button(gui_.video(),check_label,gui::button::TYPE_CHECK));
 		textbox_.check->set_check(checked);
 	}
 
@@ -2713,7 +2763,7 @@ void turn_info::create_textbox(floating_textbox::MODE mode, const std::string& l
 		return;
 	}
 
-	textbox_.box.assign(new gui::textbox(gui_,textbox_width,"",true,256,0.8,0.6));
+	textbox_.box.assign(new gui::textbox(gui_.video(),textbox_width,"",true,256,0.8,0.6));
 	textbox_.box->set_volatile(true);
 	textbox_.box->set_location(area.x + label_area.w + border_size*2,ypos);
 
@@ -2757,7 +2807,7 @@ void turn_info::enter_textbox()
 		do_command(textbox_.box->text());
 		break;
 	default:
-		lg::err(lg::display) << "unknown textbox mode\n";
+		LOG_STREAM(err, display) << "unknown textbox mode\n";
 	}
 
 	close_textbox();
@@ -2783,7 +2833,7 @@ void turn_info::tab_textbox()
 			return;
 		}
 
-		if(last_space == -1) {
+		if(last_space == std::string::npos) {
 			beginning = true;
 			semiword = text;	
 		}else{
@@ -2804,7 +2854,7 @@ void turn_info::tab_textbox()
 					if(guess.size() == 0) {
 						guess = name;
 					}else{
-						int i;
+						size_t i;
 						for(i=0; (i < guess.size()) || (i < name.size()); i++) {
 							if(guess[i] != name[i]) {
 								break;
@@ -2824,14 +2874,14 @@ void turn_info::tab_textbox()
 		break;
 	}
 	default:
-		lg::err(lg::display) << "unknown textbox mode\n";
+		LOG_STREAM(err, display) << "unknown textbox mode\n";
 	}
 }
 
 const unit_map& turn_info::visible_units() const
 {
 	if(viewing_team().uses_shroud() == false && viewing_team().uses_fog() == false) {
-		lg::info(lg::engine) << "all units are visible...\n";
+		LOG_STREAM(info, engine) << "all units are visible...\n";
 		return units_;
 	}
 
@@ -2842,7 +2892,7 @@ const unit_map& turn_info::visible_units() const
 		}
 	}
 
-	lg::info(lg::engine) << "number of visible units: " << visible_units_.size() << "\n";
+	LOG_STREAM(info, engine) << "number of visible units: " << visible_units_.size() << "\n";
 
 	return visible_units_;
 }
