@@ -1,4 +1,4 @@
-/* $Id: playturn.hpp,v 1.61 2005/02/06 10:40:12 isaaccp Exp $ */
+/* $Id: playturn.hpp,v 1.67 2005/03/24 21:18:26 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,14 +13,12 @@
 #ifndef PLAYTURN_HPP_INCLUDED
 #define PLAYTURN_HPP_INCLUDED
 
+class replay_network_sender;
+
 #include "actions.hpp"
-#include "ai_interface.hpp"
 #include "config.hpp"
-#include "dialogs.hpp"
 #include "display.hpp"
 #include "events.hpp"
-#include "game_config.hpp"
-#include "game_events.hpp"
 #include "gamestatus.hpp"
 #include "hotkeys.hpp"
 #include "key.hpp"
@@ -31,7 +29,6 @@
 #include "team.hpp"
 #include "unit_types.hpp"
 #include "unit.hpp"
-#include "video.hpp"
 
 #include "widgets/button.hpp"
 #include "widgets/textbox.hpp"
@@ -82,11 +79,12 @@ public:
 
 	enum TURN_MODE { PLAY_TURN, BROWSE_NETWORKED, BROWSE_AI };
 	
-	turn_info(game_data& gameinfo, game_state& state_of_game,
-	          gamestatus& status, const config& terrain_config, config* level,
-	          CKey& key, display& gui, gamemap& map,
-	          std::vector<team>& teams, int team_num,
-	          unit_map& units, TURN_MODE mode, floating_textbox& textbox, replay_network_sender& network_sender);
+	turn_info(const game_data& gameinfo, game_state& state_of_game,
+	          const gamestatus& status, const config& terrain_config,
+		  const config& level, CKey& key, display& gui, gamemap& map,
+		  std::vector<team>& teams, int team_num, unit_map& units,
+		  TURN_MODE mode, floating_textbox& textbox,
+		  replay_network_sender& network_sender);
 
 	void turn_slice();
 
@@ -190,7 +188,7 @@ private:
 	//function which, given the location of a potential enemy to attack, will return the location
 	//that the currently selected unit would move to and attack it from this turn. Returns an
 	//invalid location if not possible.
-	gamemap::location current_unit_attacks_from(const gamemap::location& loc, const gamemap::location* preferred) const;
+	gamemap::location current_unit_attacks_from(const gamemap::location& loc, const gamemap::location::DIRECTION preferred, const gamemap::location::DIRECTION second_preferred) const;
 
 	bool attack_enemy(unit_map::iterator attacker, unit_map::iterator defender);
 	bool move_unit_along_current_route(bool check_shroud=true);
@@ -204,11 +202,11 @@ private:
 
 	void change_side_controller(const std::string& side, const std::string& player, bool orphan_side=false);
 
-	game_data& gameinfo_;
+	const game_data& gameinfo_;
 	game_state& state_of_game_;
-	gamestatus& status_;
+	const gamestatus& status_;
 	const config& terrain_config_;
-	config* level_;
+	const config& level_;
 	CKey key_;
 	display& gui_;
 	gamemap& map_;
@@ -228,7 +226,7 @@ private:
 	paths::route current_route_;
 	bool enemy_paths_;
 	gamemap::location last_hex_;
-	gamemap::location::DIRECTION last_nearest_;
+	gamemap::location::DIRECTION last_nearest_, last_second_nearest_;
 	gamemap::location selected_hex_;
 	undo_list undo_stack_;
 	undo_list redo_stack_;
@@ -256,8 +254,9 @@ private:
 	replay_network_sender& replay_sender_;
 };
 
-void play_turn(game_data& gameinfo, game_state& state_of_game,
-               gamestatus& status, const config& terrain_config, config* level,
+void play_turn(const game_data& gameinfo, game_state& state_of_game,
+               const gamestatus& status, const config& terrain_config,
+	       const config& level,
                CKey& key, display& gui, gamemap& map,
                std::vector<team>& teams, int team_num,
                std::map<gamemap::location,unit>& units,
