@@ -1,4 +1,4 @@
-/* $Id: preferences.cpp,v 1.131 2005/02/06 10:40:12 isaaccp Exp $ */
+/* $Id: preferences.cpp,v 1.151 2005/04/05 13:37:15 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -16,21 +16,26 @@
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "cursor.hpp"
+#include "display.hpp"
 #include "events.hpp"
 #include "filesystem.hpp"
 #include "font.hpp"
+#include "gettext.hpp"
 #include "hotkeys.hpp"
-#include "language.hpp"
 #include "log.hpp"
 #include "preferences.hpp"
 #include "show_dialog.hpp"
 #include "sound.hpp"
 #include "util.hpp"
+#include "video.hpp"
+#include "wesconfig.h"
+#include "wml_separators.hpp"
+#include "serialization/parser.hpp"
+#include "serialization/string_utils.hpp"
 #include "widgets/button.hpp"
 #include "widgets/label.hpp"
 #include "widgets/menu.hpp"
 #include "widgets/slider.hpp"
-#include "wesconfig.h"
 
 #include <cstdlib>
 #include <iostream>
@@ -63,7 +68,8 @@ namespace preferences {
 
 manager::manager()
 {
-	prefs.read(read_file(get_prefs_file()));
+	scoped_istream stream = istream_file(get_prefs_file());
+	read(prefs, *stream);
 	set_music_volume(music_volume());
 	set_sound_volume(sound_volume());
 
@@ -71,10 +77,10 @@ manager::manager()
 	set_show_haloes(prefs["show_haloes"] != "no");
 
 	std::vector<std::string> v;
-	v = config::split(prefs["encountered_units"]);
+	v = utils::split(prefs["encountered_units"]);
 	std::copy(v.begin(), v.end(),
 			  std::inserter(encountered_units_set, encountered_units_set.begin()));
-	v = config::split(prefs["encountered_terrains"]);
+	v = utils::split(prefs["encountered_terrains"]);
 	std::copy(v.begin(), v.end(),
 			  std::inserter(encountered_terrains_set, encountered_terrains_set.begin()));
 }
@@ -84,15 +90,16 @@ manager::~manager()
 	
 	std::vector<std::string> v;
 	std::copy(encountered_units_set.begin(), encountered_units_set.end(), std::back_inserter(v));
-	prefs["encountered_units"] = config::join(v);
+	prefs["encountered_units"] = utils::join(v);
 	v.clear();
 	std::copy(encountered_terrains_set.begin(), encountered_terrains_set.end(),
 			  std::back_inserter(v));
-	prefs["encountered_terrains"] = config::join(v);
+	prefs["encountered_terrains"] = utils::join(v);
 	encountered_units_set.clear();
 	encountered_terrains_set.clear();
 	try {
-		write_file(get_prefs_file(),prefs.write());
+		scoped_ostream prefs_file = ostream_file(get_prefs_file());
+		write(*prefs_file, prefs);
 	} catch(io_exception&) {
 		std::cerr << "error writing to preferences file '" << get_prefs_file() << "'\n";
 	}
@@ -349,7 +356,7 @@ const std::string& official_network_host()
 
 const std::string& network_host()
 {
-	std::string& res = prefs["host"];
+	t_string& res = prefs["host"];
 	if(res.empty())
 		res = WESNOTH_DEFAULT_SERVER;
 
@@ -361,9 +368,23 @@ void set_network_host(const std::string& host)
 	prefs["host"] = host;
 }
 
+const std::string& campaign_server()
+{
+	t_string& res = prefs["campaign_server"];
+	if(res.empty())
+		res = "campaigns.wesnoth.org";
+
+	return res;
+}
+
+void set_campaign_server(const std::string& host)
+{
+	prefs["campaign_server"] = host;
+}
+
 const std::string& login()
 {
-	std::string& res = prefs["login"];
+	t_string& res = prefs["login"];
 	if(res.empty()) {
 		char* const login = getenv("USER");
 		if(login != NULL) {
@@ -637,7 +658,7 @@ const std::string& theme()
 		return null_theme;
 	}
 
-	std::string& res = prefs["theme"];
+	t_string& res = prefs["theme"];
 	if(res.empty()) {
 		res = "Default";
 	}
@@ -706,6 +727,16 @@ void set_show_haloes(bool value)
 	prefs["show_haloes"] = value ? "yes" : "no";
 }
 
+bool flip_time()
+{
+	return prefs["flip_time"] == "yes";
+}
+
+void set_flip_time(bool value)
+{
+	prefs["flip_time"] = value ? "yes" : "no";
+}
+
 bool show_fps()
 {
 	return fps;
@@ -716,6 +747,11 @@ void set_show_fps(bool value)
 	fps = value;
 }
 
+bool compress_saves()
+{
+	return prefs["compress_saves"] != "no";
+}
+
 std::set<std::string> &encountered_units() {
 	return encountered_units_set;
 }
@@ -778,36 +814,44 @@ private:
 	gui::button fullscreen_button_, turbo_button_, show_ai_moves_button_,
 	            show_grid_button_, show_floating_labels_button_, turn_dialog_button_,
 	            turn_bell_button_, show_team_colours_button_, show_colour_cursors_button_,
-	            show_haloing_button_, video_mode_button_, hotkeys_button_, gamma_button_;
+	            show_haloing_button_, video_mode_button_, hotkeys_button_, gamma_button_,
+				flip_time_button_;
 	gui::label music_label_, sound_label_, scroll_label_, gamma_label_;
 	unsigned slider_label_width_;
 
 	enum TAB { GENERAL_TAB, DISPLAY_TAB, SOUND_TAB };
 	TAB tab_;
+	display &disp_;
 };
 
 preferences_dialog::preferences_dialog(display& disp)
-	: gui::preview_pane(disp),
-	  music_slider_(disp), sound_slider_(disp),
-	  scroll_slider_(disp), gamma_slider_(disp),
-	  fullscreen_button_(disp, _("Full Screen"), gui::button::TYPE_CHECK),
-	  turbo_button_(disp, _("Accelerated Speed"), gui::button::TYPE_CHECK),
-	  show_ai_moves_button_(disp, _("Skip AI Moves"), gui::button::TYPE_CHECK),
-	  show_grid_button_(disp, _("Show Grid"), gui::button::TYPE_CHECK),
-	  show_floating_labels_button_(disp, _("Show Floating Labels"), gui::button::TYPE_CHECK),
-	  turn_dialog_button_(disp, _("Turn Dialog"), gui::button::TYPE_CHECK),
-	  turn_bell_button_(disp, _("Turn Bell"), gui::button::TYPE_CHECK),
-	  show_team_colours_button_(disp, _("Show Team Colors"), gui::button::TYPE_CHECK),
-	  show_colour_cursors_button_(disp, _("Show Color Cursors"), gui::button::TYPE_CHECK),
-	  show_haloing_button_(disp, _("Show Haloing Effects"), gui::button::TYPE_CHECK),
-	  video_mode_button_(disp, _("Video Mode")),
-	  hotkeys_button_(disp, _("Hotkeys")),
-	  gamma_button_(disp, _("Adjust Gamma"), gui::button::TYPE_CHECK),
-	  music_label_(disp, _("Music Volume:")), sound_label_(disp, _("SFX Volume:")),
-	  scroll_label_(disp, _("Scroll Speed:")), gamma_label_(disp, _("Gamma:")),
-	  slider_label_width_(0), tab_(GENERAL_TAB)
+	: gui::preview_pane(disp.video()),
+	  music_slider_(disp.video()), sound_slider_(disp.video()),
+	  scroll_slider_(disp.video()), gamma_slider_(disp.video()),
+	  fullscreen_button_(disp.video(), _("Full Screen"), gui::button::TYPE_CHECK),
+	  turbo_button_(disp.video(), _("Accelerated Speed"), gui::button::TYPE_CHECK),
+	  show_ai_moves_button_(disp.video(), _("Skip AI Moves"), gui::button::TYPE_CHECK),
+	  show_grid_button_(disp.video(), _("Show Grid"), gui::button::TYPE_CHECK),
+	  show_floating_labels_button_(disp.video(), _("Show Floating Labels"), gui::button::TYPE_CHECK),
+	  turn_dialog_button_(disp.video(), _("Turn Dialog"), gui::button::TYPE_CHECK),
+	  turn_bell_button_(disp.video(), _("Turn Bell"), gui::button::TYPE_CHECK),
+	  show_team_colours_button_(disp.video(), _("Show Team Colors"), gui::button::TYPE_CHECK),
+	  show_colour_cursors_button_(disp.video(), _("Show Color Cursors"), gui::button::TYPE_CHECK),
+	  show_haloing_button_(disp.video(), _("Show Haloing Effects"), gui::button::TYPE_CHECK),
+	  video_mode_button_(disp.video(), _("Video Mode")),
+	  hotkeys_button_(disp.video(), _("Hotkeys")),
+	  gamma_button_(disp.video(), _("Adjust Gamma"), gui::button::TYPE_CHECK),
+	  flip_time_button_(disp.video(), _("Reverse Time Graphics"), gui::button::TYPE_CHECK),
+	  music_label_(disp.video(), _("Music Volume:")), sound_label_(disp.video(), _("SFX Volume:")),
+	  scroll_label_(disp.video(), _("Scroll Speed:")), gamma_label_(disp.video(), _("Gamma:")),
+	  slider_label_width_(0), tab_(GENERAL_TAB), disp_(disp)
 {
+	// FIXME: this box should be vertically centered on the screen, but is not
+#if USE_TINY_GUI
+	set_measurements(260, 220);		  // FIXME: should compute this, but using what data ?
+#else
 	set_measurements(400, 400);
+#endif
 
 	slider_label_width_ = maximum<unsigned>(music_label_.width(),
 	                      maximum<unsigned>(sound_label_.width(),
@@ -863,6 +907,9 @@ preferences_dialog::preferences_dialog(display& disp)
 	show_team_colours_button_.set_check(show_side_colours());
 	show_team_colours_button_.set_help_string(_("Show a colored circle around the base of each unit to show which side it is on"));
 
+	flip_time_button_.set_check(flip_time());
+	flip_time_button_.set_help_string(_("Choose whether the sun moves left-to-right or right-to-left"));
+
 	show_colour_cursors_button_.set_check(use_colour_cursors());
 	show_colour_cursors_button_.set_help_string(_("Use colored mouse cursors (may be slower)"));
 
@@ -875,7 +922,13 @@ preferences_dialog::preferences_dialog(display& disp)
 void preferences_dialog::update_location(SDL_Rect const &rect)
 {
 	bg_register(rect);
-	const int border = 10;
+
+	const int border = font::relative_size(10);
+#if USE_TINY_GUI
+	const int item_interline = 20;
+#else
+	const int item_interline = 50;
+#endif
 
 	// General tab
 	int ypos = rect.y;
@@ -883,27 +936,28 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 	SDL_Rect scroll_rect = { rect.x + slider_label_width_, ypos,
 	                         rect.w - slider_label_width_ - border, 0 };
 	scroll_slider_.set_location(scroll_rect);
-	ypos += 50; turbo_button_.set_location(rect.x, ypos);
-	ypos += 50; show_ai_moves_button_.set_location(rect.x, ypos);
-	ypos += 50; turn_dialog_button_.set_location(rect.x, ypos);
-	ypos += 50; turn_bell_button_.set_location(rect.x, ypos);
-	ypos += 50; show_team_colours_button_.set_location(rect.x, ypos);
-	ypos += 50; show_grid_button_.set_location(rect.x, ypos);
-	ypos += 50; hotkeys_button_.set_location(rect.x, ypos);
+	ypos += item_interline; turbo_button_.set_location(rect.x, ypos);
+	ypos += item_interline; show_ai_moves_button_.set_location(rect.x, ypos);
+	ypos += item_interline; turn_dialog_button_.set_location(rect.x, ypos);
+	ypos += item_interline; turn_bell_button_.set_location(rect.x, ypos);
+	ypos += item_interline; show_team_colours_button_.set_location(rect.x, ypos);
+	ypos += item_interline; show_grid_button_.set_location(rect.x, ypos);
+	ypos += item_interline; hotkeys_button_.set_location(rect.x, ypos);
 
 	// Display tab
 	ypos = rect.y;
 	gamma_button_.set_location(rect.x, ypos);
-	ypos += 50;
+	ypos += item_interline;
 	gamma_label_.set_location(rect.x, ypos);
 	SDL_Rect gamma_rect = { rect.x + slider_label_width_, ypos,
 	                        rect.w - slider_label_width_ - border, 0 };
 	gamma_slider_.set_location(gamma_rect);
-	ypos += 50; show_floating_labels_button_.set_location(rect.x, ypos);
-	ypos += 50; show_colour_cursors_button_.set_location(rect.x, ypos);
-	ypos += 50; show_haloing_button_.set_location(rect.x, ypos);
-	ypos += 50; fullscreen_button_.set_location(rect.x, ypos);
-	ypos += 50; video_mode_button_.set_location(rect.x, ypos);
+	ypos += item_interline; flip_time_button_.set_location(rect.x,ypos);
+	ypos += item_interline; show_floating_labels_button_.set_location(rect.x, ypos);
+	ypos += item_interline; show_colour_cursors_button_.set_location(rect.x, ypos);
+	ypos += item_interline; show_haloing_button_.set_location(rect.x, ypos);
+	ypos += item_interline; fullscreen_button_.set_location(rect.x, ypos);
+	ypos += item_interline; video_mode_button_.set_location(rect.x, ypos);
 
 	// Sound tab
 	ypos = rect.y;
@@ -911,7 +965,7 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 	SDL_Rect music_rect = { rect.x + slider_label_width_, ypos,
 	                        rect.w - slider_label_width_ - border, 0 };
 	music_slider_.set_location(music_rect);
-	ypos += 50;
+	ypos += item_interline;
 	sound_label_.set_location(rect.x, ypos);
 	SDL_Rect sound_rect = { rect.x + slider_label_width_, ypos,
 				rect.w - slider_label_width_ - border, 0 };
@@ -943,7 +997,7 @@ void preferences_dialog::process_event()
 	if (show_team_colours_button_.pressed())
 		set_show_side_colours(show_team_colours_button_.checked());
 	if (hotkeys_button_.pressed())
-		show_hotkeys_dialog(disp());
+		show_hotkeys_dialog(disp_);
 	if (show_colour_cursors_button_.pressed())
 		set_colour_cursors(show_colour_cursors_button_.checked());
 	if (show_haloing_button_.pressed())
@@ -954,6 +1008,8 @@ void preferences_dialog::process_event()
 		gamma_slider_.hide(hide_gamma);
 		gamma_label_.hide(hide_gamma);
 	}
+	if (flip_time_button_.pressed())
+		set_flip_time(flip_time_button_.checked());
 	set_sound_volume(sound_slider_.value());
 	set_music_volume(music_slider_.value());
 	set_scroll_speed(scroll_slider_.value());
@@ -986,6 +1042,7 @@ void preferences_dialog::set_selection(int index)
 	show_haloing_button_.hide(hide_display);
 	fullscreen_button_.hide(hide_display);
 	video_mode_button_.hide(hide_display);
+	flip_time_button_.hide(hide_display);
 
 	bool hide_sound = tab_ != SOUND_TAB;
 	music_label_.hide(hide_sound);
@@ -1107,22 +1164,27 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 
 	const int centerx = disp.x()/2;
 	const int centery = disp.y()/2;
-	const int xpos = centerx  - 300;
-	const int ypos = centery  - 250;
-	const int width = 600;
+#ifdef USE_TINY_GUI
+	const int width = 300;			  // FIXME: should compute this, but using what data ?
+	const int height = 220;
+#else
+	const int width = 700;
 	const int height = 500;
+#endif
+	const int xpos = centerx  - width/2;
+	const int ypos = centery  - height/2;
 
-	gui::button close_button (disp, _("Close Window"));
+	gui::button close_button (disp.video(), _("Close Window"));
 	std::vector<gui::button*> buttons;
 	buttons.push_back(&close_button);
 
 	surface_restorer restorer;	
-	gui::draw_dialog(xpos,ypos,width,height,disp,_("Hotkey Settings"),NULL,&buttons,&restorer);
+	gui::draw_dialog(xpos,ypos,width,height,disp.video(),_("Hotkey Settings"),NULL,&buttons,&restorer);
 	
 	SDL_Rect clip_rect = { 0, 0, disp.x (), disp.y () };
 	SDL_Rect text_size = font::draw_text(NULL, clip_rect, font::SIZE_PLUS,
-			           font::NORMAL_COLOUR,_("Press desired HotKey"),
-						0, 0);
+					     font::NORMAL_COLOUR,_("Press desired HotKey"),
+					     0, 0);
 
 	std::vector<std::string> menu_items;
 
@@ -1138,15 +1200,15 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 		menu_items.push_back (str.str ());
 	}
 
-	gui::menu menu_(disp, menu_items, false, height);
-	menu_.set_width(400);	
-	menu_.set_location(xpos + 20, ypos);
+	gui::menu menu_(disp.video(), menu_items, false, height);
+	menu_.set_width(font::relative_size(400));	
+	menu_.set_location(xpos + font::relative_size(20), ypos);
 	
-	gui::button change_button (disp, _("Change Hotkey"));
-	change_button.set_location(xpos + width - change_button.width () -30,ypos + 80);
+	gui::button change_button (disp.video(), _("Change Hotkey"));
+	change_button.set_location(xpos + width - change_button.width () - font::relative_size(30),ypos + font::relative_size(80));
 
-	gui::button save_button (disp, _("Save Hotkeys"));
-	save_button.set_location(xpos + width - save_button.width () - 30,ypos + 130);
+	gui::button save_button (disp.video(), _("Save Hotkeys"));
+	save_button.set_location(xpos + width - save_button.width () - font::relative_size(30),ypos + font::relative_size(130));
 
 	for(;;) {
 
@@ -1155,7 +1217,7 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 
 		if (change_button.pressed ()) {
 			// Lets change this hotkey......
-			SDL_Rect dlgr = {centerx-text_size.w/2-30,
+			SDL_Rect dlgr = {centerx-text_size.w/2 - 30,
 								centery-text_size.h/2 - 16,
 									text_size.w+60,
 									text_size.h+32};
@@ -1163,8 +1225,8 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 		 	gui::draw_dialog_frame (centerx-text_size.w/2 - 20, 
 									centery-text_size.h/2 - 6,
 									text_size.w+40,
-									text_size.h+12,disp);
-			font::draw_text (&disp, clip_rect, font::SIZE_LARGE,font::NORMAL_COLOUR,
+									text_size.h+12,disp.video());
+			font::draw_text (&disp.video(), clip_rect, font::SIZE_LARGE,font::NORMAL_COLOUR,
 				 _("Press desired HotKey"),centerx-text_size.w/2-10,
 				 centery-text_size.h/2-3);
 			disp.update_display();
@@ -1180,7 +1242,7 @@ void show_hotkeys_dialog (display & disp, config *save_config)
 				 	mod=event.key.keysym.mod;
 				};			
 				SDL_PollEvent(&event);
-				disp.video().flip();
+				disp.flip();
 				SDL_Delay(10);
 			} while (event.type!=SDL_KEYUP);
 			restorer.restore();
