@@ -1,12 +1,15 @@
 #include "global.hpp"
 
 #include "race.hpp"
-#include "replay.hpp"
+#include "random.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <cstdlib>
 
 namespace {
 
+config::child_list empty_traits;
+
 void add_prefixes(const wide_string& str, size_t length, markov_prefix_map& res)
 {
 	for(size_t i = 0; i <= str.size(); ++i) {
@@ -22,7 +25,7 @@ markov_prefix_map markov_prefixes(const std::vector<std::string>& items, size_t
 	markov_prefix_map res;
 
 	for(std::vector<std::string>::const_iterator i = items.begin(); i != items.end(); ++i) {
-		add_prefixes(string_to_wstring(*i),length,res);
+		add_prefixes(utils::string_to_wstring(*i),length,res);
 	}
 
 	return res;
@@ -92,15 +95,16 @@ wide_string markov_generate_name(const markov_prefix_map& prefixes, size_t chain
 
 }
 
-unit_race::unit_race() : ntraits_(0), chain_size_(0), not_living_(false)
+unit_race::unit_race() : ntraits_(0), chain_size_(0), not_living_(false), traits_(&empty_traits), global_traits_(true)
 {
 }
 
 unit_race::unit_race(const config& cfg) : name_(cfg["name"]), ntraits_(atoi(cfg["num_traits"].c_str())),
-                                          not_living_(cfg["not_living"] == "yes")
+                                          not_living_(cfg["not_living"] == "yes"),
+										  traits_(&cfg.get_children("trait")), global_traits_(cfg["ignore_global_traits"] != "yes")
 {
-	names_[MALE] = config::split(cfg["male_names"]);
-	names_[FEMALE] = config::split(cfg["female_names"]);
+	names_[MALE] = utils::split(cfg["male_names"]);
+	names_[FEMALE] = utils::split(cfg["female_names"]);
 
 	chain_size_ = atoi(cfg["markov_chain_size"].c_str());
 	if(chain_size_ <= 0)
@@ -110,11 +114,21 @@ unit_race::unit_race(const config& cfg) : name_(cfg["name"]), ntraits_(atoi(cfg[
 	next_[FEMALE] = markov_prefixes(names_[FEMALE],chain_size_);
 }
 
-const std::string& unit_race::name() const { return name_; }
+const t_string& unit_race::name() const { return name_; }
 
 std::string unit_race::generate_name(unit_race::GENDER gender) const
 {
-	return wstring_to_string(markov_generate_name(next_[gender],chain_size_,12));
+	return utils::wstring_to_string(markov_generate_name(next_[gender],chain_size_,12));
+}
+
+bool unit_race::uses_global_traits() const
+{
+	return global_traits_;
+}
+
+const config::child_list& unit_race::additional_traits() const
+{
+	return *traits_;
 }
 
 int unit_race::num_traits() const { return ntraits_; }
