@@ -1,4 +1,4 @@
-/* $Id: replay.cpp,v 1.91 2005/02/17 23:47:12 j_daniel Exp $ */
+/* $Id: replay.cpp,v 1.104 2005/03/30 19:48:17 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -17,6 +17,7 @@
 #include "ai_interface.hpp"
 #include "dialogs.hpp"
 #include "game_config.hpp"
+#include "game_events.hpp"
 #include "log.hpp"
 #include "network.hpp"
 #include "pathfind.hpp"
@@ -38,16 +39,24 @@
 #include <set>
 #include <sstream>
 
+#define LOG_NW LOG_STREAM(info, network)
+#define ERR_NW LOG_STREAM(err, network)
+
 //functions to verify that the unit structure on both machines is identical
 namespace {
+	std::ostream &operator<<(std::ostream &s, gamemap::location const &l) {
+		s << (l.x + 1) << ',' << (l.y + 1);
+		return s;
+	}
+
 	void verify(const unit_map& units, const config& cfg)
 	{
-		std::cerr << "verifying unit structure...\n";
+		LOG_NW << "verifying unit structure...\n";
 
 		const size_t nunits = atoi(cfg["num_units"].c_str());
 		if(nunits != units.size()) {
-			std::cerr << "SYNC VERIFICATION FAILED: number of units from data source differ: "
-			          << nunits << " according to data source. " << units.size() << " locally\n";
+			ERR_NW << "SYNC VERIFICATION FAILED: number of units from data source differ: "
+			       << nunits << " according to data source. " << units.size() << " locally\n";
 
 			std::set<gamemap::location> locs;
 			const config::child_list& items = cfg.get_children("unit");
@@ -56,17 +65,19 @@ namespace {
 				locs.insert(loc);
 
 				if(units.count(loc) == 0) {
-					std::cerr << "data source says there is a unit at " << (loc.x+1) << "," << (loc.y+1) << " but none found locally\n";
+					ERR_NW << "data source says there is a unit at "
+					       << loc << " but none found locally\n";
 				}
 			}
 
 			for(unit_map::const_iterator j = units.begin(); j != units.end(); ++j) {
 				if(locs.count(j->first) == 0) {
-					std::cerr << "local unit at " << (j->first.x+1) << "," << (j->first.y+1) << " but none in data source\n";
+					ERR_NW << "local unit at " << j->first
+					       << " but none in data source\n";
 				}
 			}
 
-			throw replay::error();
+			if (!game_config::ignore_replay_errors) throw replay::error();
 		}
 
 		const config::child_list& items = cfg.get_children("unit");
@@ -74,10 +85,11 @@ namespace {
 			const gamemap::location loc(**i);
 			const unit_map::const_iterator u = units.find(loc);
 			if(u == units.end()) {
-				std::cerr << "SYNC VERIFICATION FAILED: data source says there is a '"
-				          << (**i)["type"] << "' (side " << (**i)["side"] << ") at " << (**i)["x"] << "," << (**i)["y"]
-						  << " but there is no local record of it\n";
-				throw replay::error();
+				ERR_NW << "SYNC VERIFICATION FAILED: data source says there is a '"
+				       << (**i)["type"] << "' (side " << (**i)["side"] << ") at "
+				       << (**i)["x"] << ',' << (**i)["y"]
+				       << " but there is no local record of it\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			config cfg;
@@ -87,19 +99,21 @@ namespace {
 			static const std::string fields[] = {"type","hitpoints","experience","side",""};
 			for(const std::string* str = fields; str->empty() == false; ++str) {
 				if(cfg[*str] != (**i)[*str]) {
-					std::cerr << "ERROR IN FIELD '" << *str << "' for unit at " << (**i)["x"] << "," << (**i)["y"]
-						      << " data source: '" << (**i)[*str] << "' local: '" << cfg[*str] << "'\n";
+					ERR_NW << "ERROR IN FIELD '" << *str << "' for unit at "
+					       << (**i)["x"] << ',' << (**i)["y"]
+					       << " data source: '" << (**i)[*str]
+					       << "' local: '" << cfg[*str] << "'\n";
 					is_ok = false;
 				}
 			}
 
 			if(!is_ok) {
-				std::cerr << "(SYNC VERIFICATION FAILED)\n";
-				throw replay::error();
+				ERR_NW << "(SYNC VERIFICATION FAILED)\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 		}
 
-		std::cerr << "verification passed\n";
+		LOG_NW << "verification passed\n";
 	}
 
 	config create_verification(const unit_map& units)
@@ -151,49 +165,15 @@ verification_manager::~verification_manager()
 	unit_map_ref = NULL;
 }
 
+// FIXME: this one now has to be assigned with set_random_generator
+// from play_level or similar.  We should surely hunt direct
+// references to it from this very file and move it out of here.
 replay recorder;
 
-namespace {
-
-replay* random_generator = &recorder;
-
-struct set_random_generator {
-
-	set_random_generator(replay* r) : old_(random_generator)
-	{
-		random_generator = r;
-	}
-
-	~set_random_generator()
-	{
-		random_generator = old_;
-	}
-
-private:
-	replay* old_;
-};
-
-}
-
-int get_random()
-{
-	return random_generator->get_random();
-}
-
-const config* get_random_results()
-{
-	return random_generator->get_random_results();
-}
-
-void set_random_results(const config& cfg)
-{
-	random_generator->set_random_results(cfg);
-}
-
-replay::replay() : pos_(0), current_(NULL), random_(NULL), skip_(0)
+replay::replay() : pos_(0), current_(NULL), skip_(0)
 {}
 
-replay::replay(const config& cfg) : cfg_(cfg), pos_(0), current_(NULL), random_(NULL), skip_(0)
+replay::replay(const config& cfg) : cfg_(cfg), pos_(0), current_(NULL), skip_(0)
 {}
 
 config& replay::get_config()
@@ -248,6 +228,12 @@ void replay::save_game(const std::string& label, const config& snapshot,
 	saveInfo_.snapshot = config();
 }
 
+void replay::add_start()
+{
+	config* const cmd = add_command();
+	cmd->add_child("start");
+}
+
 void replay::add_recruit(int value, const gamemap::location& loc)
 {
 	config* const cmd = add_command();
@@ -265,8 +251,6 @@ void replay::add_recruit(int value, const gamemap::location& loc)
 	val["y"] = buf;
 
 	cmd->add_child("recruit",val);
-
-	random_ = cmd;
 }
 
 void replay::add_recall(int value, const gamemap::location& loc)
@@ -304,9 +288,6 @@ void replay::add_disband(int value)
 void replay::add_movement(const gamemap::location& a,const gamemap::location& b)
 {
 	add_pos("move",a,b);
-	//current_->add_child("verify",make_verify_units());
-	current_ = NULL;
-	random_ = NULL;
 }
 
 void replay::add_attack(const gamemap::location& a, const gamemap::location& b, int weapon)
@@ -315,7 +296,6 @@ void replay::add_attack(const gamemap::location& a, const gamemap::location& b,
 	char buf[100];
 	sprintf(buf,"%d",weapon);
 	current_->child("attack")->values["weapon"] = buf;
-	random_ = current_;
 }
 
 void replay::add_pos(const std::string& type,
@@ -338,8 +318,6 @@ void replay::add_pos(const std::string& type,
 	move.add_child("source",src);
 	move.add_child("destination",dst);
 	cmd->add_child(type,move);
-
-	current_ = cmd;
 }
 
 void replay::add_value(const std::string& type, int value)
@@ -378,7 +356,7 @@ void replay::end_turn()
 {
 	config* const cmd = add_command();
 	cmd->add_child("end_turn");
-	//cmd->add_child("verify",make_verify_units());
+	random_ = current_;
 }
 
 void replay::speak(const config& cfg)
@@ -467,48 +445,10 @@ void replay::mark_current()
 config* replay::add_command()
 {
 	pos_ = ncommands()+1;
-	return current_ = &cfg_.add_child("command");
-}
+	current_ = &cfg_.add_child("command");
+	random_ = current_;
 
-int replay::get_random()
-{
-	if(random_ == NULL) {
-		return rand();
-	}
-
-	//random numbers are in a 'list' meaning that each random
-	//number contains another random numbers unless it's at
-	//the end of the list. Generating a new random number means
-	//nesting a new node inside the current node, and making
-	//the current node the new node
-	config* const random = random_->child("random");
-	if(random == NULL) {
-		const int res = rand();
-		random_ = &random_->add_child("random");
-
-		char buf[100];
-		sprintf(buf,"%d",res);
-		(*random_)["value"] = buf;
-
-		return res;
-	} else {
-		const int res = atol((*random)["value"].c_str());
-		random_ = random;
-		return res;
-	}
-}
-
-const config* replay::get_random_results() const
-{
-	wassert(random_ != NULL);
-	return random_->child("results");
-}
-
-void replay::set_random_results(const config& cfg)
-{
-	wassert(random_ != NULL);
-	random_->clear_children("results");
-	random_->add_child("results",cfg);
+	return current_;
 }
 
 void replay::start_replay()
@@ -521,13 +461,25 @@ config* replay::get_next_action()
 	if(pos_ >= commands().size())
 		return NULL;
 
-	std::cerr << "up to replay action " << pos_ << "/" << commands().size() << "\n";
+	LOG_NW << "up to replay action " << pos_ << "/" << commands().size() << "\n";
 
 	random_ = current_ = commands()[pos_];
 	++pos_;
 	return current_;
 }
 
+void replay::pre_replay()
+{
+
+	if(pos_ >= commands().size())
+		return;
+
+	while(commands()[pos_]->child("start") != NULL) {
+		if(get_next_action() == NULL)
+			return;
+	}
+}
+
 bool replay::at_end() const
 {
 	return pos_ >= commands().size();
@@ -596,8 +548,8 @@ replay& get_replay_source()
 
 bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
                unit_map& units,
-			   std::vector<team>& teams, int team_num, const gamestatus& state,
-			   game_state& state_of_game, replay* obj)
+	       std::vector<team>& teams, int team_num, const gamestatus& state,
+	       game_state& state_of_game, replay* obj)
 {
 	log_scope("do replay");
 
@@ -610,7 +562,7 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 	const set_random_generator generator_setter(&replayer);
 
-	update_locker lock_update(disp,replayer.skipping());
+	update_locker lock_update(disp.video(),replayer.skipping());
 
 	//a list of units that have promoted from the last attack
 	std::deque<gamemap::location> advancing_units;
@@ -627,8 +579,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 		//if we are expecting promotions here
 		if(advancing_units.empty() == false) {
 			if(cfg == NULL) {
-				std::cerr << "promotion expected, but none found\n";
-				throw replay::error();
+				ERR_NW << "promotion expected, but none found\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			//if there is a promotion, we process it and go onto the next command
@@ -655,10 +607,29 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 		//if there is nothing more in the records
 		if(cfg == NULL) {
 			replayer.set_skip(0);
+			const config current_stats = statistics::write_stats();
+			if(current_stats != statistics::replay_verify_stats) {
+				ERR_NW << "replay statistics verification failed\n";
+				//Disabling because it breaks multiplayer.
+				//Apparently, statistics::replay_verify_stats
+				//is only written to in game::load_game() (!)
+				//Re-enable when fixed / if I am wrong.
+				//
+				//By the way, making a "config" object global
+				//is highly objectable IMO.
+				//
+				// --Ayin 30 mar. 2005
+				//
+				//if (!game_config::ignore_replay_errors) 
+				//	throw replay::error();
+			}
 			return false;
 		}
 
-		else if((child = cfg->child("speak")) != NULL) {
+		else if(cfg->child("start") != NULL) {
+			//do nothing
+
+		} else if((child = cfg->child("speak")) != NULL) {
 			const std::string& team_name = (*child)["team_name"];
 			if(team_name == "" || teams[disp.viewing_team()].team_name() == team_name) {
 				if(preferences::message_bell()) {
@@ -698,30 +669,31 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			const std::set<std::string>& recruits = current_team.recruits();
 
 			if(val < 0 || val >= recruits.size()) {
-				std::cerr << "recruitment index is illegal: " << val
-				          << " while this side only has " << recruits.size() << " units available for recruitment\n";
-				throw replay::error();
+				ERR_NW << "recruitment index is illegal: " << val
+				       << " while this side only has " << recruits.size()
+				       << " units available for recruitment\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			std::set<std::string>::const_iterator itor = recruits.begin();
 			std::advance(itor,val);
 			const std::map<std::string,unit_type>::const_iterator u_type = gameinfo.unit_types.find(*itor);
 			if(u_type == gameinfo.unit_types.end()) {
-				std::cerr << "recruiting illegal unit: '" << *itor << "'\n";
-				throw replay::error();
+				ERR_NW << "recruiting illegal unit: '" << *itor << "'\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			unit new_unit(&(u_type->second),team_num,true);
 			const std::string& res = recruit_unit(map,team_num,units,new_unit,loc);
 			if(!res.empty()) {
-				std::cerr << "cannot recruit unit: " << res << "\n";
-				throw replay::error();
+				ERR_NW << "cannot recruit unit: " << res << "\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			if(u_type->second.cost() > current_team.gold()) {
-				std::cerr << "unit '" << u_type->second.name() << "' is too expensive to recruit: "
-				          << u_type->second.cost() << "/" << current_team.gold() << "\n";
-				throw replay::error();
+				ERR_NW << "unit '" << u_type->second.id() << "' is too expensive to recruit: "
+				       << u_type->second.cost() << "/" << current_team.gold() << "\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			statistics::recruit_unit(new_unit);
@@ -733,8 +705,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 		else if((child = cfg->child("recall")) != NULL) {
 			player_info* player = state_of_game.get_player(current_team.save_id());
 			if(player == NULL) {
-				std::cerr << "illegal recall\n";
-				throw replay::error();
+				ERR_NW << "illegal recall\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			std::sort(player->available_units.begin(),player->available_units.end(),compare_unit_values());
@@ -750,8 +722,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 				player->available_units.erase(player->available_units.begin()+val);
 				current_team.spend_gold(game_config::recall_cost);
 			} else {
-				std::cerr << "illegal recall\n";
-				throw replay::error();
+				ERR_NW << "illegal recall\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 			fix_shroud = true;
 		}
@@ -759,8 +731,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 		else if((child = cfg->child("disband")) != NULL) {
 			player_info* const player = state_of_game.get_player(current_team.save_id());
 			if(player == NULL) {
-				std::cerr << "illegal disband\n";
-				throw replay::error();
+				ERR_NW << "illegal disband\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			std::sort(player->available_units.begin(),player->available_units.end(),compare_unit_values());
@@ -770,8 +742,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			if(val >= 0 && val < int(player->available_units.size())) {
 				player->available_units.erase(player->available_units.begin()+val);
 			} else {
-				std::cerr << "illegal disband\n";
-				throw replay::error();
+				ERR_NW << "illegal disband\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 		}
 
@@ -781,8 +753,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			const config* const source = child->child("source");
 
 			if(destination == NULL || source == NULL) {
-				std::cerr << "no destination/source found in movement\n";
-				throw replay::error();
+				ERR_NW << "no destination/source found in movement\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			const gamemap::location src(*source);
@@ -790,16 +762,15 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 			std::map<gamemap::location,unit>::iterator u = units.find(dst);
 			if(u != units.end()) {
-				std::cerr << "destination already occupied: "
-						  << (dst.x+1) << "," << (dst.y+1) << "\n";
-				throw replay::error();
+				ERR_NW << "destination already occupied: "
+				       << dst << '\n';
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 			u = units.find(src);
 			if(u == units.end()) {
-				std::cerr << "unfound location for source of movement: "
-				          << (src.x+1) << "," << (src.y+1) << "-"
-						  << (dst.x+1) << "," << (dst.y+1) << "\n";
-				throw replay::error();
+				ERR_NW << "unfound location for source of movement: "
+				       << src << '-' << dst << '\n';
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			const bool ignore_zocs = u->second.type().is_skirmisher();
@@ -814,14 +785,12 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			if(rt == paths_list.routes.end()) {
 
 				for(rt = paths_list.routes.begin(); rt != paths_list.routes.end(); ++rt) {
-					std::cerr << "can get to: " << (rt->first.x+1) << "," << (rt->first.y+1) << "\n";
+					ERR_NW << "can get to: " << rt->first.y << '\n';
 				}
 
-				std::cerr << "src cannot get to dst: " << current_unit.movement_left() << " "
-				          << paths_list.routes.size() << " " << (src.x+1)
-				          << "," << (src.y+1) << "-" << (dst.x+1) << ","
-				          << (dst.y+1) << "\n";
-				throw replay::error();
+				ERR_NW << "src cannot get to dst: " << current_unit.movement_left() << ' '
+				       << paths_list.routes.size() << ' ' << src << '-' << dst << '\n';
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			rt->second.steps.push_back(dst);
@@ -867,8 +836,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			const config* const source = child->child("source");
 
 			if(destination == NULL || source == NULL) {
-				std::cerr << "no destination/source found in attack\n";
-				throw replay::error();
+				ERR_NW << "no destination/source found in attack\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			const gamemap::location src(*source);
@@ -879,22 +848,20 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 
 			std::map<gamemap::location,unit>::iterator u = units.find(src);
 			if(u == units.end()) {
-				std::cerr << "unfound location for source of attack\n";
-				throw replay::error();
+				ERR_NW << "unfound location for source of attack\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			if(size_t(weapon_num) >= u->second.attacks().size()) {
-				std::cerr << "illegal weapon type in attack\n";
-				throw replay::error();
+				ERR_NW << "illegal weapon type in attack\n";
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			std::map<gamemap::location,unit>::const_iterator tgt = units.find(dst);
 
 			if(tgt == units.end()) {
-				std::cerr << "unfound defender for attack: "
-				          << (src.x+1) << "," << (src.y+1) << " -> "
-						  << (dst.x+1) << "," << (dst.y+1) << "\n";
-				throw replay::error();
+				ERR_NW << "unfound defender for attack: " << src << " -> " << dst << '\n';
+				if (!game_config::ignore_replay_errors) throw replay::error();
 			}
 
 			game_events::fire("attack",src,dst);
@@ -924,8 +891,8 @@ bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
 			}
 			fix_shroud = true;
 		} else {
-			std::cerr << "unrecognized action: '" << cfg->write() << "'\n";
-			throw replay::error();
+			ERR_NW << "unrecognized action\n";
+			if (!game_config::ignore_replay_errors) throw replay::error();
 		}
 
 		//Check if we should refresh the shroud, and redraw the minimap/map tiles.
