@@ -1,4 +1,4 @@
-/* $Id: replay.hpp,v 1.28 2004/11/21 22:00:06 silene Exp $ */
+/* $Id: replay.hpp,v 1.33 2005/03/28 19:08:58 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,22 +14,20 @@
 #define REPLAY_H_INCLUDED
 
 #include "config.hpp"
-#include "display.hpp"
+#include "gamestatus.hpp"
 #include "map.hpp"
+#include "random.hpp"
 #include "unit.hpp"
 
+class display;
+
 struct verification_manager
 {
 	verification_manager(const unit_map& units);
 	~verification_manager();
 };
 
-int get_random();
-
-const config* get_random_results();
-void set_random_results(const config& cfg);
-
-class replay
+class replay: public rng
 {
 public:
 	replay();
@@ -47,6 +45,7 @@ public:
 	void save_game(const std::string& label, const config& snapshot,
 	               const config& starting_pos, bool include_replay = true);
 
+	void add_start();
 	void add_recruit(int unit_index, const gamemap::location& loc);
 	void add_recall(int unit_index, const gamemap::location& loc);
 	void add_disband(int unit_index);
@@ -73,12 +72,9 @@ public:
 
 	void undo();
 
-	int get_random();
-	const config* get_random_results() const;
-	void set_random_results(const config& cfg);
-
 	void start_replay();
 	config* get_next_action();
+	void pre_replay();
 
 	bool at_end() const;
 	void set_to_end();
@@ -108,7 +104,6 @@ private:
 	unsigned int pos_;
 
 	config* current_;
-	config* random_;
 
 	game_state saveInfo_;
 
@@ -123,8 +118,8 @@ extern replay recorder;
 //returns true if it got to the end of the turn without data running out
 bool do_replay(display& disp, const gamemap& map, const game_data& gameinfo,
                std::map<gamemap::location,unit>& units,
-			   std::vector<team>& teams, int team_num, const gamestatus& state,
-			   game_state& state_of_game, replay* obj=NULL);
+	       std::vector<team>& teams, int team_num, const gamestatus& state,
+	       game_state& state_of_game, replay* obj=NULL);
 
 //an object which can be made to undo a recorded move
 //unless the transaction is confirmed
