@@ -20,7 +20,7 @@
     slouken@libsdl.org
 */
 
-/* $Id: SDL_ttf.c,v 1.4 2004/11/18 19:22:59 gruikya Exp $ */
+/* $Id: SDL_ttf.c,v 1.7 2005/03/19 10:26:43 gruikya Exp $ */
 
 #include <math.h>
 #include <stdio.h>
@@ -71,6 +71,13 @@
 #define CACHED_BITMAP	0x01
 #define CACHED_PIXMAP	0x02
 
+/*
+ * Allows building with early libfreetype 2.1.x (x < 3)
+ */
+#ifndef FT_PIXEL_MODE_MONO
+#define FT_PIXEL_MODE_MONO ft_pixel_mode_mono
+#endif
+
 /* Cached glyph information */
 typedef struct cached_glyph {
 	int stored;
@@ -432,7 +439,9 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 	if ( ! cached->index ) {
 		cached->index = FT_Get_Char_Index( face, ch );
 	}
+	//error = FT_Load_Glyph( face, cached->index, FT_LOAD_DEFAULT | FT_LOAD_NO_BITMAP );
 	error = FT_Load_Glyph( face, cached->index, FT_LOAD_DEFAULT );
+
 	if( error ) {
 		return error;
 	}
@@ -521,7 +530,7 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 		 * a freetype2 bug because it is inconsistent with the
 		 * freetype2 documentation under FT_Render_Mode section.
 		 * */
-		if ( mono || !FT_IS_SCALABLE(face) ) {
+		if ( mono || !FT_IS_SCALABLE(face) || dst->pixel_mode == FT_PIXEL_MODE_MONO ) {
 			dst->pitch *= 8;
 		}
 
@@ -547,27 +556,18 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 			for( i = 0; i < src->rows; i++ ) {
 				int soffset = i * src->pitch;
 				int doffset = i * dst->pitch;
-				if ( mono ) {
+				if ( mono || dst->pixel_mode == FT_PIXEL_MODE_MONO ) {
 					unsigned char *srcp = src->buffer + soffset;
 					unsigned char *dstp = dst->buffer + doffset;
 					int j;
 					for ( j = 0; j < src->width; j += 8 ) {
 						unsigned char ch = *srcp++;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
+
+						int k;
+						for ( k = 0; k < 8; ++k) {
+							*dstp++ = ch & 0x80 ? 0xff : 0;
+							ch <<= 1;
+						}
 					}
 				} else if ( !FT_IS_SCALABLE(face) ) {
 					/* This special case wouldn't
