@@ -1,4 +1,4 @@
-/* $Id: sdl_utils.cpp,v 1.72 2005/02/09 23:32:01 ydirson Exp $ */
+/* $Id: sdl_utils.cpp,v 1.78 2005/03/26 17:10:32 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,19 +13,19 @@
 
 #include "global.hpp"
 
+#include "config.hpp"
+#include "log.hpp"
+#include "sdl_utils.hpp"
+#include "util.hpp"
+#include "video.hpp"
+#include "wassert.hpp"
+
 #include <algorithm>
 #include <cmath>
 #include <iostream>
 #include <map>
 
-#include "config.hpp"
-#include "game.hpp"
-#include "log.hpp"
-#include "sdl_utils.hpp"
-#include "show_dialog.hpp"
-#include "util.hpp"
-#include "video.hpp"
-#include "wassert.hpp"
+#define ERR_DP LOG_STREAM(err, display)
 
 SDLKey sdl_keysym_from_name(std::string const &keyname)
 {
@@ -842,6 +842,22 @@ bool operator!=(const SDL_Rect& a, const SDL_Rect& b)
 	return !operator==(a,b);
 }
 
+bool operator==(const SDL_Color& a, const SDL_Color& b) {
+	return a.r == b.r && a.g == b.g && a.b == b.b;
+}
+
+bool operator!=(const SDL_Color& a, const SDL_Color& b) {
+	return !operator==(a,b);
+}
+
+SDL_Color inverse(const SDL_Color& colour) {
+	SDL_Color inverse;
+	inverse.r = 255 - colour.r;
+	inverse.g = 255 - colour.g;
+	inverse.b = 255 - colour.b;
+	return inverse;
+}
+
 void pixel_data::read(const config& cfg) {
 	const std::string& red = cfg["red"];
 	const std::string& green = cfg["green"];
@@ -917,3 +933,36 @@ void surface_restorer::cancel()
 {
 	surface_.assign(NULL);
 }
+
+void draw_rectangle(int x, int y, int w, int h, Uint32 colour,surface target)
+{
+	if(x < 0 || y < 0 || x+w >= target->w || y+h >= target->h) {
+		ERR_DP << "Rectangle has illegal co-ordinates: " << x << "," << y
+		          << "," << w << "," << h << "\n";
+		return;
+	}
+
+	SDL_Rect top = {x,y,w,1};
+	SDL_Rect bot = {x,y+h-1,w,1};
+	SDL_Rect left = {x,y,1,h};
+	SDL_Rect right = {x+w-1,y,1,h};
+
+	SDL_FillRect(target,&top,colour);
+	SDL_FillRect(target,&bot,colour);
+	SDL_FillRect(target,&left,colour);
+	SDL_FillRect(target,&right,colour);
+}
+
+void draw_solid_tinted_rectangle(int x, int y, int w, int h,
+                                 int r, int g, int b,
+                                 double alpha, surface target)
+{
+	if(x < 0 || y < 0 || x+w >= target->w || y+h >= target->h) {
+		ERR_DP << "Rectangle has illegal co-ordinates: " << x << "," << y
+		          << "," << w << "," << h << "\n";
+		return;
+	}
+
+	SDL_Rect rect = {x,y,w,h};
+	fill_rect_alpha(rect,SDL_MapRGB(target->format,r,g,b),Uint8(alpha*255),target);
+}
