@@ -1,4 +1,4 @@
-/* $Id: sdl_utils.hpp,v 1.52 2005/02/09 23:32:01 ydirson Exp $ */
+/* $Id: sdl_utils.hpp,v 1.55 2005/03/23 21:30:46 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -122,6 +122,9 @@ SDL_Rect get_non_transperant_portion(surface const &surf);
 
 bool operator==(const SDL_Rect& a, const SDL_Rect& b);
 bool operator!=(const SDL_Rect& a, const SDL_Rect& b);
+bool operator==(const SDL_Color& a, const SDL_Color& b);
+bool operator!=(const SDL_Color& a, const SDL_Color& b);
+SDL_Color inverse(const SDL_Color& colour);
 
 class config; // no need to include config.hpp
 
@@ -215,4 +218,11 @@ private:
 	SDL_Rect rect;
 };
 
+
+void draw_rectangle(int x, int y, int w, int h, Uint32 colour, surface tg);
+
+void draw_solid_tinted_rectangle(int x, int y, int w, int h,
+                                 int r, int g, int b,
+				 double alpha, surface target);
+
 #endif
