@@ -1,4 +1,4 @@
-/* $Id: show_dialog.cpp,v 1.109 2005/02/12 15:29:32 ydirson Exp $ */
+/* $Id: show_dialog.cpp,v 1.124 2005/04/04 20:05:58 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -17,12 +17,14 @@
 
 #include "config.hpp"
 #include "cursor.hpp"
+#include "display.hpp"
 #include "events.hpp"
 #include "font.hpp"
+#include "gettext.hpp"
 #include "help.hpp"
 #include "hotkeys.hpp"
 #include "image.hpp"
-#include "language.hpp"
+#include "key.hpp"
 #include "log.hpp"
 #include "playlevel.hpp"
 #include "show_dialog.hpp"
@@ -30,6 +32,7 @@
 #include "sdl_utils.hpp"
 #include "tooltips.hpp"
 #include "util.hpp"
+#include "video.hpp"
 #include "widgets/button.hpp"
 #include "widgets/menu.hpp"
 #include "widgets/textbox.hpp"
@@ -40,9 +43,9 @@
 #include <iostream>
 #include <numeric>
 
-#define ERR_DP lg::err(lg::display)
-#define LOG_DP lg::info(lg::display)
-#define ERR_G  lg::err(lg::general)
+#define ERR_DP LOG_STREAM(err, display)
+#define LOG_DP LOG_STREAM(info, display)
+#define ERR_G  LOG_STREAM(err, general)
 
 namespace {
 bool is_in_dialog = false;
@@ -84,7 +87,7 @@ dialog_manager::~dialog_manager()
 	SDL_PushEvent(&pb_event);
 }
 
-void draw_dialog_frame(int x, int y, int w, int h, display& disp, const std::string* dialog_style, surface_restorer* restorer)
+void draw_dialog_frame(int x, int y, int w, int h, CVideo &video, const std::string* dialog_style, surface_restorer* restorer)
 {
 	if(dialog_style == NULL) {
 		static const std::string default_style("menu");
@@ -100,13 +103,13 @@ void draw_dialog_frame(int x, int y, int w, int h, display& disp, const std::str
 
 	if(have_border && restorer != NULL) {
 		const SDL_Rect rect = {x-top->h,y-left->w,w+left->w+right->w,h+top->h+bot->h};
-		*restorer = surface_restorer(&disp.video(),rect);
+		*restorer = surface_restorer(&video,rect);
 	} else if(restorer != NULL) {
 		const SDL_Rect rect = {x,y,w,h};
-		*restorer = surface_restorer(&disp.video(),rect);
+		*restorer = surface_restorer(&video,rect);
 	}
 
-	draw_dialog_background(x,y,w,h,disp,*dialog_style);
+	draw_dialog_background(x,y,w,h,video,*dialog_style);
 
 	if(have_border == false) {
 		return;
@@ -115,25 +118,25 @@ void draw_dialog_frame(int x, int y, int w, int h, display& disp, const std::str
 	surface top_image(scale_surface(top,w,top->h));
 
 	if(top_image != NULL) {
-		disp.blit_surface(x,y-top->h,top_image);
+		video.blit_surface(x,y-top->h,top_image);
 	}
 
 	surface bot_image(scale_surface(bot,w,bot->h));
 
 	if(bot_image != NULL) {
-		disp.blit_surface(x,y+h,bot_image);
+		video.blit_surface(x,y+h,bot_image);
 	}
 
 	surface left_image(scale_surface(left,left->w,h));
 
 	if(left_image != NULL) {
-		disp.blit_surface(x-left->w,y,left_image);
+		video.blit_surface(x-left->w,y,left_image);
 	}
 
 	surface right_image(scale_surface(right,right->w,h));
 
 	if(right_image != NULL) {
-		disp.blit_surface(x+w,y,right_image);
+		video.blit_surface(x+w,y,right_image);
 	}
 
 	update_rect(x-left->w,y-top->h,w+left->w+right->w,h+top->h+bot->h);
@@ -146,13 +149,13 @@ void draw_dialog_frame(int x, int y, int w, int h, display& disp, const std::str
 		return;
 	}
 
-	disp.blit_surface(x-top_left->w,y-top_left->h,top_left);
-	disp.blit_surface(x-bot_left->w,y+h,bot_left);
-	disp.blit_surface(x+w,y-top_right->h,top_right);
-	disp.blit_surface(x+w,y+h,bot_right);
+	video.blit_surface(x-top_left->w,y-top_left->h,top_left);
+	video.blit_surface(x-bot_left->w,y+h,bot_left);
+	video.blit_surface(x+w,y-top_right->h,top_right);
+	video.blit_surface(x+w,y+h,bot_right);
 }
 
-void draw_dialog_background(int x, int y, int w, int h, display& disp, const std::string& style)
+void draw_dialog_background(int x, int y, int w, int h, CVideo &video, const std::string& style)
 {
 	const std::string menu_background = "misc/" + style + "-background.png";
 
@@ -162,7 +165,7 @@ void draw_dialog_background(int x, int y, int w, int h, display& disp, const std
 		return;
 	}
 
-	const SDL_Rect& screen_bounds = disp.screen_area();
+	const SDL_Rect& screen_bounds = screen_area();
 	if(x < 0) {
 		w += x;
 		x = 0;
@@ -197,23 +200,21 @@ void draw_dialog_background(int x, int y, int w, int h, display& disp, const std
 			SDL_Rect dst = src;
 			dst.x = x + i;
 			dst.y = y + j;
-			SDL_BlitSurface(bg,&src,disp.video().getSurface(),&dst);
+			SDL_BlitSurface(bg,&src,video.getSurface(),&dst);
 		}
 	}
 }
 
-SDL_Rect draw_dialog_title(int x, int y, display* disp, const std::string& text)
+SDL_Rect draw_dialog_title(int x, int y, CVideo* video, const std::string& text)
 {
 	SDL_Rect rect = {0,0,10000,10000};
-	if(disp != NULL) {
-		rect = disp->screen_area();
-	}
+	rect = screen_area();
 
-	return font::draw_text(disp, rect, font::SIZE_LARGE, font::TITLE_COLOUR,
+	return font::draw_text(video, rect, font::SIZE_LARGE, font::TITLE_COLOUR,
 	                       text, x, y + 5, false, TTF_STYLE_BOLD);
 }
 
-void draw_dialog(int x, int y, int w, int h, display& disp, const std::string& title,
+void draw_dialog(int x, int y, int w, int h, CVideo &video, const std::string& title,
 				 const std::string* style, std::vector<button*>* buttons,
 				 surface_restorer* restorer, button* help_button)
 {
@@ -252,10 +253,10 @@ void draw_dialog(int x, int y, int w, int h, display& disp, const std::string& t
 
 	buttons_area.x += xpos + width;
 
-	draw_dialog_frame(xpos,ypos,width,height,disp,style,restorer);
+	draw_dialog_frame(xpos,ypos,width,height,video,style,restorer);
 
 	if (!title.empty()) {
-		draw_dialog_title(x + border_size, y - title_area.h, &disp, title);
+		draw_dialog_title(x + border_size, y - title_area.h, &video, title);
 	}
 
 	if(buttons != NULL) {
@@ -274,90 +275,8 @@ void draw_dialog(int x, int y, int w, int h, display& disp, const std::string& t
 	}
 }
 
-void draw_rectangle(int x, int y, int w, int h, Uint32 colour,surface target)
-{
-	if(x < 0 || y < 0 || x+w >= target->w || y+h >= target->h) {
-		ERR_DP << "Rectangle has illegal co-ordinates: " << x << "," << y
-		          << "," << w << "," << h << "\n";
-		return;
-	}
-
-	SDL_Rect top = {x,y,w,1};
-	SDL_Rect bot = {x,y+h-1,w,1};
-	SDL_Rect left = {x,y,1,h};
-	SDL_Rect right = {x+w-1,y,1,h};
-
-	SDL_FillRect(target,&top,colour);
-	SDL_FillRect(target,&bot,colour);
-	SDL_FillRect(target,&left,colour);
-	SDL_FillRect(target,&right,colour);
-}
-
-void draw_solid_tinted_rectangle(int x, int y, int w, int h,
-                                 int r, int g, int b,
-                                 double alpha, surface target)
-{
-	if(x < 0 || y < 0 || x+w >= target->w || y+h >= target->h) {
-		ERR_DP << "Rectangle has illegal co-ordinates: " << x << "," << y
-		          << "," << w << "," << h << "\n";
-		return;
-	}
-
-	SDL_Rect rect = {x,y,w,h};
-	fill_rect_alpha(rect,SDL_MapRGB(target->format,r,g,b),Uint8(alpha*255),target);
-}
-
 } //end namespace gui
 
-namespace gui
-{
-
-size_t text_to_lines(std::string& message, size_t max_length)
-{
-	std::string starting_markup;
-	bool at_start = true;
-
-	size_t cur_line = 0, longest_line = 0;
-	for(std::string::iterator i = message.begin(); i != message.end(); ++i) {
-		if(at_start) {
-			if(font::is_format_char(*i)) {
-				push_back(starting_markup,*i);
-			} else {
-				at_start = false;
-			}
-		}
-
-		if(*i == '\n') {
-			at_start = true;
-			starting_markup = "";
-		}
-
-		if(*i == ' ' && cur_line > max_length) {
-			*i = '\n';
-			const size_t index = i - message.begin();
-			message.insert(index+1,starting_markup);
-			i = message.begin() + index + starting_markup.size();
-
-			if(cur_line > longest_line)
-				longest_line = cur_line;
-
-			cur_line = 0;
-		}
-
-		if(*i == '\n' || i+1 == message.end()) {
-			if(cur_line > longest_line)
-				longest_line = cur_line;
-
-			cur_line = 0;
-
-		} else {
-			++cur_line;
-		}
-	}
-
-	return longest_line;
-}
-
 namespace {
 
 struct help_handler : public hotkey::command_executor
@@ -384,6 +303,9 @@ private:
 
 }
 
+namespace gui
+{
+
 void show_error_message(display &disp, std::string const &message)
 {
 	ERR_G << message << std::endl;
@@ -432,8 +354,8 @@ int show_dialog(display& disp, surface image,
 	const bool use_textbox = text_widget_text != NULL;
 	const bool editable_textbox = use_textbox && std::find(text_widget_text->begin(),text_widget_text->end(),'\n') == text_widget_text->end();
 	static const std::string default_text_string = "";
-	const unsigned int text_box_width = 350;
-	textbox text_widget(disp,text_box_width,
+	const unsigned int text_box_width = font::relative_size(350);
+	textbox text_widget(screen,text_box_width,
 	                    use_textbox ? *text_widget_text : default_text_string, editable_textbox, text_widget_max_chars);
 
 	int text_widget_width = 0;
@@ -444,24 +366,25 @@ int show_dialog(display& disp, surface image,
 
 		const SDL_Rect& area = font::text_area(*text_widget_text,message_font_size);
 
-		text_widget.set_width(minimum<size_t>(disp.x()/2,maximum<size_t>(area.w,text_widget.location().w)));
-		text_widget.set_height(minimum<size_t>(disp.y()/2,maximum<size_t>(area.h,text_widget.location().h)));
+		text_widget.set_width(minimum<size_t>(screen.getx()/2,maximum<size_t>(area.w,text_widget.location().w)));
+		text_widget.set_height(minimum<size_t>(screen.gety()/2,maximum<size_t>(area.h,text_widget.location().h)));
 		text_widget_width = font::text_area(text_widget_label,message_font_size).w + text_widget.location().w;;
 		text_widget_height = text_widget.location().h + message_font_size;
 	}
 
-	menu menu_(disp,menu_items,type == MESSAGE);
+	menu menu_(screen,menu_items,type == MESSAGE);
 
 	menu_.set_numeric_keypress_selection(use_textbox == false);
 
-#ifdef USE_TINY_GUI
-	const int max_line_length = 30;
-#else
-	const int max_line_length = 54;
-#endif
+	const size_t left_padding = font::relative_size(10);
+	const size_t right_padding = font::relative_size(10);
+	const size_t image_h_padding = font::relative_size(image != NULL ? 10 : 0);
+	const size_t top_padding = font::relative_size(10);
+	const size_t bottom_padding = font::relative_size(10);
 
-	std::string message = msg;
-	text_to_lines(message,max_line_length);
+	const std::string message = font::word_wrap_text(msg, message_font_size,
+			screen.getx() / 2,
+			screen.gety() / 2);
 
 	SDL_Rect text_size = { 0, 0, 0, 0 };
 	if(!message.empty()) {
@@ -490,14 +413,14 @@ int show_dialog(display& disp, surface image,
 
 		case YES_NO: {
 			static const char* thebuttons[] = { N_("Yes"),
-			                                          N_("No"), ""};
+							    N_("No"), ""};
 			button_list = thebuttons;
 			break;
 		}
 
 		case OK_CANCEL: {
 			static const char* thebuttons[] = { N_("Ok"),
-			                                          N_("Cancel"),""};
+							    N_("Cancel"),""};
 			button_list = thebuttons;
 			break;
 		}
@@ -518,7 +441,7 @@ int show_dialog(display& disp, surface image,
 	if(button_list != NULL) {
 		try {
 			while((*button_list)[0] != '\0') {
-				buttons.push_back(button(disp,gettext(*button_list)));
+				buttons.push_back(button(screen,gettext(*button_list)));
 
 				++button_list;
 			}
@@ -535,7 +458,7 @@ int show_dialog(display& disp, surface image,
 	std::vector<button> check_buttons;
 	if(options != NULL) {
 		for(std::vector<check_item>::const_iterator i = options->begin(); i != options->end(); ++i) {
-			button check_button(disp,i->label,button::TYPE_CHECK);
+			button check_button(screen,i->label,button::TYPE_CHECK);
 			check_button_height += check_button.height() + button_height_padding;
 			check_button_width = maximum<int>(check_button.width(),check_button_width);
 
@@ -545,7 +468,7 @@ int show_dialog(display& disp, surface image,
 
 	if(action_buttons != NULL) {
 		for(std::vector<dialog_button>::const_iterator i = action_buttons->begin(); i != action_buttons->end(); ++i) {
-			button new_button(disp,i->label);
+			button new_button(screen,i->label);
 			check_button_height += new_button.height() + button_height_padding;
 			check_button_width = maximum<int>(new_button.width(),check_button_width);
 
@@ -577,26 +500,24 @@ int show_dialog(display& disp, surface image,
 		}
 	}
 
-	const int left_padding = 10;
-	const int right_padding = 10;
-	const int image_h_padding = image != NULL ? 10 : 0;
-	const int top_padding = 10;
-	const int bottom_padding = 10;
-	const int menu_hpadding = text_size.h > 0 && menu_.height() > 0 ? 10 : 0;
-	const int padding_width = left_padding + right_padding + image_h_padding;
-	const int padding_height = top_padding + bottom_padding + menu_hpadding;
-	const int caption_width = caption_size.w;
-	const int image_width = image != NULL ? image->w : 0;
-	const int image_height = image != NULL ? image->h : 0;
-	const int total_text_height = text_size.h + caption_size.h;
+	const int menu_hpadding = font::relative_size(text_size.h > 0 && menu_.height() > 0 ? 10 : 0);
+	const size_t padding_width = left_padding + right_padding + image_h_padding;
+	const size_t padding_height = top_padding + bottom_padding + menu_hpadding;
+	const size_t caption_width = caption_size.w;
+	const size_t image_width = image != NULL ? image->w : 0;
+	const size_t image_height = image != NULL ? image->h : 0;
+	const size_t total_text_height = text_size.h + caption_size.h;
 
-	int text_width = text_size.w;
+	size_t text_width = text_size.w;
 	if(caption_width > text_width)
 		text_width = caption_width;
+	// Prevent the menu to be larger than the screen
+	if(menu_.width() + image_width + padding_width > size_t(scr->w))
+		menu_.set_width(scr->w - image_width - padding_width);
 	if(menu_.width() > text_width)
 		text_width = menu_.width();
 
-	int total_width = image_width + text_width + padding_width;
+	size_t total_width = image_width + text_width + padding_width;
 
 	if(text_widget_width+left_padding+right_padding > total_width)
 		total_width = text_widget_width+left_padding+right_padding;
@@ -646,10 +567,10 @@ int show_dialog(display& disp, surface image,
 
 	surface_restorer restorer;
 
-	button help_button(disp,_("Help"));
+	button help_button(screen,_("Help"));
 
 	const std::string& title = image == NULL ? caption : "";
-	draw_dialog(xframe,yframe,frame_width,frame_height,disp,title,dialog_style,&buttons_ptr,&restorer,help_topic.empty() ? NULL : &help_button);
+	draw_dialog(xframe,yframe,frame_width,frame_height,screen,title,dialog_style,&buttons_ptr,&restorer,help_topic.empty() ? NULL : &help_button);
 
 	//calculate the positions of the preview panes to the sides of the dialog
 	if(preview_panes != NULL) {
@@ -694,7 +615,7 @@ int show_dialog(display& disp, surface image,
 		                         text_widget_width - text_widget.location().w,
 		                         text_widget_y_unpadded);
 		events::raise_draw_event();
-		font::draw_text(&disp, clipRect, message_font_size,
+		font::draw_text(&screen, clipRect, message_font_size,
 		                font::NORMAL_COLOUR, text_widget_label,
 						xloc + left_padding,text_widget_y_unpadded);
 	}
@@ -709,15 +630,15 @@ int show_dialog(display& disp, surface image,
 		const int x = xloc + left_padding;
 		const int y = yloc + top_padding;
 
-		disp.blit_surface(x,y,image);
+		screen.blit_surface(x,y,image);
 
-		font::draw_text(&disp, clipRect, caption_font_size,
+		font::draw_text(&screen, clipRect, caption_font_size,
 		                font::NORMAL_COLOUR, caption,
 		                xloc+image_width+left_padding+image_h_padding,
 		                yloc+top_padding);
 	}
 
-	font::draw_text(&disp, clipRect, message_font_size,
+	font::draw_text(&screen, clipRect, message_font_size,
 	                font::NORMAL_COLOUR, message,
 	                xloc+image_width+left_padding+image_h_padding,
 	                yloc+top_padding+caption_size.h);
@@ -737,7 +658,7 @@ int show_dialog(display& disp, surface image,
 		}
 	}
 
-	screen.flip();
+	disp.flip();
 
 	CKey key;
 
@@ -893,7 +814,7 @@ int show_dialog(display& disp, surface image,
 			}
 		}
 
-		disp.video().flip();
+		disp.flip();
 		SDL_Delay(10);
 
 		if(action != NULL) {
@@ -936,11 +857,11 @@ int dialog_action_receive_network::do_action()
 	if(res_ != 0)
 		return CONNECTION_COMPLETE;
 	else if(network::current_transfer_stats().first != stats_.first) {
-		lg::info(lg::network) << "continuing connection...\n";
+		LOG_STREAM(info, network) << "continuing connection...\n";
 		return CONNECTION_CONTINUING;
 	} else
 		return CONTINUE_DIALOG;
-};
+}
 
 network::connection dialog_action_receive_network::result() const
 {
