@@ -1,4 +1,4 @@
-/* $Id: show_dialog.hpp,v 1.42 2004/12/19 22:06:33 silene Exp $ */
+/* $Id: show_dialog.hpp,v 1.47 2005/03/23 21:30:46 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,15 +14,14 @@
 #ifndef SHOW_DIALOG_HPP_INCLUDED
 #define SHOW_DIALOG_HPP_INCLUDED
 
-#include "config.hpp"
+class config;
+class CVideo;
+
 #include "cursor.hpp"
-#include "display.hpp"
 #include "font.hpp"
 #include "halo.hpp"
 #include "network.hpp"
 #include "tooltips.hpp"
-#include "unit.hpp"
-#include "video.hpp"
 
 #include "widgets/button.hpp"
 
@@ -47,19 +46,13 @@ private:
 	bool reset_to;
 };
 
-void draw_dialog_frame(int x, int y, int w, int h, display& disp, const std::string* dialog_style=NULL, surface_restorer* restorer=NULL);
+void draw_dialog_frame(int x, int y, int w, int h, CVideo &video, const std::string* dialog_style=NULL, surface_restorer* restorer=NULL);
 
-void draw_dialog_background(int x, int y, int w, int h, display& disp, const std::string& dialog_style);
-
-void draw_rectangle(int x, int y, int w, int h, Uint32 colour, surface tg);
-
-void draw_solid_tinted_rectangle(int x, int y, int w, int h,
-                                 int r, int g, int b,
-								 double alpha, surface target);
+void draw_dialog_background(int x, int y, int w, int h, CVideo &video, const std::string& dialog_style);
 
 //given the location of a dialog, will draw its title.
 //Returns the area the title takes up
-SDL_Rect draw_dialog_title(int x, int y, display* disp, const std::string& text);
+SDL_Rect draw_dialog_title(int x, int y, CVideo* disp, const std::string& text);
 
 //function to draw a dialog on the screen. x,y,w,h give the dimensions of the client area
 //of the dialog. 'title' is the title of the dialog. The title will be displayed at the
@@ -70,7 +63,7 @@ SDL_Rect draw_dialog_title(int x, int y, display* disp, const std::string& text)
 //below the client area.
 //if 'restorer' is present, it will be set to a restorer that will reset the screen area
 //to its original state after the dialog is drawn.
-void draw_dialog(int x, int y, int w, int h, display& disp, const std::string& title,
+void draw_dialog(int x, int y, int w, int h, CVideo &video, const std::string& title,
                  const std::string* dialog_style=NULL, std::vector<button*>* buttons=NULL,
                  surface_restorer* restorer=NULL, button* help_button=NULL);
 
@@ -117,7 +110,7 @@ struct check_item {
 //by 'show_dialog' and shows information about the selection.
 class preview_pane : public widget {
 public:
-	preview_pane(display& disp) : widget(disp) {}
+	preview_pane(CVideo &video) : widget(video) {}
 	virtual ~preview_pane() { tooltips::clear_tooltips(location()); }
 
 	virtual bool show_above() const { return false; }
@@ -125,13 +118,10 @@ public:
 	virtual void set_selection(int index) = 0;
 };
 
-//function to chop up one long string of text into lines
-size_t text_to_lines(std::string& text, size_t max_length);
-
 //if a menu is given, then returns -1 if the dialog was cancelled, and the
 //index of the selection otherwise. If no menu is given, returns the index
 //of the button that was pressed
-int show_dialog(display& screen, surface image,
+int show_dialog(display &screen, surface image,
 				const std::string& caption, const std::string& message,
 				DIALOG_TYPE type=MESSAGE,
 				const std::vector<std::string>* menu_items=NULL,
@@ -146,11 +136,11 @@ int show_dialog(display& screen, surface image,
 				const std::string& help_topic=""
 			 );
 
-void show_error_message(display &screen, std::string const &message);
+void show_error_message(display &disp, std::string const &message);
 
 network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num=0);
 
-void check_quit(display& screen);
+void check_quit(CVideo &video);
 
 }
 
