@@ -1,4 +1,4 @@
-/* $Id: sound.hpp,v 1.10 2003/12/23 22:45:30 uid66289 Exp $ */
+/* $Id: sound.hpp,v 1.11 2004/08/30 01:23:16 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -18,7 +18,7 @@
 namespace sound {
 
 struct manager {
-	manager();
+	manager(bool sound_on);
 	~manager();
 };
 
