@@ -1,8 +1,8 @@
 #ifndef STATISTICS_HPP_INCLUDED
 #define STATISTICS_HPP_INCLUDED
 
+class unit;
 #include "actions.hpp"
-#include "unit.hpp"
 
 namespace statistics
 {
@@ -82,6 +82,8 @@ namespace statistics
 
 	std::vector<std::string> get_categories();
 	stats calculate_stats(int category, int side);
+	
+	extern config replay_verify_stats;
 }
 
 #endif
