@@ -1,4 +1,4 @@
-/* $Id: team.cpp,v 1.78 2005/02/03 19:48:00 silene Exp $ */
+/* $Id: team.cpp,v 1.87 2005/03/27 23:06:17 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -15,19 +15,19 @@
 
 #include "game_config.hpp"
 #include "game_events.hpp"
-#include "gamestatus.hpp"
+#include "game_errors.hpp"
 #include "log.hpp"
 #include "network.hpp"
-#include "replay.hpp"
 #include "team.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <algorithm>
 #include <cstdlib>
 #include <sstream>
 
-#define LOG_NG lg::info(lg::engine)
+#define LOG_NG LOG_STREAM(info, engine)
 
 namespace {
 	std::vector<team>* teams = NULL;
@@ -85,6 +85,8 @@ team::team_info::team_info(const config& cfg)
 	flag = cfg["flag"];
 
 	description = cfg["description"];
+	objectives = cfg["objectives"];
+	objectives_changed = (cfg["objectives_changed"] == "yes");
 
 	const std::string& village_income = cfg["village_gold"];
 	if(village_income.empty())
@@ -94,7 +96,7 @@ team::team_info::team_info(const config& cfg)
 
 	const std::string& enemies_list = cfg["enemy"];
 	if(!enemies_list.empty()) {
-		std::vector<std::string> venemies = config::split(enemies_list);
+		std::vector<std::string> venemies = utils::split(enemies_list);
 		for(std::vector<std::string>::const_iterator i = venemies.begin(); i != venemies.end(); ++i) {
 			enemies.push_back(atoi(i->c_str()));
 		}
@@ -156,7 +158,7 @@ team::team_info::team_info(const config& cfg)
 		village_value = atof(village_val.c_str());
 	}
 
-	std::vector<std::string> recruits = config::split(cfg["recruit"]);
+	std::vector<std::string> recruits = utils::split(cfg["recruit"]);
 	for(std::vector<std::string>::const_iterator i = recruits.begin(); i != recruits.end(); ++i) {
 		can_recruit.insert(*i);
 	}
@@ -166,7 +168,7 @@ team::team_info::team_info(const config& cfg)
 		recruit_pattern = global_ai_params["recruitment_pattern"];
 	}
 
-	recruitment_pattern = config::split(recruit_pattern);
+	recruitment_pattern = utils::split(recruit_pattern);
 
 	//default recruitment pattern is to buy 2 fighters for every 1 archer
 	if(recruitment_pattern.empty()) {
@@ -208,6 +210,8 @@ void team::team_info::write(config& cfg) const
 	cfg["save_id"] = save_id;
 	cfg["flag"] = flag;
 	cfg["description"] = description;
+	cfg["objectives"] = objectives;
+	cfg["objectives_changed"] = objectives_changed ? "yes" : "no";
 
 	char buf[50];
 	sprintf(buf,"%d",income_per_village);
@@ -361,7 +365,7 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 	for(std::vector<config>::const_iterator i = info_.ai_params.begin(); i != info_.ai_params.end(); ++i) {
 		const std::string& time_of_day = (*i)["time_of_day"];
 		if(time_of_day.empty() == false) {
-			const std::vector<std::string>& times = config::split(time_of_day);
+			const std::vector<std::string>& times = utils::split(time_of_day);
 			if(std::count(times.begin(),times.end(),tod.id) == 0) {
 				continue;
 			}
@@ -371,9 +375,9 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 		if(turns.empty() == false) {
 			bool matched = false;
 
-			const std::vector<std::string>& turns_list = config::split(turns);
+			const std::vector<std::string>& turns_list = utils::split(turns);
 			for(std::vector<std::string>::const_iterator j = turns_list.begin(); j != turns_list.end(); ++j) {
-				const std::pair<int,int> range = config::parse_range(*j);
+				const std::pair<int,int> range = utils::parse_range(*j);
 				if(turn >= range.first && turn <= range.second) {
 					matched = true;
 					break;
@@ -390,9 +394,6 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 
 	aggression_ = lexical_cast_default<double>(aiparams_["aggression"],0.5);
 	caution_ = lexical_cast_default<double>(aiparams_["caution"],0.25);
-
-	LOG_NG << "for turn " << turn << ", time of day '" << tod.id << "' set ai_params to: ---\n"
-	       << aiparams_.write() << "\n---\n";
 }
 
 void team::spend_gold(int amount)
@@ -524,6 +525,28 @@ const std::string& team::save_id() const
 	return info_.save_id;
 }
 
+void team::set_objectives(const t_string& new_objectives, bool silently)
+{
+	info_.objectives = new_objectives;
+	if(!silently)
+		info_.objectives_changed = true;
+}
+
+void team::reset_objectives_changed() 
+{
+	info_.objectives_changed = false;
+}
+
+const t_string& team::objectives() const
+{
+	return info_.objectives;
+}
+
+bool team::objectives_changed() const
+{
+	return info_.objectives_changed;
+}
+
 const std::string& team::flag() const
 {
 	return info_.flag;
@@ -695,7 +718,7 @@ void validate_side(int side)
 	}
 
 	if(side < 1 || side > int(teams->size())) {
-		throw gamestatus::game_error("invalid side found in unit definition");
+		throw game::game_error("invalid side found in unit definition");
 	}
 }
 
