@@ -1,4 +1,4 @@
-/* $Id: team.hpp,v 1.52 2005/01/22 10:47:17 silene Exp $ */
+/* $Id: team.hpp,v 1.57 2005/03/27 23:06:17 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -16,10 +16,14 @@
 #include "config.hpp"
 #include "map.hpp"
 
+struct time_of_day;
+
 #include <set>
 #include <string>
 #include <vector>
 
+#include "SDL.h"
+
 //This class stores all the data for a single 'side' (in game nomenclature).
 //e.g. there is only one leader unit per team.
 class team
@@ -73,6 +77,13 @@ public:
 
 		std::string description;
 
+		t_string objectives; /** < Team's objectives for the current level. */
+
+		/** Set to true when the objectives for this time changes.
+		 * Reset to false when the objectives for this team have been
+		 * displayed to the user. */
+		bool objectives_changed;
+
 		enum CONTROLLER { HUMAN, AI, NETWORK, EMPTY };
 		CONTROLLER controller;
 		bool persistent;
@@ -116,6 +127,12 @@ public:
 	const std::string& name() const;
 	const std::string& save_id() const;
 
+	void set_objectives(const t_string& new_objectives, bool silently=false);
+	void reset_objectives_changed();
+
+	const t_string& objectives() const;
+	bool objectives_changed() const;
+
 	bool is_enemy(int n) const {
 		const size_t index = size_t(n-1);
 		if(index < enemies_.size()) {
@@ -219,8 +236,8 @@ struct teams_manager {
 
 bool is_observer();
 
-//function which will validate a side. Throws gamestatus::game_error
+//function which will validate a side. Throws game::game_error
 //if the side is invalid
-void validate_side(int side); //throw gamestatus::game_error
+void validate_side(int side); //throw game::game_error
 
 #endif
