@@ -1,4 +1,4 @@
-/* $Id: terrain.cpp,v 1.25 2004/12/31 21:01:37 isaaccp Exp $ */
+/* $Id: terrain.cpp,v 1.28 2005/03/27 23:06:17 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -12,7 +12,8 @@
 */
 
 #include "global.hpp"
-
+#include "tstring.hpp"
+#include "config.hpp"
 #include "terrain.hpp"
 
 #include <cstdlib>
@@ -45,8 +46,6 @@ terrain_type::terrain_type(const config& cfg)
 		type_ = alias;
 	}
 
-	colour_.read(cfg);
-
 	height_adjust_ = atoi(cfg["unit_height_adjust"].c_str());
 	submerge_ = atof(cfg["submerge"].c_str());
 
@@ -63,7 +62,7 @@ const std::string& terrain_type::symbol_image() const
 	return symbol_image_;
 }
 
-const std::string& terrain_type::name() const
+const t_string& terrain_type::name() const
 {
 	return name_;
 }
@@ -83,11 +82,6 @@ const std::string& terrain_type::type() const
 	return type_;
 }
 
-pixel_data terrain_type::get_rgb() const
-{
-	return colour_;
-}
-
 bool terrain_type::is_light() const
 {
 	return is_light_;
