@@ -1,4 +1,4 @@
-/* $Id: terrain.hpp,v 1.20 2004/10/19 20:49:09 gruikya Exp $ */
+/* $Id: terrain.hpp,v 1.23 2005/03/27 23:06:17 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,10 +13,12 @@
 #ifndef TERRAIN_H_INCLUDED
 #define TERRAIN_H_INCLUDED
 
-#include "config.hpp"
-#include "sdl_utils.hpp"
+class config;
+#include "tstring.hpp"
 
+#include <map>
 #include <string>
+#include <vector>
 
 class terrain_type
 {
@@ -25,7 +27,7 @@ public:
 	terrain_type(const config& cfg);
 
 	const std::string& symbol_image() const;
-	const std::string& name() const;
+	const t_string& name() const;
 	const std::string& id() const;
 
 	//the character representing this terrain
@@ -34,8 +36,6 @@ public:
 	//the underlying type of the terrain
 	const std::string& type() const;
 
-	pixel_data get_rgb() const;
-
 	bool is_light() const;
 	bool is_alias() const;
 
@@ -50,7 +50,7 @@ public:
 private:
 	std::string symbol_image_;
 	std::string id_;
-	std::string name_;
+	t_string name_;
 
 	//the 'letter' is the letter that represents this
 	//terrain type. The 'type' is a list of the 'underlying types'
@@ -58,8 +58,6 @@ private:
 	char letter_;
 	std::string type_;
 
-	pixel_data colour_;
-
 	int height_adjust_;
 
 	double submerge_;
@@ -72,6 +70,6 @@ private:
 void create_terrain_maps(const std::vector<config*>& cfgs,
                          std::vector<char>& terrain_precedence,
                          std::map<char,terrain_type>& letter_to_terrain,
-						 std::map<std::string,terrain_type>& str_to_terrain);
+			 std::map<std::string,terrain_type>& str_to_terrain);
 
 #endif
