@@ -1,5 +1,6 @@
 #include "global.hpp"
 
+#include "config.hpp"
 #include "font.hpp"
 #include "language.hpp"
 #include "log.hpp"
@@ -7,11 +8,12 @@
 #include "theme.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <cstdlib>
 #include <sstream>
 
-#define LOG_DP lg::info(lg::display)
+#define LOG_DP LOG_STREAM(info, display)
 
 namespace {
 	const SDL_Rect empty_rect = {0,0,0,0};
@@ -37,7 +39,7 @@ namespace {
 
 	_rect read_rect(const config& cfg) {
 		_rect rect = { 0, 0, 0, 0 };
-		const std::vector<std::string> items = config::split(cfg["rect"].c_str());
+		const std::vector<std::string> items = utils::split(cfg["rect"].c_str());
 		if(items.size() >= 1)
 			rect.x1 = atoi(items[0].c_str());
 
@@ -67,7 +69,7 @@ namespace {
 	std::string resolve_rect(const std::string& rect_str) {
 		_rect rect = { 0, 0, 0, 0 };
 		std::stringstream resolved;
-		const std::vector<std::string> items = config::split(rect_str.c_str());
+		const std::vector<std::string> items = utils::split(rect_str.c_str());
 		if(items.size() >= 1) {
 			rect.x1 = compute(items[0], ref_rect.x1, ref_rect.x2);
 			resolved << rect.x1;
@@ -137,7 +139,7 @@ namespace {
 			origcfg = topcfg.find_child("partialresolution", "id", cfg["inherits"]);
 			if (origcfg == NULL) {
 				throw config::error("[partialresolution] refers to non-existant [resolution] "
-						    + cfg["inherits"]);
+						    + cfg["inherits"].str());
 			}
 			// expand parent again - not so big a deal, the only thing really
 			// done again is applying he parent's changes, since we would have
@@ -335,7 +337,7 @@ theme::label::label()
 {}
 
 theme::label::label(const config& cfg)
-      : object(cfg), text_(cfg["prefix"] + cfg["text"] + cfg["postfix"]),
+      : object(cfg), text_(cfg["prefix"].str() + cfg["text"].str() + cfg["postfix"].str()),
 	    icon_(cfg["icon"]), font_(atoi(cfg["font_size"].c_str()))
 {
 	if(font_ == 0)
@@ -364,8 +366,8 @@ size_t theme::label::font_size() const
 
 theme::status_item::status_item(const config& cfg)
         : object(cfg),
-		  prefix_(cfg["prefix"] + cfg["prefix_literal"]),
-		  postfix_(cfg["postfix_literal"] + cfg["postfix"]),
+		  prefix_(cfg["prefix"].str() + cfg["prefix_literal"].str()),
+		  postfix_(cfg["postfix_literal"].str() + cfg["postfix"].str()),
           font_(atoi(cfg["font_size"].c_str()))
 {
 	if(font_ == 0)
@@ -409,9 +411,9 @@ theme::menu::menu() : context_(false)
 {}
 
 theme::menu::menu(const config& cfg) : object(cfg), context_(cfg["is_context_menu"] == "true"),
-                                       title_(cfg["title"] + cfg["title_literal"]),
-									   image_(cfg["image"]),
-									   items_(config::split(cfg["items"]))
+                                       title_(cfg["title"].str() + cfg["title_literal"].str()),
+				       image_(cfg["image"]),
+				       items_(utils::split(cfg["items"]))
 {}
 
 bool theme::menu::is_context() const { return context_; }
@@ -422,8 +424,7 @@ const std::string& theme::menu::image() const { return image_; }
 
 const std::vector<std::string>& theme::menu::items() const { return items_; }
 
-theme::theme(const config& cfg, const SDL_Rect& screen) : cfg_(resolve_rects(cfg))
-{
+theme::theme(const config& cfg, const SDL_Rect& screen):cfg_(resolve_rects(cfg)){
 	set_resolution(screen);
 }
 
@@ -454,7 +455,7 @@ bool theme::set_resolution(const SDL_Rect& screen)
 
 	if(current == resolutions.end()) {
 		if(!resolutions.empty())
-			lg::err(lg::display) << "No valid resolution found\n";
+			LOG_STREAM(err, display) << "No valid resolution found\n";
 		return false;
 	}
 
@@ -551,3 +552,30 @@ const SDL_Rect& theme::mini_map_location(const SDL_Rect& screen) const
 {
 	return mini_map_.location(screen);
 }
+
+std::map<std::string, config> theme::known_themes;
+void theme::set_known_themes(const config* cfg){
+        known_themes.clear();
+        if(cfg == NULL)
+	       return;
+	const config& v = *cfg;
+	const config::child_list& known_themes_cfg = v.get_children("theme");
+	
+	for(config::child_list::const_iterator thm = known_themes_cfg.begin(); thm != known_themes_cfg.end(); ++thm) {
+	       std::string thm_name=(**thm)["name"];
+	       if(thm_name!="null" && thm_name!="editor"){
+	              known_themes[thm_name]=(**thm);
+	       }
+	}
+}
+
+std::vector<std::string> theme::get_known_themes(){
+        std::vector<std::string> names;
+ 
+
+        for(std::map<std::string, config>::iterator p_thm=known_themes.begin();p_thm!=known_themes.end();p_thm++){
+	  names.push_back(p_thm->first);
+	}
+        return(names);
+}
+
