@@ -2,9 +2,10 @@
 #define THEME_HPP_INCLUDED
 
 #include <map>
+#include <string>
 #include <vector>
 
-#include "config.hpp"
+class config;
 
 #include "SDL.h"
 
@@ -131,8 +132,13 @@ public:
 	const SDL_Rect& main_map_location(const SDL_Rect& screen) const;
 	const SDL_Rect& mini_map_location(const SDL_Rect& screen) const;
 
+        static void set_known_themes(const config* cfg);
+        static std::vector<std::string> get_known_themes();
+
 private:
-	const config& cfg_;
+	static std::map<std::string, config> known_themes;
+        std::string cur_theme;
+        const config& cfg_;
 	std::vector<panel> panels_;
 	std::vector<label> labels_;
 	std::vector<menu> menus_;
