@@ -4,11 +4,12 @@
 #include "cursor.hpp"
 #include "display.hpp"
 #include "events.hpp"
+#include "filesystem.hpp"
 #include "font.hpp"
 #include "game_config.hpp"
 #include "hotkeys.hpp"
 #include "key.hpp"
-#include "language.hpp"
+#include "gettext.hpp"
 #include "log.hpp"
 #include "preferences.hpp"
 #include "sdl_utils.hpp"
@@ -16,11 +17,13 @@
 #include "titlescreen.hpp"
 #include "util.hpp"
 #include "video.hpp"
+#include "serialization/parser.hpp"
+#include "serialization/preprocessor.hpp"
 
 #include "SDL_ttf.h"
 
-#define LOG_DP lg::info(lg::display)
-#define ERR_DP lg::err(lg::display)
+#define LOG_DP LOG_STREAM(info, display)
+#define ERR_DP LOG_STREAM(err, display)
 
 namespace {
 
@@ -124,7 +127,8 @@ const config get_tips_of_day()
 
 	std::cerr << "Loading tips of day\n";
 	try {
-		cfg.read(preprocess_file("data/tips.cfg"));
+		scoped_istream stream = preprocess_file("data/tips.cfg");
+		read(cfg, *stream);
 	} catch(config::error&) {
 		std::cerr << "Could not read tips.cfg\n";
 	}
@@ -152,8 +156,8 @@ TITLE_RESULT show_title(display& screen, config& tips_of_day, int* ntip)
 	if(title_surface == NULL) {
 		ERR_DP << "Could not find title image\n";
 	} else {
-		screen.blit_surface(0,0,title_surface);
-		update_rect(screen.screen_area());
+		screen.video().blit_surface(0,0,title_surface);
+		update_rect(screen_area());
 
 		LOG_DP << "displayed title image\n";
 	}
@@ -165,13 +169,13 @@ TITLE_RESULT show_title(display& screen, config& tips_of_day, int* ntip)
 	const std::string& version_str = _("Version") + std::string(" ") +
 	                                 game_config::version;
 
-	const SDL_Rect version_area = font::draw_text(NULL,screen.screen_area(),
+	const SDL_Rect version_area = font::draw_text(NULL,screen_area(),
 						      font::SIZE_TINY,
 	                                    font::NORMAL_COLOUR,version_str,0,0);
 	const size_t versiony = screen.y() - version_area.h;
 
 	if(versiony < size_t(screen.y())) {
-		font::draw_text(&screen,screen.screen_area(),
+		font::draw_text(&screen.video(),screen.screen_area(),
 				font::SIZE_TINY,
 				font::NORMAL_COLOUR,version_str,0,versiony);
 	}
@@ -211,21 +215,21 @@ TITLE_RESULT show_title(display& screen, config& tips_of_day, int* ntip)
 	std::vector<button> buttons;
 	size_t b, max_width = 0;
 	for(b = 0; b != nbuttons; ++b) {
-		buttons.push_back(button(screen,sgettext(button_labels[b])));
+		buttons.push_back(button(screen.video(),sgettext(button_labels[b])));
 		buttons.back().set_help_string(sgettext(help_button_labels[b]));
 		max_width = maximum<size_t>(max_width,buttons.back().width());
 	}
 
 	SDL_Rect main_dialog_area = {menu_xbase-padding,menu_ybase-padding,max_width+padding*2,menu_yincr*(nbuttons-1)+buttons.back().height()+padding*2};
 	std::string style = "mainmenu";
-	draw_dialog_frame(main_dialog_area.x,main_dialog_area.y,main_dialog_area.w,main_dialog_area.h,screen,&style);
+	draw_dialog_frame(main_dialog_area.x,main_dialog_area.y,main_dialog_area.w,main_dialog_area.h,screen.video(),&style);
 
 	for(b = 0; b != nbuttons; ++b) {
 		buttons[b].set_width(max_width);
 		buttons[b].set_location(menu_xbase + b*menu_xincr, menu_ybase + b*menu_yincr);
 	}
 
-	gui::button next_tip_button(screen,_("More"),button::TYPE_PRESS,"lite_small");
+	gui::button next_tip_button(screen.video(),_("More"),button::TYPE_PRESS,"lite_small");
 
 	if(tips_of_day.empty()) {
 		tips_of_day = get_tips_of_day();
@@ -249,12 +253,14 @@ TITLE_RESULT show_title(display& screen, config& tips_of_day, int* ntip)
 		area.x = main_dialog_area.x - (game_config::title_tip_x*screen.x())/1024 - area.w;
 		area.y = main_dialog_area.y + main_dialog_area.h - area.h;
 
-		next_tip_button.set_location(area.x+area.w-next_tip_button.location().w - pad,area.y+area.h - pad - next_tip_button.location().h);
+		draw_dialog_frame(area.x,area.y,area.w,area.h,screen.video(),&style);
 
-		draw_dialog_frame(area.x,area.y,area.w,area.h,screen,&style);
+		next_tip_button.set_location(area.x + area.w - next_tip_button.location().w - pad,
+		                             area.y + area.h - pad - next_tip_button.location().h);
 
-		font::draw_text(&screen,area,font::SIZE_NORMAL,font::NORMAL_COLOUR,tip_of_day,area.x+pad,area.y+pad);
-		font::draw_text(&screen, area, font::SIZE_NORMAL, font::NORMAL_COLOUR,
+		font::draw_text(&screen.video(), area, font::SIZE_NORMAL, font::NORMAL_COLOUR,
+		                tip_of_day, area.x + pad, area.y + pad);
+		font::draw_text(&screen.video(), area, font::SIZE_NORMAL, font::NORMAL_COLOUR,
 		                tome, area.x + area.w - tome_area.w - pad,
 		                next_tip_button.location().y - tome_area.h - pad, false, TTF_STYLE_ITALIC);
 	}
@@ -289,7 +295,7 @@ TITLE_RESULT show_title(display& screen, config& tips_of_day, int* ntip)
 		events::raise_process_event();
 		events::raise_draw_event();
 
-		screen.video().flip();
+		screen.flip();
 
 		if(!last_escape && key[SDLK_ESCAPE])
 			return QUIT_GAME;
