@@ -1,9 +1,9 @@
 #include "global.hpp"
 
 #include "font.hpp"
-#include "show_dialog.hpp"
-#include "tooltips.hpp"
 #include "sdl_utils.hpp"
+#include "tooltips.hpp"
+#include "video.hpp"
 
 #include <vector>
 
@@ -20,13 +20,13 @@ bool rectangles_overlap(const SDL_Rect& a, const SDL_Rect& b)
 	return xoverlap && yoverlap;
 }
 
-display* display_ = NULL;
+CVideo* video_ = NULL;
 
 struct tooltip
 {
 	tooltip(const SDL_Rect& r, const std::string& msg) : rect(r), message(msg)
 	{
-		gui::text_to_lines(message,60);
+		font::text_to_lines(message,60);
 	}
 	SDL_Rect rect;
 	std::string message;
@@ -58,14 +58,14 @@ void clear_tooltip()
 
 void show_tooltip(const tooltip& tip)
 {
-	if(display_ == NULL) {
+	if(video_ == NULL) {
 		return;
 	}
 
 	clear_tooltip();
 
 	const SDL_Color bgcolour = {0,0,0,128};
-	SDL_Rect area = display_->screen_area();
+	SDL_Rect area = screen_area();
 	tooltip_handle = font::add_floating_label(tip.message,font_size,font::NORMAL_COLOUR,
 	                                          0,0,0,0,-1,area,font::LEFT_ALIGN,&bgcolour,10);
 
@@ -92,16 +92,16 @@ void show_tooltip(const tooltip& tip)
 
 namespace tooltips {
 
-manager::manager(display& disp)
+manager::manager(CVideo& video)
 {
 	clear_tooltips();
-	display_ = &disp;
+	video_ = &video;
 }
 
 manager::~manager()
 {
 	clear_tooltips();
-	display_ = NULL;
+	video_ = NULL;
 }
 
 void clear_tooltips()
@@ -155,7 +155,7 @@ void process(int mousex, int mousey)
 	current_tooltip = tips.end();
 }
 
-SDL_Rect draw_text(display* gui, const SDL_Rect& area, int size,
+SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
                    const SDL_Color& colour, const std::string& text,
                    int x, int y)
 {
