@@ -1,16 +1,17 @@
 #ifndef TOOLTIPS_HPP_INCLUDED
 #define TOOLTIPS_HPP_INCLUDED
 
-#include "font.hpp"
-#include "video.hpp"
-
 #include "SDL.h"
 
+#include <string>
+
+class CVideo;
+
 namespace tooltips {
 
 struct manager
 {
-	manager(display& disp);
+	manager(CVideo& video);
 	~manager();
 };
 
@@ -22,7 +23,7 @@ void process(int mousex, int mousey);
 
 //a function exactly the same as font::draw_text, but will also register
 //a tooltip
-SDL_Rect draw_text(display* gui, const SDL_Rect& area, int size,
+SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
                    const SDL_Color& colour, const std::string& text,
                    int x, int y);
 
