@@ -1,4 +1,4 @@
-/* $Id: unit.cpp,v 1.121 2005/02/09 23:32:01 ydirson Exp $ */
+/* $Id: unit.cpp,v 1.138 2005/04/07 21:16:50 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,24 +13,25 @@
 
 #include "global.hpp"
 
-#include "font.hpp"
 #include "game_config.hpp"
-#include "gamestatus.hpp"
-#include "language.hpp"
+#include "game_errors.hpp"
+#include "gettext.hpp"
 #include "log.hpp"
 #include "pathfind.hpp"
 #include "preferences.hpp"
-#include "replay.hpp"
+#include "random.hpp"
 #include "unit.hpp"
+#include "unit_types.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <algorithm>
 #include <cstdlib>
 #include <iostream>
 #include <sstream>
 
-#define LOG_UT lg::info(lg::engine)
+#define LOG_UT LOG_STREAM(info, engine)
 
 namespace {
 	const std::string ModificationTypes[] = { "object", "trait", "advance" };
@@ -43,8 +44,8 @@ bool compare_unit_values::operator()(const unit& a, const unit& b) const
 	const int lvla = a.type().level();
 	const int lvlb = b.type().level();
 
-	const std::string& namea = a.type().name();
-	const std::string& nameb = b.type().name();
+	const std::string& namea = a.type().id();
+	const std::string& nameb = b.type().id();
 
 	const int xpa = a.max_experience() - a.experience();
 	const int xpb = b.max_experience() - b.experience();
@@ -54,11 +55,12 @@ bool compare_unit_values::operator()(const unit& a, const unit& b) const
 }
 
 //constructor for reading a unit
-unit::unit(const game_data& data, const config& cfg) : state_(STATE_NORMAL),
-                                           moves_(0), user_end_turn_(false), facingLeft_(true),
-					   resting_(false),
-                                           recruit_(false),
-                                           guardian_(false), upkeep_(UPKEEP_FREE)
+unit::unit(const game_data& data, const config& cfg) : 
+	state_(STATE_NORMAL),
+	moves_(0), user_end_turn_(false), facingLeft_(true),
+	resting_(false),
+	recruit_(false),
+	guardian_(false), upkeep_(UPKEEP_FREE)
 {
 	read(data,cfg);
 }
@@ -76,18 +78,18 @@ unit_race::GENDER unit::generate_gender(const unit_type& type, bool gen)
 //constructor for creating a new unit
 unit::unit(const unit_type* t, int side, bool use_traits, bool dummy_unit, unit_race::GENDER gender) :
                gender_(dummy_unit ? gender : generate_gender(*t,use_traits)),
-               type_(&t->get_gender_unit_type(gender_)), state_(STATE_NORMAL),
-			   hitpoints_(type_->hitpoints()),
-			   maxHitpoints_(type_->hitpoints()),
-               backupMaxHitpoints_(type_->hitpoints()), experience_(0),
-			   maxExperience_(type_->experience_needed()),
-			   backupMaxExperience_(type_->experience_needed()),
-               side_(side), moves_(0),
-               user_end_turn_(false), facingLeft_(side != 1),
-			   maxMovement_(type_->movement()),
-			   backupMaxMovement_(type_->movement()),
-			   recruit_(false), attacks_(type_->attacks()),
-			   backupAttacks_(type_->attacks()),
+	       type_(&t->get_gender_unit_type(gender_)), state_(STATE_NORMAL),
+	       hitpoints_(type_->hitpoints()),
+	       maxHitpoints_(type_->hitpoints()),
+	       backupMaxHitpoints_(type_->hitpoints()), experience_(0),
+	       maxExperience_(type_->experience_needed()),
+	       backupMaxExperience_(type_->experience_needed()),
+	       side_(side), moves_(0),
+	       user_end_turn_(false), facingLeft_(side != 1),
+	       maxMovement_(type_->movement()),
+	       backupMaxMovement_(type_->movement()),
+	       recruit_(false), attacks_(type_->attacks()),
+	       backupAttacks_(type_->attacks()),
                guardian_(false), upkeep_(UPKEEP_FULL_PRICE),
                unrenamable_(false)
 {
@@ -105,28 +107,28 @@ unit::unit(const unit_type* t, int side, bool use_traits, bool dummy_unit, unit_
 
 //constructor for advancing a unit from a lower level
 unit::unit(const unit_type* t, const unit& u) :
-               gender_(u.gender_), variation_(u.variation_),
-               type_(&t->get_gender_unit_type(u.gender_).get_variation(u.variation_)),
-               state_(STATE_NORMAL),
-			   hitpoints_(type_->hitpoints()),
-			   maxHitpoints_(type_->hitpoints()),
-			   backupMaxHitpoints_(type_->hitpoints()),
-			   experience_(0),
-			   maxExperience_(type_->experience_needed()),
-			   backupMaxExperience_(type_->experience_needed()),
-               side_(u.side()), moves_(u.moves_),
-               user_end_turn_(false), facingLeft_(u.facingLeft_),
-			   maxMovement_(type_->movement()),
-			   backupMaxMovement_(type_->movement()),
-			   underlying_description_(u.underlying_description_),
-			   description_(u.description_), recruit_(u.recruit_),
-			   role_(u.role_), statusFlags_(u.statusFlags_),
-			   overlays_(u.overlays_), variables_(u.variables_),
-			   attacks_(type_->attacks()), backupAttacks_(type_->attacks()),
-			   modifications_(u.modifications_),
-			   traitsDescription_(u.traitsDescription_),
-			   guardian_(false), upkeep_(u.upkeep_),
-			   unrenamable_(u.unrenamable_)
+	gender_(u.gender_), variation_(u.variation_),
+	type_(&t->get_gender_unit_type(u.gender_).get_variation(u.variation_)),
+	state_(STATE_NORMAL),
+	hitpoints_(type_->hitpoints()),
+	maxHitpoints_(type_->hitpoints()),
+	backupMaxHitpoints_(type_->hitpoints()),
+	experience_(0),
+	maxExperience_(type_->experience_needed()),
+	backupMaxExperience_(type_->experience_needed()),
+	side_(u.side()), moves_(u.moves_),
+	user_end_turn_(false), facingLeft_(u.facingLeft_),
+	maxMovement_(type_->movement()),
+	backupMaxMovement_(type_->movement()),
+	underlying_description_(u.underlying_description_),
+	description_(u.description_), recruit_(u.recruit_),
+	role_(u.role_), statusFlags_(u.statusFlags_),
+	overlays_(u.overlays_), variables_(u.variables_),
+	attacks_(type_->attacks()), backupAttacks_(type_->attacks()),
+	modifications_(u.modifications_),
+	traitsDescription_(u.traitsDescription_),
+	guardian_(false), upkeep_(u.upkeep_),
+	unrenamable_(u.unrenamable_)
 {
 	validate_side(side_);
 
@@ -134,11 +136,6 @@ unit::unit(const unit_type* t, const unit& u) :
 	apply_modifications();
 	heal_all();
 	statusFlags_.clear();
-
-	//generate traits for the unit if it doesn't already have some
-	//currently removed, because we are testing not giving advancing
-	//traitless units new traits
-//	generate_traits();
 }
 
 void unit::generate_traits()
@@ -160,21 +157,10 @@ void unit::generate_traits()
 	std::vector<std::string> description;
 	
 	for(std::vector<config*>::const_iterator j = traits.begin(); j != traits.end(); ++j) {
-		add_modification("trait",**j);
-		description.push_back((**j)["name"]);
+		modifications_.add_child("trait",**j);
 	}
 
-	traitsDescription_ = "";
-
-	//we want to make sure the description always has a consistent ordering
-	std::sort(description.begin(),description.end());
-	for(std::vector<std::string>::const_iterator i = description.begin(); i != description.end(); ++i) {
-		if(i != description.begin()) {
-			traitsDescription_ += ", ";
-		}
-
-		traitsDescription_ += gettext(i->c_str());
-	}
+	apply_modifications();
 }
 
 const unit_type& unit::type() const
@@ -496,7 +482,7 @@ bool unit::matches_filter(const config& cfg) const
 		return false;
 	}
 
-	const std::string& this_type = this->type().name();
+	const std::string& this_type = this->type().id();
 
 	//the type could be a comma-seperated list of types
 	if(type.empty() == false && type != this_type) {
@@ -507,7 +493,7 @@ bool unit::matches_filter(const config& cfg) const
 		if(std::find(type.begin(),type.end(),',') != type.end() &&
 		   std::search(type.begin(),type.end(),this_type.begin(),
 			           this_type.end()) != type.end()) {
-			const std::vector<std::string>& vals = config::split(type);
+			const std::vector<std::string>& vals = utils::split(type);
 
 			if(std::find(vals.begin(),vals.end(),this_type) == vals.end()) {
 				return false;
@@ -535,7 +521,7 @@ bool unit::matches_filter(const config& cfg) const
 	if(side.empty() == false && this->side() != atoi(side.c_str()))
 	  {
 		if(std::find(side.begin(),side.end(),',') != side.end()) {
-			const std::vector<std::string>& vals = config::split(side);
+			const std::vector<std::string>& vals = utils::split(side);
 
 			std::ostringstream s;
 			s << (this->side());
@@ -616,7 +602,7 @@ void unit::read(const game_data& data, const config& cfg)
 	if(i != data.unit_types.end())
 		type_ = &i->second;
 	else
-		throw gamestatus::load_game_failed("Unit not found: '" + cfg["type"] + "'");
+		throw game::load_game_failed("Unit not found: '" + cfg["type"] + "'");
 
 	wassert(type_ != NULL);
 
@@ -649,6 +635,10 @@ void unit::read(const game_data& data, const config& cfg)
 	validate_side(side_);
 
 	description_ = cfg["user_description"];
+	if(cfg["generate_description"] == "yes") {
+		description_ = type_->generate_description();
+	}
+
 	underlying_description_ = cfg["description"];
 	if(description_.empty()) {
 		description_ = underlying_description_;
@@ -657,7 +647,7 @@ void unit::read(const game_data& data, const config& cfg)
 	custom_unit_description_ = cfg["unit_description"];
 
 	traitsDescription_ = cfg["traits_description"];
-	const std::map<std::string,std::string>::const_iterator recruit_itor = cfg.values.find("canrecruit");
+	const string_map::const_iterator recruit_itor = cfg.values.find("canrecruit");
 	if(recruit_itor != cfg.values.end() && recruit_itor->second == "1") {
 		recruit_ = true;
 	}
@@ -694,7 +684,7 @@ void unit::read(const game_data& data, const config& cfg)
 		}
 	}
 
-	overlays_ = config::split(cfg["overlays"]);
+	overlays_ = utils::split(cfg["overlays"]);
 	if(overlays_.size() == 1 && overlays_.front() == "") {
 		overlays_.clear();
 	}
@@ -736,7 +726,7 @@ void unit::read(const game_data& data, const config& cfg)
 
 void unit::write(config& cfg) const
 {
-	cfg["type"] = type_->name();
+	cfg["type"] = type_->id();
 
 	std::stringstream hp;
 	hp << hitpoints_;
@@ -764,7 +754,7 @@ void unit::write(config& cfg) const
 	cfg.add_child("variables",variables_);
 	cfg.add_child("status",status_flags);
 
-	cfg["overlays"] = config::join(overlays_);
+	cfg["overlays"] = utils::join(overlays_);
 
 	cfg["user_description"] = description_;
 	cfg["description"] = underlying_description_;
@@ -934,7 +924,7 @@ void unit::set_facing_left(bool newval)
 	facingLeft_ = newval;
 }
 
-const std::string& unit::traits_description() const
+const t_string& unit::traits_description() const
 {
 	return traitsDescription_;
 }
@@ -997,7 +987,7 @@ void unit::add_modification(const std::string& type,
 		modifications_.add_child(type,mod);
 	}
 
-	std::vector<std::string> effects_description;
+	std::vector<t_string> effects_description;
 
 	for(config::const_child_itors i = mod.child_range("effect");
 	    i.first != i.second; ++i.first) {
@@ -1005,13 +995,13 @@ void unit::add_modification(const std::string& type,
 		//see if the effect only applies to certain unit types
 		const std::string& type_filter = (**i.first)["unit_type"];
 		if(type_filter.empty() == false) {
-			const std::vector<std::string>& types = config::split(type_filter);
-			if(std::find(types.begin(),types.end(),this->type().name()) == types.end()) {
+			const std::vector<std::string>& types = utils::split(type_filter);
+			if(std::find(types.begin(),types.end(),this->type().id()) == types.end()) {
 				continue;
 			}
 		}
 
-		std::stringstream description;
+		t_string description;
 
 		const std::string& apply_to = (**i.first)["apply_to"];
 
@@ -1036,14 +1026,14 @@ void unit::add_modification(const std::string& type,
 					if(first_attack) {
 						first_attack = false;
 					} else {
-						description << "; ";
+						description += "; ";
 					}
 					
-					description << gettext(a->name().c_str()) << " " << desc;
+					description += a->name() + " " + desc;
 				}
 			}
 		} else if(apply_to == "hitpoints") {
-			LOG_UT << "applying hitpoint mod...." << hitpoints_ << "/" << maxHitpoints_ << "\n";
+			LOG_UT << "applying hitpoint mod..." << hitpoints_ << "/" << maxHitpoints_ << "\n";
 			const std::string& increase_hp = (**i.first)["increase"];
 			const std::string& heal_full = (**i.first)["heal_full"];
 			const std::string& increase_total = (**i.first)["increase_total"];
@@ -1052,7 +1042,8 @@ void unit::add_modification(const std::string& type,
 			const std::string& violate_max = (**i.first)["violate_maximum"];
 
 			if(increase_total.empty() == false) {
-				description << (increase_total[0] != '-' ? "+" : "") << increase_total << _("HP");
+				description += (increase_total[0] != '-' ? "+" : "") + increase_total +
+					t_string(N_("HP"), "wesnoth");
 
 				//a percentage on the end means increase by that many percent
 				if(increase_total[increase_total.size()-1] == '%') {
@@ -1088,7 +1079,8 @@ void unit::add_modification(const std::string& type,
 			const std::string& set_to = (**i.first)["set"];
 
 			if(increase.empty() == false) {
-				description << (increase[0] != '-' ? "+" : "") << increase << _("Moves");
+				description += (increase[0] != '-' ? "+" : "") + increase + 
+					t_string(N_("Moves"), "wesnoth");
 
 				if(increase[increase.size()-1] == '%') {
 					const std::string inc(increase.begin(),increase.end()-1);
@@ -1111,7 +1103,9 @@ void unit::add_modification(const std::string& type,
 			const std::string& increase = (**i.first)["increase"];
 
 			if(increase.empty() == false) {
-				description << (increase[0] != '-' ? "+" : "") << increase << _("XP");
+				description += (increase[0] != '-' ? "+" : "") +
+					increase + t_string(N_("XP"), "wesnoth");
+
 				if(increase[increase.size()-1] == '%') {
 					const std::string inc(increase.begin(),increase.end()-1);
 					maxExperience_ += (maxExperience_*atoi(inc.c_str()))/100;
@@ -1139,30 +1133,31 @@ void unit::add_modification(const std::string& type,
 			}
 		}
 
-		const std::string desc = description.str();
-		if(!desc.empty())
-			effects_description.push_back(desc);
+		if(!description.empty())
+			effects_description.push_back(description);
 	}
 
-	std::stringstream description;
-	description << mod["name"] << ": ";
-	if(mod["id"].empty() == false) {
-		description << mod["description"] << " ";
+	t_string& description = modificationDescriptions_[type];
+
+	// FIXME: the colon must be translatable.
+	description += mod["name"] + ": ";
+	if(!mod["description"].empty()) {
+		description += mod["description"];
+		description += " ";
 	}
 
 	if(effects_description.empty() == false) {
-		description << "(";
-		for(std::vector<std::string>::const_iterator i = effects_description.begin(); i != effects_description.end(); ++i) {
-			description << *i;
+		description += "(";
+		for(std::vector<t_string>::const_iterator i = effects_description.begin(); 
+				i != effects_description.end(); ++i) {
+			description += *i;
 			if(i+1 != effects_description.end())
-				description << "; ";
+				description += "; ";
 		}
-		description << ")";
+		description += ")";
 	}
 
-	description << "\n";
-
-	modificationDescriptions_[type] += description.str();
+	description += "\n";
 }
 
 void unit::reset_modifications()
@@ -1182,14 +1177,33 @@ void unit::apply_modifications()
 	log_scope("apply mods");
 	modificationDescriptions_.clear();
 
+	std::vector<t_string> descriptions;
+
 	for(size_t i = 0; i != NumModificationTypes; ++i) {
 		const std::string& mod = ModificationTypes[i];
 		const config::child_list& mods = modifications_.get_children(mod);
 		for(config::child_list::const_iterator j = mods.begin(); j != mods.end(); ++j) {
 			log_scope("add mod");
 			add_modification(ModificationTypes[i],**j,true);
+
+			const t_string& name = (**j)["name"];
+			if(name.empty() == false) {
+				descriptions.push_back(name);
+			}
 		}
 	}
+
+	traitsDescription_ = "";
+
+	//we want to make sure the description always has a consistent ordering
+	std::sort(descriptions.begin(),descriptions.end());
+	for(std::vector<t_string>::const_iterator j = descriptions.begin(); j != descriptions.end(); ++j) {
+		if(j != descriptions.begin()) {
+			traitsDescription_ += ", ";
+		}
+
+		traitsDescription_ += *j;
+	}
 }
 
 void unit::remove_temporary_modifications()
@@ -1206,11 +1220,11 @@ void unit::remove_temporary_modifications()
 	}
 }
 
-const std::string& unit::modification_description(const std::string& type) const
+const t_string& unit::modification_description(const std::string& type) const
 {
 	const string_map::const_iterator i = modificationDescriptions_.find(type);
 	if(i == modificationDescriptions_.end()) {
-		static const std::string empty_string;
+		static const t_string empty_string;
 		return empty_string;
 	} else {
 		return i->second;
@@ -1221,7 +1235,7 @@ int unit::upkeep() const
 {
 	switch(upkeep_) {
 	case UPKEEP_FREE: return 0;
-	case UPKEEP_LOYAL: return minimum<int>(1,type().level());
+	case UPKEEP_LOYAL: return 0;
 	case UPKEEP_FULL_PRICE: return type().level();
 	default: wassert(false); return 0;
 	}
