@@ -1,4 +1,4 @@
-/* $Id: unit.hpp,v 1.60 2005/02/09 23:32:01 ydirson Exp $ */
+/* $Id: unit.hpp,v 1.67 2005/04/07 21:16:50 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -15,6 +15,7 @@
 
 #include "config.hpp"
 #include "map.hpp"
+#include "race.hpp"
 #include "team.hpp"
 #include "unit_types.hpp"
 
@@ -51,9 +52,7 @@ public:
 	int experience() const;
 	int max_experience() const;
 	bool get_experience(int xp);
-	//the name of this unit is fixed by scenario, it is impossible to
-	//change it.
-	bool unrenamable() const;
+	bool unrenamable() const; /** < Set to true for some scenario-specific units which should not be renamed */
 	bool advances() const;
 	int side() const;
 	unit_race::GENDER gender() const;
@@ -101,6 +100,12 @@ public:
 	void remove_overlay(const std::string& overlay);
 	const std::vector<std::string>& overlays() const;
 
+	/**
+	 * Initializes this unit from a cfg object.
+	 *
+	 * \param data The global game_data object
+	 * \param cfg  Configuration object from which to read the unit
+	 */
 	void read(const game_data& data, const config& cfg);
 
 	void write(config& cfg) const;
@@ -128,7 +133,7 @@ public:
 	bool facing_left() const;
 	void set_facing_left(bool newval);
 
-	const std::string& traits_description() const;
+	const t_string& traits_description() const;
 
 	int value() const;
 	bool is_guardian() const;
@@ -148,7 +153,7 @@ public:
 	void add_modification(const std::string& type, const config& modification,
 	                      bool no_add=false);
 
-	const std::string& modification_description(const std::string& type) const;
+	const t_string& modification_description(const std::string& type) const;
 
 	bool move_interrupted() const;
 	const gamemap::location& get_interrupted_move() const;
@@ -206,7 +211,7 @@ private:
 
 	config modifications_;
 
-	std::string traitsDescription_;
+	t_string traitsDescription_;
 
 	string_map modificationDescriptions_;
 
@@ -227,6 +232,7 @@ private:
 	void apply_modifications();
 	void remove_temporary_modifications();
 	void generate_traits();
+	void generate_traits_description();
 };
 
 //object which temporarily resets a unit's movement
