@@ -1,7 +1,8 @@
 #ifndef UNIT_DISPLAY_HPP_INCLUDED
 #define UNIT_DISPLAY_HPP_INCLUDED
 
-#include "display.hpp"
+class display;
+
 #include "unit.hpp"
 
 ///the unit_display namespace contains a number of free functions
@@ -16,7 +17,7 @@ void move_unit(display& disp, const gamemap& map, const std::vector<gamemap::loc
 
 ///a function to show a unit fading out. Note that this only shows the effect, it doesn't
 ///actually kill the unit.
-void unit_die(display& disp, const gamemap::location& loc, const unit& u);
+void unit_die(display& disp, const gamemap::location& loc, const unit& u, const attack_type* attack=NULL);
 
 ///a function to make the unit on tile 'a' attack the unit on tile 'b'.
 ///the 'damage' will be subtracted from the unit's hitpoints, and a die effect will be
