@@ -1,4 +1,4 @@
-/* $Id: unit_types.cpp,v 1.79 2005/02/19 09:26:31 isaaccp Exp $ */
+/* $Id: unit_types.cpp,v 1.96 2005/03/29 17:41:09 darthfool Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,24 +14,17 @@
 #include "global.hpp"
 
 #include "game_config.hpp"
-#include "gamestatus.hpp"
-#include "language.hpp"
+#include "gettext.hpp"
 #include "log.hpp"
 #include "unit_types.hpp"
 #include "util.hpp"
 #include "wassert.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <algorithm>
 #include <cstdlib>
 #include <iostream>
 
-//these headers are used to check for file existence
-#ifdef linux
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#endif
-
 unit_animation::frame::frame(const config& cfg)
 {
 	xoffset = atoi(cfg["xoffset"].c_str()); 
@@ -146,7 +139,7 @@ attack_type::attack_type(const config& cfg)
 		if(dir == "") {
 			animation_.push_back(unit_animation(**an));
 		} else {
-			const std::vector<std::string>& directions = config::split(dir);
+			const std::vector<std::string>& directions = utils::split(dir);
 			for(std::vector<std::string>::const_iterator i = directions.begin(); i != directions.end(); ++i) {
 				const gamemap::location::DIRECTION d = gamemap::location::parse_direction(*i);
 				if(d != gamemap::location::NDIRECTIONS) {
@@ -181,7 +174,7 @@ attack_type::attack_type(const config& cfg)
 	  defense_weight_ = 1.0;
 }
 
-const std::string& attack_type::name() const
+const t_string& attack_type::name() const
 {
 	return name_;
 }
@@ -251,7 +244,7 @@ const unit_animation& attack_type::animation(const gamemap::location::DIRECTION
 bool attack_type::matches_filter(const config& cfg) const
 {
 	const std::string& filter_range = cfg["range"];
-	const std::string& filter_name = cfg["name"];
+	const t_string& filter_name = cfg["name"];
 	const std::string& filter_type = cfg["type"];
 	const std::string& filter_special = cfg["special"];
 
@@ -343,9 +336,9 @@ unit_movement_type::unit_movement_type(const config& cfg, const unit_movement_ty
              : cfg_(cfg), parent_(parent)
 {}
 
-const std::string& unit_movement_type::name() const
+const t_string& unit_movement_type::name() const
 {
-	const std::string& res = cfg_["name"];
+	const t_string& res = cfg_["name"];
 	if(res == "" && parent_ != NULL)
 		return parent_->name();
 	else
@@ -381,13 +374,13 @@ int unit_movement_type::movement_cost(const gamemap& map,gamemap::TERRAIN terrai
 		return min_value;
 	}
 
-	const config* movement_costs = cfg_.child("movement costs");
+	const config* movement_costs = cfg_.child("movement_costs");
 
 	int res = -1;
 
 	if(movement_costs != NULL) {
 		if(underlying.size() != 1) {
-			lg::err(lg::config) << "terrain '" << terrain << "' has " << underlying.size() << " underlying names - 0 expected\n";
+			LOG_STREAM(err, config) << "terrain '" << terrain << "' has " << underlying.size() << " underlying names - 0 expected\n";
 			return impassable;
 		}
 
@@ -446,7 +439,7 @@ int unit_movement_type::defense_modifier(const gamemap& map,gamemap::TERRAIN ter
 
 	if(defense != NULL) {
 		if(underlying.size() != 1) {
-			lg::err(lg::config) << "terrain '" << terrain << "' has " << underlying.size() << " underlying names - 0 expected\n";
+			LOG_STREAM(err, config) << "terrain '" << terrain << "' has " << underlying.size() << " underlying names - 0 expected\n";
 			return 100;
 		}
 
@@ -534,10 +527,10 @@ unit_type::unit_type(const unit_type& o)
       leadership_(o.leadership_), illuminates_(o.illuminates_),
       skirmish_(o.skirmish_), teleport_(o.teleport_),
       nightvision_(o.nightvision_), steadfast_(o.steadfast_),
-      can_advance_(o.can_advance_),
+      can_advance_(o.can_advance_), alignment_(o.alignment_),
       movementType_(o.movementType_), possibleTraits_(o.possibleTraits_),
       genders_(o.genders_), defensive_animations_(o.defensive_animations_),
-      teleport_animations_(o.teleport_animations_)
+      teleport_animations_(o.teleport_animations_), death_animations_(o.death_animations_)
 {
 	gender_types_[0] = o.gender_types_[0] != NULL ? new unit_type(*o.gender_types_[0]) : NULL;
 	gender_types_[1] = o.gender_types_[1] != NULL ? new unit_type(*o.gender_types_[1]) : NULL;
@@ -569,7 +562,7 @@ unit_type::unit_type(const config& cfg, const movement_type_map& mv_types,
 		gender_types_[unit_race::FEMALE] = new unit_type(*female_cfg,mv_types,races,traits);
 	}
 
-	const std::vector<std::string> genders = config::split(cfg["gender"]);
+	const std::vector<std::string> genders = utils::split(cfg["gender"]);
 	for(std::vector<std::string>::const_iterator i = genders.begin();
 	    i != genders.end(); ++i) {
 		if(*i == "male") {
@@ -586,12 +579,24 @@ unit_type::unit_type(const config& cfg, const movement_type_map& mv_types,
 	const race_map::const_iterator race_it = races.find(cfg["race"]);
 	if(race_it != races.end()) {
 		race_ = &race_it->second;
+		if(race_ != NULL) {
+			if(race_->uses_global_traits() == false) {
+				possibleTraits_.clear();
+			}
+
+			const config::child_list& traits = race_->additional_traits();
+			possibleTraits_.insert(possibleTraits_.end(),traits.begin(),traits.end());
+		}
 	} else {
 		static const unit_race dummy_race;
 		race_ = &dummy_race;
 	}
 
-	abilities_ = config::split(cfg_["ability"]);
+	//insert any traits that are just for this unit type
+	const config::child_list& unit_traits = cfg.get_children("trait");
+	possibleTraits_.insert(possibleTraits_.end(),unit_traits.begin(),unit_traits.end());
+
+	abilities_ = utils::split(cfg_["ability"]);
 
 	//if the string was empty, split will give us one empty string in the list,
 	//remove it.
@@ -616,6 +621,18 @@ unit_type::unit_type(const config& cfg, const movement_type_map& mv_types,
 	teleport_ = has_ability("teleport");
 	nightvision_ = has_ability("night vision");
 	steadfast_ = has_ability("steadfast");
+	
+	const std::string& align = cfg_["alignment"];
+	if(align == "lawful")
+		alignment_ = LAWFUL;
+	else if(align == "chaotic")
+		alignment_ = CHAOTIC;
+	else if(align == "neutral")
+		alignment_ = NEUTRAL;
+	else {
+		LOG_STREAM(err, config) << "Invalid alignment found for " << id() << ": '" << align << "'\n";
+		alignment_ = NEUTRAL;
+	}
 
 	const std::string& alpha_blend = cfg_["alpha"];
 	if(alpha_blend.empty() == false) {
@@ -640,6 +657,11 @@ unit_type::unit_type(const config& cfg, const movement_type_map& mv_types,
 	for(config::child_list::const_iterator t = teleports.begin(); t != teleports.end(); ++t) {
 		teleport_animations_.push_back(unit_animation(**t));
 	}
+
+	const config::child_list& deaths = cfg_.get_children("death");
+	for(config::child_list::const_iterator death = deaths.begin(); death != deaths.end(); ++death) {
+		death_animations_.push_back(death_animation(**death));
+	}
 }
 
 unit_type::~unit_type()
@@ -689,21 +711,23 @@ const std::string& unit_type::id() const
 			id_ = cfg_["name"];
 		}
 
-		id_.erase(std::remove(id_.begin(),id_.end(),' '),id_.end());
+		//id_.erase(std::remove(id_.begin(),id_.end(),' '),id_.end());
 	}
 
 	return id_;
 }
 
-std::string unit_type::language_name() const
+const t_string& unit_type::language_name() const
 {
 	return cfg_["name"];
 }
 
+#if 0
 const std::string& unit_type::name() const
 {
 	return cfg_["id"];
 }
+#endif
 
 const std::string& unit_type::image() const
 {
@@ -868,9 +892,21 @@ std::vector<std::string> unit_type::advances_to() const
 	if(val == "null" || val == "")
 		return std::vector<std::string>();
 	else
-		return config::split(val);
+		return utils::split(val);
 }
 
+const config::child_list& unit_type::modification_advancements() const
+{
+	return cfg_.get_children("advancement");
+}
+
+const std::string& unit_type::undead_variation() const
+{
+        return cfg_["undead_variation"];
+}
+
+
+
 const std::string& unit_type::usage() const
 {
 	return cfg_["usage"];
@@ -888,17 +924,7 @@ int unit_type::movement() const
 
 unit_type::ALIGNMENT unit_type::alignment() const
 {
-	const std::string& align = cfg_["alignment"];
-	if(align == "lawful")
-		return LAWFUL;
-	else if(align == "chaotic")
-		return CHAOTIC;
-	else if(align == "neutral")
-		return NEUTRAL;
-	else {
-		lg::err(lg::config) << "Invalid alignment found for " << name() << ": '" << align << "'\n";
-		return NEUTRAL;
-	}
+	return alignment_;
 }
 
 const char* unit_type::alignment_description(unit_type::ALIGNMENT align)
@@ -1030,15 +1056,49 @@ bool unit_type::defensive_animation::matches(bool h, attack_type::RANGE r) const
 	}
 }
 
+unit_type::death_animation::death_animation(const config& cfg)
+: damage_type(utils::split(cfg["damage_type"])), special(utils::split(cfg["attack_special"])), animation(cfg)
+{
+}
+
+bool unit_type::death_animation::matches(const attack_type* attack) const
+{
+	if(attack == NULL) {
+		return true;
+	}
+
+	if(damage_type.empty() == false && std::find(damage_type.begin(),damage_type.end(),attack->type()) == damage_type.end()) {
+		return false;
+	}
+
+	if(special.empty() == false && std::find(special.begin(),special.end(),attack->special()) == special.end()) {
+		return false;
+	}
+
+	return true;
+}
+
 const unit_animation* unit_type::defend_animation(bool hits, attack_type::RANGE range) const
 {
+	//select one of the matching animations at random
+	const unit_animation* res = NULL;
+	std::vector<const unit_animation*> options;
 	for(std::vector<defensive_animation>::const_iterator i = defensive_animations_.begin(); i != defensive_animations_.end(); ++i) {
 		if(i->matches(hits,range)) {
-			return &i->animation;
+			if(res != NULL) {
+				options.push_back(res);
+			}
+
+			res = &i->animation;
 		}
 	}
 
-	return NULL;
+	if(options.empty()) {
+		return res;
+	} else {
+		options.push_back(res);
+		return options[rand()%options.size()];
+	}
 }
 
 const unit_animation* unit_type::teleport_animation( ) const
@@ -1047,6 +1107,36 @@ const unit_animation* unit_type::teleport_animation( ) const
 	return &teleport_animations_[rand() % teleport_animations_.size()];
 }
 
+const unit_animation* unit_type::die_animation(const attack_type* attack) const
+{
+	if(death_animations_.empty()) {
+		return NULL;
+	}
+
+	if(attack == NULL) {
+		return &death_animations_[rand()%death_animations_.size()].animation;
+	}
+
+	const unit_animation* res = NULL;
+	std::vector<const unit_animation*> options;
+	for(std::vector<death_animation>::const_iterator i = death_animations_.begin(); i != death_animations_.end(); ++i) {
+		if(i->matches(attack)) {
+			if(res != NULL) {
+				options.push_back(res);
+			}
+
+			res = &i->animation;
+		}
+	}
+
+	if(options.empty()) {
+		return res;
+	} else {
+		options.push_back(res);
+		return options[rand()%options.size()];
+	}
+}
+
 game_data::game_data()
 {}
 
@@ -1078,7 +1168,7 @@ void game_data::set_config(const config& cfg)
 	for(config::const_child_itors j = cfg.child_range("unit");
 	    j.first != j.second; ++j.first) {
 		const unit_type u_type(**j.first,movement_types,races,unit_traits);
-		unit_types.insert(std::pair<std::string,unit_type>(u_type.name(),u_type));
+		unit_types.insert(std::pair<std::string,unit_type>(u_type.id(),u_type));
 	}
 }
 
