@@ -1,4 +1,4 @@
-/* $Id: unit_types.hpp,v 1.59 2005/02/19 09:26:31 isaaccp Exp $ */
+/* $Id: unit_types.hpp,v 1.67 2005/03/29 17:41:09 darthfool Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -13,11 +13,12 @@
 #ifndef UNIT_TYPES_H_INCLUDED
 #define UNIT_TYPES_H_INCLUDED
 
+#include "animated.hpp"
 #include "config.hpp"
 #include "map.hpp"
 #include "race.hpp"
 #include "team.hpp"
-#include "animated.hpp"
+#include "util.hpp"
 
 #include <string>
 #include <vector>
@@ -78,7 +79,7 @@ public:
 	enum RANGE { SHORT_RANGE, LONG_RANGE };
 
 	attack_type(const config& cfg);
-	const std::string& name() const;
+	const t_string& name() const;
 	const std::string& type() const;
 	const std::string& special() const;
 	const std::string& icon() const;
@@ -101,7 +102,7 @@ public:
 private:
 	std::vector<unit_animation> animation_;
 	std::vector<unit_animation> direction_animation_[6];
-	std::string name_;
+	t_string name_;
 	std::string type_;
 	std::string special_;
 	std::string icon_;
@@ -128,7 +129,7 @@ public:
 	//for at least as long as the class instance
 	unit_movement_type(const config& cfg, const unit_movement_type* parent=NULL);
 
-	const std::string& name() const;
+	const t_string& name() const;
 	int movement_cost(const gamemap& map, gamemap::TERRAIN terrain, int recurse_count=0) const;
 	int defense_modifier(const gamemap& map, gamemap::TERRAIN terrain, int recurse_count=0) const;
 	int damage_against(const attack_type& attack) const;
@@ -164,17 +165,21 @@ public:
 
 	const unit_type& get_gender_unit_type(unit_race::GENDER gender) const;
 	const unit_type& get_variation(const std::string& name) const;
+        //info on the type of unit that the unit reanimates as
+        const std::string& undead_variation() const;
 
 	int num_traits() const;
 
 	std::string generate_description() const;
 
 	//the name of the unit in the current language setting
-	std::string language_name() const;
+	const t_string& language_name() const;
 
-	//unique identifier that doesn't have any whitespace
 	const std::string& id() const;
-	const std::string& name() const;
+	//Disabling this one for consistency with other similar structures,
+	//where name() is always the user-visible, translated, name.
+	//language_name should eventually be renamed name()
+	// const std::string& name() const;
 	const std::string& image() const;
 	const std::string& image_halo() const;
 	const std::string& image_moving() const;
@@ -193,7 +198,7 @@ public:
 
 	int experience_needed() const;
 	std::vector<std::string> advances_to() const;
-	const config::child_list& modification_advancements() const { return cfg_.get_children("advancement"); }
+	const config::child_list& modification_advancements() const;
 	const std::string& usage() const;
 
 	struct experience_accelerator {
@@ -244,6 +249,7 @@ public:
 
 	const unit_animation* defend_animation(bool hits, attack_type::RANGE range) const;
 	const unit_animation* teleport_animation() const;
+	const unit_animation* die_animation(const attack_type* attack) const;
 
 private:
 	void operator=(const unit_type& o);
@@ -273,16 +279,17 @@ private:
 	bool nightvision_;
 	bool steadfast_;
 	bool can_advance_;
+	ALIGNMENT alignment_;
 
 	unit_movement_type movementType_;
 
-	const std::vector<config*>& possibleTraits_;
+	std::vector<config*> possibleTraits_;
 
 	std::vector<unit_race::GENDER> genders_;
 
 	struct defensive_animation
 	{
-		defensive_animation(const config& cfg);
+		explicit defensive_animation(const config& cfg);
 		bool matches(bool hits, attack_type::RANGE range) const;
 
 		enum { HIT, MISS, HIT_OR_MISS } hits;
@@ -293,6 +300,17 @@ private:
 	std::vector<defensive_animation> defensive_animations_;
 
 	std::vector<unit_animation> teleport_animations_;
+
+	struct death_animation
+	{
+		explicit death_animation(const config& cfg);
+		bool matches(const attack_type* attack) const;
+
+		std::vector<std::string> damage_type, special;
+		unit_animation animation;
+	};
+
+	std::vector<death_animation> death_animations_;
 };
 
 struct game_data
