@@ -1,4 +1,4 @@
-/* $Id: button.cpp,v 1.53 2005/01/20 22:49:46 silene Exp $ */
+/* $Id: button.cpp,v 1.62 2005/03/28 08:27:01 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,14 +14,13 @@
 #include "../global.hpp"
 
 #include "button.hpp"
-#include "menu.hpp"
-#include "../display.hpp"
-#include "../game.hpp"
 #include "../font.hpp"
 #include "../image.hpp"
 #include "../log.hpp"
 #include "../util.hpp"
 #include "../video.hpp"
+#include "../wml_separators.hpp"
+#include "serialization/string_utils.hpp"
 
 namespace gui {
 
@@ -30,9 +29,9 @@ const int horizontal_padding = font::SIZE_SMALL;
 const int checkbox_horizontal_padding = font::SIZE_SMALL / 2;
 const int vertical_padding = font::SIZE_SMALL / 2;
 
-button::button(display& disp, const std::string& label, button::TYPE type,
+button::button(CVideo& video, const std::string& label, button::TYPE type,
                std::string button_image_name, SPACE_CONSUMPTION spacing)
-	: widget(disp), label_(label),
+	: widget(video), label_(label),
 	  image_(NULL), pressedImage_(NULL), activeImage_(NULL), pressedActiveImage_(NULL),
 	  button_(true), state_(NORMAL), type_(type), enabled_(true), pressed_(false)
 {
@@ -48,29 +47,28 @@ button::button(display& disp, const std::string& label, button::TYPE type,
 	surface button_image(image::get_image(button_image_file,image::UNSCALED));
 	surface pressed_image(image::get_image("buttons/" + button_image_name + "-pressed.png", image::UNSCALED));
 	surface active_image(image::get_image("buttons/" + button_image_name + "-active.png", image::UNSCALED));
-	surface pressed_active_image(image::get_image("buttons/" + button_image_name + "-active-pressed.png", image::UNSCALED));
+	surface pressed_active_image;
 
-	if(pressed_image == NULL) {
-		pressed_image.assign(image::get_image(button_image_file,image::UNSCALED));
+	if (pressed_image.null())
+		pressed_image.assign(button_image);
+
+	if (active_image.null())
+		active_image.assign(button_image);
+
+	if (type == TYPE_CHECK) {
+		pressed_active_image.assign(image::get_image("buttons/" + button_image_name + "-active-pressed.png",
+		                                             image::UNSCALED));
+		if (pressed_active_image.null())
+			pressed_active_image.assign(pressed_image);
 	}
 
-	if(active_image == NULL) {
-		active_image.assign(image::get_image(button_image_file,image::UNSCALED));
-	}
-
-	if(pressed_active_image == NULL) {
-		pressed_active_image.assign(image::get_image(button_image_file,image::UNSCALED));
-	}
-
-	if(button_image == NULL) {
-		std::cerr << "could not find button image: '" << button_image_file << "'\n";
+	if (button_image.null())
 		throw error();
-	}
 
 	textRect_.x = 0;
 	textRect_.y = 0;
-	textRect_.w = disp.x();
-	textRect_.h = disp.y();
+	textRect_.w = video.getx();
+	textRect_.h = video.gety();
 
 	textRect_ = font::draw_text(NULL,textRect_,font_size,
 	                            font::BUTTON_COLOUR,label_,0,0);
@@ -93,14 +91,13 @@ button::button(display& disp, const std::string& label, button::TYPE type,
 		image_.assign(scale_surface(button_image,location().w,location().h));
 		pressedImage_.assign(scale_surface(pressed_image,location().w,location().h));
 		activeImage_.assign(scale_surface(active_image,location().w,location().h));
-		pressedActiveImage_.assign(scale_surface(pressed_active_image,location().w,location().h));
-
 	} else {
 		set_width(checkbox_horizontal_padding + textRect_.w + button_image->w);
 		image_.assign(scale_surface(button_image,button_image->w,button_image->h));
 		pressedImage_.assign(scale_surface(pressed_image,button_image->w,button_image->h));
 		activeImage_.assign(scale_surface(active_image,button_image->w,button_image->h));
-		pressedActiveImage_.assign(scale_surface(pressed_active_image,button_image->w,button_image->h));
+		if (type == TYPE_CHECK)
+			pressedActiveImage_.assign(scale_surface(pressed_active_image, button_image->w, button_image->h));
 	}
 }
 
@@ -157,7 +154,7 @@ void button::draw_contents()
 		break;
 	}
 
-	SDL_Rect const &clipArea = disp().screen_area();
+	SDL_Rect const &clipArea = screen_area();
 	SDL_Rect const &loc = location();
 	const int texty = loc.y + loc.h / 2 - textRect_.h / 2 + offset;
 	int textx;
@@ -170,9 +167,9 @@ void button::draw_contents()
 	if (!enabled_)
 		image = greyscale_image(image);
 
-	disp().blit_surface(loc.x, loc.y, image);
+	video().blit_surface(loc.x, loc.y, image);
 	const std::string etext = font::make_text_ellipsis(label_, font_size, loc.w);
-	font::draw_text(&disp(), clipArea, font_size, font::BUTTON_COLOUR, etext, textx, texty);
+	font::draw_text(&video(), clipArea, font_size, font::BUTTON_COLOUR, etext, textx, texty);
 
 	update_rect(loc);
 }
@@ -192,14 +189,14 @@ void button::set_label(const std::string& val)
 
 	//if we have a list of items, use the first one that isn't an image
 	if (std::find(label_.begin(), label_.end(), COLUMN_SEPARATOR) != label_.end()) {
-		const std::vector<std::string>& items = config::split(label_, COLUMN_SEPARATOR);
+		const std::vector<std::string>& items = utils::split(label_, COLUMN_SEPARATOR);
 		const std::vector<std::string>::const_iterator i = std::find_if(items.begin(),items.end(),not_image);
 		if(i != items.end()) {
 			label_ = *i;
 		}
 	}
 
-	textRect_ = disp().screen_area();
+	textRect_ = screen_area();
 	const std::string etext = font::make_text_ellipsis(label_, font_size, width());
 	textRect_ = font::draw_text(NULL,textRect_,font_size,
 	                            font::BUTTON_COLOUR,etext,0,0);
