@@ -1,4 +1,4 @@
-/* $Id: button.hpp,v 1.29 2004/11/06 13:13:16 silene Exp $ */
+/* $Id: button.hpp,v 1.30 2005/03/18 21:21:48 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -23,8 +23,6 @@
 #include <vector>
 #include <string>
 
-class display;
-
 namespace gui {
 
 class button : public widget
@@ -36,7 +34,7 @@ public:
 
 	enum SPACE_CONSUMPTION { DEFAULT_SPACE, MINIMUM_SPACE };
 
-	button(display& disp, const std::string& label, TYPE type=TYPE_PRESS,
+	button(CVideo& video, const std::string& label, TYPE type=TYPE_PRESS,
 	       std::string button_image="", SPACE_CONSUMPTION spacing=DEFAULT_SPACE);
 
 	virtual ~button() {}
