@@ -1,4 +1,4 @@
-/* $Id: combo.hpp,v 1.19 2005/01/12 21:33:58 silene Exp $ */
+/* $Id: combo.hpp,v 1.22 2005/03/18 21:21:48 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -15,8 +15,9 @@
 
 #include "SDL.h"
 
+class display;
+
 #include "button.hpp"
-#include "../display.hpp"
 
 namespace gui {
 
@@ -27,6 +28,7 @@ public:
 
 	void set_selected(int val);
 	void set_items(const std::vector<std::string>& items);
+	size_t items_size() const;
 	int selected() const;
 	bool changed();
 
@@ -37,6 +39,7 @@ private:
 	void set_selected_internal(int val);
 	std::vector<std::string> items_;
 	int selected_, oldSelected_;
+	display* disp_;
 }; //end class combo
 
 }
