@@ -18,6 +18,8 @@
 #include "textbox.hpp"
 #include "widget.hpp"
 
+class display;
+
 namespace gui {
 
 /// A widget where the user may navigate through directories and choose
@@ -93,6 +95,7 @@ private:
 	label current_path_label_;
 	bool choice_made_;
 	int last_selection_;
+	display& disp_;
 };
 
 }
