@@ -1,4 +1,4 @@
-/* $Id: label.hpp,v 1.3 2004/11/06 13:13:16 silene Exp $ */
+/* $Id: label.hpp,v 1.4 2005/03/18 21:21:48 ydirson Exp $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -23,7 +23,7 @@ namespace gui {
 class label : public widget
 {
 public:
-	label(display& d, const std::string& text, int size=font::SIZE_NORMAL, 
+	label(CVideo& video, const std::string& text, int size=font::SIZE_NORMAL, 
 			const SDL_Color& colour=font::NORMAL_COLOUR);
 	const std::string& set_text(const std::string& text);
 	const std::string& get_text() const;
