@@ -1,4 +1,4 @@
-/* $Id: scrollbar.cpp,v 1.26 2005/02/06 10:40:13 isaaccp Exp $*/
+/* $Id: scrollbar.cpp,v 1.31 2005/03/18 21:21:48 ydirson Exp $*/
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
                  2004 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -15,7 +15,6 @@
 #include "../global.hpp"
 
 #include "scrollbar.hpp"
-#include "../display.hpp"
 #include "../image.hpp"
 #include "../video.hpp"
 
@@ -39,10 +38,10 @@ namespace {
 
 namespace gui {
 
-scrollbar::scrollbar(display &d)
-	: widget(d), mid_scaled_(NULL), groove_scaled_(NULL),
-	  uparrow_(d, "", button::TYPE_TURBO, "uparrow-button"),
-	  downarrow_(d, "", button::TYPE_TURBO, "downarrow-button"),
+scrollbar::scrollbar(CVideo &video)
+	: widget(video), mid_scaled_(NULL), groove_scaled_(NULL),
+	  uparrow_(video, "", button::TYPE_TURBO, "uparrow-button"),
+	  downarrow_(video, "", button::TYPE_TURBO, "downarrow-button"),
 	  state_(NORMAL),
 	  grip_position_(0), grip_height_(0), full_height_(0), scroll_rate_(1)
 {
@@ -138,6 +137,17 @@ void scrollbar::set_scroll_rate(unsigned r)
 	scroll_rate_ = r;
 }
 
+bool scrollbar::is_valid_height(int height) const
+{
+	int uh = uparrow_.height();
+	int dh = downarrow_.height();
+	if(uh + dh >= height) {
+		return false;
+	} else {
+		return true;
+	}
+}
+
 void scrollbar::process_event()
 {
 	if (uparrow_.pressed())
@@ -150,8 +160,13 @@ SDL_Rect scrollbar::groove_area() const
 {
 	SDL_Rect loc = location();
 	int uh = uparrow_.height();
-	loc.y += uh;
-	loc.h -= uh + downarrow_.height();
+	int dh = downarrow_.height();
+	if(uh + dh >= loc.h) {
+		loc.h = 0;
+	} else {
+		loc.y += uh;
+		loc.h -= uh + dh;
+	}
 	return loc;
 }
 
@@ -220,17 +235,17 @@ void scrollbar::draw_contents()
 		return;
 	}
 
-	surface const screen = disp().video().getSurface();
+	surface const screen = video().getSurface();
 
 	// draw scrollbar "groove"
-	disp().blit_surface(groove.x, groove.y, top_grv);
-	disp().blit_surface(groove.x, groove.y + top_grv->h, groove_scaled_);
-	disp().blit_surface(groove.x, groove.y + top_grv->h + groove_height, bottom_grv);
+	video().blit_surface(groove.x, groove.y, top_grv);
+	video().blit_surface(groove.x, groove.y + top_grv->h, groove_scaled_);
+	video().blit_surface(groove.x, groove.y + top_grv->h + groove_height, bottom_grv);
 
 	// draw scrollbar "grip"
-	disp().blit_surface(grip.x, grip.y, top_img);
-	disp().blit_surface(grip.x, grip.y + top_img->h, mid_scaled_);
-	disp().blit_surface(grip.x, grip.y + top_img->h + mid_height, bottom_img);
+	video().blit_surface(grip.x, grip.y, top_img);
+	video().blit_surface(grip.x, grip.y + top_img->h, mid_scaled_);
+	video().blit_surface(grip.x, grip.y + top_img->h + mid_height, bottom_img);
 
 	update_rect(groove);
 }
