@@ -1,4 +1,4 @@
-/* $Id: scrollbar.hpp,v 1.14 2005/02/06 10:40:13 isaaccp Exp $ */
+/* $Id: scrollbar.hpp,v 1.17 2005/03/18 21:21:48 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
                  2004 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -31,7 +31,7 @@ public:
 	/// \param d the display object
 	/// \param pane the widget where wheel events take place
 	/// \param callback a callback interface for warning that the grip has been moved
-	scrollbar(display &d);
+	scrollbar(CVideo &video);
 
 	virtual void hide(bool value = true);
 
@@ -60,6 +60,9 @@ public:
 	/// Set scroll rate.
 	void set_scroll_rate(unsigned r);
 
+	/// Return true if the scrollbar has a valid size.
+	bool is_valid_height(int height) const;
+
 protected:
 	virtual void update_location(SDL_Rect const &rect);
 	virtual void handle_event(const SDL_Event& event);
