@@ -1,4 +1,4 @@
-/* $Id: slider.cpp,v 1.36 2004/11/22 23:54:27 ydirson Exp $ */
+/* $Id: slider.cpp,v 1.39 2005/03/18 21:21:48 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,7 +14,6 @@
 #include "../global.hpp"
 
 #include "slider.hpp"
-#include "../display.hpp"
 #include "../image.hpp"
 #include "../video.hpp"
 
@@ -28,8 +27,8 @@ namespace {
 
 namespace gui {
 
-slider::slider(display& d)
-	: widget(d), image_(image::get_image(slider_image, image::UNSCALED)),
+slider::slider(CVideo &video)
+	: widget(video), image_(image::get_image(slider_image, image::UNSCALED)),
 	  highlightedImage_(image::get_image(selected_image, image::UNSCALED)),
 	  min_(-100000), max_(100000), value_(0), 
 	  increment_(1), state_(NORMAL)
@@ -120,13 +119,13 @@ void slider::draw_contents()
 	if (image->w >= loc.w)
 		return;
 
-	surface const screen = disp().video().getSurface();
+	surface const screen = video().getSurface();
 
 	SDL_Rect line_rect = { loc.x + image->w / 2, loc.y + loc.h / 2, loc.w - image->w, 1 };
 	SDL_FillRect(screen, &line_rect, SDL_MapRGB(screen->format, 255, 255, 255));
 
 	SDL_Rect const &slider = slider_area();
-	disp().blit_surface(slider.x, slider.y, image);
+	video().blit_surface(slider.x, slider.y, image);
 }
 
 void slider::set_slider_position(int x)
