@@ -1,4 +1,4 @@
-/* $Id: slider.hpp,v 1.22 2004/11/06 13:42:25 silene Exp $ */
+/* $Id: slider.hpp,v 1.23 2005/03/18 21:21:48 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -26,7 +26,7 @@ namespace gui {
 class slider : public widget
 {
 public:
-	slider(display& d);
+	slider(CVideo &video);
 
 	void set_min(int value);
 	void set_max(int value);
