@@ -1,4 +1,4 @@
-/* $Id: textbox.hpp,v 1.36 2004/11/13 00:27:26 isaaccp Exp $ */
+/* $Id: textbox.hpp,v 1.44 2005/03/18 21:21:48 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,9 +14,7 @@
 #ifndef TEXTBOX_HPP_INCLUDED
 #define TEXTBOX_HPP_INCLUDED
 
-#include "../events.hpp"
-#include "../key.hpp"
-#include "../language.hpp"
+#include "../serialization/string_utils.hpp"
 #include "../sdl_utils.hpp"
 
 #include "scrollarea.hpp"
@@ -28,7 +26,7 @@ namespace gui {
 class textbox : public scrollarea
 {
 public:
-	textbox(display& d, int width, const std::string& text="", bool editable=true, size_t max_size = 256, double alpha = 0.4, double alpha_focus = 0.2);
+	textbox(CVideo &video, int width, const std::string& text="", bool editable=true, size_t max_size = 256, double alpha = 0.4, double alpha_focus = 0.2);
 
 	const std::string text() const;
 	void set_text(const std::string& text);
@@ -43,6 +41,12 @@ public:
 
 	void set_wrap(bool val);
 
+	void set_location(const SDL_Rect& rect);
+
+	//VC++ doesn't like a 'using scrollarea::set_location' directive here, so we declare
+	//an inline forwarding function instead
+	void set_location(int x, int y) { widget::set_location(x,y); }
+
 protected:
 	virtual void draw_contents();
 	virtual void set_inner_location(SDL_Rect const &);
@@ -82,7 +86,7 @@ private:
 
 	void handle_event(const SDL_Event& event);
 
-	void draw_cursor(int pos, display &disp) const;
+	void draw_cursor(int pos, CVideo &video) const;
 	void update_text_cache(bool reset = false);
 	surface add_text_line(const wide_string& text);
 	bool is_selection();
