@@ -1,7 +1,7 @@
 #include "../global.hpp"
 
 #include "widget.hpp"
-#include "../display.hpp"
+#include "../video.hpp"
 
 namespace {
 	const SDL_Rect EmptyRect = {-1234,-1234,0,0};
@@ -10,21 +10,17 @@ namespace {
 namespace gui {
 
 widget::widget(const widget &o)
-	: events::handler(), disp_(o.disp_), restorer_(o.restorer_), rect_(o.rect_),
+	: events::handler(), video_(o.video_), restorer_(o.restorer_), rect_(o.rect_),
 	  focus_(o.focus_), needs_restore_(o.needs_restore_),
-	  state_(o.state_), clip_rect_(o.clip_rect_), volatile_(o.volatile_),
+	  state_(o.state_), clip_(o.clip_), clip_rect_(o.clip_rect_), volatile_(o.volatile_),
 	  help_text_(o.help_text_), help_string_(o.help_string_)
 {
 }
 
-widget::widget(display& disp)
-	: disp_(&disp), rect_(EmptyRect), focus_(true), needs_restore_(false),
-	  state_(UNINIT), volatile_(false), help_string_(0)
+widget::widget(CVideo& video)
+	: video_(&video), rect_(EmptyRect), focus_(true), needs_restore_(false),
+	  state_(UNINIT), clip_(false), volatile_(false), help_string_(0)
 {
-	clip_rect_.x = 0;
-	clip_rect_.y = 0;
-	clip_rect_.w = disp.screen_area().w;
-	clip_rect_.h = disp.screen_area().h;
 }
 
 widget::~widget()
@@ -60,7 +56,7 @@ void widget::update_location(SDL_Rect const &rect)
 
 void widget::bg_register(SDL_Rect const &rect)
 {
-	restorer_.push_back(surface_restorer(&disp_->video(), rect));
+	restorer_.push_back(surface_restorer(&video(), rect));
 }
 
 void widget::set_location(int x, int y)
@@ -131,6 +127,7 @@ void widget::hide(bool value)
 void widget::set_clip_rect(const SDL_Rect& rect)
 {
 	clip_rect_ = rect;
+	clip_ = true;
 	set_dirty(true);
 }
 
@@ -163,7 +160,9 @@ void widget::bg_update()
 
 void widget::bg_restore() const
 {
-	clip_rect_setter set_clip_rect(disp().video().getSurface(), clip_rect_);
+	util::scoped_ptr<clip_rect_setter> clipper(NULL);
+	if (clip_)
+		clipper.assign(new clip_rect_setter(video().getSurface(), clip_rect_));
 
 	if (needs_restore_) {
 		for(std::vector< surface_restorer >::const_iterator i = restorer_.begin(),
@@ -179,7 +178,9 @@ void widget::bg_restore() const
 
 void widget::bg_restore(SDL_Rect const &rect) const
 {
-	clip_rect_setter set_clip_rect(disp().video().getSurface(), clip_rect_);
+	util::scoped_ptr<clip_rect_setter> clipper(NULL);
+	if (clip_)
+		clipper.assign(new clip_rect_setter(video().getSurface(), clip_rect_));
 
 	for(std::vector< surface_restorer >::const_iterator i = restorer_.begin(),
 	    i_end = restorer_.end(); i != i_end; ++i)
@@ -200,7 +201,10 @@ void widget::draw()
 
 	bg_restore();
 
-	clip_rect_setter set_clip_rect(disp().video().getSurface(), clip_rect_);
+	util::scoped_ptr<clip_rect_setter> clipper(NULL);
+	if (clip_)
+		clipper.assign(new clip_rect_setter(video().getSurface(), clip_rect_));
+
 	draw_contents();
 
 	update_rect(rect_);
@@ -233,10 +237,10 @@ void widget::process_help_string(int mousex, int mousey)
 	if (!hidden() && point_in_rect(mousex, mousey, rect_)) {
 		if(help_string_ == 0 && help_text_ != "") {
 			//std::cerr << "setting help string to '" << help_text_ << "'\n";
-			help_string_ = disp().set_help_string(help_text_);
+			help_string_ = video().set_help_string(help_text_);
 		}
 	} else if(help_string_ > 0) {
-		disp().clear_help_string(help_string_);
+		video().clear_help_string(help_string_);
 		help_string_ = 0;
 	}
 }
