@@ -6,9 +6,10 @@
 
 #include "SDL.h"
 
+#include <string>
 #include <vector>
 
-class display;
+class CVideo;
 
 namespace gui {
 
@@ -52,7 +53,7 @@ public:
 
 protected:
 	widget(widget const &o);
-	widget(display& disp);
+	widget(CVideo& video);
 	virtual ~widget();
 
 	// During each relocation, this function should be called to register
@@ -63,7 +64,7 @@ protected:
 	void bg_update();
 	void bg_cancel();
 
-	display& disp() const { return *disp_; }
+	CVideo& video() const { return *video_; };
 
 	virtual void handle_event(SDL_Event const &event) {}
 
@@ -75,13 +76,14 @@ private:
 	void volatile_draw();
 	void volatile_undraw();
 
-	display* disp_;
+	CVideo* video_;
 	std::vector< surface_restorer > restorer_;
 	SDL_Rect rect_;
 	bool focus_;		// Should user input be ignored?
 	mutable bool needs_restore_; // Have we drawn ourselves, so that if moved, we need to restore the background?
 
 	enum { UNINIT, HIDDEN, DIRTY, DRAWN } state_;
+	bool clip_;
 	SDL_Rect clip_rect_;
 
 	bool volatile_;
