@@ -10,9 +10,9 @@ namespace xzipios {
 	public:
 		// explicit XCColl
 		bool hasSubdir(const std::string) const ;
-		void xzipios::XCColl::childrenOf(std::string path, 
-						 std::vector<std::string>* files,
-						 std::vector<std::string>* dirs) const;
+		void childrenOf(std::string path, 
+		                std::vector<std::string>* files,
+		                std::vector<std::string>* dirs) const;
 	};
 }
 
